@@ -177,9 +177,9 @@ colourdisplay_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 		imageinfo_to_text( imageinfo, &buf );
 
 		gtk_selection_data_set( selection_data,
-			gdk_atom_intern( "text/plain", FALSE ),
-			8, buf_all( &buf ), strlen( buf_all( &buf ) ) );
-		
+			gdk_atom_intern( "text/plain", FALSE ), 8, 
+			(guchar *) buf_all( &buf ), strlen( buf_all( &buf ) ) );
+
 #ifdef DEBUG
 		printf( "colourdisplay_drag_data_get: sending text/plain\n" );
 #endif /*DEBUG*/
@@ -242,7 +242,8 @@ colourdisplay_drag_data_received( GtkWidget *widget, GdkDragContext *context,
 		printf( "colourdisplay_drag_data_received: seen text/plain\n" );
 #endif /*DEBUG*/
 
-		if( !imageinfo_from_text( imageinfo, selection_data->data ) )
+		if( !imageinfo_from_text( imageinfo, 
+			(char *) selection_data->data ) )
 			box_alert( widget );
 		break;
 
@@ -253,21 +254,15 @@ colourdisplay_drag_data_received( GtkWidget *widget, GdkDragContext *context,
 }
 
 static void
-colourdisplay_set_tooltip( Colourdisplay *colourdisplay )
+colourdisplay_generate_tooltip( Colourdisplay *colourdisplay, BufInfo *buf )
 {
 	Imagedisplay *id = IMAGEDISPLAY( colourdisplay );
 
 	if( id->conv && id->conv->ii ) {
-		char buf_text[256];
-		BufInfo buf;
-
-		buf_init_static( &buf, buf_text, 256 );
-		imageinfo_to_text( id->conv->ii, &buf );
-		buf_appends( &buf, "\n" );
-		buf_appends( &buf, _( "Double-click to edit this color, or "
+		imageinfo_to_text( id->conv->ii, buf );
+		buf_appends( buf, "\n" );
+		buf_appends( buf, _( "Double-click to edit this color, or "
 			"drag-and-drop between colors" ) );
-
-		set_tooltip( GTK_WIDGET( colourdisplay ), buf_all( &buf ) );
 	}
 }
 
@@ -282,15 +277,6 @@ colourdisplay_conversion_changed( Imagedisplay *id )
 		conversion_set_mag( id->conv, 5000 );
 
 	colourdisplay_set_drag_type( colourdisplay );
-	colourdisplay_set_tooltip( colourdisplay );
-}
-
-static void
-colourdisplay_area_changed( Imagedisplay *id, Rect *display )
-{
-	colourdisplay_set_tooltip( COLOURDISPLAY( id ) );
-
-	IMAGEDISPLAY_CLASS( parent_class )->area_changed( id, display );
 }
 
 static void
@@ -312,7 +298,6 @@ colourdisplay_class_init( ColourdisplayClass *class )
 
 	imagedisplay_class->conversion_changed =
 		colourdisplay_conversion_changed;
-	imagedisplay_class->area_changed = colourdisplay_area_changed;
 }
 
 static void
@@ -326,7 +311,9 @@ colourdisplay_init( Colourdisplay *colourdisplay )
 	 */
 	GTK_WIDGET_UNSET_FLAGS( GTK_WIDGET( colourdisplay ), GTK_CAN_FOCUS );
 
-	colourdisplay_set_tooltip( colourdisplay );
+	set_tooltip_generate( GTK_WIDGET( colourdisplay ), 
+		(TooltipGenerateFn) colourdisplay_generate_tooltip, 
+			NULL, NULL );
 }
 
 GtkType
