@@ -1,4 +1,4 @@
-/* a view of a coloumn
+/* a view of a column
  */
 
 /*
@@ -39,26 +39,6 @@ static ViewClass *parent_class = NULL;
  */
 static GtkWidget *columnview_menu = NULL;
 
-static void
-columnview_destroy( GtkObject *object )
-{
-	Columnview *cview;
-
-#ifdef DEBUG
-	printf( "columnview_destroy\n" );
-#endif /*DEBUG*/
-
-	g_return_if_fail( object != NULL );
-	g_return_if_fail( IS_COLUMNVIEW( object ) );
-
-	cview = COLUMNVIEW( object );
-
-	if( cview->wview && cview->wview->front == cview )
-		cview->wview->front = NULL;
-
-	GTK_OBJECT_CLASS( parent_class )->destroy( object );
-}
-
 /* Edit caption ... right button menu on title bar.
  */
 static void
@@ -153,7 +133,7 @@ columnview_save_as_cb( GtkWidget *wid, GtkWidget *host, Columnview *cview )
 		_( "Save Column \"%s\"" ), IOBJECT( col )->name );
 	filesel_set_flags( FILESEL( filesel ), FALSE, TRUE );
 	filesel_set_filetype( FILESEL( filesel ), filesel_type_workspace, 0 ); 
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( iwnd ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( iwnd ) );
 	idialog_set_iobject( IDIALOG( filesel ), IOBJECT( col ) );
 	filesel_set_done( FILESEL( filesel ), columnview_save_as_sub, col );
 	iwindow_build( IWINDOW( filesel ) );
@@ -167,7 +147,7 @@ columnview_save_as_cb( GtkWidget *wid, GtkWidget *host, Columnview *cview )
 static void
 columnview_filename( char *file, const char *caption )
 {
-	size_t i;
+	int i;
 	char name[FILENAME_MAX];
 
 	im_strncpy( name, caption, 10 );
@@ -177,7 +157,7 @@ columnview_filename( char *file, const char *caption )
 
 	for( i = 0; ; i++ ) {
 		im_snprintf( file, FILENAME_MAX, 
-			"$SAVEDIR" IM_DIR_SEP_STR "data" IM_DIR_SEP_STR 
+			"$SAVEDIR" G_DIR_SEPARATOR_S "data" G_DIR_SEPARATOR_S 
 			"%s-%d.ws", name, i );
 		if( !existsf( "%s", file ) )
 			break;
@@ -231,7 +211,7 @@ columnview_to_menu_done_cb( iWindow *iwnd, void *client,
 		return;
 	}
 
-	SETSTR( columnview_to_menu_last_toolkit, toolkit_text );
+	IM_SETSTR( columnview_to_menu_last_toolkit, toolkit_text );
 
 	nfn( sys, IWINDOW_TRUE );
 }
@@ -271,7 +251,7 @@ columnview_to_menu_cb( GtkWidget *wid, GtkWidget *host, Columnview *cview )
 	idialog_set_help_tag( IDIALOG( ss ), "sec:diaref" );
 	idialog_add_ok( IDIALOG( ss ), columnview_to_menu_done_cb, 
 		_( "Menuize" ) );
-	idialog_set_parent( IDIALOG( ss ), GTK_WIDGET( iwnd ) );
+	iwindow_set_parent( IWINDOW( ss ), GTK_WIDGET( iwnd ) );
 	iwindow_build( IWINDOW( ss ) );
 
 	gtk_widget_show( ss );
@@ -283,11 +263,9 @@ void
 columnview_get_position( Columnview *cview, int *x, int *y, int *w, int *h )
 {
 	Column *col = COLUMN( VOBJECT( cview )->iobject );
-	Workspaceview *wview = cview->wview;
 
-        if( (wview->vp.width == 0 && wview->vp.height == 0) ||
-		(GTK_WIDGET( cview )->allocation.x == -1 && 
-		GTK_WIDGET( cview )->allocation.y == -1) ) {
+        if( GTK_WIDGET( cview )->allocation.x < 2 || 
+		GTK_WIDGET( cview )->allocation.y < 2 ) {
                 /* Nothing there yet, guess.
                  */
 		*x = col->x; 
@@ -309,25 +287,6 @@ columnview_get_position( Columnview *cview, int *x, int *y, int *w, int *h )
         }
 }
 
-/* Pop columnview to front.
- */
-static void
-columnview_front( Columnview *cview )
-{
-	Workspaceview *wview = cview->wview;
-
-	if( wview->front != cview ) {
-		gtk_widget_ref( GTK_WIDGET( cview ) );
-		gtk_container_remove( GTK_CONTAINER( wview->fixed ),
-			GTK_WIDGET( cview ) );
-		gtk_fixed_put( GTK_FIXED( wview->fixed ),
-			GTK_WIDGET( cview ), cview->lx, cview->ly );
-		gtk_widget_unref( GTK_WIDGET( cview ) );
-
-		wview->front = cview;
-	}
-}
-
 /* Transition functions for mouse stuff on columnviews.
  */
 static void
@@ -394,6 +353,41 @@ columnview_left_press( Columnview *cview, GdkEvent *ev )
         }
 }
 
+typedef struct {
+	Column *col;		/* Column we are snapping */
+	Column *best;		/* Closest we've found so far */
+} ColumnviewSnap;
+
+static void *
+columnview_snap_column( Column *col, ColumnviewSnap *snap )
+{
+	if( col != snap->col )
+		if( !snap->best ||
+			IM_ABS( col->x - snap->col->x ) < 
+				IM_ABS( snap->best->x - snap->col->x ) ) 
+			snap->best = col;
+
+	return( NULL );
+}
+
+static void
+columnview_snap( Column *col )
+{
+	const int SNAP_THRESHOLD = 20;
+	Workspace *ws = col->ws;
+	ColumnviewSnap snap;
+	
+	/* Find the column closest to this one.
+	 */
+	snap.col = col;
+	snap.best = NULL;
+	(void) workspace_map_column( ws, 
+		(column_map_fn) columnview_snap_column, &snap );
+
+	if( snap.best && IM_ABS( snap.best->x - col->x ) < SNAP_THRESHOLD ) 
+		col->x = snap.best->x;
+}
+
 static void
 columnview_left_motion( Columnview *cview, GdkEvent *ev )
 {
@@ -444,6 +438,11 @@ columnview_left_motion( Columnview *cview, GdkEvent *ev )
         case COL_DRAG:
 		col->x = xnew;
 		col->y = ynew;
+
+		/* Snap to other column x positions.
+		 */
+		columnview_snap( col );
+
 		iobject_changed( IOBJECT( col ) );
 
 #ifdef DEBUG
@@ -490,7 +489,7 @@ columnview_left_release( Columnview *cview, GdkEvent *ev )
         case COL_SELECT:
                 cview->state = COL_WAIT;
                 workspace_column_select( ws, col );
-                columnview_front( cview );
+                view_child_front( VIEW( cview ) );
 
                 break;
 
@@ -575,39 +574,6 @@ columnview_updown_cb( GtkWidget *wid, Columnview *cview )
 	column_set_open( col, !col->open );
 }
 
-/* Calculate the xywh of the column title bar in the workspaceview.
- */
-static void
-columnview_title_get_position( Columnview *cview,
-        int *x, int *y, int *w, int *h )
-{
-        Column *col = COLUMN( VOBJECT( cview )->iobject );
-
-        *x = col->x;
-        *y = col->y;
-        *w = cview->title->allocation.width;
-        *h = cview->title->allocation.height; 
-}
-
-/* Scroll to show the column title bar.
- */
-static gboolean
-columnview_focus_cb( GtkWidget *widget, GtkDirectionType dir,
-        Columnview *cview )
-{
-        Workspaceview *wview = cview->wview;
-        int x, y, w, h;
-
-#ifdef DEBUG
-	printf( "columnview_focus_cb\n" );
-#endif /*DEBUG*/
-
-	columnview_title_get_position( cview, &x, &y, &w, &h );
-	workspaceview_scroll( wview, x, y, w, h );
-
-        return( FALSE );
-}
-
 /* Delete this column from the popup menu.
  */
 static void
@@ -640,6 +606,10 @@ columnview_caption_enter_cb( GtkWidget *wid, Columnview *cview )
         cview->state = COL_WAIT;
 	iobject_set( IOBJECT( col ), NULL, text );
 	filemodel_set_modified( FILEMODEL( ws ), TRUE );
+
+	/* The ws view needs to update the jumpto menus.
+	 */
+	iobject_changed( IOBJECT( ws ) );
 }
 
 /* Detect cancel in a caption field.
@@ -702,26 +672,6 @@ columnview_text_enter_cb( GtkWidget *wid, Columnview *cview )
 	set_gentry( cview->text, NULL );
 }
 
-/* Scroll to show the new def widget.
- */
-static gboolean
-columnview_text_focus_cb( GtkWidget *widget, GtkDirectionType dir,
-        Columnview *cview )
-{
-        Workspaceview *wview = cview->wview;
-        int x, y, w, h;
-
-#ifdef DEBUG
-	printf( "columnview_text_focus_cb\n" );
-#endif /*DEBUG*/
-
-        columnview_title_get_position( cview, &x, &y, &w, &h );
-        y += cview->main->allocation.height - h;
-        workspaceview_scroll( wview, x, y, w, h );
-
-        return( FALSE );
-}
-
 /* Add bottom entry widget.
  */
 static void
@@ -743,19 +693,15 @@ columnview_add_text( Columnview *cview )
 		cview->text, TRUE, TRUE, 0 );
         gtk_signal_connect( GTK_OBJECT( cview->text ), "activate",
                 GTK_SIGNAL_FUNC( columnview_text_enter_cb ), cview );
-        gtk_signal_connect( GTK_OBJECT( cview->text ), "focus",
-                GTK_SIGNAL_FUNC( columnview_text_focus_cb ), cview );
         gtk_widget_show( cview->text );
         set_tooltip( cview->text, _( "Enter expressions here" ) );
 }
 
-
 static void 
 columnview_refresh( vObject *vobject )
 {
 	Columnview *cview = COLUMNVIEW( vobject );
 	Column *col = COLUMN( VOBJECT( cview )->iobject );
-	Workspaceview *wview = cview->wview;
 	gboolean editable = col->ws->mode != WORKSPACE_MODE_NOEDIT;
 
 	/* Update position.
@@ -768,8 +714,7 @@ columnview_refresh( vObject *vobject )
 
 		cview->lx = col->x;
 		cview->ly = col->y;
-		gtk_fixed_move( GTK_FIXED( wview->fixed ),
-			GTK_WIDGET( cview ), cview->lx, cview->ly );
+		view_child_position( VIEW( cview ) ); 
 
 		/* Update the save offset hints too.
 		 */
@@ -785,10 +730,12 @@ columnview_refresh( vObject *vobject )
 		GtkWidget *label = gtk_frame_get_label_widget( 
 			GTK_FRAME( cview->frame ) );
 		char buf[256];
+		char buf2[256];
 
 		gtk_frame_set_label( GTK_FRAME( cview->frame ), "x" );
 		label = gtk_frame_get_label_widget( GTK_FRAME( cview->frame ) );
-		im_snprintf( buf, 256, "<b>%s</b>", IOBJECT( col )->caption );
+		escape_markup( IOBJECT( col )->caption, buf2, 256 );
+		im_snprintf( buf, 256, "<b>%s</b>", buf2 );
 		gtk_label_set_markup( GTK_LABEL( label ), buf );
 		gtk_misc_set_padding( GTK_MISC( label ), 2, 6 );
 	}
@@ -802,13 +749,13 @@ columnview_refresh( vObject *vobject )
 	 */
 	if( col->open ) {
                 gtk_arrow_set( GTK_ARROW( cview->updown ),
-                        GTK_ARROW_UP, GTK_SHADOW_OUT );
-                set_tooltip( cview->updownb, _( "Fold the column up" ) );
+                        GTK_ARROW_DOWN, GTK_SHADOW_OUT );
+                set_tooltip( cview->updownb, _( "Fold the column away" ) );
 	}
 	else {
                 gtk_arrow_set( GTK_ARROW( cview->updown ),
-                        GTK_ARROW_DOWN, GTK_SHADOW_OUT );
-                set_tooltip( cview->updownb, _( "Fold the column down" ) );
+                        GTK_ARROW_RIGHT, GTK_SHADOW_OUT );
+                set_tooltip( cview->updownb, _( "Open the column" ) );
 	}
 
 	/* Closed columns are hidden in NOEDIT mode.
@@ -847,72 +794,69 @@ columnview_refresh( vObject *vobject )
         if( col->selected && !cview->selected ) {
                 set_name( cview->title, "selected_widget" );
 		cview->selected = TRUE;
-
-		/* Grab focus the 1st time we're selected.
-		 */
-		if( !cview->stale && cview->text ) {
-#ifdef DEBUG
-			printf( "columnview_refresh: grabbing focus for %s\n",
-				IOBJECT( col )->name );
-#endif /*DEBUG*/
-
+		if( cview->textfr )
 			gtk_widget_grab_focus( cview->text );
-			cview->stale = TRUE;
-		}
 	}
-        else if( !col->selected && cview->selected ) {
-                set_name( cview->title, "default_widget" );
+        else if( !col->selected ) {
+		/* Always do this, even if cview->selected, so we set on the
+		 * first _refresh().
+		 */
+                set_name( cview->title, "column_widget" );
 		cview->selected = FALSE;
 	}
 
 	VOBJECT_CLASS( parent_class )->refresh( vobject );
 }
 
+static void
+columnview_link( View *view, Model *model, View *parent )
+{
+	Columnview *cview = COLUMNVIEW( view );
+	Workspaceview *wview = WORKSPACEVIEW( parent );
+
+	VIEW_CLASS( parent_class )->link( view, model, parent );
+
+	cview->wview = wview;
+}
+
+static void
+columnview_child_add( View *parent, View *child )
+{
+	Columnview *cview = COLUMNVIEW( parent );
+	Subcolumnview *sview = SUBCOLUMNVIEW( child );
+
+	VIEW_CLASS( parent_class )->child_add( parent, child );
+
+	gtk_container_add( GTK_CONTAINER( cview->frame ), GTK_WIDGET( sview ) );
+}
+
 /* Scroll to keep the text entry at the bottom of the columnview on screen.
  * We can't use the position/size of the text widget for positioning, since it
  * may not be properly realized yet ... make the bottom of the column visible
  * instead.
  */
 static void
-columnview_scrollto( View *view )
+columnview_scrollto( View *view, ModelScrollPosition position )
 {
 	Columnview *cview = COLUMNVIEW( view );
 	Workspaceview *wview = cview->wview;
 	int x, y, w, h;
 
 	columnview_get_position( cview, &x, &y, &w, &h );
-	workspaceview_scroll( wview, x, y + h, w, 25 );
-}
 
-static void
-columnview_link( View *view, Model *model, View *parent )
-{
-	Columnview *cview = COLUMNVIEW( view );
-	Column *col = COLUMN( model );
-	Workspaceview *wview = WORKSPACEVIEW( parent );
-
-	VIEW_CLASS( parent_class )->link( view, model, parent );
-
-	cview->wview = wview;
-
-	/* Pick start xy pos. 
-	 */
-        workspaceview_pick_xy( wview, &col->x, &col->y );
-	gtk_fixed_put( GTK_FIXED( wview->fixed ),
-		GTK_WIDGET( cview ), cview->lx, cview->ly );
-	cview->lx = -1;
-	cview->ly = -1;
-	wview->front = cview;
-
-	/* Refresh to make sure ws->area is updated.
-	 */
-        vobject_refresh_queue( VOBJECT( cview ) );
+	if( position == MODEL_SCROLL_BOTTOM )
+		/* 35 is supposed to be enough to ensure the whole of the edit
+		 * box gets on the screen.
+		 */
+		workspaceview_scroll( wview, x, y + h, w, 35 );
+	else
+		workspaceview_scroll( wview, 
+			x, y, w, cview->title->allocation.height );
 }
 
 static void
 columnview_class_init( ColumnviewClass *class )
 {
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
 	vObjectClass *vobject_class = (vObjectClass *) class;
 	ViewClass *view_class = (ViewClass *) class;
 
@@ -920,8 +864,6 @@ columnview_class_init( ColumnviewClass *class )
 
 	parent_class = g_type_class_peek_parent( class );
 
-	object_class->destroy = columnview_destroy;
-
 	/* Create signals.
 	 */
 
@@ -929,8 +871,9 @@ columnview_class_init( ColumnviewClass *class )
 	 */
 	vobject_class->refresh = columnview_refresh;
 
-	view_class->scrollto = columnview_scrollto;
 	view_class->link = columnview_link;
+	view_class->child_add = columnview_child_add;
+	view_class->scrollto = columnview_scrollto;
 
 	pane = columnview_menu = popup_build( _( "Column menu" ) );
 	popup_add_but( pane, _( "_Edit Caption" ), 
@@ -965,7 +908,6 @@ columnview_init( Columnview *cview )
 	cview->state = COL_WAIT;
 
 	cview->selected = FALSE;
-	cview->stale = FALSE;
 
         /* Make outer vb.
          */
@@ -983,7 +925,7 @@ columnview_init( Columnview *cview )
         set_tooltip( cview->title, _( "Left-drag to move, left-double-click to "
                 "set title, right-click for menu" ) );
         frame = gtk_frame_new( NULL );
-        gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );
+        gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_NONE );
         gtk_container_add( GTK_CONTAINER( cview->title ), frame );
         popup_attach( cview->title, columnview_menu, cview );
         gtk_signal_connect( GTK_OBJECT( cview->title ), "event",
@@ -999,19 +941,17 @@ columnview_init( Columnview *cview )
         cview->updownb = gtk_button_new();
         gtk_button_set_relief( GTK_BUTTON( cview->updownb ), GTK_RELIEF_NONE );
         gtk_container_set_border_width( GTK_CONTAINER( cview->updownb ), 0 );
-        gtk_box_pack_end( GTK_BOX( cview->titlehb ),
+        gtk_box_pack_start( GTK_BOX( cview->titlehb ),
                 cview->updownb, FALSE, FALSE, 0 );
-        cview->updown = gtk_arrow_new( GTK_ARROW_UP, GTK_SHADOW_OUT );
+        cview->updown = gtk_arrow_new( GTK_ARROW_DOWN, GTK_SHADOW_OUT );
         gtk_container_add( GTK_CONTAINER( cview->updownb ), cview->updown );
         gtk_signal_connect( GTK_OBJECT( cview->updownb ), "clicked",
                 GTK_SIGNAL_FUNC( columnview_updown_cb ), cview );
-        gtk_signal_connect( GTK_OBJECT( cview->updownb ), "focus",
-                GTK_SIGNAL_FUNC( columnview_focus_cb ), cview );
 
         /* Remove columnview button.
          */
         sb = gtk_vbox_new( FALSE, 0 );
-        gtk_box_pack_start( GTK_BOX( cview->titlehb ), sb, FALSE, FALSE, 1 );
+        gtk_box_pack_end( GTK_BOX( cview->titlehb ), sb, FALSE, FALSE, 1 );
         but = gtk_button_new();
         gtk_button_set_relief( GTK_BUTTON( but ), GTK_RELIEF_NONE );
         gtk_box_pack_start( GTK_BOX( sb ), but, TRUE, FALSE, 0 );
@@ -1021,20 +961,6 @@ columnview_init( Columnview *cview )
         gtk_signal_connect( GTK_OBJECT( but ), "clicked",
                 GTK_SIGNAL_FUNC( columnview_destroy2_cb ), cview );
 
-	/*
-		
-		FIXME ... turning this on makes nip scroll to a funny place
-		when loading a workspace :-( part of the strange way
-		scrolledwindows scroll before they're shown
-
-		fix this, making sure focus starts in a sensible place when we
-		load a workspace, and that we scroll to make the focus point
-		visible
-
-        gtk_signal_connect( GTK_OBJECT( but ), "focus",
-                GTK_SIGNAL_FUNC( columnview_focus_cb ), cview );
-	 */
-
         /* Columnview name.
          */
         cview->lab = gtk_label_new( "" );
@@ -1068,10 +994,10 @@ columnview_init( Columnview *cview )
 GtkType
 columnview_get_type( void )
 {
-	static GtkType columnview_type = 0;
+	static GtkType type = 0;
 
-	if( !columnview_type ) {
-		static const GtkTypeInfo cview_info = {
+	if( !type ) {
+		static const GtkTypeInfo info = {
 			"Columnview",
 			sizeof( Columnview ),
 			sizeof( ColumnviewClass ),
@@ -1082,10 +1008,10 @@ columnview_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		columnview_type = gtk_type_unique( TYPE_VIEW, &cview_info );
+		type = gtk_type_unique( TYPE_VIEW, &info );
 	}
 
-	return( columnview_type );
+	return( type );
 }
 
 View *
