@@ -50,10 +50,6 @@
 
 #include "ip.h"
 
-/* Turn on to output i18n strings at the end of compile.
- */
-static gboolean compile_find_i18n = FALSE;
-
 static iContainerClass *parent_class = NULL;
 
 Compile *
@@ -68,7 +64,7 @@ compile_get_parent( Compile *compile )
 void *
 compile_name_print( Compile *compile )
 {
-	printf( "compile(0x%x) ", GPOINTER_TO_UINT( compile ) );
+	printf( "compile(%p) ", compile );
 	symbol_name_print( compile->sym );
 	printf( " " );
 
@@ -79,7 +75,7 @@ static void *
 compile_name_sub( Expr *expr, BufInfo *buf )
 {
 	if( expr->row ) {
-		if( !buf_isempty( buf ) )
+		if( !buf_is_empty( buf ) )
 			buf_appends( buf, ", " );
 		row_qualified_name( expr->row, buf );
 	}
@@ -100,7 +96,7 @@ compile_name( Compile *compile, BufInfo *buf )
 	buf_init_static( &buf2, txt, 256 );
 	slist_map( compile->exprs,
 		(SListMapFn) compile_name_sub, &buf2 );
-	if( !buf_isempty( &buf2 ) ) 
+	if( !buf_is_empty( &buf2 ) ) 
 		buf_appendf( buf, " (%s)", buf_all( &buf2 ) );
 }
 
@@ -276,7 +272,7 @@ compile_finalize( GObject *gobject )
 	/* Remove static strings we created.
 	 */
 	slist_map( compile->statics, 
-		(SListMapFn) heap_static_string_destroy, NULL );
+		(SListMapFn) heap_static_string_unref, NULL );
 	IM_FREEF( g_slist_free, compile->statics );
 
 	/* Junk heap.
@@ -449,27 +445,55 @@ compile_new_local( Expr *expr )
 /* Code generation.
  */
 
-/* Generate a (.sym x) pair. Set x to be NULL and point rhs at it .. caller
+/* Generate a binop. Point arg1 and arg2 at the elements to be filled in:
+ * caller sets them later. First arg is the compile that this operator came 
+ * from. 
+ */
+static gboolean
+compile_binop( Compile *compile, 
+	BinOp bop, PElement *arg1, PElement *arg2, PElement *out )
+{
+	Heap *heap = compile->heap;
+
+	HeapNode *hn1, *hn2, *hn3;
+	PElement e1, e2;
+
+	if( NEWNODE( heap, hn1 ) )
+		return( FALSE );
+	hn1->type = TAG_APPL;
+	PPUT( hn1, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL );
+	PEPUTP( out, ELEMENT_NODE, hn1 );
+	PEPOINTLEFT( hn1, &e1 );
+	PEPOINTRIGHT( hn1, arg2 );
+
+	if( NEWNODE( heap, hn2 ) )
+		return( FALSE );
+	hn2->type = TAG_APPL;
+	PPUT( hn2, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL );
+	PEPUTP( &e1, ELEMENT_NODE, hn2 );
+	PEPOINTRIGHT( hn2, arg1 );
+	PEPOINTLEFT( hn2, &e2 );
+
+	if( NEWNODE( heap, hn3 ) )
+		return( FALSE );
+	hn3->type = TAG_APPL;
+	PPUT( hn3, ELEMENT_BINOP, bop, ELEMENT_COMPILEREF, compile );
+	PEPUTP( &e2, ELEMENT_NODE, hn3 );
+
+	return( TRUE );
+}
+
+/* Generate "x.sym". Set x to be NULL and point rhs at it .. caller
  * fills in later.
  */
 static gboolean
 compile_dotsym( Compile *compile, Symbol *sym, PElement *rhs, PElement *out )
 {
-	Heap *heap = compile->heap;
-	HeapNode *hn1;
-	PElement lhs;
+	PElement e;
 
-	if( NEWNODE( heap, hn1 ) )
+	if( !compile_binop( compile, BI_DOT, rhs, &e, out ) )
 		return( FALSE );
-	hn1->type = TAG_APPL;
-	PPUT( hn1, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL ); 
-	PEPUTP( out, ELEMENT_NODE, hn1 );
-
-	PEPOINTLEFT( hn1, &lhs );
-	if( !heap_dot_sym_new( heap, sym, compile, &lhs ) )
-		return( FALSE );
-
-	PEPOINTRIGHT( hn1, rhs );
+	PEPUTP( &e, ELEMENT_SYMREF, sym );
 
 	return( TRUE );
 }
@@ -563,35 +587,6 @@ compile_reference( Compile *compile, Symbol *sym, PElement *out )
 	return( TRUE );
 }
 
-/* Fwd.
- */
-static gboolean
-compile_graph( Compile *compile, ParseNode *pn, PElement *out );
-
-static gboolean
-compile_dot( Compile *compile, ParseNode *arg, const char *str, PElement *out )
-{
-	Heap *heap = compile->heap;
-	HeapNode *hn1;
-	PElement e1, e2;
-
-	if( NEWNODE( heap, hn1 ) )
-		return( FALSE );
-	hn1->type = TAG_APPL;
-	PPUT( hn1, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL );
-	PEPUTP( out, ELEMENT_NODE, hn1 );
-
-	PEPOINTLEFT( hn1, &e1 );
-	if( !heap_dot_tag_new( heap, str, compile, &e1 ) )
-		return( FALSE );
-
-	PEPOINTRIGHT( hn1, &e2 );
-	if( !compile_graph( compile, arg, &e2 ) )
-		return( FALSE );
-
-	return( TRUE );
-}
-
 /* Build a graph with vars still in it. Write result to *out.
  */
 static gboolean
@@ -647,45 +642,10 @@ compile_graph( Compile *compile, ParseNode *pn, PElement *out )
 		break;
 
 	case NODE_BINOP:
-		if( pn->biop == BI_DOT ) {
-			assert( pn->arg2->type == NODE_TAG );
-
-			if( !compile_dot( compile, 
-				pn->arg1, pn->arg2->tag, out ) )
-				return( FALSE );
-		}
-		else {
-			/* Build apply nodes.
-			 */
-			if( NEWNODE( heap, hn1 ) )
-				return( FALSE );
-			hn1->type = TAG_APPL;
-			PPUT( hn1, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL );
-			PEPUTP( out, ELEMENT_NODE, hn1 );
-			PEPOINTLEFT( hn1, &e1 );
-
-			if( NEWNODE( heap, hn2 ) )
-				return( FALSE );
-			hn2->type = TAG_APPL;
-			PPUT( hn2, ELEMENT_ELIST, NULL, ELEMENT_ELIST, NULL );
-			PEPUTP( &e1, ELEMENT_NODE, hn2 );
-			PEPOINTLEFT( hn2, &e1 );
-
-			if( NEWNODE( heap, hn3 ) )
-				return( FALSE );
-			hn3->type = TAG_APPL;
-			PPUT( hn3, ELEMENT_BINOP, pn->biop, 
-				ELEMENT_COMPILEREF, compile );
-			PEPUTP( &e1, ELEMENT_NODE, hn3 );
-
-			/* Build args.
-			 */
-			PEPOINTRIGHT( hn1, &e2 );
-			PEPOINTRIGHT( hn2, &e3 );
-			if( !compile_graph( compile, pn->arg1, &e3 ) ||
-				!compile_graph( compile, pn->arg2, &e2 ) )
-				return( FALSE );
-		}
+		if( !compile_binop( compile, pn->biop, &e1, &e2, out ) ||
+			!compile_graph( compile, pn->arg1, &e1 ) ||
+			!compile_graph( compile, pn->arg2, &e2 ) )
+			return( FALSE );
 
 		break;
 
@@ -861,6 +821,11 @@ compile_graph( Compile *compile, ParseNode *pn, PElement *out )
 				return( FALSE );
 			break;
 
+		case PARSE_CONST_COMPLEX:
+			if( !heap_complex_new( heap, 0, pn->con.val.num, out ) )
+				return( FALSE );
+			break;
+
 		default:
 			assert( FALSE );
 		}
@@ -941,9 +906,9 @@ compile_abstract_body( Compile *compile,
 		case TAG_COMPLEX:
 		case TAG_CLASS:
 		case TAG_GEN:
-		case TAG_DOT:
 			break;
 
+		case TAG_FILE:
 		case TAG_FREE:	
 		default:
 			assert( FALSE );
@@ -973,7 +938,6 @@ compile_abstract_body( Compile *compile,
 	case ELEMENT_UNOP:
 	case ELEMENT_COMB:
 	case ELEMENT_ELIST:
-	case ELEMENT_IMAGE:
 	case ELEMENT_SYMREF:
 	case ELEMENT_COMPILEREF:
 	case ELEMENT_NOVAL:
@@ -983,6 +947,9 @@ compile_abstract_body( Compile *compile,
 		 */
 		break;
 
+	/* Should not exist at compile time.
+	 */
+	case ELEMENT_MANAGED:
 	default:
 		assert( FALSE );
 	}
@@ -1065,6 +1032,10 @@ typedef gpointer CompileHash;
 #define COMPILEHASH_ADD( A, B ) \
 	GUINT_TO_POINTER( GPOINTER_TO_UINT( A ) +  GPOINTER_TO_UINT( B ) )
 
+/* An int to a hash.
+ */
+#define INT_TO_HASH GUINT_TO_POINTER
+
 /* Build one of these during sharing analysis. From node pointers to
  * hash codes, and from hash codes to a list of matching node pointers.
  */
@@ -1139,7 +1110,7 @@ compile_share_add( CompileShare *share, HeapNode *hn, CompileHash hash )
 	/* Make sure hash is non-zero (very unlikely).
 	 */
 	if( !hash )
-		hash = GUINT_TO_POINTER( 1 );
+		hash = INT_TO_HASH( 1 );
 
 	if( !g_hash_table_lookup( share->node2hash, hn ) ) {
 		GSList *nodel;
@@ -1194,24 +1165,24 @@ compile_share_scan_element( CompileShare *share, PElement *e )
 	case ELEMENT_BINOP:
 	case ELEMENT_UNOP:
 	case ELEMENT_COMB:
-	case ELEMENT_IMAGE:
 	case ELEMENT_CONSTRUCTOR:
-		hash = GUINT_TO_POINTER( PEGETTYPE( e ) + PEGETVAL( e ) );
+		hash = INT_TO_HASH( PEGETTYPE( e ) + PEGETVAL( e ) );
 		break;
 
 	case ELEMENT_ELIST:
-		hash = GUINT_TO_POINTER( ELEMENT_ELIST );
+		hash = INT_TO_HASH( ELEMENT_ELIST );
 		break;
 
 	case ELEMENT_TAG:
-		hash = GUINT_TO_POINTER( g_str_hash( PEGETTAG( e ) ) );
+		hash = INT_TO_HASH( g_str_hash( PEGETTAG( e ) ) );
 		break;
 
 	case ELEMENT_STATIC:
-		hash = GUINT_TO_POINTER( g_str_hash( PEGETSTATIC( e )->text ) );
+		hash = INT_TO_HASH( g_str_hash( PEGETSTATIC( e )->text ) );
 		break;
 
 	case ELEMENT_NOVAL:
+	case ELEMENT_MANAGED:
 	default:
 		assert( 0 );
 	}
@@ -1229,12 +1200,11 @@ compile_share_scan_node( CompileShare *share, HeapNode *hn )
 	CompileHash hash;
 	PElement a;
 
-	hash = 0;
+	hash = INT_TO_HASH( 0 );
 	switch( hn->type ) {
 	case TAG_CONS:
 	case TAG_GEN:
 	case TAG_CLASS:
-	case TAG_DOT:
 	case TAG_COMPLEX:
 	case TAG_APPL:
 		PEPOINTLEFT( hn, &a );
@@ -1243,17 +1213,18 @@ compile_share_scan_node( CompileShare *share, HeapNode *hn )
 		PEPOINTRIGHT( hn, &a );
 		hash = COMPILEHASH_ADD( hash, 
 			compile_share_scan_element( share, &a ) );
-		hash += (int) hn->type;
+		hash = COMPILEHASH_ADD( hash, 
+			INT_TO_HASH( (int) hn->type ) );
 		break;
 
 	case TAG_DOUBLE:
 		hash = COMPILEHASH_ADD( hash, 
-			hn->body.ptrs.left );
+			INT_TO_HASH( (int) hn->body.num ) );
 		hash = COMPILEHASH_ADD( hash, 
-			hn->body.ptrs.right );
-		hash += (int) hn->type;
+			INT_TO_HASH( (int) hn->type ) );
 		break;
 
+	case TAG_FILE:
 	case TAG_REFERENCE:
 	case TAG_SHARED:
 	case TAG_FREE:
@@ -1396,7 +1367,6 @@ compile_share_trim( CompileShare *share, HeapNode *hn )
 	case TAG_CONS:
 	case TAG_GEN:
 	case TAG_CLASS:
-	case TAG_DOT:
 	case TAG_COMPLEX:
 	case TAG_APPL:
 		PEPOINTLEFT( hn, &a );
@@ -1413,6 +1383,7 @@ compile_share_trim( CompileShare *share, HeapNode *hn )
 
 	case TAG_SHARED:
 	case TAG_FREE:
+	case TAG_FILE:
 	default:
 		assert( FALSE );
 	}
@@ -1659,7 +1630,7 @@ compile_check_i18n( Compile *compile, ParseNode *pn )
 			pn->arg2->con.type == PARSE_CONST_STR ) {
 			const char *text = pn->arg2->con.val.str;
 
-			if( compile_find_i18n ) {
+			if( main_option_i18n ) {
 				/* Remove msgid duplicates with this.
 				 */
 				static GHashTable *msgid = NULL;
@@ -1985,116 +1956,3 @@ compile_get_member_string( Compile *compile, const char *name )
 
 	return( NULL );
 }
-
-/* Output constant i18n strings during compile.
- */
-void
-compile_set_i18n( gboolean i18n )
-{
-	compile_find_i18n = i18n;
-}
-
-/* Lev's stack trace:
-
-654            switch( pn->type ) {
-(gdb) where
-#0  0x0807a1e1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddd940)
-    at compile.c:1654
-#1  0x0807a39d in compile_check_i18n (compile=0x9dbce98, pn=0x9ddd9c0)
-    at compile.c:1699
-#2  0x0807a39d in compile_check_i18n (compile=0x9dbce98, pn=0x9dddac0)
-    at compile.c:1699
-#3  0x0807a39d in compile_check_i18n (compile=0x9dbce98, pn=0x9dde980)
-    at compile.c:1699
-#4  0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9dde9c0)
-    at compile.c:1712
-#5  0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddea00)
-    at compile.c:1700
-#6  0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddea40)
-    at compile.c:1712
-#7  0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddea80)
-    at compile.c:1700
-#8  0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddeac0)
-    at compile.c:1712
-#9  0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddeb00)
-    at compile.c:1700
-#10 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddeb40)
-    at compile.c:1712
-#11 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddeb80)
-    at compile.c:1700
-#12 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddebc0)
-    at compile.c:1712
-#13 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddec00)
-    at compile.c:1700
-#14 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddec40)
-    at compile.c:1712
-#15 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddec80)
-    at compile.c:1700
-#16 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddecc0)
-    at compile.c:1712
-#17 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9dded00)
-    at compile.c:1700
-#18 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9dded40)
-    at compile.c:1712
-#19 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9dded80)
-    at compile.c:1700
-#20 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddedc0)
-    at compile.c:1712
-#21 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddee00)
-    at compile.c:1700
-#22 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddee40)
-    at compile.c:1712
-#23 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddee80)
-    at compile.c:1700
-#24 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddeec0)
-    at compile.c:1712
-#25 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddef00)
-    at compile.c:1700
-#26 0x0807a3f1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddef40)
-    at compile.c:1712
-#27 0x0807a3b1 in compile_check_i18n (compile=0x9dbce98, pn=0x9ddef80)
-    at compile.c:1700
-#28 0x0807a4c1 in compile_check (compile=0x9dbce98) at compile.c:1755
-#29 0x080a7874 in yyparse () at parse.y:728
-#30 0x080a8ea4 in parse_input (ch=46, sym=0x8a110bc, kit=0x8c0bcb0, pos=0)
-    at parse.y:1286
-#31 0x080a8efa in parse_toplevel (kit=0x8c0bcb0, pos=0) at parse.y:1304
-#32 0x080db5e5 in toolkit_load_text (model=0x8c0bcb0, parent=0x8a18018,
-    of=0x9da85c0) at toolkit.c:112
-#33 0x080b6035 in model_load_text (model=0x8c0bcb0, parent=0x8a18018,
-    of=0x9da85c0) at model.c:395
-#34 0x080825e4 in filemodel_load_all_text (filemodel=0x8c0bcb0,
-    parent=0x8a18018,
-    filename=0xbfbf6120 "$VIPSHOME/share/nip2/start/_stdenv.def")
-    at filemodel.c:525
-#35 0x0808265a in filemodel_load_all (filemodel=0x8c0bcb0, parent=0x8a18018,
-    filename=0xbfbf6120 "$VIPSHOME/share/nip2/start/_stdenv.def")
-    at filemodel.c:550
-#36 0x080db9a8 in toolkit_new_from_file (kitg=0x8a18018,
-    filename=0xbfbf6120 "$VIPSHOME/share/nip2/start/_stdenv.def")
-    at toolkit.c:247
-#37 0x080ac674 in main_load_def (
-    filename=0xbfbf6120 "$VIPSHOME/share/nip2/start/_stdenv.def")
-    at main.c:426
-#38 0x080bb7be in path_next_item (
-    filename=0xbfbf6120 "$VIPSHOME/share/nip2/start/_stdenv.def",
-    previous=0xbfbf65f0, fn=0x80ac620 <main_load_def>, a=0x0) at path.c:201
-#39 0x080bb641 in path_match_pattern (wild=0x8c45270,
-    dir_name=0x830a620 "$VIPSHOME/share/nip2/start",
-    name=0x8c48180 "_stdenv.def", fn=0x80bb734 <path_next_item>,
-    a=0xbfbf65f0, b=0x80ac620, c=0x0) at path.c:148
-#40 0x080bb6c7 in path_scan_dir (
-    dir_name=0x830a620 "$VIPSHOME/share/nip2/start", wild=0x8c45270,
-    fn=0x80bb734 <path_next_item>, a=0xbfbf65f0, b=0x80ac620, c=0x0)
-    at path.c:170
-#41 0x080e0d6e in slist_map5 (list=0x8318830, fn=0x80bb660 <path_scan_dir>,
-    a=0x8c45270, b=0x80bb734, c=0xbfbf65f0, d=0x80ac620, e=0x0) at util.c:575
-#42 0x080bb82f in path_map_exact (path=0x8318830, patt=0x80fee03 "*.def",
-    fn=0x80ac620 <main_load_def>, a=0x0) at path.c:230
-#43 0x080aca33 in main_load_startup () at main.c:549
-#44 0x080ad988 in main (argc=1, argv=0xbfbfecf4) at main.c:1177
-(gdb) quit
-
- */
-
-
