@@ -117,16 +117,16 @@ conversion_finalize( GObject *gobject )
 	IM_FREEF( im_region_free, conv->ireg );
 	IM_FREEF( im_region_free, conv->reg );
 
-	IM_FREEF( imageinfo_destroy_nonheap, conv->repaint_ii );
-	IM_FREEF( imageinfo_destroy_nonheap, conv->display_ii );
-	IM_FREEF( imageinfo_destroy_nonheap, conv->visual_ii );
-	IM_FREEF( imageinfo_destroy_nonheap, conv->ii );
+	MANAGED_UNREF( conv->repaint_ii );
+	MANAGED_UNREF( conv->display_ii );
+	MANAGED_UNREF( conv->visual_ii );
+	MANAGED_UNREF( conv->ii );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
 /* Make the visualisation image ... eg. we im_histplot histograms, and we
- * rotate foureier images.
+ * log scale fourier images.
  */
 static IMAGE *
 conversion_make_visualise( Conversion *conv, IMAGE *in )
@@ -216,7 +216,10 @@ conversion_paint_callback( IMAGE *im, Rect *area, void *client )
 	update->area = *area;
 
 	g_async_queue_push( conversion_update_queue, update );
-	g_main_context_wakeup( NULL );
+
+	/* Don't need g_main_context_wakeup( NULL ); ... not sure it's
+	 * reentrant anyway.
+	 */
 }
 
 static gboolean
@@ -253,6 +256,12 @@ conversion_update_callback( gpointer data )
 		image.height += 1;
 	}
 
+	/* We need to invalid the main image too, since those regions will
+	 * have black in from the failed first calc.
+	 */
+	im_invalidate( conv->mask );
+	im_invalidate( imageinfo_get( FALSE, conv->display_ii ) );
+
 	imageinfo_area_changed( conv->ii, &image );
 
 	return( TRUE );
@@ -376,7 +385,7 @@ conversion_make_display( Conversion *conv, IMAGE *in, IMAGE **mask_out )
 		if( im_render_fade( in, out, mask, 
 			conv->tile_size, conv->tile_size, 
 				conversion_get_default_tiles( conv ),
-			20, 10,
+			20, conv->fade_steps,
 			conv->priority,
 			conversion_paint_callback, conv ) ) {
 			im_close( out );
@@ -452,9 +461,9 @@ typedef struct {
 	} \
 }
 
-/* Complex input, uchar output. 
+/* Complex input, uchar output.
  */
-#define LOOPCMPLX(IN) {\
+#define LOOPCMPLX(IN) { \
         IN *p = (IN *) in; \
         PEL *q = (PEL *) out; \
         \
@@ -469,7 +478,7 @@ typedef struct {
 			t = 0; \
 		\
                 q[x] = t; \
-        } \
+	} \
 }
 
 /* Lintra a buffer, 1 set of scale/offset.
@@ -652,85 +661,98 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		return( NULL );
 	}
 
-	if( tconv &&
-		in->Bands == 3 && in->Type == IM_TYPE_LCH ) {
-		IMAGE *t[2];
-
-                if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
-			im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
-			im_LCh2Lab( t[0], t[1] ) ) {
-			im_close( out );
-			return( NULL );
-		}
-
-		in = t[1];
-	}
-
-	if( tconv &&
-		in->Bands == 3 && in->Type == IM_TYPE_YXY ) {
-		IMAGE *t[2];
-
-                if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
-			im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
-			im_Yxy2XYZ( t[0], t[1] ) ) {
-			im_close( out );
-			return( NULL );
-		}
-
-		in = t[1];
-	}
-
-	if( tconv &&
-		in->Bands == 3 && in->Type == IM_TYPE_UCS ) {
-		IMAGE *t[2];
-
-                if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
-			im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
-			im_UCS2XYZ( t[0], t[1] ) ) {
-			im_close( out );
-			return( NULL );
-		}
-
-		in = t[1];
-	}
-
-	if( tconv &&
-		in->Bands == 3 && in->Type == IM_TYPE_LAB ) {
-		IMAGE *t[2];
-
-                if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
-			im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
-			im_Lab2XYZ( t[0], t[1] ) ) {
-			im_close( out );
-			return( NULL );
-		}
-
-		in = t[1];
-	}
-
-	if( tconv &&
-		in->Bands == 3 && in->Type == IM_TYPE_XYZ ) {
-		IMAGE *t[2];
-
-                if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
-			im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
-			im_XYZ2disp( t[0], t[1], display ) ) {
-			im_close( out );
-			return( NULL );
-		}
-
-		in = t[1];
-	}
-
-	/* Scale and offset?
+	/* One of the colorimetric types?
 	 */
-	if( conv && conv->enabled ) {
-		if( conv->scale != 1.0 || conv->offset != 0.0 ) {
+	if( tconv && in->Bands == 3 && 
+		(in->Type == IM_TYPE_LCH ||
+		in->Type == IM_TYPE_YXY ||
+		in->Type == IM_TYPE_UCS ||
+		in->Type == IM_TYPE_LAB ||
+		in->Type == IM_TYPE_XYZ) ) {
+		IMAGE *t[2];
+
+		/* We need to scale/offset before we go to 8 bit to work well 
+		 * with HDR.
+		 */
+		if( conv && conv->enabled && 
+			(conv->scale != 1.0 || conv->offset != 0.0) ) {
+			IMAGE *t = im_open_local( out, "conv:1", "p" );
+
+			if( !t || im_lintra( conv->scale, in, 
+				conv->offset, t ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t;
+		}
+
+		if( in->Type == IM_TYPE_LCH ) {
+			if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
+				im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
+				im_LCh2Lab( t[0], t[1] ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t[1];
+		}
+
+		if( in->Type == IM_TYPE_YXY ) {
+			if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
+				im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
+				im_Yxy2XYZ( t[0], t[1] ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t[1];
+		}
+
+		if( in->Type == IM_TYPE_UCS ) {
+			if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
+				im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
+				im_UCS2XYZ( t[0], t[1] ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t[1];
+		}
+
+		if( in->Type == IM_TYPE_LAB ) {
+			if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
+				im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
+				im_Lab2XYZ( t[0], t[1] ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t[1];
+		}
+
+		if( in->Type == IM_TYPE_XYZ ) {
+			if( im_open_local_array( out, t, 2, "conv-1", "p" ) ||
+				im_clip2fmt( in, t[0], IM_BANDFMT_FLOAT ) ||
+				im_XYZ2disp( t[0], t[1], display ) ) {
+				im_close( out );
+				return( NULL );
+			}
+
+			in = t[1];
+		}
+	}
+	else {
+		/* Not colorimetric. We can use a special ->uchar lintra for
+		 * scale/offset. Don't scale/offset fourier or histogram 
+		 * images, they are presented above.
+		 */
+		if( conv && conv->enabled &&
+			(!tconv || in->Type != IM_TYPE_FOURIER) &&
+			(!tconv || in->Type != IM_TYPE_HISTOGRAM) &&
+			(conv->scale != 1.0 || conv->offset != 0.0) ) {
 			IMAGE *t = im_open_local( out, "conv:1", "p" );
 
-			/* Use a special lintra()  which writes uchar and
-			 * doesn't do the _vec stuff. About 5 times faster (!!)
-			 */
 			if( !t || im_lintrauc( conv->scale, in, 
 				conv->offset, t ) ) {
 				im_close( out );
@@ -741,6 +763,22 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 		}
 	}
 
+	/* im_msb() only works for the int formats.
+	 */
+	if( tconv && 
+		(in->Type == IM_TYPE_RGB16 || in->Type == IM_TYPE_GREY16) &&
+		im_isint( in ) ) {
+		IMAGE *t[1];
+
+                if( im_open_local_array( out, t, 1, "conv-1", "p" ) ||
+			im_msb( in, t[0] ) ) {
+			im_close( out );
+			return( NULL );
+		}
+
+		in = t[0];
+	}
+
 	/* Clip to uchar if not there already.
 	 */
 	if( in->BandFmt != IM_BANDFMT_UCHAR ) {
@@ -774,7 +812,7 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 
 			in = t2;
 		}
-		
+
 		if( im_falsecolour( in, t1 ) ) {
 			im_close( out );
 			return( NULL );
@@ -786,13 +824,17 @@ conversion_make_repaint( Conversion *conv, IMAGE *in )
 	/* Combine with background checkerboard and mask to make final image.
 	 */
 	if( conv->synchronous ) {
-		if( im_copy( in, out ) )
+		if( im_copy( in, out ) ) {
+			im_close( out );
 			return( NULL );
+		}
 	}
 	else
 		if( conversion_make_bg( conv, in, bg ) ||
-			im_blend( conv->mask, in, bg, out ) )
+			im_blend( conv->mask, in, bg, out ) ) {
+			im_close( out );
 			return( NULL );
+		}
 
 	return( out );
 }
@@ -809,7 +851,7 @@ conversion_rebuild_repaint( Conversion *conv )
 	REGION *new_ireg;
 
 #ifdef DEBUG
-	g_print( "conversion_remake_repaint: 0x%x\n", (unsigned int) conv );
+	g_print( "conversion_remake_repaint: %p\n", conv );
 #endif /*DEBUG*/
 
 	if( conv->display_ii )
@@ -833,18 +875,19 @@ conversion_rebuild_repaint( Conversion *conv )
 			im_close( new_repaint_im );
 			return;
 		}
-		imageinfo_sub_add( new_repaint_ii, conv->display_ii );
+		managed_sub_add( MANAGED( new_repaint_ii ), 
+			MANAGED( conv->display_ii ) );
 
 		if( !(new_ireg = im_region_create( new_repaint_im )) ) 
 			return;
 	}
 
 	IM_FREEF( im_region_free, conv->ireg );
-	IM_FREEF( imageinfo_destroy_nonheap, conv->repaint_ii );
+	MANAGED_UNREF( conv->repaint_ii );
 
 	if( display_im ) {
 		conv->repaint_ii = new_repaint_ii;
-		imageinfo_dup_nonheap( conv->repaint_ii );
+		MANAGED_REF( conv->repaint_ii );
 		conv->ireg = new_ireg;
 	}	
 }
@@ -861,7 +904,7 @@ conversion_rebuild_display( Conversion *conv )
 	IMAGE *mask;
 
 #ifdef DEBUG
-	g_print( "conversion_remake_display: 0x%x\n", (unsigned int) conv );
+	g_print( "conversion_remake_display: %p\n", conv );
 #endif /*DEBUG*/
 
 	if( conv->visual_ii )
@@ -885,15 +928,16 @@ conversion_rebuild_display( Conversion *conv )
 			im_close( new_display_im );
 			return;
 		}
-		imageinfo_sub_add( new_display_ii, conv->visual_ii );
+		managed_sub_add( MANAGED( new_display_ii ), 
+			MANAGED( conv->visual_ii ) );
 	}
 
-	IM_FREEF( imageinfo_destroy_nonheap, conv->display_ii );
+	MANAGED_UNREF( conv->display_ii );
 
 	if( visual_im ) {
 		conv->display_ii = new_display_ii;
 		conv->mask = mask;
-		imageinfo_dup_nonheap( conv->display_ii );
+		MANAGED_REF( conv->display_ii );
 
 		conv->canvas.width = new_display_im->Xsize;
 		conv->canvas.height = new_display_im->Ysize;
@@ -915,7 +959,7 @@ conversion_rebuild_visual( Conversion *conv )
 	REGION *new_reg;
 
 #ifdef DEBUG
-	g_print( "conversion_rebuild_visual: 0x%x\n", (unsigned int) conv );
+	g_print( "conversion_rebuild_visual: %p\n", conv );
 #endif /*DEBUG*/
 
 	/* Keep gcc quiet about annoying possible-used-before-set warnings.
@@ -934,7 +978,8 @@ conversion_rebuild_visual( Conversion *conv )
 			im_close( new_visual_im );
 			return;
 		}
-		imageinfo_sub_add( new_visual_ii, conv->ii );
+		managed_sub_add( MANAGED( new_visual_ii ), 
+			MANAGED( conv->ii ) );
 
 		if( !(new_reg = im_region_create( im )) ) {
 			IM_FREEF( im_region_free, new_reg );
@@ -945,13 +990,13 @@ conversion_rebuild_visual( Conversion *conv )
 	/* Junk old stuff.
 	 */
 	IM_FREEF( im_region_free, conv->reg );
-	IM_FREEF( imageinfo_destroy_nonheap, conv->visual_ii );
+	MANAGED_UNREF( conv->visual_ii );
 
 	/* Install new stuff.
 	 */
 	if( im ) {
 		conv->visual_ii = new_visual_ii;
-		imageinfo_dup_nonheap( conv->visual_ii );
+		MANAGED_REF( conv->visual_ii );
 		conv->image.width = new_visual_im->Xsize;
 		conv->image.height = new_visual_im->Ysize;
 		conv->reg = new_reg;
@@ -973,7 +1018,7 @@ conversion_changed( iObject *iobject )
 	gboolean rebuild_repaint = FALSE;
 
 #ifdef DEBUG
-	g_print( "conversion_changed: 0x%x\n", (unsigned int) conv );
+	g_print( "conversion_changed: %p\n", conv );
 #endif /*DEBUG*/
 
 	/* Need to remake the display image if mag has changed.
@@ -1058,6 +1103,11 @@ conversion_init( Conversion *conv )
 	 */
 	conv->tile_size = 128;
 
+	/* Default to the preference ... iimageview sets this to less to stop
+	 * the tile fade animation on thumbnails.
+	 */
+	conv->fade_steps = DISPLAY_FADE_STEPS;
+
 	conv->underlay = emptyrect;
 	conv->image = emptyrect;
 	conv->canvas = emptyrect;
@@ -1160,7 +1210,7 @@ conversion_set_image( Conversion *conv, Imageinfo *ii )
 		 */
 		FREESID( conv->changed_sid, conv->ii );
 		FREESID( conv->area_changed_sid, conv->ii );
-		IM_FREEF( imageinfo_destroy_nonheap, conv->ii );
+		MANAGED_UNREF( conv->ii );
 		conv->image.width = -1;
 		conv->image.height = -1;
 
@@ -1168,7 +1218,7 @@ conversion_set_image( Conversion *conv, Imageinfo *ii )
 		 */
 		if( ii ) {
 			conv->ii = ii;
-			imageinfo_dup_nonheap( conv->ii );
+			MANAGED_REF( conv->ii );
 			conv->changed_sid = g_signal_connect( 
 				G_OBJECT( ii ), "changed",
 				G_CALLBACK( conversion_ii_changed_cb ), 
