@@ -61,10 +61,29 @@ expr_error_print_sub( Expr *expr, BufInfo *buf )
 {
 	assert( expr->err );
 
-	if( !buf_isempty( buf )  )
+	if( !buf_is_empty( buf )  )
 		buf_appends( buf, "\n" );
-	buf_appendf( buf, _( "error in \"%s\": %s %s" ), 
-		IOBJECT( expr->sym )->name, expr->error_top, expr->error_sub );
+
+	buf_appendf( buf, _( "error in \"%s\"" ), IOBJECT( expr->sym )->name );
+	if( expr->sym->tool ) {
+		Tool *tool = expr->sym->tool;
+
+		if( tool->lineno != -1 && FILEMODEL( tool->kit )->filename )
+			buf_appendf( buf, " (%s:%d)", 
+				FILEMODEL( tool->kit )->filename, 
+				tool->lineno );
+	}
+	else if( expr->row ) {
+		Workspace *ws = expr->row->ws;
+
+		buf_appendf( buf, " (" );
+		row_qualified_name( expr->row, buf );
+		if( FILEMODEL( ws )->filename )
+			buf_appendf( buf, " - %s", FILEMODEL( ws )->filename );
+		buf_appendf( buf, ")" );
+	}
+
+	buf_appendf( buf, ": %s\n%s", expr->error_top, expr->error_sub );
 
 	return( NULL );
 }
@@ -111,7 +130,7 @@ expr_map_all( Expr *expr, map_expr_fn fn, void *a )
 void *
 expr_name_print( Expr *expr )
 {
-	printf( "expr(0x%x) ", GPOINTER_TO_UINT( expr ) );
+	printf( "expr(%p) ", expr );
 	symbol_name_print( expr->sym );
 	printf( " " );
 
@@ -301,6 +320,18 @@ expr_dispose( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
 
+static void
+expr_info( iObject *iobject, BufInfo *buf )
+{
+	Expr *expr = EXPR( iobject );
+
+	if( expr->err ) {
+		buf_appends( buf, _( "Error" ) );
+		buf_appendf( buf, ": %s\n%s\n", 
+			expr->error_top, expr->error_sub );
+	}
+}
+
 static void
 expr_real_new_value( Expr *expr )
 {
@@ -321,6 +352,7 @@ static void
 expr_class_init( ExprClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
+	iObjectClass *iobject_class = (iObjectClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
 
@@ -338,6 +370,8 @@ expr_class_init( ExprClass *class )
 	 */
 	gobject_class->dispose = expr_dispose;
 
+	iobject_class->info = expr_info;
+
 	class->new_value = expr_real_new_value;
 
 	/* Static init.
@@ -444,8 +478,8 @@ expr_error_set( Expr *expr )
 		printf( ": %s %s\n", error_get_top(), error_get_sub() );
 #endif /*DEBUG_ERROR*/
 
-		SETSTR( expr->error_top, error_get_top() );
-		SETSTR( expr->error_sub, error_get_sub() );
+		IM_SETSTR( expr->error_top, error_get_top() );
+		IM_SETSTR( expr->error_sub, error_get_sub() );
 
 		/* Zap the value of the expr ... it may contain pointers to
 		 * dead stuff.
