@@ -554,21 +554,15 @@ filesel_get_filetype( Filesel *filesel )
 		return( filesel->default_type );
 	else {
 		GtkFileFilter *filter;
-		const char *name;
 		int i;
 
 		filter = gtk_file_chooser_get_filter( 
 			GTK_FILE_CHOOSER( filesel->chooser ) );
 		assert( filter );
-		name = gtk_file_filter_get_name( filter );
 
 		for( i = 0; filesel->type[i]; i++ ) 
-			if( strcmp( _( filesel->type[i]->name ), name ) == 0 )
-				break;
-
-		assert( filesel->type[i] );
-
-		return( i );
+			if( filter == filesel->filter[i] )
+				return( i );
 	}
 }
 
