@@ -177,8 +177,7 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 			*back = g_slist_prepend( *back, hn );
 			buf_appendf( buf, "<" ); 
 			buf_appendf( buf, _( "circular" ) ); 
-			buf_appendf( buf, " (0x%x)>", 
-				GPOINTER_TO_UINT( hn ) );
+			buf_appendf( buf, " (%p)>", hn );
 		}
 		else {
 			buf_appendf( buf, "<" );
@@ -244,8 +243,7 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 	case TAG_CLASS:
 		if( fn ) {
 			buf_appendf( buf, "\n%s", spc( indent ) );
-			buf_appendf( buf, _( "class (0x%x)" ), 
-				GPOINTER_TO_UINT( hn ) );
+			buf_appendf( buf, _( "class (%p)" ), hn );
 			buf_appendf( buf, " " );
 		}
 
@@ -277,18 +275,6 @@ lisp_node( BufInfo *buf, HeapNode *hn, GSList **back, gboolean fn, int indent )
 
 		break;
 
-	case TAG_DOT:
-		PEPOINTLEFT( hn, &p1 );
-		PEPOINTRIGHT( hn, &p2 );
-		buf_appends( buf, ".(" );
-		lisp_pelement( buf, &p1, back, fn, indent );
-		buf_appends( buf, " " );
-		buf_appends( buf, _( "from" ) );
-		buf_appends( buf, " " );
-		lisp_pelement( buf, &p2, back, fn, indent );
-		buf_appends( buf, ")" );
-		break;
-
 	case TAG_GEN:
 		buf_appendf( buf, "[%g,%g...", 
 			GETLEFT( hn )->body.num, 
@@ -325,7 +311,6 @@ static void
 lisp_pelement( BufInfo *buf, PElement *base, 
 	GSList **back, gboolean fn, int indent )
 {
-	Imageinfo *ii;
 	HeapNode *hn;
 	EType type = PEGETTYPE( base );
 
@@ -395,21 +380,6 @@ lisp_pelement( BufInfo *buf, PElement *base,
 		buf_appends( buf, decode_UnOp( PEGETUNOP( base ) ) );
 		break;
 
-	case ELEMENT_IMAGE:
-		if( (ii = PEGETII( base )) ) {
-			buf_appendf( buf, "<" ); 
-			buf_appendf( buf, _( "image \"%s\"" ),
-				ii->im->filename );
-			buf_appendf( buf, ">" ); 
-		}
-		else {
-			buf_appends( buf, "<" ); 
-			buf_appends( buf, _( "no image" ) ); 
-			buf_appends( buf, ">" ); 
-		}
-
-		break;
-
 	case ELEMENT_ELIST:
 		buf_appends( buf, "[ ]" ); 
 		break;
@@ -427,6 +397,10 @@ lisp_pelement( BufInfo *buf, PElement *base,
 		buf_appendf( buf, ">" );
 		break;
 
+	case ELEMENT_MANAGED:
+		buf_appendf( buf, "<Managed* %p>", PEGETVAL( base ) );
+		break;
+
 	case ELEMENT_COMB:
 		buf_appends( buf, decode_CombinatorType( PEGETCOMB( base ) ) );
 		break;
@@ -513,14 +487,67 @@ graph_pointer( PElement *root )
 	printf( "%s\n", buf_all( &buf ) );
 }
 
+static void
+save_image( Imageinfo *ii, char *filename )
+{
+#ifdef DEBUG
+	printf( "set_output: writing image to %s\n", filename );
+#endif /*DEBUG*/
+
+	if( !imageinfo_write( ii, NULL, filename ) ) 
+		error( error_get_top() );
+
+	increment_filename( filename );
+}
+
+/* Write all images to files. Generate filenames with increment-file,
+ * overwriting filename.
+ */
+void *
+save_objects( PElement *base, char *filename )
+{
+	gboolean result;
+	Imageinfo *ii;
+
+	if( PEISIMAGE( base ) && (ii = PEGETII( base )) ) 
+		save_image( ii, filename );
+	else if( PEISCLASS( base ) ) {
+		if( !heap_is_instanceof( CLASS_IMAGE, base, &result ) )
+			error( error_get_top() );
+		if( result ) {
+			if( !class_get_member_image( base, MEMBER_VALUE, &ii ) )
+				error( error_get_top() );
+			if( ii )
+				save_image( ii, filename );
+		}
+
+		if( !heap_is_instanceof( CLASS_MATRIX, base, &result ) )
+			error( error_get_top() );
+		if( result ) {
+			DOUBLEMASK *m;
+
+			if( !(m = matrix_ip_to_dmask( base )) ) 
+				error( error_get_top() );
+			if( im_write_dmask_name( m, filename ) ) 
+				error( error_get_top() );
+			increment_filename( filename );
+		}
+	}
+	else if( PEISLIST( base ) )
+		heap_map_list( base, 
+			(heap_map_list_fn) save_objects, filename, NULL );
+
+	return( NULL );
+}
+
 /* Fwd ref.
  */
-static void csh_pelement( PElement *base );
+static void shell_pelement( PElement *base );
 
-/* Print a graph csh-style.
+/* Print a graph shell-style.
  */
 static void
-csh_node( HeapNode *hn )
+shell_node( HeapNode *hn )
 {
 	PElement p1, p2;
 
@@ -538,22 +565,33 @@ csh_node( HeapNode *hn )
 	case TAG_REFERENCE:
 	case TAG_SHARED:
 	case TAG_GEN:
-	case TAG_DOT:
 		break;
 		
 	case TAG_CONS:
+{
+		gboolean string_mode;
+
+		PEPOINTLEFT( hn, &p1 );
+		string_mode = PEISCHAR( &p1 );
+
 		for(;;) {
-			PEPOINTLEFT( hn, &p1 );
-			csh_pelement( &p1 );
+			if( string_mode )
+				printf( "%c", PEGETCHAR( &p1 ) );
+			else
+				shell_pelement( &p1 );
 
 			PEPOINTRIGHT( hn, &p2 );
 			if( PEISELIST( &p2 ) ) 
 				break;
-			else {
+
+			if( !string_mode )
 				printf( "\n" );
-				hn = PEGETVAL( &p2 );
-			}
+			hn = PEGETVAL( &p2 );
+			PEPOINTLEFT( hn, &p1 );
+			if( string_mode && !PEISCHAR( &p1 ) )
+				string_mode = FALSE;
 		}
+}
 		break;
 
 	case TAG_DOUBLE:
@@ -571,13 +609,11 @@ csh_node( HeapNode *hn )
 	}
 }
 
-/* Print a pelement csh-style.
+/* Print a pelement shell-style.
  */
 static void
-csh_pelement( PElement *base )
+shell_pelement( PElement *base )
 {
-	Imageinfo *ii;
-
 	switch( PEGETTYPE( base ) ) {
 	/* Only allow concrete base types.
 	 */
@@ -594,7 +630,7 @@ csh_pelement( PElement *base )
 		break;
 
 	case ELEMENT_NODE:
-		csh_node( PEGETVAL( base ) );
+		shell_node( PEGETVAL( base ) );
 		break;
 
 	case ELEMENT_CHAR:
@@ -609,24 +645,21 @@ csh_pelement( PElement *base )
 		printf( "%s", bool_to_char( PEGETBOOL( base ) ) );
 		break;
 
-	case ELEMENT_IMAGE:
-		if( (ii = PEGETII( base )) ) 
-			printf( "%s", imageinfo_get( FALSE, ii )->filename );
-		else
-			printf( "no-image" ); 
-
-		break;
-
 	case ELEMENT_ELIST:
 		printf( "[ ]" ); 
 		break;
 
+	case ELEMENT_MANAGED:
+		if( PEISIMAGE( base ) )
+			printf( "%s", PEGETIMAGE( base )->filename );
+		break;
+
 	default:
 		assert( FALSE );
 	}
 }
 
-/* Print a pelement csh-style.
+/* Print a pelement shell-style.
  */
 void
 graph_value( PElement *root )
@@ -639,27 +672,8 @@ graph_value( PElement *root )
 	}
 
 	heap_clear( reduce_context->heap, FLAG_PRINT );
-	csh_pelement( root );
+	shell_pelement( root );
 	printf( "\n" );
 }
 
-/* Force all images to files, set no-delete.
- */
-void *
-set_output( PElement *base )
-{
-	if( PEISIMAGE( base ) ) {
-		Imageinfo *ii;
 
-		if( (ii = PEGETII( base )) ) {
-			if( !imageinfo_file( ii, NULL ) ) 
-				error( error_get_top() );
-			ii->dfile = FALSE;
-		}
-	}
-	else if( PEISLIST( base ) )
-		heap_map_list( base, 
-			(heap_map_list_fn) set_output, NULL, NULL );
-
-	return( NULL );
-}
