@@ -217,6 +217,23 @@ menu_add_sep( GtkWidget *menu )
 	return( sep );
 }
 
+/* Add a pullright.
+ */
+GtkWidget *
+menu_add_pullright( GtkWidget *menu, const char *stock_id )
+{
+	GtkWidget *pullright;
+	GtkWidget *subpane;
+
+	subpane = gtk_menu_new();
+	pullright = gtk_image_menu_item_new_from_stock( stock_id, NULL );
+	gtk_menu_item_set_submenu( GTK_MENU_ITEM( pullright ), subpane );
+	gtk_menu_shell_append( GTK_MENU_SHELL( menu ), pullright );
+	gtk_widget_show( pullright );
+
+	return( subpane );
+}
+
 /* Four quarks: each menu item has a quark linking back to the main pane, 
  * plus a quark for the user signal. The main pane has a quark linking to the
  * widget the menu was popped from, and that has the userdata for this context.
@@ -285,6 +302,18 @@ popup_add_tog( GtkWidget *popup, const char *name, PopupFunc cb )
 	return( tog );
 }
 
+/* Add a pullright item to a popup. Return the empty sub-pane.
+ */
+GtkWidget *
+popup_add_pullright( GtkWidget *popup, const char *name )
+{
+	GtkWidget *pullright = menu_add_pullright( popup, name );
+
+	gtk_object_set_data_by_id( GTK_OBJECT( pullright ), quark_main, popup );
+
+	return( pullright );
+}
+
 /* Show the popup.
  */
 void
@@ -386,6 +415,85 @@ set_tooltip( GtkWidget *wid, const char *fmt, ... )
 	g_free( txt );
 }
 
+/* Track tooltips we generate with one of these.
+ */
+typedef struct _TooltipGenerate {
+	GtkWidget *widget;
+
+	TooltipGenerateFn generate;
+	void *a;
+	void *b;
+
+	BufInfo buf;
+	char txt[256];
+} TooltipGenerate;
+
+static void
+tooltip_generate_free( GtkWidget *widget, TooltipGenerate *gen )
+{
+	gen->widget = NULL;
+	gen->generate = NULL;
+	gen->a = NULL;
+	gen->b = NULL;
+
+	IM_FREE( gen );
+}
+
+static gboolean
+tooltip_generate_rebuild( GtkWidget *widget, 
+	GdkEventCrossing *event, TooltipGenerate *gen )
+{
+	gboolean handled = FALSE;
+
+	if( gen->widget ) {
+		buf_rewind( &gen->buf );
+		gen->generate( widget, &gen->buf, gen->a, gen->b );
+		set_tooltip( gen->widget, "%s", buf_all( &gen->buf ) );
+	}
+
+	return( handled );
+}
+
+static void
+tooltip_generate_attach( GtkWidget *widget, TooltipGenerate *gen )
+{
+	/* Must have enter/leave.
+	 */
+	gtk_widget_add_events( widget, 
+		GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK );
+
+	/* On enter, regenerate the tooltip.
+	 */
+	g_signal_connect( widget, "enter_notify_event",
+		G_CALLBACK( tooltip_generate_rebuild ), gen );
+}
+
+/* Set a callback to be used to generate the tooltip.
+ */
+void
+set_tooltip_generate( GtkWidget *widget, 
+	TooltipGenerateFn generate, void *a, void *b )
+{
+	TooltipGenerate *gen;
+
+	if( !(gen = INEW( NULL, TooltipGenerate )) )
+		return;
+
+	gen->widget = widget;
+	gen->generate = generate;
+	gen->a = a;
+	gen->b = b;
+	buf_init_static( &gen->buf, gen->txt, 256 );
+	g_signal_connect( widget, "destroy", 
+		G_CALLBACK( tooltip_generate_free ), gen );
+
+	if( !GTK_WIDGET_REALIZED( widget ) )
+		g_signal_connect( widget, "realize",
+			G_CALLBACK( tooltip_generate_attach ), gen );
+	else
+		tooltip_generate_attach( widget, gen );
+}
+
 /* Set a GtkEditable.
  */
 void
@@ -442,6 +550,24 @@ set_glabel( GtkWidget *label, const char *fmt, ... )
 	gtk_label_set_text( GTK_LABEL( label ), buf );
 }
 
+/* Like set_glabel(), but don't display multi-line strings (just display the
+ * first line).
+ */
+void
+set_glabel1( GtkWidget *label, const char *fmt, ... )
+{
+	va_list ap;
+	char txt[1000];
+	BufInfo buf;
+
+	va_start( ap, fmt );
+	buf_init_static( &buf, txt, 1000 );
+	buf_vappendf( &buf, fmt, ap );
+	va_end( ap );
+
+	gtk_label_set_text( GTK_LABEL( label ), buf_firstline( &buf ) );
+}
+
 /* Like set_glabel, but do it caption-style.
  */
 void 
@@ -479,7 +605,7 @@ get_geditable_name( GtkWidget *text, char *out, int sz )
 		return( FALSE );
 	}
 	im_strncpy( out, tname, sz );
-	IM_FREEF( g_free, name );
+	g_free( name );
 
 	return( TRUE );
 }
@@ -491,7 +617,7 @@ get_geditable_string( GtkWidget *text, char *out, int sz )
 
 	str = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	im_strncpy( out, str, sz );
-	IM_FREEF( g_free, str );
+	g_free( str );
 
 	return( TRUE );
 }
@@ -505,11 +631,11 @@ get_geditable_filename( GtkWidget *text, char *out, int sz )
 	filename = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	tfilename = trim_white( filename );
 	if( !is_valid_filename( tfilename ) ) {
-		IM_FREEF( g_free, filename );
+		g_free( filename );
 		return( FALSE );
 	}
 	im_strncpy( out, tfilename, sz );
-	IM_FREEF( g_free, filename );
+	g_free( filename );
 
 	return( TRUE );
 }
@@ -519,26 +645,29 @@ get_geditable_filename( GtkWidget *text, char *out, int sz )
 gboolean
 get_geditable_double( GtkWidget *text, double *out )
 {
-	char *txt = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
+	char *txt;
 	char *end;
 	double t;
 
+	txt = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	t = strtod( txt, &end );
 	if( end == txt ) {
 		error_top( _( "Bad floating point number." ) );
 		error_sub( _( "\"%s\" is not a floating point number." ), txt );
-		IM_FREEF( g_free, txt );
+		g_free( txt );
+
 		return( FALSE );
 	}
 
 	if( strspn( end, WHITESPACE ) != strlen( end ) ) {
 		error_top( _( "Bad floating point number." ) );
 		error_sub( _( "Extra characters \"%s\" after number." ), end );
-		IM_FREEF( g_free, txt );
+		g_free( txt );
+
 		return( FALSE );
 	}
+	g_free( txt );
 
-	IM_FREEF( g_free, txt );
 	*out = t;
 
 	return( TRUE );
@@ -550,17 +679,18 @@ gboolean
 get_geditable_int( GtkWidget *text, int *n )
 {
 	int i;
-	char *txt = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
+	char *txt;
 
 	/* Parse values.
 	 */
+	txt = gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 );
 	if( sscanf( txt, "%i", &i ) != 1 ) {
 		error_top( _( "Bad integer." ) );
 		error_sub( _( "\"%s\" is not an integer." ), txt );
-		IM_FREEF( g_free, txt );
+		g_free( txt );
 		return( FALSE );
 	}
-	IM_FREEF( g_free, txt );
+	g_free( txt );
 	*n = i;
 
 	return( TRUE );
@@ -822,7 +952,11 @@ filedrop_register( GtkWidget *widget, FiledropFunc fn, void *client )
 	gtk_drag_dest_set( GTK_WIDGET( widget ),
 		GTK_DEST_DEFAULT_ALL,
 		target_table, IM_NUMBER( target_table ),
-		GDK_ACTION_COPY );
+		GDK_ACTION_COPY |
+		/* That's all you need to get draggable URIs in GNOME and
+		 * win32, but KDE needs these other flags too, apparently.
+		 */
+		GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK );
 	gtk_signal_connect( GTK_OBJECT( widget ), "drag_data_received",
 		GTK_SIGNAL_FUNC( filedrop_drag_data_received ), fdi );
 }
@@ -912,8 +1046,7 @@ widget_update_pointer( GtkWidget *widget, GdkEvent *ev )
 void *
 gobject_print( GObject *gobject )
 {
-	printf( "%s (0x%x)\n", 
-		G_OBJECT_TYPE_NAME( gobject ), GPOINTER_TO_UINT( gobject ) );
+	printf( "%s (%p)\n", G_OBJECT_TYPE_NAME( gobject ), gobject );
 
 	return( NULL );
 }
@@ -942,8 +1075,8 @@ image_new_from_file( const char *name )
 	char *file;
 
 	if( (file = path_find_file( PATH_SEARCH, name )) ) {
-		image = (GtkWidget *) call_string_filename( 
-			(call_string_fn) gtk_image_new_from_file, 
+		image = (GtkWidget *) callv_string_filename( 
+			(callv_string_fn) gtk_image_new_from_file, 
 			file, NULL, NULL, NULL );
 		im_free( file );
 	}
@@ -954,3 +1087,16 @@ image_new_from_file( const char *name )
 
 	return( image );
 }
+
+void
+vfatal( GError **error )
+{
+	fprintf( stderr, PACKAGE ": fatal error\n" );
+
+	if( *error ) {
+		fprintf( stderr, "%s\n", (*error)->message );
+		IM_FREEF( g_error_free, *error );
+	}
+
+	exit( -1 );
+} 
