@@ -41,7 +41,7 @@ typedef unsigned char NodeType;
 #define TAG_CLASS (8)		/* Class object */
 #define TAG_SHARED (9)		/* Root of a common sub-expression */
 #define TAG_REFERENCE (10)	/* Reference to a common sub-expression */
-#define TAG_DOT (11)		/* 'dot' member reference */
+#define TAG_FILE (12)		/* Generate list from file */
 
 /* Element types. Generally represent operators.
  */
@@ -57,7 +57,7 @@ typedef unsigned char EType;
 #define ELEMENT_UNOP (8)	/* Unary operator */
 #define ELEMENT_COMB (9)	/* Combinator */
 #define ELEMENT_TAG (10)	/* RHS of '.' operator */
-#define ELEMENT_IMAGE (11)	/* A pointer into the image list */
+#define ELEMENT_MANAGED (11)	/* A managed object */
 #define ELEMENT_CONSTRUCTOR (12)/* Class constructor */
 #define ELEMENT_ELIST (13)	/* Empty list */
 #define ELEMENT_STATIC (14)	/* A static string */
@@ -93,14 +93,14 @@ typedef enum combinator_type {
 /* An element ... a tag plus a pointer. Use one of these to hold a pointer
  * into a heap.
  */
-typedef struct element {
+typedef struct _Element {
 	EType type;
 	void *ele;
 } Element;
 
 /* A node on the heap. Should fit in 12 bytes on most machines.
  */
-typedef struct heap_node {
+typedef struct _HeapNode {
 	/* Elements: either a pair of pointers, or a double. Sensible on most
 	 * 32-bit systems, not so great on 64 bitters.
 	 */
@@ -216,13 +216,18 @@ typedef struct pelement {
 #define PEISCHAR(P) (PEGETTYPE(P) == ELEMENT_CHAR)
 #define PEISCLASS(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_CLASS)
 #define PEISCONSTRUCTOR(P) (PEGETTYPE(P) == ELEMENT_CONSTRUCTOR)
-#define PEISDOT(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_DOT)
 #define PEISCOMB(P) (PEGETTYPE(P) == ELEMENT_COMB)
 #define PEISCOMPLEX(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_COMPLEX)
 #define PEISELIST(P) (PEGETTYPE(P) == ELEMENT_ELIST)
 #define PEISFLIST(P) (PEISNODE(P) && PEGETVAL(P)->type == TAG_CONS)
 #define PEISTAG(P) (PEGETTYPE(P) == ELEMENT_TAG)
-#define PEISIMAGE(P) (PEGETTYPE(P) == ELEMENT_IMAGE)
+#define PEISMANAGED(P) (PEGETTYPE(P) == ELEMENT_MANAGED)
+#define PEISMANAGEDGOBJECT(P) (PEISMANAGED(P) && \
+	IS_MANAGEDGOBJECT( PEGETVAL(P) ))
+#define PEISIMAGE(P) (PEISMANAGED(P) && IS_IMAGEINFO( PEGETVAL(P) ))
+#define PEISVIPSOBJECT(P) \
+		(PEISMANAGEDGOBJECT(P) && VIPS_IS_OBJECT( PEGETMANAGEDGOBJECT(P) )) 
+#define PEISFILE(P) (PEISMANAGED(P) && IS_MANAGEDFILE( PEGETVAL(P) ))
 #define PEISLIST(P) (PEISELIST(P) || PEISFLIST(P))
 #define PEISNOVAL(P) (PEGETTYPE(P) == ELEMENT_NOVAL)
 #define PEISNUM(P) (PEISREAL(P) || PEISCOMPLEX(P))
@@ -239,7 +244,7 @@ typedef struct pelement {
 #define PEGETSTATIC(P) ((HeapStaticString*)PEGETVAL(P))
 #define PEGETSYMBOL(P) ((Symbol*)PEGETVAL(P))
 #define PEGETSYMREF(P) ((Symbol*)PEGETVAL(P))
-#define PEGETCOMPILE(P) (COMPILE(PEGETVAL(P)))
+#define PEGETCOMPILE(P) ((Compile*)(PEGETVAL(P)))
 #define PEGETBINOP(P) ((BinOp)PEGETVAL(P))
 #define PEGETUNOP(P) ((UnOp)PEGETVAL(P))
 #define PEGETCOMB(P) ((CombinatorType)PEGETVAL(P))
@@ -249,6 +254,11 @@ typedef struct pelement {
 #define PEGETCHAR(P) ((unsigned char)(GPOINTER_TO_UINT(PEGETVAL(P))))
 #define PEGETIMAGE(P) (((Imageinfo*)PEGETVAL(P))->im)
 #define PEGETII(P) ((Imageinfo*)PEGETVAL(P))
+#define PEGETFILE(P) ((Managedfile*)PEGETVAL(P))
+#define PEGETMANAGED(P) ((Managed*)PEGETVAL(P))
+#define PEGETMANAGEDGOBJECT(P) (((Managedgobject*)PEGETVAL(P))->object)
+#define PEGETVIPSOBJECT(P) \
+		((VipsObject*)(((Managedgobject*)PEGETVAL(P))->object))
 
 #define PEGETHD(P1,P2) PEPOINTLEFT(PEGETVAL(P2), P1)
 #define PEGETTL(P1,P2) PEPOINTRIGHT(PEGETVAL(P2), P1)
@@ -256,9 +266,6 @@ typedef struct pelement {
 #define PEGETREALPART(P) (GETLEFT(PEGETVAL(P))->body.num)
 #define PEGETIMAGPART(P) (GETRIGHT(PEGETVAL(P))->body.num)
 
-#define PEGETDOTCOMPILE(P) (COMPILE(GETLEFT(PEGETVAL(P))))
-#define PEGETDOTTAG(P) ((char*)GETRIGHT(PEGETVAL(P)))
-
 #define PEGETCLASSCOMPILE(P) (COMPILE(GETLEFT(PEGETVAL(P))))
 #define PEGETCLASSSECRET(P1,P2) PEPOINTLEFT(GETRIGHT(PEGETVAL(P2)),P1)
 #define PEGETCLASSMEMBER(P1,P2) PEPOINTRIGHT(GETRIGHT(PEGETVAL(P2)),P1)
@@ -316,9 +323,9 @@ struct _Heap {
 	int serial;		/* Last serial number we used */
 	gboolean filled;	/* Set on heap full */
 
-	GHashTable *emark;	/* List of elements to mark on GC */
-	GSList *rmark;		/* List of Reduce to mark on GC */
-	GSList *itable;		/* Imageinfos associated with this heap */
+	GHashTable *emark;	/* Set of elements to mark on GC */
+	GHashTable *rmark;	/* Set of Reduce to mark on GC */
+	GHashTable *mtable;	/* Managed associated with this heap */
 	GHashTable *statics;	/* Static string table */
 
         guint gc_tid;		/* id of gc delay timer */
@@ -378,15 +385,10 @@ int heap_serial_new( Heap *heap );
 
 gboolean heap_bool_new( Heap *heap, gboolean val, PElement *out );
 gboolean heap_real_new( Heap *heap, double in, PElement *out );
-gboolean heap_image_new( Heap *heap, Imageinfo *ii, PElement *out );
 gboolean heap_element_new( Heap *heap, Element *e, PElement *out );
 gboolean heap_complex_element_new( Heap *heap, 
 	PElement *rp, PElement *ip, PElement *out );
 gboolean heap_complex_new( Heap *heap, double rp, double ip, PElement *out );
-gboolean heap_dot_tag_new( Heap *heap, 
-	const char *tag, Compile *compile, PElement *out );
-gboolean heap_dot_sym_new( Heap *heap, 
-	Symbol *sym, Compile *compile, PElement *out );
 gboolean heap_realvec_new( Heap *heap, int n, double *vec, PElement *out );
 void heap_list_init( PElement *list );
 gboolean heap_list_add( Heap *heap, PElement *list, PElement *data );
@@ -398,12 +400,16 @@ gboolean heap_matrix_new( Heap *heap,
 	int xsize, int ysize, double *vec, PElement *out );
 gboolean heap_string_new( Heap *heap, const char *str, PElement *out );
 gboolean heap_lstring_new( Heap *heap, GSList *labels, PElement *out );
+gboolean heap_file_new( Heap *heap, const char *filename, PElement *out );
 
 gboolean heap_error_typecheck( PElement *e, 
 	const char *name, const char *type );
 typedef void *(*heap_map_list_fn)( PElement *, void *, void * );
 void *heap_map_list( PElement *base, heap_map_list_fn fn, void *a, void *b );
+typedef void *(*heap_map_dict_fn)( const char *, PElement *, void *a, void *b );
+void *heap_map_dict( PElement *base, heap_map_dict_fn fn, void *a, void *b );
 
+gboolean heap_get_list_next( PElement *list, PElement *data );
 gboolean heap_get_string( PElement *base, char *buf, int n );
 gboolean heap_get_lstring( PElement *base, GSList **labels );
 gboolean heap_get_bool( PElement *base, gboolean *out );
@@ -415,15 +421,15 @@ gboolean heap_get_matrix_size( PElement *base, int *xsize, int *ysize );
 gboolean heap_get_matrix( PElement *base, 
 	double *buf, int n, int *xsize, int *ysize );
 
-gboolean heap_iselist( PElement *base, gboolean *out );
-gboolean heap_islist( PElement *base, gboolean *out );
-gboolean heap_isstring( PElement *base, gboolean *out );
-gboolean heap_isrealvec( PElement *base, gboolean *out );
-gboolean heap_ismatrix( PElement *base, gboolean *out );
-gboolean heap_isclass( PElement *base, gboolean *out );
-gboolean heap_isinstanceof_exact( const char *name, PElement *klass, 
+gboolean heap_is_elist( PElement *base, gboolean *out );
+gboolean heap_is_list( PElement *base, gboolean *out );
+gboolean heap_is_string( PElement *base, gboolean *out );
+gboolean heap_is_realvec( PElement *base, gboolean *out );
+gboolean heap_is_matrix( PElement *base, gboolean *out );
+gboolean heap_is_class( PElement *base, gboolean *out );
+gboolean heap_is_instanceof_exact( const char *name, PElement *klass, 
 	gboolean *out);
-gboolean heap_isinstanceof( const char *name, PElement *klass, gboolean *out );
+gboolean heap_is_instanceof( const char *name, PElement *klass, gboolean *out );
 
 int heap_list_length( PElement *base );
 gboolean heap_list_index( PElement *base, int n, PElement *out );
@@ -432,4 +438,7 @@ gboolean heap_reduce_strict( PElement *base );
 gboolean heap_copy( Heap *heap, Compile *compile, PElement *out );
 
 HeapStaticString *heap_static_string_new( Heap *heap, const char *text );
-void *heap_static_string_destroy( HeapStaticString *string );
+void *heap_static_string_unref( HeapStaticString *string );
+
+gboolean heap_ip_to_gvalue( PElement *in, GValue *out );
+gboolean heap_gvalue_to_ip( GValue *in, PElement *out );
