@@ -102,15 +102,6 @@ idialog_popdown_notify( void *sys, iWindowResult result )
 	 */
 	iwindow_susp_return( susp, result );
 
-#ifdef HAVE_WINDOWS_H
-	/* Pop the parent back up to the top of the window stack. Helps win32
-	 * keep window ordering if you alt-tab away from the app while a
-	 * dialog is being shown.
-	 */
-	if( idlg->parent_window && GTK_WIDGET_MAPPED( idlg->parent_window ) )
-		gtk_window_present( GTK_WINDOW( idlg->parent_window ) );
-#endif /*HAVE_WINDOWS_H*/
-
 	/* Housekeeping.
 	 */
 	iwindow_notify_return( IWINDOW( idlg ) );
@@ -287,7 +278,6 @@ idialog_destroy( GtkObject *object )
 		idlg->destroy_cb = NULL;
 	}
 
-	FREESID( idlg->parent_sid, idlg->parent );
 	FREESID( idlg->destroy_sid, idlg->iobject );
 	slist_map( idlg->ok_txt_l, (SListMapFn) im_free, NULL );
 	IM_FREEF( g_slist_free, idlg->ok_cb_l );
@@ -312,23 +302,6 @@ idialog_realize( GtkWidget *widget )
 		gtk_widget_grab_focus( GTK_WIDGET( idlg->entry ) );
 }
 
-/* Our parent has been destroyed, kill us too.
- */
-static void
-idialog_parent_destroy( GtkWidget *par, iDialog *idlg )
-{
-#ifdef DEBUG
-	printf( "idialog_parent_destroy: %s\n", IWINDOW( idlg )->title );
-#endif /*DEBUG*/
-
-	/* Here for dead parent ... if parent is dead, we won't need to remove
-	 * the dead-dad signal. 
-	 */
-	idlg->parent_sid = 0;
-
-	iwindow_kill( IWINDOW( idlg ) );
-}
-
 /* The object we represent has been destroyed, kill us too.
  */
 static void
@@ -358,6 +331,19 @@ idialog_build_ok( const char *txt, iDialog *idlg )
 	return( NULL );
 }
 
+/* Set a button to be the dialog default. Turn off button_focus for complex
+ * dialogs like file_chooser which have their on focus systems.
+ */
+static void
+idialog_set_default( iDialog *idlg, GtkWidget *widget )
+{
+	if( idlg->button_focus ) 
+		gtk_widget_grab_focus( widget );
+
+	GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
+	gtk_window_set_default( GTK_WINDOW( idlg ), widget );
+}
+
 static void 
 idialog_build( GtkWidget *widget )
 {
@@ -445,10 +431,8 @@ idialog_build( GtkWidget *widget )
 		/* Cancel grabs default if it's the only button. Set focus
 		 * too; user build can change this later.
 		 */
-                if( !idlg->ok_l ) {
-                        gtk_widget_grab_default( idlg->but_cancel );
-			gtk_widget_grab_focus( idlg->but_cancel );
-		}
+                if( !idlg->ok_l ) 
+			idialog_set_default( idlg, idlg->but_cancel );
 	}
 
 	/* Make OK1
@@ -461,10 +445,8 @@ idialog_build( GtkWidget *widget )
 
 	/* OK1 grabs the default.
 	 */
-	if( idlg->ok_l ) {
-		gtk_widget_grab_default( GTK_WIDGET( idlg->ok_l->data ) );
-		gtk_widget_grab_focus( GTK_WIDGET( idlg->ok_l->data ) );
-	}
+	if( idlg->ok_l ) 
+		idialog_set_default( idlg, idlg->ok_l->data );
 
 	/* Escape triggers cancel, if there is a cancel.
 	 */
@@ -493,20 +475,17 @@ idialog_build( GtkWidget *widget )
 		idlg->build( iwnd, idlg->work, 
 			idlg->build_a, idlg->build_b, idlg->build_c );
 
-	/* Link to parent.
-	 */
-        if( idlg->parent ) {
-                gtk_window_set_transient_for( GTK_WINDOW( idlg ),
-                        GTK_WINDOW( iwindow_get_root( idlg->parent ) ) );
-		idlg->parent_sid = gtk_signal_connect( 
-			GTK_OBJECT( idlg->parent ), 
-			"destroy",
-			GTK_SIGNAL_FUNC( idialog_parent_destroy ), idlg );
-	}
 	if( idlg->iobject )
 		idlg->destroy_sid = g_signal_connect( idlg->iobject, "destroy", 
 			G_CALLBACK( idialog_iobject_destroy ), idlg );
 
+	/* Set transient hint ... this may not have been set by our iwindow
+	 * superclass if the lock-in-front pref is off.
+	 */
+        if( iwnd->parent_window ) 
+		gtk_window_set_transient_for( GTK_WINDOW( iwnd ),
+			GTK_WINDOW( iwnd->parent_window ) );
+
         gtk_widget_show( idlg->work );
 }
 
@@ -543,9 +522,6 @@ idialog_init( iDialog *idlg )
 
 	/* Init our instance fields.
 	 */
-	idlg->parent = NULL;
-	idlg->parent_window = NULL;
-        idlg->parent_sid = 0;
 	idlg->iobject = NULL;
         idlg->destroy_sid = 0;
 
@@ -562,6 +538,7 @@ idialog_init( iDialog *idlg )
 	idlg->modal = FALSE;
 	idlg->pinup = FALSE;
 	idlg->nosep = FALSE;
+	idlg->button_focus = TRUE;
 
 	idlg->help_tag = NULL;
 
@@ -585,10 +562,10 @@ idialog_init( iDialog *idlg )
 GtkType
 idialog_get_type( void )
 {
-	static GtkType idialog_type = 0;
+	static GtkType type = 0;
 
-	if (!idialog_type) {
-		static const GtkTypeInfo idlg_info = {
+	if( !type ) {
+		static const GtkTypeInfo info = {
 			"iDialog",
 			sizeof( iDialog ),
 			sizeof( iDialogClass ),
@@ -599,10 +576,10 @@ idialog_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		idialog_type = gtk_type_unique( TYPE_IWINDOW, &idlg_info );
+		type = gtk_type_unique( TYPE_IWINDOW, &info );
 	}
 
-	return( idialog_type );
+	return( type );
 }
 
 GtkWidget *
@@ -618,19 +595,6 @@ idialog_new()
 	return( GTK_WIDGET( idlg ) );
 }
 
-void 
-idialog_set_parent( iDialog *idlg, GtkWidget *parent )
-{
-	idlg->parent = parent;
-
-	/* Get parent window now, we sometimes need it after parent has been
-	 * destroyed.
-	 */
-	if( parent )
-		idlg->parent_window = 
-			IWINDOW( iwindow_get_root( GTK_WIDGET( parent ) ) );
-}
-
 void 
 idialog_set_iobject( iDialog *idlg, iObject *iobject )
 {
@@ -659,10 +623,16 @@ idialog_set_nosep( iDialog *idlg, gboolean nosep )
 	idlg->nosep = nosep;
 }
 
+void 
+idialog_set_button_focus( iDialog *idlg, gboolean button_focus )
+{
+	idlg->button_focus = button_focus;
+}
+
 void 
 idialog_set_help_tag( iDialog *idlg, const char *help_tag )
 {
-	SETSTR( idlg->help_tag, help_tag );
+	IM_SETSTR( idlg->help_tag, help_tag );
 }
 
 void 
@@ -720,17 +690,8 @@ idialog_set_cancel_text( iDialog *idlg, const char *cancel_text )
 void 
 idialog_set_default_entry( iDialog *idlg, GtkEntry *entry )
 {
-	if( idlg->ok_l ) {
-		gtk_signal_connect_object( GTK_OBJECT( entry ), 
-			"focus_in_event", 
-			GTK_SIGNAL_FUNC( gtk_widget_grab_default ),
-			GTK_OBJECT( idlg->ok_l->data ) );
-		gtk_signal_connect_object( GTK_OBJECT( entry ), 
-			"activate", 
-			GTK_SIGNAL_FUNC( gtk_button_clicked ),
-			GTK_OBJECT( idlg->ok_l->data ) );
-		idlg->entry = entry;
-	}
+	gtk_entry_set_activates_default( entry, TRUE );
+	idlg->entry = entry;
 }
 
 /* Set up an entry inside a dialog ... set tooltip, set start
@@ -748,18 +709,3 @@ idialog_init_entry( iDialog *idlg, GtkWidget *entry,
         set_tooltip( entry, "%s", tip );
 	idialog_set_default_entry( idlg, GTK_ENTRY( entry ) );
 }
-
-/* Get the root (non-dialog) window for a widget.
- */
-GtkWidget *
-idialog_get_root( GtkWidget *widget )
-{
-	GtkWidget *toplevel = iwindow_get_root( widget );
-
-	/* If this is a idialog, get the window we popped up from.
-	 */
-	if( IS_IDIALOG( toplevel ) && IDIALOG( toplevel )->parent ) 
-		return( idialog_get_root( IDIALOG( toplevel )->parent ) );
-
-	return( toplevel );
-}
