@@ -35,63 +35,6 @@
 
 static ClassmodelClass *parent_class = NULL;
 
-static void
-iimage_instance_init( iImageInstance *instance, Classmodel *classmodel )
-{
-	instance->ii = NULL;
-	instance->file_changed_sid = 0;
-	instance->classmodel = classmodel;
-}
-
-static void
-iimage_instance_destroy( iImageInstance *instance )
-{
-	FREESID( instance->file_changed_sid, instance->ii );
-	IM_FREEF( imageinfo_destroy_nonheap, instance->ii );
-}
-
-static void
-iimage_instance_file_changed( Imageinfo *imageinfo, iImageInstance *instance )
-{
-	Row *row = HEAPMODEL( instance->classmodel )->row;
-
-#ifdef DEBUG
-	printf( "iimage_instance_file_changed: " );
-	iobject_print( IOBJECT( instance->classmodel ) );
-#endif /*DEBUG*/
-
-	if( CALC_RELOAD ) {
-		(void) expr_dirty( row->expr, link_serial_new() );
-		symbol_recalculate_all();
-	}
-}
-
-static gboolean
-iimage_instance_update( iImageInstance *instance, PElement *root )
-{
-	Imageinfo *ii;
-
-	if( !class_get_member_image( root, MEMBER_VALUE, &ii ) )
-		return( FALSE );
-
-	iimage_instance_destroy( instance );
-
-	instance->ii = ii;
-
-	if( ii ) {
-		imageinfo_dup_nonheap( ii );
-		instance->file_changed_sid = g_signal_connect( 
-			G_OBJECT( instance->ii ), "file_changed", 
-			G_CALLBACK( iimage_instance_file_changed ), instance );
-	}
-
-#ifdef DEBUG
-	printf( "iimage_instance_update: ii = 0x%x\n", (unsigned int) ii );
-#endif /*DEBUG*/
-
-	return( TRUE );
-}
-
 static void
 iimage_dispose( GObject *gobject )
 {
@@ -127,55 +70,58 @@ iimage_finalize( GObject *gobject )
 
 	iimage = IIMAGE( gobject );
 
-	/* My instance finalize stuff.
-	 */
-	iimage_instance_destroy( &iimage->instance );
+	image_value_destroy( &iimage->value );
 	IM_FREEF( g_slist_free, iimage->views );
 	buf_destroy( &iimage->caption_buffer );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
+/* Return the main caption. 
+ */
 static const char *
 iimage_generate_caption( iObject *iobject ) 
 {
 	iImage *iimage = IIMAGE( iobject );
-	Row *row = HEAPMODEL( iimage )->row;
-	Expr *expr;
+	Imageinfo *ii = iimage->value.ii;
+	BufInfo *buf = &iimage->caption_buffer;
 
-	if( !HEAPMODEL( iimage )->row ||
-		!(expr = HEAPMODEL( iimage )->row->expr) ) 
-		return( _( "No image" ) );
+	buf_rewind( buf );
 
-	buf_rewind( &iimage->caption_buffer );
+	image_value_caption( &iimage->value, buf );
 
-	/* Show the filename, if we know it, otherwise something from
-	 * the class.
-	 */
-	if( CLASSMODEL( iimage )->filename )
-		buf_appends( &iimage->caption_buffer, 
-			im_skip_dir( CLASSMODEL( iimage )->filename ) );
-	else if( expr && PEISCLASS( &expr->root ) ) {
-		Symbol *sym = PEGETCLASSCOMPILE( &expr->root )->sym;
-		Toolitem *toolitem;
-
-		/* If this is an action member, we should be able to look up
-		 * it's sym and get a descriptive label.
-		 */
-		if( (toolitem = toolitem_lookup( row->ws->kitg, sym )) )
-			buf_appends( &iimage->caption_buffer, toolitem->name );
-		else 
-			symbol_qualified_name_relative( row->ws->sym,
-				sym, &iimage->caption_buffer );
+	if( ii ) {
+		buf_appends( buf, ", " );
+		iobject_info( IOBJECT( iimage->value.ii ), buf );
 	}
-	else
-		buf_appends( &iimage->caption_buffer, CLASS_IMAGE );
-	buf_appends( &iimage->caption_buffer, ", " );
 
-	if( iimage->instance.ii )
-		imageinfo_info( iimage->instance.ii, &iimage->caption_buffer );
+	return( buf_all( buf ) );
+}
 
-	return( buf_all( &iimage->caption_buffer ) );
+static void
+iimage_info( iObject *iobject, BufInfo *buf )
+{
+	iImage *iimage = IIMAGE( iobject );
+	Imageinfo *ii = iimage->value.ii;
+	IMAGE *im;
+
+	if( ii && (im = imageinfo_get( FALSE, ii )) ) {
+		char *filename;
+
+		if( im_header_get_type( im, ORIGINAL_FILENAME ) != 0 ) {
+			if( !im_header_string( im, 
+				ORIGINAL_FILENAME, &filename ) ) {
+				buf_appends( buf, _( "Original filename" ) );
+				buf_appendf( buf, ": %s\n", filename );
+			}
+		}
+	}
+}
+
+static View *
+iimage_view_new( Model *model, View *parent )
+{
+	return( iimageview_new() );
 }
 
 static void
@@ -183,14 +129,17 @@ iimage_edit( GtkWidget *parent, Model *model )
 {
         iImage *iimage = IIMAGE( model );
 
-	if( iimage->instance.ii ) 
-		imageview_new( iimage );
+	if( iimage->value.ii ) {
+		Imageview *iv;
+
+		iv = imageview_new( iimage, parent );
+	}
 }
 
 static xmlNode *
 iimage_save( Model *model, xmlNode *xnode )
 {
-        iImage *iimage = IIMAGE( model );
+	iImage *iimage = IIMAGE( model );
 	xmlNode *xthis;
 
 	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
@@ -198,25 +147,22 @@ iimage_save( Model *model, xmlNode *xnode )
 
 	/* We always rebuild the value from the expr ... don't save.
 	 */
-	if( !set_prop( xthis, "window_x", "%d", iimage->window_x ) ||
-		!set_prop( xthis, "window_y", "%d", iimage->window_y ) ||
-		!set_prop( xthis, "window_width", 
-			"%d", iimage->window_width ) ||
-		!set_prop( xthis, "window_height", 
-			"%d", iimage->window_height ) ||
-		!set_prop( xthis, "image_left", "%d", iimage->image_left ) ||
+	if( !set_prop( xthis, "image_left", "%d", iimage->image_left ) ||
 		!set_prop( xthis, "image_top", "%d", iimage->image_top ) ||
 		!set_prop( xthis, "image_mag", "%d", iimage->image_mag ) ||
-		!set_prop( xthis, "show_status", 
+		!set_prop( xthis, "show_status",
 			bool_to_char( iimage->show_status ) ) ||
-		!set_prop( xthis, "show_paintbox", 
+		!set_prop( xthis, "show_paintbox",
 			bool_to_char( iimage->show_paintbox ) ) ||
-		!set_prop( xthis, "show_convert", 
+		!set_prop( xthis, "show_convert",
 			bool_to_char( iimage->show_convert ) ) ||
-		!set_prop( xthis, "show_rulers", 
+		!set_prop( xthis, "show_rulers",
 			bool_to_char( iimage->show_rulers ) ) ||
-		!set_prop( xthis, "scale", "%g", iimage->scale ) ||
-		!set_prop( xthis, "offset", "%g", iimage->offset ) )
+		!set_dprop( xthis, "scale", iimage->scale ) ||
+		!set_dprop( xthis, "offset", iimage->offset ) ||
+		!set_prop( xthis, "falsecolour", "%s",
+			bool_to_char( iimage->falsecolour ) ) ||
+		!set_prop( xthis, "type", "%s", bool_to_char( iimage->type ) ) )
 		return( NULL );
 
 	return( xthis );
@@ -230,10 +176,6 @@ iimage_load( Model *model,
 
 	assert( IS_RHS( parent ) );
 
-	(void) get_iprop( xnode, "window_x", &iimage->window_x );
-	(void) get_iprop( xnode, "window_y", &iimage->window_y );
-	(void) get_iprop( xnode, "window_width", &iimage->window_width );
-	(void) get_iprop( xnode, "window_height", &iimage->window_height );
 	(void) get_iprop( xnode, "image_left", &iimage->image_left );
 	(void) get_iprop( xnode, "image_top", &iimage->image_top );
 	(void) get_iprop( xnode, "image_mag", &iimage->image_mag );
@@ -243,6 +185,8 @@ iimage_load( Model *model,
 	(void) get_bprop( xnode, "show_rulers", &iimage->show_rulers );
 	(void) get_dprop( xnode, "scale", &iimage->scale );
 	(void) get_dprop( xnode, "offset", &iimage->offset );
+	(void) get_bprop( xnode, "falsecolour", &iimage->falsecolour );
+	(void) get_bprop( xnode, "type", &iimage->type );
 
 	return( MODEL_CLASS( parent_class )->load( model, 
 		state, parent, xnode ) );
@@ -256,9 +200,10 @@ iimage_update_heap( Heapmodel *heapmodel )
 {
 	Expr *expr = heapmodel->row->expr;
         iImage *iimage = IIMAGE( heapmodel );
-	iImageInstance *instance = &iimage->instance;
+	ImageValue *value = &iimage->value;
 
 	PElement pe;
+	Imageinfo *ii;
 
 #ifdef DEBUG
 	printf( "iimage_update_heap: " );
@@ -270,14 +215,15 @@ iimage_update_heap( Heapmodel *heapmodel )
 	 */
 	if( !class_get_exact( &expr->root, IOBJECT( heapmodel )->name, &pe ) )
 		return( FALSE );
-	if( !iimage_instance_update( instance, &pe ) )
-		return( heapmodel );
+	if( !class_get_member_image( &pe, MEMBER_VALUE, &ii ) )
+		return( FALSE );
+	image_value_set( value, ii );
 
 	IM_FREE( CLASSMODEL( iimage )->filename );
-        if( instance->ii && instance->ii->from_file &&
-                IOBJECT( instance->ii )->name )
-                SETSTR( CLASSMODEL( iimage )->filename, 
-                        IOBJECT( instance->ii )->name );
+
+        if( value->ii && imageinfo_is_from_file( value->ii ) ) 
+                IM_SETSTR( CLASSMODEL( iimage )->filename, 
+                        IOBJECT( value->ii )->name );
 
 	/* Classmodel _update_heap() will do _instance_new() from the fixed up
 	 * model.
@@ -291,7 +237,9 @@ static gboolean
 iimage_class_get( Classmodel *classmodel, PElement *root )
 {
         iImage *iimage = IIMAGE( classmodel );
-	iImageInstance *instance = &iimage->instance;
+	ImageValue *value = &iimage->value;
+
+	Imageinfo *ii;
 
 #ifdef DEBUG
 	printf( "iimage_class_get: " );
@@ -299,14 +247,31 @@ iimage_class_get( Classmodel *classmodel, PElement *root )
 	printf( "\n" );
 #endif /*DEBUG*/
 
-	if( !iimage_instance_update( instance, root ) )
+	if( !class_get_member_image( root, MEMBER_VALUE, &ii ) )
 		return( FALSE );
+	image_value_set( value, ii );
 
+	/* Try to update the filename for this row ... get from the meta if we
+	 * can.
+	 */
 	IM_FREE( classmodel->filename );
-        if( instance->ii && instance->ii->from_file &&
-                IOBJECT( instance->ii )->name )
-                SETSTR( classmodel->filename, 
-                        IOBJECT( instance->ii )->name );
+        if( ii ) {
+		IMAGE *im;
+		char *filename;
+
+		if( (im = imageinfo_get( FALSE, ii )) &&
+			im_header_get_type( im, ORIGINAL_FILENAME ) != 0 ) {
+			if( im_header_string( im, 
+				ORIGINAL_FILENAME, &filename ) )
+				return( FALSE );
+		}
+		else if( imageinfo_is_from_file( ii ) )
+			filename = IOBJECT( ii )->name;
+		else
+			filename = NULL;
+
+		IM_SETSTR( classmodel->filename, filename ); 
+	}
 
 	return( CLASSMODEL_CLASS( parent_class )->class_get( 
 		classmodel, root ) );
@@ -319,7 +284,7 @@ iimage_class_new( Classmodel *classmodel, PElement *fn, PElement *out )
 {
 	Heap *heap = reduce_context->heap;
         iImage *iimage = IIMAGE( classmodel );
-	iImageInstance *instance = &iimage->instance;
+	ImageValue *value = &iimage->value;
 
 	PElement rhs;
 
@@ -332,10 +297,11 @@ iimage_class_new( Classmodel *classmodel, PElement *fn, PElement *out )
 	/* Make application nodes.
 	 */
 	heap_appl_init( out, fn );
-	if( !heap_appl_add( heap, out, &rhs ) ||
-		!heap_image_new( heap, instance->ii, &rhs ) )
+	if( !heap_appl_add( heap, out, &rhs ) )
 		return( FALSE );
 
+	PEPUTP( &rhs, ELEMENT_MANAGED, value->ii );
+
 	return( TRUE );
 }
 
@@ -344,7 +310,7 @@ iimage_graphic_save( Classmodel *classmodel,
 	GtkWidget *parent, const char *filename )
 {
 	iImage *iimage = IIMAGE( classmodel );
-	iImageInstance *instance = &iimage->instance;
+	ImageValue *value = &iimage->value;
 	char buf[MAX_STRSIZE];
 
 	/* Can't happen nested-ly, so a static is OK. 
@@ -358,12 +324,13 @@ iimage_graphic_save( Classmodel *classmodel,
 		timer = g_timer_new();
 	g_timer_reset( timer );
 
-	if( !imageinfo_write( instance->ii, parent, buf ) )
-		return( FALSE );
+	if( value->ii )
+		if( !imageinfo_write( value->ii, parent, buf ) )
+			return( FALSE );
 
 	mainw_recent_add( &mainw_recent_image, filename );
 
-	if( main_time_save ) {
+	if( main_option_time_save ) {
 		double elapsed;
 
 		elapsed = g_timer_elapsed( timer, NULL );
@@ -426,8 +393,9 @@ iimage_class_init( iImageClass *class )
 	gobject_class->finalize = iimage_finalize;
 
 	iobject_class->generate_caption = iimage_generate_caption;
+	iobject_class->info = iimage_info;
 
-	model_class->view_new = iimageview_new;
+	model_class->view_new = iimage_view_new;
 	model_class->edit = iimage_edit;
 	model_class->save = iimage_save;
 	model_class->load = iimage_load;
@@ -451,19 +419,12 @@ iimage_class_init( iImageClass *class )
 static void
 iimage_init( iImage *iimage )
 {
-	iimage_instance_init( &iimage->instance, CLASSMODEL( iimage ) );
+	image_value_init( &iimage->value, CLASSMODEL( iimage ) );
 
 	iimage->classmodels = NULL;
 
 	iimage->views = NULL;
 
-	/* Magic: -1 means none of these saved settings are valid. It'd be
-	 * nice to do something better, but we'd break old workspaces.
-	 */
-	iimage->window_x = 0;
-	iimage->window_y = 0;
-	iimage->window_width = -1;	
-	iimage->window_height = 0;
 	iimage->image_left = 0;
 	iimage->image_top = 0;
 	iimage->image_mag = 0;
@@ -475,6 +436,8 @@ iimage_init( iImage *iimage )
 
 	iimage->scale = 0.0;
 	iimage->offset = 0.0;
+	iimage->falsecolour = FALSE;
+	iimage->type = TRUE;
 
 	buf_init_dynamic( &iimage->caption_buffer, MAX_LINELENGTH );
 
@@ -505,14 +468,3 @@ iimage_get_type( void )
 
 	return( type );
 }
-
-Classmodel *
-iimage_new( Rhs *rhs )
-{
-	iImage *iimage;
-
-	iimage = IIMAGE( g_object_new( TYPE_IIMAGE, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( iimage ), -1 );
-
-	return( CLASSMODEL( iimage ) );
-}
