@@ -118,8 +118,8 @@ iimageview_drag_data_get( GtkWidget *widget, GdkDragContext *context,
 		row_qualified_name_relative( main_workspacegroup->sym, 
 			row, &buf );
 		gtk_selection_data_set( selection_data,
-			gdk_atom_intern( "text/symbol", FALSE ),
-			8, buf_all( &buf ), strlen( buf_all( &buf ) ) );
+			gdk_atom_intern( "text/symbol", FALSE ), 8, 
+			(guchar *) buf_all( &buf ), strlen( buf_all( &buf ) ) );
 	}
 }
 
@@ -187,8 +187,8 @@ iimageview_edit( GtkWidget *parent, iImageview *iimageview )
 {
 	iImage *iimage = IIMAGE( VOBJECT( iimageview )->iobject );
 
-	if( IS_IREGION( iimage ) && iimage->instance.ii ) 
-		imageview_new( iimage );
+	if( IS_IREGION( iimage ) && iimage->value.ii ) 
+		imageview_new( iimage, parent );
 	else 
 		model_edit( parent, MODEL( iimage ) );
 }
@@ -202,9 +202,8 @@ iimageview_link( View *view, Model *model, View *parent )
 
 	VIEW_CLASS( parent_class )->link( view, model, parent );
 
-	iimageview->popup_sid = rowview_menu_attach( rview,
-		GTK_WIDGET( iimageview->id ) );
-	
+	rowview_menu_attach( rview, GTK_WIDGET( iimageview->id ) );
+
 	if( row->popup && row->top_row == row ) {
 		row->popup = FALSE;
 		iimageview_edit( GTK_WIDGET( view ), iimageview );
@@ -217,9 +216,11 @@ iimageview_refresh( vObject *vobject )
 	iImageview *iimageview = IIMAGEVIEW( vobject );
 	iImage *iimage = IIMAGE( vobject->iobject );
 	Row *row = HEAPMODEL( iimage )->row;
+
 	int w, h;
 	gboolean enabled;
 	double scale, offset;
+	gboolean falsecolour, type;
 
 #ifdef DEBUG
 	printf( "iimageview_refresh\n" );
@@ -228,27 +229,35 @@ iimageview_refresh( vObject *vobject )
 	w = IM_MAX( GTK_WIDGET( iimageview->id )->requisition.width,
 		DISPLAY_THUMBNAIL );
 	h = DISPLAY_THUMBNAIL;
-	conversion_set_image( iimageview->conv, iimage->instance.ii );
+	conversion_set_image( iimageview->conv, iimage->value.ii );
 	gtk_widget_set_size_request( GTK_WIDGET( iimageview->id ), w, h );
 	gtk_widget_queue_draw( GTK_WIDGET( iimageview->id ) );
+
 	set_gcaption( iimageview->label, "%s", 
 		NN( IOBJECT( iimage )->caption ) );
 
 	/* Set scale/offset for the thumbnail. Use the prefs setting, or if
 	 * there's a setting for this image, override with that.
 	 */
-	enabled = DISPLAY_CONVERT;
+	enabled = DISPLAY_CONVERSION;
 	scale = row->ws->scale;
 	offset = row->ws->offset;
+	falsecolour = FALSE;
+	type = TRUE;
 
-	if( iimage->window_width != -1 ) {
+	/* If the image_width has been set, a viewer must have popped down and
+	 * set it, so the recorded settings must be valid.
+	 */
+	if( CLASSMODEL( iimage )->window_width != -1 ) {
 		enabled = iimage->show_convert;
 		scale = iimage->scale;
 		offset = iimage->offset;
+		falsecolour = iimage->falsecolour;
+		type = iimage->type;
 	}
 
 	conversion_set_params( iimageview->conv, 
-		enabled, scale, offset, FALSE, TRUE );
+		enabled, scale, offset, falsecolour, type );
 
 	VOBJECT_CLASS( parent_class )->refresh( vobject );
 }
@@ -310,6 +319,25 @@ iimageview_filedrop( iImageview *iimageview, const char *file )
 	return( result );
 }
 
+static void
+iimageview_tooltip_generate( GtkWidget *widget, 
+	BufInfo *buf, iImageview *iimageview )
+{
+	iImage *iimage = IIMAGE( VOBJECT( iimageview )->iobject );
+	Imageinfo *ii = iimage->value.ii;
+	IMAGE *im = imageinfo_get( FALSE, ii );
+
+	buf_rewind( buf );
+	buf_appends( buf, buf_all( &iimage->caption_buffer ) );
+	if( im ) {
+		double size = (double) im->Ysize * IM_IMAGE_SIZEOF_LINE( im );
+
+		buf_appends( buf, ", " );
+		to_size( buf, size );
+		buf_appendf( buf, ", %.3gx%.3g p/mm", im->Xres, im->Yres );
+	}
+}
+
 static void
 iimageview_init( iImageview *iimageview )
 {
@@ -320,8 +348,6 @@ iimageview_init( iImageview *iimageview )
 	printf( "iimageview_init\n" );
 #endif /*DEBUG*/
 
-        iimageview->popup_sid = 0; 
-
         eb = gtk_event_box_new();
         gtk_box_pack_start( GTK_BOX( iimageview ), eb, FALSE, FALSE, 0 );
 	vbox = gtk_vbox_new( FALSE, 0 );
@@ -330,12 +356,11 @@ iimageview_init( iImageview *iimageview )
 
 	iimageview->conv = conversion_new( NULL );
 	iimageview->conv->tile_size = 16;
+	iimageview->conv->fade_steps = 1;
         iimageview->id = imagedisplay_new( iimageview->conv );
 	imagedisplay_set_shrink_to_fit( iimageview->id, TRUE );
         gtk_box_pack_start( GTK_BOX( vbox ), 
 		GTK_WIDGET( iimageview->id ), FALSE, FALSE, 0 );
-        set_tooltip( GTK_WIDGET( iimageview->id ), 
-		_( "Double-click to open a viewer on this thumbnail" ) );
 	gtk_widget_show( GTK_WIDGET( iimageview->id ) );
 
 	/* Need these events in the enclosing workspaceview.
@@ -360,6 +385,10 @@ iimageview_init( iImageview *iimageview )
 		DOUBLECLICK_FUNC( iimageview_doubleclick_one_cb ), iimageview,
 		DOUBLECLICK_FUNC( iimageview_doubleclick_two_cb ), iimageview );
 
+        set_tooltip_generate( eb,
+		(TooltipGenerateFn) iimageview_tooltip_generate, 
+		iimageview, NULL );
+
 	set_name( eb, "caption_widget" );
         gtk_widget_show( GTK_WIDGET( eb ) );
 }
