@@ -31,18 +31,13 @@
 
 jobs:
 
-- reference counting layer ... we im_close() the underlying image when the
-  final reference has gone
-
-  imageinfo_new makes images with count == 0, therefore the image will be
-  GCed away next time around ... you need to store the pointer somewhere
-  and dup it to make it permanent
+- reference counting layer ... in Managed base class
 
 - filesystem tracking: we stat open files and signal file_changed if we see a 
   change
 
-- cache: several open( "fred.v" )s share a single IMAGE, provided their mtimes
-  are all the same
+- cache: several open( "fred.v" )s share a single Imageinfo, provided their 
+  mtimes are all the same
 
 - lookup table management ... if an operation can work with pixel lookup 
   tables (found by examining a flag in the VIPS function descriptor), then
@@ -66,17 +61,35 @@ every Imageinfo is associated with a heap: when we GC a heap, we
 imageinfo_mark() any references to images inside that heap; after GC we
 imageinfo_destroy_heap() any unreferenced imageinfo
 
-extra wrinkle: we allow non-heap pointers to imageinfos (eg. Imagedisplay
-classes will hold a pointer to the image they are displaying) ... again,
-delay destroy until all non-heap pointers are done as well
+ */
+
+/* 
+
+more stuff:
+
+while we transition to vips8, also use imageinfo to wrap VipsImage
+
+most of the jobs above are pushed down into vips8 now ... except for
+
+- reference counting layer ... in Managed base class
+
+- filesystem tracking: we stat open files and signal file_changed if we see a
+  change
+
+- cache: several open( "fred.v" )s share a single Imageinfo, provided their 
+  mtimes are all the same
+
+
+- 
 
  */
 
+
 #include "ip.h"
 
 /*
-#define DEBUG
 #define DEBUG_OPEN
+#define DEBUG
 #define DEBUG_RGB
 #define DEBUG_CHECK
 #define DEBUG_MAKE
@@ -152,6 +165,7 @@ imageinfogroup_class_init( ImageinfogroupClass *class )
 	imageinfogroup_parent_class = g_type_class_peek_parent( class );
 
 	gobject_class->finalize = imageinfogroup_finalize;
+
 	icontainer_class->child_add = imageinfogroup_child_add;
 	icontainer_class->child_remove = imageinfogroup_child_remove;
 }
@@ -241,7 +255,7 @@ enum {
 	SIG_LAST
 };
 
-static iContainerClass *parent_class = NULL;
+static ManagedClass *parent_class = NULL;
 
 static guint imageinfo_signals[SIG_LAST] = { 0 };
 
@@ -250,10 +264,10 @@ static guint imageinfo_signals[SIG_LAST] = { 0 };
 static void
 imageinfo_print( Imageinfo *imageinfo )
 {
-	printf( " \"%s\" mtime = %d (0x%x)\n",
+	printf( " \"%s\" mtime = %d (%p)\n",
 		IOBJECT( imageinfo )->name,
 		(int) imageinfo->mtime,
-		GPOINTER_TO_UINT( imageinfo ) );
+		imageinfo );
 }
 #endif
 
@@ -318,103 +332,6 @@ imageinfo_file_changed( Imageinfo *imageinfo )
 	return( NULL );
 }
 
-/* Prettyprint an imageinfo.
- */
-void *
-imageinfo_dump( Imageinfo *imageinfo )
-{
-	printf( "imageinfo->heap = 0x%x\n", 
-		GPOINTER_TO_UINT( imageinfo->heap ) );
-	if( imageinfo->heap ) 
-		iobject_dump( IOBJECT( imageinfo->heap ) );
-
-	printf( "imageinfo->marked (refed in heap) = %s\n", 
-		bool_to_char( imageinfo->marked ) );
-	printf( "imageinfo->count (# non-heap pointers) = %d\n", 
-		imageinfo->count );
-
-	printf( "imageinfo->im = 0x%x\n", 
-		GPOINTER_TO_UINT( imageinfo->im ) );
-	printf( "imageinfo->mapped_im  = 0x%x\n", 
-		GPOINTER_TO_UINT( imageinfo->mapped_im ) );
-	printf( "imageinfo->identity_lut = 0x%x\n", 
-		GPOINTER_TO_UINT( imageinfo->identity_lut ) );
-	printf( "imageinfo->underlying = 0x%x\n", 
-		GPOINTER_TO_UINT( imageinfo->underlying ) );
-
-	printf( "imageinfo->subii (imageinfo depending on us) = %d\n",
-		g_slist_length( imageinfo->subii ) );
-
-	printf( "imageinfo->dfile (unlink on close) = %s\n",
-		bool_to_char( imageinfo->dfile ) );
-	printf( "imageinfo->from_file (was loaded from file) = %s\n",
-		bool_to_char( imageinfo->from_file ) );
-	printf( "imageinfo->mtime (mod time at load) = %d\n",
-		(int) imageinfo->mtime );
-	printf( "imageinfo->name = \"%s\"\n", IOBJECT( imageinfo )->name );
-
-	printf( "imageinfo->exprs = " );
-	slist_map( imageinfo->exprs, 
-		(SListMapFn) expr_name_print, NULL );
-	printf( "\n" );
-
-	printf( "imageinfo->ok_to_paint = %s\n",
-		bool_to_char( imageinfo->ok_to_paint ) );
-
-	if( imageinfo->underlying ) {
-		printf( "underlying:\n" );
-		imageinfo_dump( imageinfo->underlying );
-	}
-
-	return( NULL );
-}
-
-/* Debugging ... check that all imageinfos have been closed, dump any which
- * haven't.
- */
-void
-imageinfo_check_all_destroyed( Imageinfogroup *imageinfogroup )
-{
-	icontainer_map( ICONTAINER( imageinfogroup ), 
-		(icontainer_map_fn) imageinfo_dump, NULL, NULL );
-}
-
-/* imageinfo no longer depends upon in.
- */
-static void *
-imageinfo_sub_remove( Imageinfo *in, Imageinfo *imageinfo )
-{
-	assert( g_slist_find( imageinfo->subii, in ) );
-
-	imageinfo->subii = g_slist_remove( imageinfo->subii, in );
-	imageinfo_destroy_nonheap( in );
-
-	return( NULL );
-}
-
-/* imageinfo depends on in ... add a dependency.
- */
-void 
-imageinfo_sub_add( Imageinfo *imageinfo, Imageinfo *in )
-{
-	assert( imageinfo && in );
-
-	imageinfo->subii = g_slist_prepend( imageinfo->subii, in );
-	imageinfo_dup_nonheap( in );
-}
-
-/* out needs all of in[], add to sub-mark-list.
- */
-void 
-imageinfo_sub_add_all( Imageinfo *out, int nin, Imageinfo **in )
-{
-	int i;
-
-	if( out )
-		for( i = 0; i < nin; i++ )
-			imageinfo_sub_add( out, in[i] );
-}
-
 void
 imageinfo_expr_add( Imageinfo *imageinfo, Expr *expr )
 {
@@ -528,18 +445,10 @@ imageinfo_dispose( GObject *gobject )
 	imageinfo_print( imageinfo );
 #endif /*DEBUG_OPEN*/
 
-	assert( imageinfo->count == 0 );
-
-	slist_map( imageinfo->subii, 
-		(SListMapFn) imageinfo_sub_remove, imageinfo );
-	assert( !imageinfo->subii );
 	slist_map( imageinfo->exprs, 
 		(SListMapFn) imageinfo_expr_remove, imageinfo );
 	assert( !imageinfo->exprs );
 
-	imageinfo->heap->itable = 
-		g_slist_remove( imageinfo->heap->itable, imageinfo );
-
 	imageinfo_dispose_eval( imageinfo );
 
 	IM_FREEF( g_source_remove, imageinfo->check_tid );
@@ -583,64 +492,25 @@ imageinfo_finalize( GObject *gobject )
 		iobject_changed( IOBJECT( main_imageinfogroup ) );
 	}
 
-	IM_FREEF( imageinfo_destroy_nonheap, imageinfo->underlying );
+	MANAGED_UNREF( imageinfo->underlying );
 
 	imageinfo_undo_free( imageinfo );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
-/* From heap_gc() ... no heap pointers left, delete if there are no
- * non-heap pointers either.
+/* Make an info string about an imageinfo.
  */
-void
-imageinfo_destroy_heap( Imageinfo *imageinfo )
+static void
+imageinfo_info( iObject *iobject, BufInfo *buf )
 {
-#ifdef DEBUG
-	printf( "imageinfo_destroy_heap: " );
-	iobject_print( IOBJECT( imageinfo ) );
-#endif /*DEBUG*/
+	Imageinfo *imageinfo = IMAGEINFO( iobject );
 
-	/* All non-heaps gone too?
+	buf_appendi( buf, imageinfo_get( FALSE, imageinfo ) );
+
+	/* Don't chain up to parent->info(), we don't want all the other
+	 * stuff, this is going to be used for a caption.
 	 */
-	if( !imageinfo->count ) 
-		IDESTROY( imageinfo );
-}
-
-/* destroy() for non-heap pointers.
- */
-void
-imageinfo_destroy_nonheap( Imageinfo *imageinfo )
-{
-	assert( imageinfo->count > 0 );
-
-#ifdef DEBUG
-	printf( "imageinfo_destroy_nonheap: count = %d ", imageinfo->count );
-	iobject_print( IOBJECT( imageinfo ) );
-#endif /*DEBUG*/
-
-	imageinfo->count--;
-
-	/* We can't destroy the imageinfo if count == 0 && it's not marked,
-	 * since a heap pointer might have been created to it since the last
-	 * GC. Wait for the next GC to clean off stray imageinfos.
-	 */
-	heap_gc_request( imageinfo->heap );
-}
-
-/* Create a new non-heap pointer.
- */
-void
-imageinfo_dup_nonheap( Imageinfo *imageinfo )
-{
-	assert( imageinfo->count >= 0 );
-
-	imageinfo->count++;
-
-#ifdef DEBUG
-	printf( "imageinfo_dup_nonheap: count = %d ", imageinfo->count );
-	iobject_print( IOBJECT( imageinfo ) );
-#endif /*DEBUG*/
 }
 
 static void
@@ -671,12 +541,15 @@ static void
 imageinfo_class_init( ImageinfoClass *class )
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	iObjectClass *iobject_class = IOBJECT_CLASS( class );
 
 	parent_class = g_type_class_peek_parent( class );
 
 	gobject_class->dispose = imageinfo_dispose;
 	gobject_class->finalize = imageinfo_finalize;
 
+	iobject_class->info = imageinfo_info;
+
 	class->area_changed = imageinfo_real_area_changed;
 	class->area_painted = imageinfo_real_area_painted;
 	class->undo_changed = imageinfo_real_undo_changed;
@@ -720,21 +593,13 @@ static void
 imageinfo_init( Imageinfo *imageinfo )
 {
 #ifdef DEBUG_MAKE
-	printf( "imageinfo_init: 0x%x\n", GPOINTER_TO_UINT( imageinfo ) );
+	printf( "imageinfo_init: %p\n", imageinfo );
 #endif /*DEBUG_MAKE*/
 
-	imageinfo->heap = NULL;
-
-	/* Init to TRUE, so we won't close until (at least) the next GC.
-	 */
-	imageinfo->marked = TRUE;
-
-	imageinfo->count = 0;
 	imageinfo->im = NULL;
 	imageinfo->mapped_im = NULL;
 	imageinfo->identity_lut = NULL;
 	imageinfo->underlying = NULL;
-	imageinfo->subii = NULL;
 	imageinfo->dfile = FALSE;
 	imageinfo->from_file = FALSE;
 	imageinfo->mtime = 0;
@@ -769,7 +634,7 @@ imageinfo_get_type( void )
 			(GInstanceInitFunc) imageinfo_init,
 		};
 
-		type = g_type_register_static( TYPE_ICONTAINER, 
+		type = g_type_register_static( TYPE_MANAGED, 
 			"Imageinfo", &info, 0 );
 	}
 
@@ -791,12 +656,7 @@ imageinfo_new( Imageinfogroup *imageinfogroup,
 	printf( "imageinfo_new: \"%s\"\n", im->filename );
 #endif /*DEBUG_OPEN*/
 
-	imageinfo->im = im;
-
-	if( heap == NULL )
-		heap = reduce_context->heap;
-	imageinfo->heap = heap;
-	heap->itable = g_slist_prepend( heap->itable, imageinfo );
+	managed_link_heap( MANAGED( imageinfo ), heap );
 
 	if( !name ) {
 		if( !temp_name( buf, "v" ) ) 
@@ -808,6 +668,11 @@ imageinfo_new( Imageinfogroup *imageinfogroup,
 	}
 	iobject_set( IOBJECT( imageinfo ), name, NULL );
 
+	/* Only record the pointer when we know we will make the imageinfo
+	 * successfully.
+	 */
+	imageinfo->im = im;
+
 	icontainer_child_add( ICONTAINER( imageinfogroup ),
 		ICONTAINER( imageinfo ), -1 );
 
@@ -851,7 +716,7 @@ imageinfo_new_modlut( Imageinfogroup *imageinfogroup,
 	/* Link together.
 	 */
 	top_imageinfo->underlying = imageinfo;
-	imageinfo_dup_nonheap( top_imageinfo->underlying );
+	MANAGED_REF( top_imageinfo->underlying );
 
 	return( top_imageinfo );
 }
@@ -891,8 +756,7 @@ imageinfo_progress_eval( Imageinfo *imageinfo )
 	else {
 		if( elapsed > start_threshold ) {
 			imageinfo->eval_progress = progress_new( imageinfo );
-			idialog_set_parent( 
-				IDIALOG( imageinfo->eval_progress ),
+			iwindow_set_parent( IWINDOW( imageinfo->eval_progress ),
 				imageinfo->eval_parent );
 			idialog_set_iobject( 
 				IDIALOG( imageinfo->eval_progress ), 
@@ -975,7 +839,9 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		 */
 		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
 			open->heap, open->filename, "p" )) ||
-			im_tiff2vips( filename, imageinfo->im ) )
+			im_tiff2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_tiff2vips %s %s",
+				filename, imageinfo->im->filename ) ) 
 			return( NULL );
 
 #ifdef DEBUG_OPEN
@@ -983,7 +849,7 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 			"opened tiled TIFF \"%s\"\n", filename );
 #endif /*DEBUG_OPEN*/
 
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 	}
 
 	else if( im_istiff( filename ) && !im_istifftiled( filename ) ) {
@@ -992,10 +858,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
 			open->heap, open->filename, "w" )) )
 			return( NULL );
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 		imageinfo_progress_add( imageinfo, NULL );
-		if( im_tiff2vips( filename, imageinfo->im ) ) {
-			imageinfo_destroy_nonheap( imageinfo );
+		if( im_tiff2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_tiff2vips %s %s",
+				filename, imageinfo->im->filename ) ) {
+			MANAGED_UNREF( imageinfo );
 			return( NULL );
 		}
 
@@ -1009,10 +877,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
 			open->heap, open->filename, "w" )) )
 			return( NULL );
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 		imageinfo_progress_add( imageinfo, NULL );
-		if( im_jpeg2vips( filename, imageinfo->im ) ) {
-			imageinfo_destroy_nonheap( imageinfo );
+		if( im_jpeg2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_jpeg2vips %s %s",
+				filename, imageinfo->im->filename ) ) {
+			MANAGED_UNREF( imageinfo );
 			return( NULL );
 		}
 
@@ -1026,10 +896,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
 			open->heap, open->filename, "w" )) )
 			return( NULL );
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 		imageinfo_progress_add( imageinfo, NULL );
-		if( im_png2vips( filename, imageinfo->im ) ) {
-			imageinfo_destroy_nonheap( imageinfo );
+		if( im_png2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_png2vips %s %s",
+				filename, imageinfo->im->filename ) ) {
+			MANAGED_UNREF( imageinfo );
 			return( NULL );
 		}
 
@@ -1041,14 +913,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 
 	else if( im_isppm( filename ) ) {
 		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
-			open->heap, open->filename, "w" )) )
+			open->heap, open->filename, "p" )) ||
+			im_ppm2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_ppm2vips %s %s",
+				filename, imageinfo->im->filename ) ) 
 			return( NULL );
-		imageinfo_dup_nonheap( imageinfo );
-		imageinfo_progress_add( imageinfo, NULL );
-		if( im_ppm2vips( filename, imageinfo->im ) ) {
-			imageinfo_destroy_nonheap( imageinfo );
-			return( NULL );
-		}
+		MANAGED_REF( imageinfo );
 
 #ifdef DEBUG_OPEN
 		printf( "imageinfo_open_image_input: "
@@ -1056,14 +926,63 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 #endif /*DEBUG_OPEN*/
 	}
 
+	else if( im_isexr( filename ) ) {
+		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
+			open->heap, open->filename, "p" )) ||
+			im_exr2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_exr2vips %s %s",
+				filename, imageinfo->im->filename ) ) 
+			return( NULL );
+		MANAGED_REF( imageinfo );
+
+#ifdef DEBUG_OPEN
+		printf( "imageinfo_open_image_input: "
+			"opened OpenEXR \"%s\"\n", filename );
+#endif /*DEBUG_OPEN*/
+	}
+
+	else if( im_isanalyze( filename ) ) {
+		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
+			open->heap, open->filename, "p" )) ||
+			im_analyze2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_analyze2vips %s %s",
+				filename, imageinfo->im->filename ) ) 
+			return( NULL );
+		MANAGED_REF( imageinfo );
+
+#ifdef DEBUG_OPEN
+		printf( "imageinfo_open_image_input: "
+			"opened Analyze \"%s\"\n", filename );
+#endif /*DEBUG_OPEN*/
+	}
+
+	/* CSV ... test off the filename.
+	 */
+	else if( is_file_type( &filesel_sfile_type, filename ) ) {
+		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
+			open->heap, open->filename, "p" )) ||
+			im_csv2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_csv2vips %s %s",
+				filename, imageinfo->im->filename ) ) 
+			return( NULL );
+		MANAGED_REF( imageinfo );
+
+#ifdef DEBUG_OPEN
+		printf( "imageinfo_open_image_input: "
+			"opened CSV \"%s\"\n", filename );
+#endif /*DEBUG_OPEN*/
+	}
+
 	else if( im_ismagick( filename ) ) {
 		if( !(imageinfo = imageinfo_new_temp( open->imageinfogroup, 
 			open->heap, open->filename, "w" )) )
 			return( NULL );
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 		imageinfo_progress_add( imageinfo, NULL );
-		if( im_magick2vips( filename, imageinfo->im ) ) {
-			imageinfo_destroy_nonheap( imageinfo );
+		if( im_magick2vips( filename, imageinfo->im ) ||
+			im_histlin( imageinfo->im, "im_magick2vips %s %s",
+				filename, imageinfo->im->filename ) ) {
+			MANAGED_UNREF( imageinfo );
 			return( NULL );
 		}
 
@@ -1080,18 +999,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 		if( !(im = im_open( filename, "r" )) ) 
 			return( NULL );
 
-		/* Junk the old history, we don't want stuff from before this
-		 * session confusing the balancer.
-		 */
-		IM_FREE( im->Hist );
-		im_histlin( im, "" );
-
 		if( !(imageinfo = imageinfo_new( open->imageinfogroup, 
 			open->heap, im, open->filename )) ) {
 			im_close( im );
 			return( NULL );
 		}
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 
 #ifdef DEBUG_OPEN
 		printf( "imageinfo_open_image_input: opened VIPS \"%s\"\n", 
@@ -1110,6 +1023,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 	if( im_pincheck( imageinfo->im ) ) 
 		return( NULL );
 
+	/* Attach the origiinal filename ... pick this up again later as a
+	 * save default.
+	 */
+	if( im_meta_set_string( imageinfo->im, ORIGINAL_FILENAME, filename ) )
+		return( NULL );
+
 	return( imageinfo );
 }
 
@@ -1129,16 +1048,24 @@ imageinfo_open_image_output( const char *filename,
 		im_close( im );
 		return( NULL );
 	}
-	imageinfo_dup_nonheap( imageinfo );
+	MANAGED_REF( imageinfo );
 
 	return( imageinfo );
 }
 
+/* Was this ii loaded from a file (ie. ->name contains a filename the user
+ * might recognise).
+ */
+gboolean 
+imageinfo_is_from_file( Imageinfo *imageinfo )
+{
+	return( IOBJECT( imageinfo )->name && imageinfo->from_file );
+}
+
 static gint
 imageinfo_attach_check_cb( Imageinfo *imageinfo )
 {
-	if( imageinfo->from_file && imageinfo->check_tid && 
-		IOBJECT( imageinfo )->name ) {
+	if( imageinfo_is_from_file( imageinfo ) && imageinfo->check_tid ) {
 		struct stat buf;
 
 		if( !stat( IOBJECT( imageinfo )->name, &buf ) &&
@@ -1156,7 +1083,7 @@ imageinfo_attach_check_cb( Imageinfo *imageinfo )
 static void
 imageinfo_attach_check( Imageinfo *imageinfo )
 {
-	if( imageinfo->from_file && !imageinfo->check_tid ) {
+	if( imageinfo_is_from_file( imageinfo ) && !imageinfo->check_tid ) {
 		struct stat buf;
 
 		/* Need to be able to state to be able to track a file.
@@ -1190,7 +1117,7 @@ imageinfo_new_input( Imageinfogroup *imageinfogroup, GtkWidget *parent,
 	if( (imageinfo = imageinfogroup_lookup( imageinfogroup, filename )) ) {
 		/* We always make a new non-heap pointer.
 		 */
-		imageinfo_dup_nonheap( imageinfo );
+		MANAGED_REF( imageinfo );
 		return( imageinfo );
 	}
 
@@ -1199,8 +1126,8 @@ imageinfo_new_input( Imageinfogroup *imageinfogroup, GtkWidget *parent,
 	open.filename = filename;
 	open.parent = parent;
 
-        if( !(imageinfo = (Imageinfo *) call_string_filename( 
-		(call_string_fn) imageinfo_open_image_input, 
+        if( !(imageinfo = (Imageinfo *) callv_string_filename( 
+		(callv_string_fn) imageinfo_open_image_input, 
 		filename, &open, NULL, NULL )) ) {
 		error_top( _( "Unable to open image." ) );
 		error_sub( _( "Unable to open file \"%s\" as image." ), 
@@ -1234,8 +1161,8 @@ imageinfo_new_output( Imageinfogroup *imageinfogroup,
                 return( NULL );
 	}
 
-        if( !(imageinfo = (Imageinfo *) call_string_filename( 
-		(call_string_fn) imageinfo_open_image_output, 
+        if( !(imageinfo = (Imageinfo *) callv_string_filename( 
+		(callv_string_fn) imageinfo_open_image_output, 
 		name, imageinfogroup, heap, (char *) filename )) ) {
                 error_top( _( "Unable to write to file." ) );
 		error_sub( _( "Unable to open \"%s\" for write." ), filename );
@@ -1265,7 +1192,11 @@ imageinfo_get_identity_lut( Imageinfo *imageinfo )
 			imageinfo->identity_lut = im;
 
 			if( im_identity( imageinfo->identity_lut, 
-				imageinfo->im->Bands ) ) 
+				imageinfo->im->Bands ) || 
+				im_histlin( imageinfo->identity_lut, 
+					"im_identity %s %d",
+					imageinfo->identity_lut->filename,
+					imageinfo->im->Bands ) ) 
 				return( NULL );
 		}
 
@@ -1282,11 +1213,17 @@ imageinfo_get_mapped( Imageinfo *imageinfo )
 		IMAGE *im = imageinfo_get_underlying( imageinfo );
 		IMAGE *mapped_im;
 		char name[FILENAME_MAX];
+		char *argv[4];
 
 		if( !temp_name( name, "v" ) || 
 			!(mapped_im = im_open( name, "p" )) ) 
 			return( NULL );
-		if( im_maplut( im, mapped_im, imageinfo->im ) ) {
+		argv[0] = im->filename;
+		argv[1] = mapped_im->filename;
+		argv[2] = imageinfo->im->filename;
+		argv[3] = NULL;
+		if( im_maplut( im, mapped_im, imageinfo->im ) ||
+			im_updatehist( mapped_im, "im_maplut", 3, argv ) ) {
 			im_close( mapped_im );
 			error_vips_all();
 			return( NULL );
@@ -1365,7 +1302,7 @@ imageinfo_write_im( Imageinfo *out, IMAGE *im )
 gboolean
 imageinfo_write( Imageinfo *imageinfo, GtkWidget *parent, const char *filename )
 {
-	Heap *heap = imageinfo->heap;
+	Heap *heap = MANAGED( imageinfo )->heap;
 	Imageinfogroup *imageinfogroup = 
 		IMAGEINFOGROUP( ICONTAINER( imageinfo )->parent );
 	IMAGE *in = imageinfo_get( FALSE, imageinfo );
@@ -1373,11 +1310,11 @@ imageinfo_write( Imageinfo *imageinfo, GtkWidget *parent, const char *filename )
 
 	if( !(out = imageinfo_new_output( imageinfogroup, heap, filename )) ) 
 		return( FALSE );
-	imageinfo_sub_add( out, imageinfo );
+	managed_sub_add( MANAGED( out ), MANAGED( imageinfo ) );
 	imageinfo_progress_add( out, parent );
 
 	if( imageinfo_write_im( out, in ) ) {
-		imageinfo_destroy_nonheap( out );
+		MANAGED_UNREF( out );
 		error_top( _( "Unable to write to file." ) );
 		error_sub( _( "Error writing image to file \"%s\"." ), 
 			filename );
@@ -1385,18 +1322,18 @@ imageinfo_write( Imageinfo *imageinfo, GtkWidget *parent, const char *filename )
 		return( FALSE );
 	}
 
-	imageinfo_destroy_nonheap( out );
+	MANAGED_UNREF( out );
 
 	return( TRUE );
 }
 
-/* Change an imageinfo to be a file, rather than a memory object.
+/* Change an imageinfo to be a file, rather than a memory object. 
  */
-gboolean
+static gboolean
 imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent )
 {
 	IMAGE *im;
-	char name[FILENAME_MAX];
+	char filename[FILENAME_MAX];
 
 	/* Check image type.
 	 */
@@ -1407,21 +1344,23 @@ imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent )
 
 	/* Save it.
 	 */
-	if( !temp_name( name, "v" ) )
+	if( !temp_name( filename, "v" ) )
 		return( FALSE );
-	if( !imageinfo_write( imageinfo, parent, name ) )
+	if( !imageinfo_write( imageinfo, parent, filename ) )
 		return( FALSE );
-
-	if( !(im = im_open( name, "r" )) ) {
+	if( !(im = im_open( filename, "r" )) ) {
 		error_vips_all();
 		return( FALSE );
 	}
 
 	IM_FREEF( im_close, imageinfo->im );
 	IM_FREEF( im_close, imageinfo->mapped_im );
-	IM_FREEF( imageinfo_destroy_nonheap, imageinfo->underlying );
+	MANAGED_UNREF( imageinfo->underlying );
 
 	imageinfo->im = im;
+
+	/* It's now a temp file.
+	 */
 	imageinfo->dfile = TRUE;
 
 	iobject_changed( IOBJECT( imageinfo ) );
@@ -1504,18 +1443,6 @@ imageinfo_check_paintable( Imageinfo *imageinfo, GtkWidget *parent,
 	return( TRUE );
 }
 
-/* Mark an imageinfo as being used ... also mark all sub-images.
- */
-void 
-imageinfo_mark( Imageinfo *imageinfo )
-{
-	if( !imageinfo->marked ) {
-		imageinfo->marked = TRUE;
-		(void) slist_map( imageinfo->subii, 
-			(SListMapFn) imageinfo_mark, NULL );
-	}
-}
-
 /* Try to get an Imageinfo from a symbol.
  */
 Imageinfo *
@@ -1529,34 +1456,6 @@ imageinfo_sym_image( Symbol *sym )
                 return( NULL );
 }
 
-/* Make an info string about an imageinfo.
- */
-void
-imageinfo_info( Imageinfo *imageinfo, BufInfo *buf )
-{
-	IMAGE *im;
-
-	if( !(im = imageinfo_get( FALSE, imageinfo )) ) {
-		buf_appends( buf, _( "No image value" ) );
-		return;
-	}
-
-	/* Coded? Special warning.
-	 */
-	if( im->Coding != IM_CODING_NONE ) 
-		buf_appendf( buf, "%s, ", NN( im_Coding2char( im->Coding ) ) );
-
-	/* Main stuff. Format string expands to (eg.) "2000x3000 128-bit
-	 * complex, 3 bands, Lab"
-	 */
-	buf_appendf( buf, ngettext( 
-			"%dx%d %s, %d band, %s",
-			"%dx%d %s, %d bands, %s",
-			im->Bands ),
-		im->Xsize, im->Ysize, decode_bandfmt( im->BandFmt ),
-		im->Bands, decode_type( im->Type ) );
-}
-
 /* Brush definitions.
  */
 static PEL imageinfo_brush1[] = {
@@ -2288,7 +2187,7 @@ imageinfo_to_text( Imageinfo *imageinfo, BufInfo *buf )
 	int i;
 
 #define PRINT_INT( T, I ) buf_appendf( buf, "%d", ((T *)p)[I] );
-#define PRINT_FLOAT( T, I ) buf_appendf( buf, "%g", ((T *)p)[I] );
+#define PRINT_FLOAT( T, I ) buf_appendg( buf, ((T *)p)[I] );
 
 	for( i = 0; i < im->Bands; i++ ) {
 		if( i )
@@ -2440,15 +2339,15 @@ imageinfo_from_text( Imageinfo *imageinfo, const char *text )
 		if( p[0] == '(' ) {
 			/* Complex constant.
 			 */
-			re = atof( p + 1 );
+			re = g_ascii_strtod( p + 1, NULL );
 			p = break_token( p, "," );
-			im = atof( p );
+			im = g_ascii_strtod( p, NULL );
 			p = break_token( p, ")" );
 		}
 		else {
 			/* Real constant.
 			 */
-			re = atof( p );
+			re = g_ascii_strtod( p, NULL );
 			im = 0;
 		}
 
@@ -2621,6 +2520,12 @@ imageinfo_from_rgb( Imageinfo *imageinfo, double *rgb )
 			return;
 		break;
 
+	case IM_TYPE_RGB16:
+	case IM_TYPE_GREY16:
+		if( im_lintra( 1.0 / 256.0, in->im, 0.0, out->im ) )
+			return;
+		break;
+
 	case IM_TYPE_RGB:
 	case IM_TYPE_sRGB:
 	default:
@@ -2629,7 +2534,6 @@ imageinfo_from_rgb( Imageinfo *imageinfo, double *rgb )
 		break;
 	}
 
-
 #define SET( TYPE, i ) ((TYPE *) im->data)[i] = ((float *) out->im->data)[i];
 
 	/* Now ... overwrite imageinfo.
@@ -2687,6 +2591,7 @@ imageinfo_from_rgb( Imageinfo *imageinfo, double *rgb )
 				assert( FALSE );
 			}
 	}
+	im_invalidate( im );
 
 #ifdef DEBUG_RGB
 {
@@ -2772,7 +2677,7 @@ imageinfo_colour_edit( GtkWidget *parent, Imageinfo *imageinfo )
 		iwindow_true_cb, NULL, idialog_free_client, eds );
 	idialog_add_ok( IDIALOG( idlg ), 
 		imageinfo_colour_done_cb, "Set Colour" );
-	idialog_set_parent( IDIALOG( idlg ), parent );
+	iwindow_set_parent( IWINDOW( idlg ), parent );
 	idialog_set_iobject( IDIALOG( idlg ), IOBJECT( imageinfo ) );
 	iwindow_build( IWINDOW( idlg ) );
 
