@@ -27,6 +27,10 @@
 
  */
 
+/* Meta we attach for the filename we loaded from.
+ */
+#define ORIGINAL_FILENAME "original-filename"
+
 /* Group imageinfo with this.
  */
 
@@ -96,24 +100,16 @@ typedef struct _Undobuffer {
 	Rect bbox;			/* Bounding box for frags */
 } Undobuffer;
 
-/* Table of images associated with a heap ... ELEMENT_IMAGE values are
- * pointers to these guys; we mark and delete unused images on GC.
+/* A VIPS image wrapped up nicely.
  */
 struct _Imageinfo {
-	iContainer parent_object;
-
-	Heap *heap;		/* Heap we are attached to */
-
-	gboolean marked;	/* For mark-sweep */
-	int count;		/* Number of non-heap pointers to us */
+	Managed parent_object;
 
 	IMAGE *im;		/* Image we manage, LUT if delayed */
 	IMAGE *mapped_im;	/* Cache image mapped-thru-lut here */
 	IMAGE *identity_lut;	/* For base images, keep an id lut if poss */
 	Imageinfo *underlying;	/* If we're a LUT, the image we are a LUT of */
 
-	GSList *subii;		/* Sub-images ... mark these if we mark this */
-
 	gboolean dfile;		/* delete_file on final close */
 
 	gboolean from_file;	/* Set if ->name is a user file */
@@ -150,7 +146,7 @@ struct _Imageinfo {
 };
 
 typedef struct _ImageinfoClass {
-	iContainerClass parent_class;
+	ManagedClass parent_class;
 
 	/* An area of the screen needs repainting.
 	 */
@@ -173,21 +169,12 @@ typedef struct _ImageinfoClass {
 
 void *imageinfo_area_changed( Imageinfo *imageinfo, Rect *dirty );
 void *imageinfo_area_painted( Imageinfo *imageinfo, Rect *dirty );
-void *imageinfo_dump( Imageinfo *imageinfo );
-void imageinfo_check_all_destroyed( Imageinfogroup *imageinfogroup );
-
-void imageinfo_sub_add( Imageinfo *imageinfo, Imageinfo *in );
-void imageinfo_sub_add_all( Imageinfo *out, int nin, Imageinfo **in );
 
 void *imageinfo_expr_remove( Expr *expr, Imageinfo *imageinfo );
 void imageinfo_expr_add( Imageinfo *imageinfo, Expr *expr );
 GSList *imageinfo_expr_which( Imageinfo *imageinfo );
 IMAGE *imageinfo_get_underlying( Imageinfo *imageinfo );
 
-void imageinfo_destroy_heap( Imageinfo *imageinfo );
-void imageinfo_destroy_nonheap( Imageinfo *imageinfo );
-void imageinfo_dup_nonheap( Imageinfo *imageinfo );
-
 GType imageinfo_get_type( void );
 Imageinfo *imageinfo_new( Imageinfogroup *imageinfogroup, 
 	Heap *heap, IMAGE *im, const char *name );
@@ -197,6 +184,7 @@ Imageinfo *imageinfo_new_lut( Imageinfogroup *imageinfogroup,
 	Heap *heap, Imageinfo *imageinfo );
 Imageinfo *imageinfo_new_modlut( Imageinfogroup *imageinfogroup, 
 	Heap *heap, Imageinfo *imageinfo, IMAGE *im );
+gboolean imageinfo_is_from_file( Imageinfo *imageinfo );
 Imageinfo *imageinfo_new_input( Imageinfogroup *imageinfogroup, 
 	GtkWidget *parent, Heap *heap, const char *name );
 
@@ -205,7 +193,6 @@ gboolean imageinfo_same_underlying( Imageinfo *imageinfo[], int n );
 
 gboolean imageinfo_write( Imageinfo *imageinfo, 
 	GtkWidget *parent, const char *filename );
-gboolean imageinfo_file( Imageinfo *imageinfo, GtkWidget *parent );
 gboolean imageinfo_check_paintable( Imageinfo *imageinfo, 
 	GtkWidget *parent, iWindowNotifyFn nfn, void *sys );
 
@@ -218,8 +205,6 @@ void imageinfo_mark( Imageinfo *imageinfo );
 
 Imageinfo *imageinfo_sym_image( Symbol *sym );
 
-void imageinfo_info( Imageinfo *imageinfo, BufInfo *buf );
-
 void imageinfo_undo_mark( Imageinfo *imageinfo );
 gboolean imageinfo_undo( Imageinfo *imageinfo );
 gboolean imageinfo_redo( Imageinfo *imageinfo );
