@@ -72,7 +72,7 @@ static ImagepresentKeymap imagepresent_keymap[] = {
 	{ GDK_4, 4 },
 	{ GDK_5, 5 },
 	{ GDK_6, 6 },
-	{ GDK_7, 6 },
+	{ GDK_7, 7 },
 	{ GDK_8, 8 },
 	{ GDK_9, 9 }
 };
@@ -236,53 +236,32 @@ imagepresent_hruler_rethink( Imagepresent *ip )
 	Conversion *conv = imagemodel->conv;
 	IMAGE *im = imageinfo_get( FALSE, conv->ii );
 
-	/* Try to get the ruler width. If it looks like it may not have been
-	 * set yet, use the visible image width instead.
+	/* Try to get the ruler width: same as the whole of the scrolled
+	 * window.
 	 */
-	int ruler_width = GTK_WIDGET( ip->hrule )->allocation.width == 1 ?
-		imagemodel->visible.width :
-		GTK_WIDGET( ip->hrule )->allocation.width;
+	int ruler_width = GTK_WIDGET( ip->swin )->allocation.width;
 
-	/* Difference between width of ruler and width of image display area.
-	 */
-	int margin = ruler_width - imagemodel->visible.width;
+	double from = imagemodel->visible.left;
+	double to = from + ruler_width;
+	double pos = ip->last_x;
 
-	/* Display position.
-	 */
-	int dx_from = imagemodel->visible.left;
-	int dx_to = IM_RECT_RIGHT( &imagemodel->visible ) + margin;
-
-	/* Scale factor.
-	 */
 	double scale;
 
-	/* Position in image.
-	 */
-	double ix_from;
-	double ix_to;
-	double ix_pos;
-
-	ix_from = dx_from;
-	ix_to = dx_to;
-	ix_pos = ip->last_x;
-
 	if( imagemodel->rulers_offset && im ) {
-		ix_from -= im->Xoffset;
-		ix_to -= im->Xoffset;
-		ix_pos -= im->Xoffset;
+		from -= im->Xoffset;
+		to -= im->Xoffset;
+		pos -= im->Xoffset;
 	}
 
 	scale = conversion_dmag( conv->mag );
-
 	if( imagemodel->rulers_mm && im ) 
 		scale *= im->Xres;
 
-	ix_from /= scale;
-	ix_to /= scale;
-	ix_pos /= scale;
+	from /= scale;
+	to /= scale;
+	pos /= scale;
 
-	gtk_ruler_set_range( GTK_RULER( ip->hrule ), 
-		ix_from, ix_to, ix_pos, ix_to - ix_from );
+	gtk_ruler_set_range( GTK_RULER( ip->hrule ), from, to, pos, to - from );
 }
 
 static void
@@ -292,42 +271,32 @@ imagepresent_vruler_rethink( Imagepresent *ip )
 	Conversion *conv = imagemodel->conv;
 	IMAGE *im = imageinfo_get( FALSE, conv->ii );
 
-	int ruler_height = GTK_WIDGET( ip->vrule )->allocation.height == 1 ?
-		imagemodel->visible.height :
-		GTK_WIDGET( ip->vrule )->allocation.height;
+	/* Try to get the ruler height: same as the whole of the scrolled
+	 * window.
+	 */
+	int ruler_height = GTK_WIDGET( ip->swin )->allocation.height;
 
-	int margin = ruler_height - imagemodel->visible.height;
-
-	int dy_from = imagemodel->visible.top;
-	int dy_to = IM_RECT_BOTTOM( &imagemodel->visible ) + margin;
+	double from = imagemodel->visible.top;
+	double to = from + ruler_height;
+	double pos = ip->last_y;
 
 	double scale;
 
-	double iy_from;
-	double iy_to;
-	double iy_pos;
-
-	iy_from = dy_from;
-	iy_to = dy_to;
-	iy_pos = ip->last_y;
-
 	if( imagemodel->rulers_offset && im ) {
-		iy_from -= im->Yoffset;
-		iy_to -= im->Yoffset;
-		iy_pos -= im->Yoffset;
+		from -= im->Yoffset;
+		to -= im->Yoffset;
+		pos -= im->Yoffset;
 	}
 
 	scale = conversion_dmag( conv->mag );
-
 	if( imagemodel->rulers_mm && im ) 
 		scale *= im->Yres;
 
-	iy_from /= scale;
-	iy_to /= scale;
-	iy_pos /= scale;
+	from /= scale;
+	to /= scale;
+	pos /= scale;
 
-	gtk_ruler_set_range( GTK_RULER( ip->vrule ), 
-		iy_from, iy_to, iy_pos, iy_to - iy_from );
+	gtk_ruler_set_range( GTK_RULER( ip->vrule ), from, to, pos, to - from );
 }
 
 /* Zoom with the mouse clicked at position x, y in canvas coordinates.
@@ -377,13 +346,15 @@ imagepresent_scroll_event_cb( GtkWidget *widget,
 {
 	Imagemodel *imagemodel = ip->imagemodel;
 	Rect *visible = &imagemodel->visible;
-	gboolean handled = FALSE;
+	gboolean handled;
 
 	/* Gimp uses page_incr / 4 I think, but then scroll speed varies with
 	 * window size, which is pretty odd. Just use a constant.
 	 */
 	const int incr = 50;
 
+	handled = FALSE;
+
 	if( ev->direction == GDK_SCROLL_UP || 
 		ev->direction == GDK_SCROLL_DOWN ) {
 		if( ev->state & GDK_CONTROL_MASK ) {
@@ -481,6 +452,11 @@ imagepresent_floating_new( Imagepresent *ip,
 	regionview_attach( ip->regionview, x, y );
 }
 
+/* Need to fwd ref this.
+ */
+static void imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, 
+	int x, int y );
+
 static gint
 imagepresent_hruler_event( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 {
@@ -516,6 +492,19 @@ imagepresent_hruler_event( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 		}
 		break;
 
+	case GDK_BUTTON_RELEASE:
+		switch( ev->button.button ) {
+		case 1:
+			imagepresent_left_release( ip, ev, 
+				ev->button.x, ev->button.y );
+			handled = TRUE;
+			break;
+
+		default:
+			break;
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -552,6 +541,19 @@ imagepresent_vruler_event( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 		}
 		break;
 
+	case GDK_BUTTON_RELEASE:
+		switch( ev->button.button ) {
+		case 1:
+			imagepresent_left_release( ip, ev, 
+				ev->button.x, ev->button.y );
+			handled = TRUE;
+			break;
+
+		default:
+			break;
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -899,7 +901,7 @@ imagepresent_paint_stop( Imagepresent *ip, int x, int y )
 {
 	Imagemodel *imagemodel = ip->imagemodel;
 	Conversion *conv = imagemodel->conv;
-	Imageinfo *imageinfo = imagemodel->iimage->instance.ii;
+	Imageinfo *imageinfo = imagemodel->iimage->value.ii;
         Rect oper;
         int ix, iy;
 
@@ -1114,7 +1116,7 @@ imagepresent_button_motion( Imagepresent *ip, GdkEvent *ev )
 {
 	Imagemodel *imagemodel = ip->imagemodel;
 	Conversion *conv = imagemodel->conv;
-	Imageinfo *imageinfo = imagemodel->iimage->instance.ii;
+	Imageinfo *imageinfo = imagemodel->iimage->value.ii;
 	Rect oper;
 	int x, y;
 	int ix, iy;
@@ -1190,12 +1192,14 @@ static gboolean
 imagepresent_event_cb( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 {
 	Imagemodel *imagemodel = ip->imagemodel;
-	gboolean handled = FALSE;
+	gboolean handled;
 
 #ifdef EVENT
 	printf( "imagepresent_event_cb %d\n", ev->type );
 #endif /*EVENT*/
 
+	handled = FALSE;
+
 	switch( ev->type ) {
 	case GDK_BUTTON_PRESS:
 		if( !GTK_WIDGET_HAS_FOCUS( GTK_WIDGET( ip ) ) )
@@ -1244,7 +1248,7 @@ imagepresent_event_cb( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 				ev->button.x, ev->button.y );
 #endif /*EVENT*/
 
-			/* Should always suceed.
+			/* Should always succeed.
 			 */
 			(void) imagemodel_set_state( imagemodel, 
 				imagemodel->save_state, NULL );
@@ -1296,7 +1300,7 @@ imagepresent_event_cb( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 	return( handled );
 }
 
-static gint
+static gboolean
 imagepresent_key_press_event_cb( GtkWidget *widget, GdkEventKey *event, 
 	Imagepresent *ip )
 {
@@ -1305,13 +1309,15 @@ imagepresent_key_press_event_cb( GtkWidget *widget, GdkEventKey *event,
 	Rect *visible = &imagemodel->visible;
 	GtkAdjustment *hadj = ip->hadj;
 	GtkAdjustment *vadj = ip->vadj;
-	gboolean handled = FALSE;
+	gboolean handled;
 	int i;
 
 #ifdef DEBUG
 	printf( "imagepresent_key_press_event_cb\n" );
 #endif /*DEBUG*/
 
+	handled = FALSE;
+
 	switch( event->keyval ) {
 	case GDK_Left:
 		if( !(event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK)) )
@@ -1395,7 +1401,7 @@ imagepresent_key_press_event_cb( GtkWidget *widget, GdkEventKey *event,
 		break;
 	}
 
-	/* Chek the number zoom keys too.
+	/* Check the number zoom keys too.
 	 */
 	if( !handled ) 
 		for( i = 0; i < IM_NUMBER( imagepresent_keymap ); i++ )
@@ -1448,7 +1454,6 @@ imagepresent_ruler_hide_cb( GtkWidget *wid, GtkWidget *host, Imagepresent *ip )
 static void
 imagepresent_init( Imagepresent *ip )
 {
-	GtkScrolledWindow *swin;
 	GtkWidget *port;
 	GtkWidget *bar;
 	GtkWidget *table;
@@ -1496,16 +1501,16 @@ imagepresent_init( Imagepresent *ip )
 	gtk_signal_connect( GTK_OBJECT( ip ), "key_press_event",
 		GTK_SIGNAL_FUNC( imagepresent_key_press_event_cb ), ip );
 
-	swin = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new( NULL, NULL ) );
-	gtk_scrolled_window_add_with_viewport( swin, GTK_WIDGET( ip->id ) );
-        gtk_scrolled_window_set_policy( swin,
+	ip->swin = GTK_SCROLLED_WINDOW( gtk_scrolled_window_new( NULL, NULL ) );
+	gtk_scrolled_window_add_with_viewport( ip->swin, GTK_WIDGET( ip->id ) );
+        gtk_scrolled_window_set_policy( ip->swin,
 		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-	ip->hadj = gtk_scrolled_window_get_hadjustment( swin );
-	ip->vadj = gtk_scrolled_window_get_vadjustment( swin );
-	gtk_signal_connect( GTK_OBJECT( swin ), "scroll_event",
+	ip->hadj = gtk_scrolled_window_get_hadjustment( ip->swin );
+	ip->vadj = gtk_scrolled_window_get_vadjustment( ip->swin );
+	gtk_signal_connect( GTK_OBJECT( ip->swin ), "scroll_event",
 		GTK_SIGNAL_FUNC( imagepresent_scroll_event_cb ), ip );
 
-	port = gtk_bin_get_child( GTK_BIN( swin ) );
+	port = gtk_bin_get_child( GTK_BIN( ip->swin ) );
 	assert( GTK_IS_VIEWPORT( port ) );
 	gtk_signal_connect( GTK_OBJECT( ip->hadj ), "changed",
 		GTK_SIGNAL_FUNC( imagepresent_hadj_changed_cb ), ip );
@@ -1516,11 +1521,11 @@ imagepresent_init( Imagepresent *ip )
 	gtk_signal_connect( GTK_OBJECT( ip->vadj ), "value_changed",
 		GTK_SIGNAL_FUNC( imagepresent_vadj_changed_cb ), ip );
 
-	bar = swin->hscrollbar;
-	assert( GTK_IS_SCROLLBAR( bar ) );
+	bar = ip->swin->hscrollbar;
+	g_assert( GTK_IS_SCROLLBAR( bar ) );
 	GTK_WIDGET_UNSET_FLAGS( bar, GTK_CAN_FOCUS );
-	bar = swin->vscrollbar;
-	assert( GTK_IS_SCROLLBAR( bar ) );
+	bar = ip->swin->vscrollbar;
+	g_assert( GTK_IS_SCROLLBAR( bar ) );
 	GTK_WIDGET_UNSET_FLAGS( bar, GTK_CAN_FOCUS );
 
 	/* Need one menu per image window (could have a single menu for all
@@ -1572,13 +1577,13 @@ imagepresent_init( Imagepresent *ip )
 		GTK_FILL, 
 		GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 		2, 2 );
-	gtk_table_attach( GTK_TABLE( table ), GTK_WIDGET( swin ), 
+	gtk_table_attach( GTK_TABLE( table ), GTK_WIDGET( ip->swin ), 
 		1, 2, 1, 2,
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK, 
 		GTK_FILL | GTK_EXPAND | GTK_SHRINK,
 		2, 2 );
 	gtk_widget_show( GTK_WIDGET( ip->id ) );
-	gtk_widget_show( GTK_WIDGET( swin ) );
+	gtk_widget_show( GTK_WIDGET( ip->swin ) );
 
 	/* Set initial ruler visibility on first refresh from imagemodel.
 	 */
