@@ -36,7 +36,7 @@
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_IMAGEVIEW ))
 
 typedef struct _Imageview {
-	iWindow parent_class;
+	Floatwindow parent_class;
 
 	/* Model stuff here.
 	 */
@@ -54,7 +54,7 @@ typedef struct _Imageview {
 } Imageview;
 
 typedef struct _ImageviewClass {
-	iWindowClass parent_class;
+	FloatwindowClass parent_class;
 
 	/* My methods.
 	 */
@@ -62,6 +62,5 @@ typedef struct _ImageviewClass {
 
 GtkType imageview_get_type( void );
 void imageview_set_paint( Imageview *iv, gboolean paint );
-Imageview *imageview_new( iImage *iimage );
-Imageview *imageview_new_area( iImage *iimage, Rect *area );
-
+Imageview *imageview_new( iImage *iimage, GtkWidget *parent );
+Imageview *imageview_new_area( iImage *iimage, Rect *area, GtkWidget *parent );
