@@ -608,16 +608,6 @@ iwindow_popdown_notify( iWindow *iwnd, iWindowResult result )
 		iwnd->destroy = FALSE;
 	}
 	else {
-#ifdef OS_WIN32
-		/* Pop the parent back up to the top of the window stack. 
-		 * Helps win32 keep window ordering if you alt-tab away from 
-		 * the app while a dialog is being shown.
-		 */
-		if( iwnd->parent_window && 
-			GTK_WIDGET_MAPPED( iwnd->parent_window ) )
-			gtk_window_present( GTK_WINDOW( iwnd->parent_window ) );
-#endif /*OS_WIN32*/
-
 		/* Popdown confirmed! Trigger class popdown. _real_popdown()
 		 * does an unmap to hide the window during the rest of the
 		 * destroy.
