@@ -45,10 +45,19 @@
 #define DEBUG_TIME
  */
 
-/* On quit, make sure we free stuff we can free.
+/* On quit, make sure we free stuff we can free. Define PROFILE too to get
+ * glib's mem use profiler. But it's broken on windows and will cause strange
+ * problems, careful.
 #define DEBUG_LEAK
+#define PROFILE
  */
 
+/* Need im__print_all() for leak testing.
+ */
+#ifdef DEBUG_LEAK
+extern void im__print_all( void );
+#endif /*DEBUG_LEAK*/
+
 /* General stuff. 
  */
 GdkWindow *main_window_gdk;			/* Top GdkWindow */
@@ -64,24 +73,66 @@ void *main_c_stack_base = NULL;			/* Base of C stack */
 
 gboolean main_starting = TRUE;			/* In startup */
 
-gboolean main_time_save = FALSE;		/* Time save image ops */
-
-/* Action on main_quit()
- */
-typedef enum {
-	EXIT_ACTION_NONE,			/* Just quit */
-	EXIT_ACTION_PRINT_MAIN			/* Print symbol "main" */
-} ExitAction;
-
-static ExitAction action_on_exit = EXIT_ACTION_NONE;
-
 static const char *main_argv0 = NULL;		/* argv[0] */
 static iOpenFile *main_stdin = NULL;		/* stdin as an iOpenFile */
-static gboolean main_interactive = TRUE;	/* In GUI mode */
-static gboolean main_load_all = TRUE;		/* Load all .defs */
 static GtkIconFactory *main_icon_factory = NULL;/* Add stocks to this */
 static Splash *main_splash = NULL;
 
+static char *main_option_script = NULL;
+static char *main_option_expression = NULL;
+gboolean main_option_batch = FALSE;
+static gboolean main_option_no_load_menus = FALSE;
+static gboolean main_option_no_load_args = FALSE;
+static gboolean main_option_stdin_ws = FALSE;
+static gboolean main_option_stdin_def = FALSE;
+static char *main_option_output = NULL;
+static gboolean main_option_benchmark = FALSE;
+gboolean main_option_time_save = FALSE;
+gboolean main_option_i18n = FALSE;
+static gboolean main_option_print_main = FALSE;
+static gboolean main_option_version = FALSE;
+static gboolean main_option_verbose = FALSE;
+
+static GOptionEntry main_option[] = {
+	{ "expression", 'e', 0, G_OPTION_ARG_STRING, &main_option_expression, 
+		N_( "evaluate and print EXPRESSION" ), 
+		"EXPRESSION" },
+	{ "script", 's', 0, G_OPTION_ARG_FILENAME, &main_option_script, 
+		N_( "load FILE as a set of definitions" ), 
+		"FILE" },
+	{ "output", 'o', 0, G_OPTION_ARG_FILENAME, &main_option_output, 
+		N_( "write value of 'main' to FILE" ), "FILE" },
+	{ "batch", 'b', 0, G_OPTION_ARG_NONE, &main_option_batch, 
+		N_( "run in batch mode" ), NULL },
+	{ "verbose", 'V', 0, G_OPTION_ARG_NONE, &main_option_verbose, 
+		N_( "verbose error output" ), NULL },
+	{ "no-load-menus", 'm', 0, G_OPTION_ARG_NONE, 
+		&main_option_no_load_menus, 
+		N_( "don't load menu definitions" ), NULL },
+	{ "no-load-args", 'a', 0, G_OPTION_ARG_NONE, &main_option_no_load_args, 
+		N_( "don't try to load extra cmd-line args as files" ), NULL },
+	{ "stdin-ws", 'w', 0, G_OPTION_ARG_NONE, &main_option_stdin_ws, 
+		N_( "load stdin as a workspace" ), NULL },
+	{ "stdin-def", 'd', 0, G_OPTION_ARG_NONE, &main_option_stdin_def, 
+		N_( "load stdin as a set of definitions" ), NULL },
+	{ "print-main", 'p', 0, G_OPTION_ARG_NONE, &main_option_print_main, 
+		N_( "print value of 'main' to stdout" ), 
+		NULL },
+	{ "benchmark", 'c', 0, G_OPTION_ARG_NONE, &main_option_benchmark, 
+		N_( "start up and shut down" ), 
+		NULL },
+	{ "time-save", 't', 0, G_OPTION_ARG_NONE, &main_option_time_save, 
+		N_( "time image save operations" ), 
+		NULL },
+	{ "i18n", 'i', 0, G_OPTION_ARG_NONE, &main_option_i18n, 
+		N_( "output strings for internationalisation" ), 
+		NULL },
+	{ "version", 'v', 0, G_OPTION_ARG_NONE, &main_option_version, 
+		N_( "print version number" ), 
+		NULL },
+	{ NULL }
+};
+
 /* Accumulate startup errors here.
  */
 static char main_start_error_txt[MAX_STRSIZE];
@@ -105,89 +156,50 @@ main_log_get( void )
 }
 
 static gboolean
-main_log_isempty( void )
+main_log_is_empty( void )
 {
-	return( buf_isempty( &main_start_error ) );
-}
-
-static void
-main_usage( void )
-{
-	printf( _(
-"%s: usage:\n"
-"%s filename1 filename2 ...\n"
-"\tstart in GUI mode, loading the named files" ),
-		g_get_prgname(), g_get_prgname() );
-	printf( "\n" );
-
-	printf( _(
-"%s -script filename arg1 arg2 ...\n"
-"\tread in filename as a set of definitions, set list argv to\n"
-"\t[\"filename\", \"arg1\", \"arg2\", ...], set argc to length of list;\n"
-"\tprint the value of symbol \"main\" to stdout; exit; useful for\n"
-"\trunning %s as an interpreter on unix" ),
-		g_get_prgname(), g_get_prgname() );
-	printf( "\n" );
-
-	printf( _(
-"%s -main arg1 arg2 ...\n"
-"\tas -script, but read the definitions from stdin rather than from a\n"
-"\tfile; useful for here-is shell scripts" ),
-		g_get_prgname() );
-	printf( "\n" );
-
-	printf( _(
-"%s -workspace arg1 arg2 ...\n"
-"\tas -main, but read a workspace save file instead; run in GUI mode\n"
-"\tprint the value of the final row in the save file on exit" ),
-		g_get_prgname() );
-	printf( "\n" );
-
-	printf( _(
-"%s -i18n\n"
-"\toutput all (_"") strings in all toolkits and quit" ),
-		g_get_prgname() );
-	printf( "\n" );
-
-	printf( _(
-"%s -benchmark\n"
-"\tload all start objects and quit; useful for bechmarking the compiler" ),
-		g_get_prgname() );
-	printf( "\n" );
-
-	printf( _(
-"%s -time_save\n"
-"\ttime all save image operations, useful for benchmarking image\n" 
-"\tprocessing functions" ),
-		g_get_prgname() );
-	printf( "\n" );
-
-	printf( _( "%s version: %s" ), PACKAGE, VERSION );
-	printf( "\n" );
-
-	printf( _( "linked to vips library: %s" ), im_version_string() );
-	printf( "\n" );
+	return( buf_is_empty( &main_start_error ) );
 }
 
 /* NULL log handler. Used to suppress output on win32 without DEBUG_FATAL.
  */
 #ifndef DEBUG_FATAL
-#ifdef HAVE_WINDOWS_H 
+#ifdef OS_WIN32 
 static void
 main_log_null( const char *log_domain, GLogLevelFlags log_level,
 	const char *message, void *user_data )
 {
 }
-#endif /*HAVE_WINDOWS_H*/ 
+#endif /*OS_WIN32*/ 
 #endif /*!DEBUG_FATAL*/
 
+/* Print all errors and quit. Batch mode only.
+ */
+static void
+main_error_exit( const char *msg )
+{
+	fprintf( stderr, "%s %s\n", 
+		error_get_top(), error_get_sub() );
+
+	if( main_option_verbose ) {
+		BufInfo buf;
+		char txt[MAX_STRSIZE];
+
+		buf_init_static( &buf, txt, MAX_STRSIZE );
+		expr_error_print_all( &buf );
+		fprintf( stderr, "%s\n", buf_all( &buf ) );
+		error( msg );
+	}
+
+	exit( 1 );
+}
+
 /* Clean up our application and quit. Not interactive! Do any "has been
  * modified, OK to quit?" stuff before this.
  */
 static void
 main_quit( void )
 {
-	Symbol *sym;
 #if HAVE_FFTW || HAVE_FFTW3
 	iOpenFile *of;
 #endif /*HAVE_FFTW || HAVE_FFTW3*/
@@ -196,41 +208,38 @@ main_quit( void )
 	printf( "main_quit: cleaning up ...\n" );
 #endif/*DEBUG*/
 
-	/* Any exit actions?
-	 */
-	switch( action_on_exit ) {
-	case EXIT_ACTION_NONE:	
-		break;
+	if( main_option_print_main || main_option_output ) {
+		Symbol *sym;
+		PElement *root;
+
+		symbol_recalculate_all();
 
-	case EXIT_ACTION_PRINT_MAIN:
 		sym = compile_lookup( symbol_root->expr->compile, "main" );
 
 		if( !sym )
-			error( _( "symbol \"main\" not found" ) );
-		if( symbol_recalculate( sym ) ||
-			!reduce_pelement( reduce_context, reduce_spine_strict, 
-				&sym->expr->root ) ) {
-			BufInfo buf;
-			char txt[MAX_STRSIZE];
+			main_error_exit( _( "symbol \"main\" not found" ) );
+		root = &sym->expr->root;
+		if( symbol_recalculate_check( sym ) || 
+			!reduce_pelement( reduce_context, 
+				reduce_spine_strict, root ) ) 
+			main_error_exit( _( "error calculating \"main\"" ) );
 
-			buf_init_static( &buf, txt, MAX_STRSIZE );
-			expr_error_print_all( &buf );
-			fprintf( stderr, "%s\n", buf_all( &buf ) );
-			error( _( "error calculating \"main\"" ) );
+		if( main_option_output ) {
+			char filename[FILENAME_MAX];
+
+			im_strncpy( filename, main_option_output, 
+				FILENAME_MAX );
+			(void) save_objects( root, filename );
 		}
 
-		(void) set_output( &sym->expr->root );
-		(void) graph_value( &sym->expr->root );
-		break;
-
-	default:
-		assert( FALSE );
+		if( main_option_print_main )
+			graph_value( root );
 	}
 
 	/* Force all windows down. 
-	
+
 		FIXME 
-		
+
 		this can cause a recursive call to us from mainw_destroy()
 
 	 */
@@ -240,7 +249,7 @@ main_quit( void )
 	/* Dump wisdom back again.
 	 */
 #if HAVE_FFTW || HAVE_FFTW3
-	if( (of = file_open_write( "%s" IM_DIR_SEP_STR "wisdom", 
+	if( (of = file_open_write( "%s" G_DIR_SEPARATOR_S "wisdom", 
 		get_savedir() )) ) {
 		fftw_export_wisdom_to_file( of->fp );
 		file_close( of );
@@ -264,7 +273,7 @@ main_quit( void )
 	reduce_destroy( reduce_context );
 
 #ifdef DEBUG_LEAK
-	printf( "DEBUG_LEAK: testing for leaks ...\n" );
+	fprintf( stderr, "DEBUG_LEAK: testing for leaks ...\n" );
 
 	/* Free other GTK stuff.
 	 */
@@ -279,12 +288,19 @@ main_quit( void )
 		FIXME ... #ifdef this lot out at some point
 
 	 */
-	imageinfo_check_all_destroyed( main_imageinfogroup );
 	UNREF( main_imageinfogroup );
 	heap_check_all_destroyed();
 	im__print_all();
+	managed_check_all_destroyed();
 	util_check_all_destroyed();
 	vips_check_all_destroyed();
+#ifdef HAVE_VIPS8
+	vips_object_print_all();
+#endif /*HAVE_VIPS8*/
+
+#ifdef PROFILE
+	g_mem_profile();
+#endif /*PROFILE*/
 #endif /*DEBUG_LEAK*/
 
 #ifdef DEBUG
@@ -324,7 +340,7 @@ main_quit_test( void )
 gboolean
 main_splash_enabled( void )
 {
-	return( !existsf( "%s" IM_DIR_SEP_STR "%s", 
+	return( !existsf( "%s" G_DIR_SEPARATOR_S "%s", 
 		get_savedir(), NO_SPLASH ) );
 }
 
@@ -345,15 +361,11 @@ main_splash_update( const char *fmt, ... )
 static void
 main_watchgroup_changed_cb( void )
 {
-	static int last_max_cpus = -1;
-
-	/* Try to avoid setting too often ... we can get leaks on some
-	 * platforms.
+	/* Only set this in GUI mode. Otherwise, let the user control CPUs 
+	 * with the env variable and --vips-concurrency args.
 	 */
-	if( last_max_cpus != VIPS_CPUS ) {
-		last_max_cpus = VIPS_CPUS;
-		setenvf( "IM_CONCURRENCY", "%d", VIPS_CPUS );
-	}
+	if( !main_option_batch )
+		im_concurrency_set( VIPS_CPUS );
 }
 
 /* Try to load a thing, anything at all. Actually, we don't load plugins
@@ -370,23 +382,29 @@ main_load( Workspace *ws, const char *filename )
 		workspace_new_from_file( main_workspacegroup, filename )) ) {
 		Mainw *new_mainw;
 
-		new_mainw = mainw_new( new_ws );
-		gtk_widget_show( GTK_WIDGET( new_mainw ) );
+		if( !main_option_batch ) {
+			new_mainw = mainw_new( new_ws );
+			gtk_widget_show( GTK_WIDGET( new_mainw ) );
+		}
 		mainw_recent_add( &mainw_recent_workspace, filename );
 
 		return( TRUE );
 	}
 	error_clear();
 
+	/* workspace_load_file() needs to recalc to work, try to avoid that by
+	 * doing .defs first.
+	 */
+	if( is_file_type( &filesel_dfile_type, filename ) ) {
+		if( toolkit_new_from_file( main_toolkitgroup, filename ) )
+			return( TRUE );
+	}
+
 	/* Try as matrix or image. Have to do these via definitions.
 	 */
 	if( workspace_load_file( ws, filename ) )
 		return( TRUE );
 
-	/* Toolkits are about the least fussy file format ... try this last.
-	 */
-	if( toolkit_new_from_file( main_toolkitgroup, filename ) )
-		return( TRUE );
 	error_clear();
 
 	error_top( _( "Unknown file type." ) );
@@ -399,7 +417,7 @@ main_load( Workspace *ws, const char *filename )
 static void *
 main_load_plug( char *name )
 {
-	if( !call_string_filename( (call_string_fn) im_load_plugin,
+	if( !calli_string_filename( (calli_string_fn) im_load_plugin,
 		name, NULL, NULL, NULL ) ) {
 		error_top( _( "Unable to load." ) );
 		error_sub( _( "Error loading plug-in \"%s\"." ), name );
@@ -416,15 +434,12 @@ main_load_def( const char *filename )
 {	
 	Toolkit *kit;
 
-	/* If we're in command-line mode, only load toolkits which 
-	 * start with '_'. For interactive mode, load everything.
-	 */
-	if( main_load_all || im_skip_dir( filename )[0] == '_' ) {
+	if( !main_option_no_load_menus || im_skip_dir( filename )[0] == '_' ) {
 		main_splash_update( _( "Loading toolkit \"%s\"" ), 
 			im_skip_dir( filename ) );
 
-		if( !(kit = 
-			toolkit_new_from_file( main_toolkitgroup, filename )) )
+		if( !(kit = toolkit_new_from_file( main_toolkitgroup, 
+			filename )) )
 			box_alert( NULL );
 		else 
 			filemodel_set_auto_load( FILEMODEL( kit ) );
@@ -494,8 +509,6 @@ main_link_package( im_package *pack)
 static void
 main_load_startup( void )
 {
-	const char *prefix;
-
 	mainw_recent_freeze();
 
 /* Stop load of builtins, plugs and vips ... handy for debugging if you're
@@ -519,18 +532,6 @@ main_load_startup( void )
 
 	main_splash_update( "%s", _( "Loading plugins" ) );
 
-	/* Load up any standard plugs.
-	 */
-	if( (prefix = get_vipshome( main_argv0 )) &&
-		im_load_plugins( "%s" IM_DIR_SEP_STR "lib", prefix ) ) {
-		error_top( _( "Unable to load plugins." ) );
-		error_sub( _( "Error loading plug-ins in "
-			"directory \"%s" IM_DIR_SEP_STR "lib\"" ),
-			NN( prefix ) );
-		error_vips();
-		box_alert( NULL );
-	}
-
 	/* Load any plug-ins on PATH_START. 
 	 */
 	(void) path_map_exact( PATH_START, "*.plg", 
@@ -630,28 +631,19 @@ main_register_icons( void )
  *   { GTK_STOCK_COPY, N_("_Copy"), GDK_CONTROL_MASK, 'c', GETTEXT_PACKAGE },
  *
  */
-		{ STOCK_DROPPER, N_( "Ink dropper" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_DUPLICATE, N_( "D_uplicate" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_PAINTBRUSH, N_( "Pen" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_LINE, N_( "Line" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_TEXT, N_( "Text" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_SMUDGE, N_( "Smudge" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_FLOOD, N_( "Flood" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_FLOOD_BLOB, N_( "Flood Blob" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_RECT, N_( "Fill Rectangle" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_MOVE, N_( "Pan" ), 
-			0, 0, GETTEXT_PACKAGE },
-		{ STOCK_SELECT, N_( "Select" ), 
-			0, 0, GETTEXT_PACKAGE },
+		{ STOCK_NEXT_ERROR, 
+			N_( "Next _Error" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_DROPPER, N_( "Ink dropper" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_DUPLICATE, N_( "D_uplicate" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_PAINTBRUSH, N_( "Pen" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_LINE, N_( "Line" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_TEXT, N_( "Text" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_SMUDGE, N_( "Smudge" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_FLOOD, N_( "Flood" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_FLOOD_BLOB, N_( "Flood Blob" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_RECT, N_( "Fill Rectangle" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_MOVE, N_( "Pan" ), 0, 0, GETTEXT_PACKAGE },
+		{ STOCK_SELECT, N_( "Select" ), 0, 0, GETTEXT_PACKAGE },
 
 		/* And the LEDs we use.
 		 */
@@ -675,6 +667,11 @@ main_register_icons( void )
 	icon_set = gtk_icon_factory_lookup_default( GTK_STOCK_COLOR_PICKER );
 	gtk_icon_factory_add( main_icon_factory, STOCK_DROPPER, icon_set );
 
+	/* For Next Error, use JUMP_TO.
+	 */
+	icon_set = gtk_icon_factory_lookup_default( GTK_STOCK_JUMP_TO );
+	gtk_icon_factory_add( main_icon_factory, STOCK_NEXT_ERROR, icon_set );
+
 	/* For clone, use the DND_MULTIPLE icon (close enough).
 	 */
 	icon_set = gtk_icon_factory_lookup_default( GTK_STOCK_DND_MULTIPLE );
@@ -731,10 +728,11 @@ main_x_init( int *argc, char ***argv )
 	printf( "X11 init\n" );
 #endif/*DEBUG*/
 
-	(void) call_string_filename( (call_string_fn) gtk_rc_add_default_file, 
-		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR 
-		PACKAGE IM_DIR_SEP_STR "rc" IM_DIR_SEP_STR 
-		"ipgtkrc", NULL, NULL, NULL );
+	(void) calli_string_filename( 
+		(calli_string_fn) gtk_rc_add_default_file, 
+			"$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S 
+			PACKAGE G_DIR_SEPARATOR_S "rc" G_DIR_SEPARATOR_S 
+			"ipgtkrc", NULL, NULL, NULL );
 	gtk_init( argc, argv );
 
 	/* Set the default icon. 
@@ -774,8 +772,8 @@ main_x_init( int *argc, char ***argv )
 
 	/* Load up any saved accelerators.
 	 */
-	call_string_filenamef( (call_string_fn) gtk_accel_map_load,
-		"%s" IM_DIR_SEP_STR "accel_map", get_savedir() );
+	calli_string_filenamef( (calli_string_fn) gtk_accel_map_load,
+		"%s" G_DIR_SEPARATOR_S "accel_map", get_savedir() );
 }
 
 static void *
@@ -797,8 +795,6 @@ main_toobig_done( iWindow *iwnd,
 	 */
 	path_map_dir( PATH_TMP, "*.v",
 		(path_map_fn) main_toobig_done_sub, NULL );
-	path_map_dir( PATH_TMP, "*.desc",
-		(path_map_fn) main_toobig_done_sub, NULL );
 	path_map_dir( PATH_TMP, "*.ws",
 		(path_map_fn) main_toobig_done_sub, NULL );
 
@@ -843,50 +839,60 @@ main_check_temp( double total )
 	}
 }
 
+/* Make sure a savedir exists. Used to build the "~/.nip2-xx/tmp" etc.
+ * directory tree.
+ */
+static void
+main_mkdir( const char *dir )
+{
+	if( !existsf( "%s" G_DIR_SEPARATOR_S "%s", get_savedir(), dir ) )
+		if( !mkdirf( "%s" G_DIR_SEPARATOR_S "%s", get_savedir(), dir ) )
+			error_exit( _( "unable to make %s %s: %s" ),
+				get_savedir(), dir, g_strerror( errno ) );
+}
+
 /* Start here!
  */
 int
 main( int argc, char *argv[] )
 {
-	/* Flags which trigger the usage message.
-	 */
-	static const char *usage_flags[] = {
-		"-?", "--?", 
-		"-help", "--help", 
-		"-v", "-version", "--version", "--v"
-	};
-	
 	gboolean welcome_message = FALSE;
 	Workspace *ws;
-	int i, j;
+	GError *error = NULL;
+	GOptionContext *context;
+	const char *prefix;
+	int i;
 	double total = 0.0;
 #ifdef HAVE_GETRLIMIT
 	struct rlimit rlp;
 #endif /*HAVE_GETRLIMIT*/
-	const char *prefix;
 	char name[256];
 #if HAVE_FFTW || HAVE_FFTW3
 	iOpenFile *of;
 #endif /*HAVE_FFTW*/
+	Toolkit *kit;
+	char txt[MAX_STRSIZE];
+	BufInfo buf;
+
+#ifdef DEBUG_LEAK
+#ifdef PROFILE
+	g_mem_set_vtable( glib_mem_profiler_table );
+#endif /*PROFILE*/
+#endif /*DEBUG_LEAK*/
 
 #ifdef DEBUG_TIME
-	static GTimer *startup_timer = NULL;
-
-	if( !startup_timer )
-		startup_timer = g_timer_new();
-
-	g_timer_reset( startup_timer );
+	GTimer *startup_timer = g_timer_new();
+	printf( "DEBUG_TIME: startup timer zeroed ...\n" );
+#endif /*DEBUG_TIME*/
 
 	/* In startup phase.
 	 */
 	main_starting = TRUE;
 
-	printf( "%s: startup timer zeroed ...\n", argv[0] );
-#endif /*DEBUG_TIME*/
-
 	/* Want numeric locale to be "C", so we have C rules for doing 
 	 * double <-> string (ie. no "," for decimal point).
 	 */
+	setlocale( LC_ALL, "" );
 	setlocale( LC_NUMERIC, "C" );
 
 	/* Make sure our LC_NUMERIC setting is not trashed.
@@ -897,28 +903,12 @@ main( int argc, char *argv[] )
 	 */
 	g_set_application_name( _( PACKAGE ) );
 
-	/* Set prgname ... do this ourselves in case we don't start gtk.
-	 */
-	{
-		char *prgname;
-
-		prgname = g_path_get_basename( argv[0] );
-		g_set_prgname( prgname );
-		g_free( prgname );
-	}
-
-	/* Init object system. Need to call this explicitly in case we don't
-	 * start the GUI.
-	 */
-	g_type_init();
-
 #ifdef DEBUG
-	printf( "main: sizeof( struct heap_node ) == %d\n",
-		sizeof( struct heap_node ) );
+	printf( "main: sizeof( HeapNode ) == %d\n", sizeof( HeapNode ) );
 
 	/* Should be 3 pointers, hopefully.
 	 */
-	if( sizeof( struct heap_node ) != 3 * sizeof( void * ) )
+	if( sizeof( HeapNode ) != 3 * sizeof( void * ) )
 		printf( "*** struct packing problem!\n" );
 #endif/*DEBUG*/
 
@@ -938,58 +928,100 @@ main( int argc, char *argv[] )
 	setenvf( "PACKAGE", "%s", PACKAGE );
 	setenvf( "VERSION", "%s", VERSION );
 
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
+{
         /* No HOME on windows ... make one from HOMEDRIVE and HOMEDIR (via
          * glib).
          */
-        if( !g_getenv( "HOME" ) )
-                setenvf( "HOME", "%s", g_get_home_dir() );
-#endif /*HAVE_WINDOWS_H*/
+	const char *home;
+	char buf[FILENAME_MAX];
+
+	if( !(home = g_getenv( "HOME" )) ) 
+		home = g_get_home_dir();
+
+	/* We need native paths.
+	 */
+	strncpy( buf, home, FILENAME_MAX );
+	nativeize_path( buf );
+	setenvf( "HOME", "%s", buf );
+}
+#endif /*OS_WIN32*/
 
 	/* Name of the dir we store our config stuff in. This can get used by
 	 * Preferences.ws.
 	 */
 	setenvf( "SAVEDIR", "%s", get_savedir() );
 
+	/* Path separator on this platform.
+	 */
+	setenvf( "SEP", "%s", G_DIR_SEPARATOR_S );
+
+#ifdef HAVE_VIPS8
+	if( !vips_init( &argc, &argv, &error ) ) 
+		vfatal( &error );
+#endif /*HAVE_VIPS8*/
+
 	/* Start up vips.
 	 */
 	if( im_init_world( main_argv0 ) )
 		error_exit( "unable to start VIPS" );
 
+	/* Init i18n ... get catalogues from $VIPSHOME/share/locale so we're
+	 * relocatable.
+	 */
+	prefix = im_guess_prefix( main_argv0, "VIPSHOME" );
+	im_snprintf( name, 256, 
+		"%s" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S "locale", 
+		prefix );
+#ifdef DEBUG
+	printf( "bindtextdomain: %s\n", name );
+#endif /*DEBUG*/
+	textdomain( GETTEXT_PACKAGE );
+	bindtextdomain( GETTEXT_PACKAGE, name );
+	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
+
+#ifdef G_THREADS_ENABLED
 	/* If VIPS has been configured without threads, we will need to start
 	 * the thread system ourselves so that we can make the async queue in
 	 * conversion.c
 	 */
-#ifdef G_THREADS_ENABLED
         if( !g_thread_supported() )
                 g_thread_init( NULL );
 #endif /*G_THREADS_ENABLED*/
 
-	/* Try to discover our data files.
-	 */
-	if( !(prefix = get_vipshome( main_argv0 )) ) 
-		error_exit( "unable to guess VIPSHOME" );
+	context = g_option_context_new( _( "- image processing spreadsheet" ) );
+	g_option_context_add_main_entries( context, 
+		main_option, GETTEXT_PACKAGE );
 
-	/* Init i18n ... get catalogues from $VIPSHOME/share/locale so we're
-	 * relocatable.
+	/* Don't start X here! We may be in batch mode.
 	 */
-	im_snprintf( name, 256, 
-		"%s" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR "locale", prefix );
-#ifdef DEBUG
-	printf( "bindtextdomain: %s\n", name );
-#endif /*DEBUG*/
-	bindtextdomain( GETTEXT_PACKAGE, name );
-	bind_textdomain_codeset( GETTEXT_PACKAGE, "UTF-8" );
-	textdomain( GETTEXT_PACKAGE );
+	g_option_context_add_group( context, gtk_get_option_group( FALSE ) );
 
-	/* Look for any usage args.
-	 */
-	for( i = 1; i < argc; i++ )
-		for( j = 0; j < IM_NUMBER( usage_flags ); j++ )
-			if( strcmp( argv[i], usage_flags[j] ) == 0 ) {
-				main_usage();
-				exit( 0 );
-			}
+#ifdef HAVE_VIPS8
+	g_option_context_add_group( context, vips_get_option_group() );
+#endif /*HAVE_VIPS8*/
+	g_option_context_add_group( context, im_get_option_group() );
+
+	if( !g_option_context_parse( context, &argc, &argv, &error ) ) 
+		vfatal( &error );
+
+	g_option_context_free( context );
+
+	if( main_option_version ) {
+		printf( "%s-%s", PACKAGE, VERSION );
+		printf( "\n" );
+
+		printf( _( "linked to vips-%s" ), 
+			im_version_string() );
+		printf( "\n" );
+
+		exit( 0 );
+	}
+
+#ifdef DEBUG_LEAK
+	fprintf( stderr, 
+		"*** DEBUG_LEAK is on ... will leaktest on exit\n" );
+#endif /*DEBUG_LEAK*/
 
 #ifdef DEBUG_FATAL
 	/* Set masks for debugging ... stop on any problem. 
@@ -1001,9 +1033,10 @@ main( int argc, char *argv[] )
 		G_LOG_LEVEL_CRITICAL |
 		G_LOG_LEVEL_WARNING );
 
-	printf( "*** DEBUG_FATAL is on ... will abort() on first warning\n" );
+	fprintf( stderr, 
+		"*** DEBUG_FATAL is on ... will abort() on first warning\n" );
 #else /*!DEBUG_FATAL*/
-#ifdef HAVE_WINDOWS_H 
+#ifdef OS_WIN32 
 	/* No logging output ... on win32, log output pops up a very annoying
  	 * console text box.
 	 */
@@ -1016,7 +1049,7 @@ main( int argc, char *argv[] )
 	g_log_set_handler( NULL,
 		G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, 
 		main_log_null, NULL );
-#endif /*HAVE_WINDOWS_H*/ 
+#endif /*OS_WIN32*/ 
 #endif /*DEBUG_FATAL*/
 
 	main_stdin = file_open_read_stdin();
@@ -1048,20 +1081,17 @@ main( int argc, char *argv[] )
 	 */
 	path_init();
 
-	/* Do we have a save dir? make one if not.
+	/* First time we've been run? Welcome message.
 	 */
-	if( !existsf( "%s", get_savedir() ) ) {
-		if( !mkdirf( "%s", get_savedir() ) || 
-			!mkdirf( "%s" IM_DIR_SEP_STR "tmp", get_savedir() ) || 
-			!mkdirf( "%s" IM_DIR_SEP_STR "start", get_savedir() ) || 
-			!mkdirf( "%s" IM_DIR_SEP_STR "data", get_savedir() ) ) 
-			error_exit( _( "unable to make %s area: %s" ),
-				get_savedir(), g_strerror( errno ) );
-
-		/* Can't pop up the welcome message now, wait until we start X.
-		 */
+	if( !existsf( "%s", get_savedir() ) ) 
 		welcome_message = TRUE;
-	}
+
+	/* Always make these in case some got deleted.
+	 */
+	main_mkdir( "" );
+	main_mkdir( "tmp" );
+	main_mkdir( "start" );
+	main_mkdir( "data" );
 
 	/* Init other stuff.
 	 */
@@ -1069,12 +1099,13 @@ main( int argc, char *argv[] )
 	fftw_import_system_wisdom();
 #endif /*HAVE_FFTW3*/
 #if HAVE_FFTW || HAVE_FFTW3
-	if( (of = file_open_read( "%s" IM_DIR_SEP_STR "wisdom", 
+	if( (of = file_open_read( "%s" G_DIR_SEPARATOR_S "wisdom", 
 		get_savedir() )) ) {
 		fftw_import_wisdom_from_file( of->fp );
 		file_close( of );
 	}
 #endif /*HAVE_FFTW*/
+
 	mainw_startup();
 	reduce_context = reduce_new();
 	main_symbol_root = symbol_root_init();
@@ -1094,11 +1125,6 @@ main( int argc, char *argv[] )
 	g_object_ref( G_OBJECT( main_imageinfogroup ) );
 	iobject_sink( IOBJECT( main_imageinfogroup ) );
 
-	/* Set IM_CONCURRENCY if a watch changes.
-	 */
-	g_signal_connect( main_watchgroup, "watch_changed", 
-		G_CALLBACK( main_watchgroup_changed_cb ), NULL );
-
 #ifdef DEBUG
 	printf( "arg processing\n" );
 #endif/*DEBUG*/
@@ -1108,72 +1134,66 @@ main( int argc, char *argv[] )
 	 */
 	ws = NULL;
 
-	/* Look for non-interactive modes first. Just look at argv[1].
-	 * Interactive modes may have random extra X or toolkit arguments.
+	/* Interpret compound options first: these imply other options.
 	 */
-	if( argc >= 2 ) {
-		if( strcmp( argv[1], "-main" ) == 0 ) {
-			Toolkit *kit;
+	if( main_option_script ) {
+		if( !toolkit_new_from_file( main_toolkitgroup,
+			main_option_script ) )
+			main_log_add( "%s\n", error_get_sub() );
 
-			main_interactive = FALSE;
-			main_load_all = FALSE;
-			action_on_exit = EXIT_ACTION_PRINT_MAIN;
+		main_option_batch = TRUE;
+		main_option_no_load_menus = TRUE;
+		main_option_no_load_args = TRUE;
+		main_option_print_main = TRUE;
+	}
 
-			/* Load stdin as a load of defs. Good for HEREIS 
-			 * shell scripts.
-			 */
-			if( !(kit = toolkit_new_from_openfile( 
-				main_toolkitgroup, main_stdin )) )
-				main_log_add( "%s\n", error_get_sub() );
-		}
-		else if( strcmp( argv[1], "-workspace" ) == 0 ) {
-			/* Load stdin as if it were a workspace. On dispose(),
-			 * print the value of the last loaded symbol.
-			 */
-			if( !(ws = workspace_new_from_openfile( 
-				main_workspacegroup, main_stdin )) ) 
-				main_log_add( "%s\n", error_get_sub() );
-			else {
-				filemodel_set_filename( FILEMODEL( ws ), NULL );
-				FILEMODEL( ws )->nosave = TRUE;
-				ws->print_last = TRUE;
-			}
-		}
-		else if( strcmp( argv[1], "-script" ) == 0 ) {
-			action_on_exit = EXIT_ACTION_PRINT_MAIN;
-			main_interactive = FALSE;
-			main_load_all = FALSE;
+	if( main_option_expression ) {
+		kit = toolkit_new( main_toolkitgroup, "_expression" );
 
-			if( argc < 3 )
-				error( _( "-script needs an argument" ) );
-			if( !toolkit_new_from_file( main_toolkitgroup,
-				argv[2] ) )
-				main_log_add( "%s\n", error_get_sub() );
+		buf_init_static( &buf, txt, MAX_STRSIZE );
+		buf_appendf( &buf, "main = %s;", main_option_expression );
+		attach_input_string( buf_all( &buf ) );
+		(void) parse_onedef( kit, -1 );
 
-			/* Shift args. We want argv[0] to be the name of the
-			 * script we are interpreting.
-			 */
-			for( i = 0; i < argc; i++ )
-				argv[i] = argv[i + 1];
-			argv[0] = argv[1];
-			argc -= 1;
-		}
-		else if( strcmp( argv[1], "-benchmark" ) == 0 ) {
-			/* Just start up and shutdown, no X.
-			 */
-			main_interactive = FALSE;
-		}
-		else if( strcmp( argv[1], "-time_save" ) == 0 ) {
-			/* Pop up save timers.
-			 */
-			main_time_save = TRUE;
-		}
-		else if( strcmp( argv[1], "-i18n" ) == 0 ) {
-			/* Just start up and shutdown, no X. Output constant
-			 * i18n strings.
-			 */
-			main_interactive = FALSE;
-			compile_set_i18n( TRUE );
+		filemodel_set_modified( FILEMODEL( kit ), FALSE );
+
+		main_option_batch = TRUE;
+		main_option_no_load_menus = TRUE;
+		main_option_no_load_args = TRUE;
+		main_option_print_main = TRUE;
+	}
+
+	if( main_option_benchmark ) {
+		main_option_batch = TRUE;
+		main_option_no_load_menus = FALSE;
+	}
+
+	if( main_option_i18n ) {
+		/* Just start up and shutdown, no X. Output constant
+		 * i18n strings.
+		 */
+		main_option_batch = TRUE;
+		main_option_no_load_menus = FALSE;
+	}
+
+	if( main_option_stdin_def ) {
+		/* Load stdin as a set of defs. Good for HEREIS 
+		 * shell scripts.
+		 */
+		if( !(kit = toolkit_new_from_openfile( 
+			main_toolkitgroup, main_stdin )) )
+			main_log_add( "%s\n", error_get_sub() );
+	}
+
+	if( main_option_stdin_ws ) {
+		/* Load stdin as a workspace.
+		 */
+		if( !(ws = workspace_new_from_openfile( 
+			main_workspacegroup, main_stdin )) ) 
+			main_log_add( "%s\n", error_get_sub() );
+		else {
+			filemodel_set_filename( FILEMODEL( ws ), "stdin" );
+			FILEMODEL( ws )->nosave = TRUE;
 		}
 	}
 
@@ -1185,59 +1205,79 @@ main( int argc, char *argv[] )
 	}
 
 #ifdef DEBUG
-	if( !main_interactive ) 
+	if( main_option_batch ) 
 		printf( "non-interactive mode\n" );
 #endif /*DEBUG*/
 
 	/* Start the X connection. We need this before _load_all(), so that
 	 * we can pop up error dialogs.
 	 */
-	if( main_interactive ) 
+	if( !main_option_batch )
 		main_x_init( &argc, &argv );
 
+	/* Reset IM_CONCURRENCY if a watch changes. Need to do this after
+	 * parsing options so we skip in batch mode.
+	 */
+	g_signal_connect( main_watchgroup, "watch_changed", 
+		G_CALLBACK( main_watchgroup_changed_cb ), NULL );
+
 	/* Load start-up stuff.
 	 */
 	main_load_startup();
 
 	/* Recalc to build all classes.
 	 */
-	main_splash_update( "%s", _( "First recalculation ..." ) );
-	symbol_recalculate_all();
+	if( !main_option_batch ) {
+		main_splash_update( "%s", _( "First recalculation ..." ) );
+		symbol_recalculate_all();
+	}
 
 	/* Measure amount of stuff in temp area ... need this for checking
-	 * temps later.
+	 * temps later. We pop a diglog if there are too many, so only useful
+	 * in interactive mode.
 	 */
-	if( main_interactive ) 
+	if( !main_option_batch )
 		total = directory_size( PATH_TMP );
 
-	if( main_interactive ) {
-		/* Load args as files, if we can.
+	/* Make nip's argc/argv[].
+	 */
+	kit = toolkit_new( main_toolkitgroup, "_args" );
+	buf_init_static( &buf, txt, MAX_STRSIZE );
+	buf_appendf( &buf, "argc = %d;", argc );
+	attach_input_string( buf_all( &buf ) );
+	(void) parse_onedef( kit, -1 );
+
+	buf_init_static( &buf, txt, MAX_STRSIZE );
+	buf_appendf( &buf, "argv = [" );
+	for( i = 0; i < argc; i++ ) {
+		if( i > 0 )
+			buf_appendf( &buf, ", " );
+		buf_appendf( &buf, "\"%s\"", argv[i] );
+	}
+	buf_appendf( &buf, "];" );
+
+	attach_input_string( buf_all( &buf ) );
+	if( !parse_onedef( kit, -1 ) ) 
+		main_log_add( "%s\n", error_get_sub() );
+
+	filemodel_set_modified( FILEMODEL( kit ), FALSE );
+
+	if( !main_option_no_load_args ) {
+		/* Load args as files, if we can. 
 		 */
 		for( i = 1; i < argc; i++ )
-			if( argv[i][0] != '-' && !main_load( ws, argv[i] ) ) 
+			if( !main_load( ws, argv[i] ) ) 
 				main_log_add( "%s\n", error_get_sub() );
 	}
-	else {
-		Toolkit *kit = toolkit_new( main_toolkitgroup, "args" );
-		char txt[MAX_STRSIZE];
-		BufInfo buf;
 
-		/* Make argc/argv[].
-		 */
-		buf_init_static( &buf, txt, MAX_STRSIZE );
-		buf_appendf( &buf, "argc = %d;", argc - 1 );
-		attach_input_string( buf_all( &buf ) );
-		(void) parse_onedef( kit, -1 );
-
-		buf_init_static( &buf, txt, MAX_STRSIZE );
-		buf_appendf( &buf, "argv = [ \"%s\"", argv[0] );
-		for( i = 2; i < argc; i++ ) 
-			buf_appendf( &buf, ", \"%s\"", argv[i] );
-		buf_appendf( &buf, " ];" );
-
-		attach_input_string( buf_all( &buf ) );
-		if( !parse_onedef( kit, -1 ) ) 
-			main_log_add( "%s\n", error_get_sub() );
+	/* Abandon batch mode if there are startup errors.
+	 */
+	if( main_option_batch ) {
+		if( !main_log_is_empty() ) {
+			fprintf( stderr, _( "Startup error log:\n%s" ), 
+				main_log_get() );
+			exit( 1 );
+		}
 	}
 
 	/* Recalc to load any args.
@@ -1251,14 +1291,13 @@ main( int argc, char *argv[] )
 	filemodel_set_modified( FILEMODEL( ws ), FALSE );
 
 #ifdef DEBUG_TIME
-	printf( "%s: main init in %gs\n",  
-		main_argv0,
+	printf( "DEBUG_TIME: main init in %gs\n",  
 		g_timer_elapsed( startup_timer, NULL ) );
 #endif /*DEBUG_TIME*/
 
 	/* Are we running interactively? Start the main window and loop.
 	 */
-	if( main_interactive ) {
+	if( !main_option_batch ) {
 		/* Only display our initial ws if it's not blank, or if it is
 		 * blank, if there are no other windows up.
 		 */
@@ -1282,7 +1321,7 @@ main( int argc, char *argv[] )
 			main_splash = NULL;
 		}
 
-		if( !main_log_isempty() ) {
+		if( !main_log_is_empty() ) {
 			error_top( _( "Startup error." ) );
 			error_sub( _( "Startup error log:\n%s" ), 
 				main_log_get() );
@@ -1327,7 +1366,7 @@ _( "A new directory has been created in your home directory to hold startup, "
 	return( 0 );
 }
 
-#ifdef HAVE_WINDOWS_H 
+#ifdef OS_WIN32 
 /* Get non-cmd line args on win32.
  */
 static int 
@@ -1388,4 +1427,4 @@ WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
 	return( main( argc, argv ) );
 } 
-#endif /*HAVE_WINDOWS_H*/ 
+#endif /*OS_WIN32*/ 
