@@ -39,18 +39,23 @@
 #define RECENT_IMAGE "recent_image"
 #define RECENT_MATRIX "recent_matrix"
 
-static iWindowClass *parent_class = NULL;
-
-/* All the mainw.
- */
-static GSList *mainw_all = NULL;
-
 /* Recently loaded/saved workspaces, images and matricies.
  */
 GSList *mainw_recent_workspace = NULL;
 GSList *mainw_recent_image = NULL;
 GSList *mainw_recent_matrix = NULL;
 
+/* Auto-recalc state. Don't do this as a preference, since preferences are
+ * workspaces and need to have recalc working to operate.
+ */
+gboolean mainw_auto_recalc = TRUE;
+
+static iWindowClass *parent_class = NULL;
+
+/* All the mainw.
+ */
+static GSList *mainw_all = NULL;
+
 void
 mainw_startup( void )
 {
@@ -99,7 +104,7 @@ mainw_recent_add( GSList **recent, const char *filename )
 
 		expand_variables( PATH_TMP, buf );
 		if( filename && strcmp( filename, "" ) != 0 &&
-			!isprefix( buf, filename ) )
+			!is_prefix( buf, filename ) )
 			*recent = recent_add( *recent, filename );
 	}
 }
@@ -145,6 +150,7 @@ mainw_destroy( GtkObject *object )
 	FREESID( mainw->imageinfo_changed_sid, main_imageinfogroup );
 	FREESID( mainw->heap_changed_sid, reduce_context->heap );
 	FREESID( mainw->watch_changed_sid, main_watchgroup );
+	DESTROY_GTK( mainw->popup );
 	UNREF( mainw->kitgview );
 	IM_FREEF( g_source_remove, mainw->pane_animate_timeout );
 
@@ -177,13 +183,25 @@ mainw_map( GtkWidget *widget )
 
 	assert( !mainw->compat_timeout );
 
-	if( mainw->ws->compatibility ) 
+	if( mainw->ws->compat_major )
 		mainw->compat_timeout = g_timeout_add( 500, 
 			(GSourceFunc) mainw_compat_timeout, mainw );
 
 	GTK_WIDGET_CLASS( parent_class )->map( widget );
 }
 
+static gboolean
+mainw_configure_event( GtkWidget *widget, GdkEventConfigure *event )
+{
+	Mainw *mainw = MAINW( widget );
+
+	mainw->ws->window_width = event->width;
+	mainw->ws->window_height = event->height;
+
+	return( GTK_WIDGET_CLASS( parent_class )->
+		configure_event( widget, event ) );
+}
+
 static void
 mainw_class_init( MainwClass *class )
 {
@@ -195,6 +213,7 @@ mainw_class_init( MainwClass *class )
 	object_class->destroy = mainw_destroy;
 
 	widget_class->map = mainw_map;
+	widget_class->configure_event = mainw_configure_event;
 }
 
 static void
@@ -304,10 +323,10 @@ static void
 mainw_free_update( Mainw *mainw )
 {
 	Heap *heap = reduce_context->heap;
-	char txt[MAX_STRSIZE];
+	char txt[80];
 	BufInfo buf;
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
+	buf_init_static( &buf, txt, 80 );
 
 	if( workspace_selected_any( mainw->ws ) ) {
 		/* Display select message instead.
@@ -329,14 +348,6 @@ mainw_free_update( Mainw *mainw )
 	}
 
 	set_glabel( mainw->space_free, "%s", buf_all( &buf ) );
-
-	buf_rewind( &buf );
-	mainw_find_disc( &buf );
-        set_tooltip( mainw->space_free_eb, 
-                _( "%s in \"%s\", "
-		"%d cells in heap, %d cells free, %d cells maximum" ),
-		buf_all( &buf ), PATH_TMP,
-                heap->ncells, heap->nfree, heap->max_fn( heap ) );
 }
 
 static void
@@ -346,14 +357,13 @@ mainw_title_update( Mainw *mainw )
 	char txt[512];
 
 	buf_init_static( &buf, txt, 512 );
-	buf_appendf( &buf, "%s", NN( IOBJECT( mainw->ws )->name ) );
-	if( mainw->ws->compatibility ) {
+	buf_appendf( &buf, "%s", NN( IOBJECT( mainw->ws->sym )->name ) );
+	if( mainw->ws->compat_major ) {
 		buf_appends( &buf, " - " );
 		buf_appends( &buf, _( "compatibility mode" ) );
 		buf_appendf( &buf, " %d.%d", 
-			FILEMODEL( mainw->ws )->major,
-			FILEMODEL( mainw->ws )->minor,
-			FILEMODEL( mainw->ws )->micro );
+			mainw->ws->compat_major, 
+			mainw->ws->compat_minor ); 
 	}
 	if( FILEMODEL( mainw->ws )->filename )
 		buf_appendf( &buf, " - %s", FILEMODEL( mainw->ws )->filename );
@@ -366,6 +376,20 @@ mainw_title_update( Mainw *mainw )
 	iwindow_set_title( IWINDOW( mainw ), buf_all( &buf ) );
 }
 
+static void 
+mainw_status_update( Mainw *mainw )
+{
+	Workspace *ws = mainw->ws;
+
+	gtk_statusbar_pop( GTK_STATUSBAR( mainw->statusbar ), 0 );
+	if( symbol_busy() )
+		gtk_statusbar_push( GTK_STATUSBAR( mainw->statusbar ), 0, 
+			_( "Calculating ..." ) );
+	else if( ws->status )
+		gtk_statusbar_push( GTK_STATUSBAR( mainw->statusbar ), 0, 
+			ws->status );
+}
+
 static gboolean
 mainw_pane_animate_timeout_cb( Mainw *mainw )
 {
@@ -425,6 +449,66 @@ mainw_pane_animate( Mainw *mainw, int target_position )
 			(GSourceFunc) mainw_pane_animate_timeout_cb, mainw );
 }
 
+static gint
+mainw_jump_name_compare( iContainer *a, iContainer *b )
+{
+	int la = strlen( IOBJECT( a )->name );
+	int lb = strlen( IOBJECT( b )->name );
+
+	/* Smaller names first.
+	 */
+	if( la == lb )
+		return( strcmp( IOBJECT( a )->name, IOBJECT( b )->name ) );
+	else
+		return( la - lb );
+}
+
+static void
+mainw_jump_column_cb( GtkWidget *item, Column *column )
+{
+	model_scrollto( MODEL( column ), MODEL_SCROLL_TOP );
+}
+
+static void *
+mainw_jump_build( Column *column, GtkWidget *menu )
+{
+	GtkWidget *item;
+	char txt[256];
+	BufInfo buf;
+
+	buf_init_static( &buf, txt, 256 );
+	buf_appendf( &buf, "%s - %s", 
+		IOBJECT( column )->name, IOBJECT( column )->caption );
+	item = gtk_menu_item_new_with_label( buf_all( &buf ) );
+	g_signal_connect( item, "activate",
+		G_CALLBACK( mainw_jump_column_cb ), column );
+	gtk_menu_append( GTK_MENU( menu ), item );
+	gtk_widget_show( item );
+
+	return( NULL );
+}
+
+static void
+mainw_jump_update( Mainw *mainw, GtkWidget *menu )
+{
+	GtkWidget *item;
+	GSList *columns;
+
+	gtk_container_foreach( GTK_CONTAINER( menu ),
+		(GtkCallback) gtk_widget_destroy, NULL );
+
+	item = gtk_tearoff_menu_item_new();
+	gtk_menu_append( GTK_MENU( menu ), item );
+	gtk_widget_show( item );
+
+	columns = icontainer_get_children( ICONTAINER( mainw->ws ) );
+        columns = g_slist_sort( columns, 
+		(GCompareFunc) mainw_jump_name_compare );
+	slist_map( columns, (SListMapFn) mainw_jump_build, menu );
+
+	g_slist_free( columns );
+}
+
 static void
 mainw_refresh( Mainw *mainw )
 {
@@ -448,10 +532,11 @@ mainw_refresh( Mainw *mainw )
 	int pref = IM_CLIP( 0, MAINW_TOOLBAR_STYLE, IM_NUMBER( styles ) - 1 );
         GtkAction *action;
 
-	gtk_statusbar_pop( GTK_STATUSBAR( mainw->statusbar ), 0 );
-	if( ws->status )
-		gtk_statusbar_push( GTK_STATUSBAR( mainw->statusbar ), 
-			0, ws->status );
+#ifdef DEBUG
+	printf( "mainw_refresh: %p\n", mainw );
+#endif /*DEBUG*/
+
+	mainw_status_update( mainw );
 	mainw_free_update( mainw );
 	mainw_title_update( mainw );
 
@@ -461,6 +546,11 @@ mainw_refresh( Mainw *mainw )
 
 	gtk_toolbar_set_style( GTK_TOOLBAR( mainw->toolbar ), styles[pref] );
 
+	action = gtk_action_group_get_action( mainw->action_group, 
+		"AutoRecalculate" );
+	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
+		mainw_auto_recalc );
+
 	action = gtk_action_group_get_action( mainw->action_group, 
 		"Toolbar" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
@@ -506,6 +596,9 @@ mainw_refresh( Mainw *mainw )
 			mainw_pane_animate( mainw, 10000 );
 		}
 	}
+
+	mainw_jump_update( mainw, mainw->jump_to_column_menu );
+	mainw_jump_update( mainw, mainw->popup_jump );
 }
 
 static void
@@ -527,10 +620,36 @@ mainw_space_free_event( GtkWidget *widget, GdkEvent *ev, Mainw *mainw )
 	return( FALSE );
 }
 
+static void
+mainw_space_free_tooltip_generate( GtkWidget *widget, BufInfo *buf, 
+	Mainw *mainw )
+{
+	Heap *heap = reduce_context->heap;
+	Symbol *sym = mainw->ws->sym;
+
+	mainw_find_disc( buf );
+	/* Expands to (eg.) "14GB free in /pics/tmp" */
+        buf_appendf( buf, _( " in \"%s\"" ), PATH_TMP );
+        buf_appends( buf, ", " );
+
+        buf_appendf( buf, 
+		_( "%d cells in heap, %d cells free, %d cells maximum" ),
+                heap->ncells, heap->nfree, heap->max_fn( heap ) );
+        buf_appends( buf, ", " );
+
+        buf_appendf( buf, _( "%d objects in workspace" ),
+		g_slist_length( ICONTAINER( sym->expr->compile )->children ) );
+        buf_appends( buf, ", " );
+
+        buf_appendf( buf, _( "%d vips calls cached" ),
+		vips_history_size );
+}
+
 static void
 mainw_free_changed_cb( gpointer *dummy, Mainw *mainw )
 {
 	mainw_free_update( mainw );
+	mainw_status_update( mainw );
 }
 
 /* Go to home page.
@@ -554,7 +673,7 @@ mainw_about_action_cb( GtkAction *action, iWindow *iwnd )
 void
 mainw_guide_action_cb( GtkAction *action, iWindow *iwnd )
 {
-	box_url( GTK_WIDGET( iwnd ), "file://" NIP_DOCPATH "/index.html" );
+	box_url( GTK_WIDGET( iwnd ), "file://" NIP_DOCPATH "/nipguide.html" );
 }
 
 static void
@@ -586,7 +705,7 @@ mainw_clone( Mainw *mainw )
 	workspace_deselect_all( ws );
 	set_pointer();
 
-	model_scrollto( MODEL( ws->current ) );
+	model_scrollto( MODEL( ws->current ), MODEL_SCROLL_TOP );
 }
 
 static void
@@ -638,6 +757,12 @@ mainw_group_action_cb( GtkAction *action, Mainw *mainw )
 	workspace_deselect_all( ws );
 }
 
+static void
+mainw_group_action_cb2( GtkWidget *wid, GtkWidget *host, Mainw *mainw )
+{
+	mainw_group_action_cb( NULL, mainw );
+}
+
 /* Select all objects.
  */
 static void
@@ -717,7 +842,7 @@ mainw_next_error_action_cb( GtkAction *action, Mainw *mainw )
 	 */
 	assert( mainw->row_last_error && mainw->row_last_error->err );
 
-	model_scrollto( MODEL( mainw->row_last_error ) );
+	model_scrollto( MODEL( mainw->row_last_error ), MODEL_SCROLL_TOP );
 
 	buf_init_static( &buf, str, MAX_LINELENGTH );
 	row_qualified_name( mainw->row_last_error->expr->row, &buf );
@@ -726,6 +851,12 @@ mainw_next_error_action_cb( GtkAction *action, Mainw *mainw )
 	workspace_set_status( mainw->ws, "%s", buf_firstline( &buf ) );
 }
 
+static void
+mainw_next_error_action_cb2( GtkWidget *wid, GtkWidget *host, Mainw *mainw )
+{
+	mainw_next_error_action_cb( NULL, mainw );
+}
+
 /* Callback for box_yesno in mainw_force_calc_cb. Recalc selected items.
  */
 static void
@@ -766,7 +897,7 @@ mainw_force_calc_action_cb( GtkAction *action, Mainw *mainw )
 	else 
 		/* Recalculate.
 		 */
-		symbol_recalculate_all_force();
+		symbol_recalculate_all_force( FALSE );
 }
 
 /* Callback for save workspace.
@@ -908,7 +1039,7 @@ mainw_open( Mainw *mainw )
 	filesel_set_filetype( FILESEL( filesel ), 
 		filesel_type_mainw, IMAGE_FILE_TYPE );
 	filesel_set_filetype_pref( FILESEL( filesel ), "IMAGE_FILE_TYPE" );
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( mainw ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( mainw ) );
 	filesel_set_done( FILESEL( filesel ), 
 		mainw_open_done_cb, mainw );
 	filesel_set_multi( FILESEL( filesel ), TRUE );
@@ -934,15 +1065,15 @@ mainw_open_examples( Mainw *mainw )
 	filesel_set_flags( FILESEL( filesel ), TRUE, FALSE );
 	filesel_set_filetype( FILESEL( filesel ), 
 		filesel_type_workspace, 0 );
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( mainw ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( mainw ) );
 	filesel_set_done( FILESEL( filesel ), 
 		mainw_open_done_cb, mainw );
 	filesel_set_multi( FILESEL( filesel ), TRUE );
 	iwindow_build( IWINDOW( filesel ) );
 	filesel_set_filename( FILESEL( filesel ), 
-		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR 
-		PACKAGE IM_DIR_SEP_STR "data" IM_DIR_SEP_STR 
-		"examples" IM_DIR_SEP_STR "1_point_mosaic" );
+		"$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S 
+		PACKAGE G_DIR_SEPARATOR_S "data" G_DIR_SEPARATOR_S 
+		"examples" G_DIR_SEPARATOR_S "1_point_mosaic" );
 
 	/* Reset the filetype ... setting the filename will have changed it to
 	 * 'all', since there's no suffix.
@@ -1012,11 +1143,11 @@ mainw_recent_clear_cb( GtkWidget *widget, const char *filename )
 
 	/* Need to remove files too to prevent merge on quit.
 	 */
-	(void) unlinkf( "%s" IM_DIR_SEP_STR "%s", 
+	(void) unlinkf( "%s" G_DIR_SEPARATOR_S "%s", 
 		get_savedir(), RECENT_WORKSPACE );
-	(void) unlinkf( "%s" IM_DIR_SEP_STR "%s", 
+	(void) unlinkf( "%s" G_DIR_SEPARATOR_S "%s", 
 		get_savedir(), RECENT_IMAGE );
-	(void) unlinkf( "%s" IM_DIR_SEP_STR "%s", 
+	(void) unlinkf( "%s" G_DIR_SEPARATOR_S "%s", 
 		get_savedir(), RECENT_MATRIX );
 }
 
@@ -1029,14 +1160,6 @@ mainw_recent_map_cb( GtkWidget *widget, Mainw *mainw )
 	gtk_container_foreach( GTK_CONTAINER( menu ),
 		(GtkCallback) gtk_widget_destroy, NULL );
 
-	if( mainw_recent_workspace ) {
-		item = gtk_menu_item_new_with_label( _( "Recent Workspaces" ) );
-		gtk_menu_append( GTK_MENU( menu ), item );
-		gtk_widget_show( item );
-		gtk_widget_set_sensitive( item, FALSE );
-		mainw_recent_build( menu, mainw_recent_workspace );
-	}
-
 	if( mainw_recent_image ) {
 		item = gtk_menu_item_new_with_label( _( "Recent Images" ) );
 		gtk_menu_append( GTK_MENU( menu ), item );
@@ -1045,6 +1168,14 @@ mainw_recent_map_cb( GtkWidget *widget, Mainw *mainw )
 		mainw_recent_build( menu, mainw_recent_image );
 	}
 
+	if( mainw_recent_workspace ) {
+		item = gtk_menu_item_new_with_label( _( "Recent Workspaces" ) );
+		gtk_menu_append( GTK_MENU( menu ), item );
+		gtk_widget_show( item );
+		gtk_widget_set_sensitive( item, FALSE );
+		mainw_recent_build( menu, mainw_recent_workspace );
+	}
+
 	if( mainw_recent_matrix ) {
 		item = gtk_menu_item_new_with_label( _( "Recent Matricies" ) );
 		gtk_menu_append( GTK_MENU( menu ), item );
@@ -1123,7 +1254,7 @@ mainw_workspace_merge( Mainw *mainw )
 	iwindow_set_title( IWINDOW( filesel ), _( "Merge Workspace" ) );
 	filesel_set_flags( FILESEL( filesel ), FALSE, FALSE );
 	filesel_set_filetype( FILESEL( filesel ), filesel_type_workspace, 0 ); 
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( mainw ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( mainw ) );
 	filesel_set_done( FILESEL( filesel ), 
 		mainw_workspace_merge_done_cb, mainw );
 	filesel_set_multi( FILESEL( filesel ), TRUE );
@@ -1181,12 +1312,85 @@ mainw_column_new_action_cb( GtkAction *action, Mainw *mainw )
 	IM_FREE( name );
 }
 
+/* Same, but from the popup.
+ */
+static void
+mainw_column_new_action_cb2( GtkWidget *wid, GtkWidget *host, Mainw *mainw )
+{
+	mainw_column_new_action_cb( NULL, mainw );
+}
+
+/* Done button hit.
+ */
+static void
+mainw_column_new_cb( iWindow *iwnd, void *client, 
+	iWindowNotifyFn nfn, void *sys )
+{
+	Mainw *mainw = MAINW( client );
+	Stringset *ss = STRINGSET( iwnd );
+	StringsetChild *name = stringset_child_get( ss, _( "Name" ) );
+	StringsetChild *caption = stringset_child_get( ss, _( "Caption" ) );
+
+	Column *col;
+
+	char name_text[1024];
+	char caption_text[1024];
+
+	if( !get_geditable_name( name->entry, name_text, 1024 ) ||
+		!get_geditable_string( caption->entry, caption_text, 1024 ) ) {
+		nfn( sys, IWINDOW_ERROR );
+		return;
+	}
+
+	if( !(col = column_new( mainw->ws, name_text )) ) {
+		nfn( sys, IWINDOW_ERROR );
+		return;
+	}
+
+	iobject_set( IOBJECT( col ), NULL, caption_text );
+	workspace_column_select( mainw->ws, col );
+
+	nfn( sys, IWINDOW_TRUE );
+}
+
+/* Make a new column with a specified name.
+ */
+static void
+mainw_column_new_named_action_cb( GtkAction *action, Mainw *mainw )
+{
+	GtkWidget *ss = stringset_new();
+	char *name;
+
+	name = workspace_column_name_new( mainw->ws, NULL );
+
+	stringset_child_new( STRINGSET( ss ), 
+		_( "Name" ), name, _( "Set column name here" ) );
+	stringset_child_new( STRINGSET( ss ), 
+		_( "Caption" ), "", _( "Set column caption here" ) );
+
+	iwindow_set_title( IWINDOW( ss ), _( "New Column" ) );
+	idialog_set_callbacks( IDIALOG( ss ), 
+		iwindow_true_cb, NULL, NULL, mainw );
+	idialog_add_ok( IDIALOG( ss ), 
+		mainw_column_new_cb, _( "Create Column" ) );
+	iwindow_set_parent( IWINDOW( ss ), GTK_WIDGET( mainw ) );
+	iwindow_build( IWINDOW( ss ) );
+
+	gtk_widget_show( ss );
+
+	IM_FREE( name );
+}
+
 /* Callback from program.
  */
 static void
 mainw_program_new_action_cb( GtkAction *action, Mainw *mainw )
 {
-	program_new( mainw->ws->kitg );
+	Program *program;
+
+	program = program_new( mainw->ws->kitg );
+
+	gtk_widget_show( GTK_WIDGET( program ) ); 
 }
 
 /* New ws callback.
@@ -1199,6 +1403,25 @@ mainw_workspace_new_action_cb( GtkAction *action, Mainw *mainw )
 	workspacegroup_workspace_new( wsg, GTK_WIDGET( mainw ) );
 }
 
+/* Callback from auto-recalc toggle.
+ */
+static void
+mainw_autorecalc_action_cb( GtkToggleAction *action, Mainw *mainw )
+{
+	GSList *i;
+
+	mainw_auto_recalc = gtk_toggle_action_get_active( action );
+
+	/* Yuk! We have to ask all mainw to refresh by hand, since we're not 
+	 * using the prefs system for auto_recalc for reasons noted at top.
+	 */
+	for( i = mainw_all; i; i = i->next ) 
+		mainw_refresh( MAINW( i->data ) );
+
+	if( mainw_auto_recalc )
+		symbol_recalculate_all();
+}
+
 /* Callback from show toolbar toggle.
  */
 static void
@@ -1240,6 +1463,12 @@ mainw_layout_action_cb( GtkAction *action, Mainw *mainw )
 	model_layout( MODEL( mainw->ws ) );
 }
 
+static void
+mainw_layout_action_cb2( GtkWidget *wid, GtkWidget *host, Mainw *mainw )
+{
+	mainw_layout_action_cb( NULL, mainw );
+}
+
 /* Remove selected items.
  */
 static void
@@ -1248,18 +1477,65 @@ mainw_selected_remove_action_cb( GtkAction *action, Mainw *mainw )
 	workspace_selected_remove_yesno( mainw->ws, GTK_WIDGET( mainw ) );
 }
 
+void 
+mainw_revert_ok_cb( iWindow *iwnd, void *client, 
+	iWindowNotifyFn nfn, void *sys ) 
+{
+	Mainw *mainw = MAINW( client );
+
+	if( FILEMODEL( mainw->ws )->filename ) {
+		(void) unlinkf( "%s", FILEMODEL( mainw->ws )->filename );
+		main_reload();
+		symbol_recalculate_all();
+	}
+
+	nfn( sys, IWINDOW_TRUE );
+}
+
+void 
+mainw_revert_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys ) 
+{
+	Mainw *mainw = MAINW( client );
+
+	box_yesno( GTK_WIDGET( iwnd ),
+		mainw_revert_ok_cb, iwindow_true_cb, mainw,
+		nfn, sys,
+		_( "Revert to Defaults" ), 
+		_( "Revert to installation defaults?" ),
+		_( "Would you like to reset all preferences to their factory "
+		"settings? This will delete any changes you have ever made "
+		"to your preferences and may take a few seconds." ) );
+}
+
 static void
 mainw_preferences_action_cb( GtkAction *action, Mainw *mainw )
 {
 	Prefs *prefs;
 
-	if( !(prefs = prefs_new()) ) {
+	/* Can fail if there's no prefs ws, or an error on load.
+	 */
+	if( !(prefs = prefs_new( NULL )) ) {
 		box_alert( GTK_WIDGET( mainw ) );
 		return;
 	}
 
-	idialog_set_parent( IDIALOG( prefs ), GTK_WIDGET( mainw ) );
+	iwindow_set_title( IWINDOW( prefs ), _( "Preferences" ) );
+	iwindow_set_parent( IWINDOW( prefs ), GTK_WIDGET( mainw ) );
+	idialog_set_callbacks( IDIALOG( prefs ), 
+		NULL, NULL, NULL, mainw );
+	idialog_add_ok( IDIALOG( prefs ), 
+		mainw_revert_cb, _( "Revert to Defaults ..." ) );
+	idialog_add_ok( IDIALOG( prefs ), iwindow_true_cb, GTK_STOCK_CLOSE );
 	iwindow_build( IWINDOW( prefs ) );
+
+	/* Just big enough to avoid a horizontal scrollbar on my machine. 
+
+		FIXME ... Yuk! There must be a better way to do this! 
+		Maybe a setting in prefs to suppress the h scrollbar?
+
+	 */
+	gtk_window_set_default_size( GTK_WINDOW( prefs ), 730, 480 );
+
 	gtk_widget_show( GTK_WIDGET( prefs ) );
 }
 
@@ -1300,13 +1576,14 @@ mainw_mode_action_cb( GtkRadioAction *action, GtkRadioAction *current,
 static GtkActionEntry mainw_actions[] = {
 	/* Menu items.
 	 */
-	{ "FileMenu", NULL, "_File" },
-	{ "FileNewMenu", NULL, "_New" },
-	{ "FileRecentMenu", NULL, "Open _Recent" },
-	{ "EditMenu", NULL, "_Edit" },
-	{ "ViewMenu", NULL, "_View" },
-	{ "ToolkitsMenu", NULL, "_Toolkits" },
-	{ "HelpMenu", NULL, "_Help" },
+	{ "FileMenu", NULL, N_( "_File" ) },
+	{ "NewMenu", NULL, N_( "_New" ) },
+	{ "RecentMenu", NULL, N_( "Open _Recent" ) },
+	{ "EditMenu", NULL, N_( "_Edit" ) },
+	{ "JumpToColumnMenu", NULL, N_( "Jump to _Column" ) },
+	{ "ViewMenu", NULL, N_( "_View" ) },
+	{ "ToolkitsMenu", NULL, N_( "_Toolkits" ) },
+	{ "HelpMenu", NULL, N_( "_Help" ) },
 
 	/* Dummy action ... replaced at runtime.
 	 */
@@ -1320,6 +1597,11 @@ static GtkActionEntry mainw_actions[] = {
 		N_( "Create a new column" ), 
 		G_CALLBACK( mainw_column_new_action_cb ) },
 
+	{ "NewColumnName", 
+		GTK_STOCK_NEW, N_( "New Named C_olumn" ), NULL, 
+		N_( "Create a new column with a specified name" ), 
+		G_CALLBACK( mainw_column_new_named_action_cb ) },
+
 	{ "NewWorkspace", 
 		GTK_STOCK_NEW, N_( "New _Workspace" ), NULL, 
 		N_( "Create a new workspace" ), 
@@ -1401,7 +1683,7 @@ static GtkActionEntry mainw_actions[] = {
 		G_CALLBACK( mainw_find_again_action_cb ) },
 
 	{ "NextError", 
-		GTK_STOCK_JUMP_TO, N_( "Next _Error" ), NULL,
+		STOCK_NEXT_ERROR, NULL, NULL,
 		N_( "Jump to next error" ), 
 		G_CALLBACK( mainw_next_error_action_cb ) },
 
@@ -1442,6 +1724,11 @@ static GtkActionEntry mainw_actions[] = {
 };
 
 static GtkToggleActionEntry mainw_toggle_actions[] = {
+	{ "AutoRecalculate",
+		NULL, N_( "Au_to Recalculate" ), NULL,
+		N_( "Recalculate automatically on change" ),
+		G_CALLBACK( mainw_autorecalc_action_cb ), TRUE },
+
 	{ "Toolbar",
 		NULL, N_( "_Toolbar" ), NULL,
 		N_( "Show window toolbar" ),
@@ -1456,7 +1743,6 @@ static GtkToggleActionEntry mainw_toggle_actions[] = {
 		NULL, N_( "Toolkit _Browser" ), NULL,
 		N_( "Show toolkit browser" ),
 		G_CALLBACK( mainw_toolkitbrowser_action_cb ), FALSE },
-
 };
 
 static GtkRadioActionEntry mainw_radio_actions[] = {
@@ -1480,12 +1766,12 @@ static const char *mainw_menubar_ui_description =
 "<ui>"
 "  <menubar name='MainwMenubar'>"
 "    <menu action='FileMenu'>"
-"      <menu action='FileNewMenu'>"
-"        <menuitem action='NewColumn'/>"
+"      <menu action='NewMenu'>"
+"        <menuitem action='NewColumnName'/>"
 "        <menuitem action='NewWorkspace'/>"
 "      </menu>"
 "      <menuitem action='Open'/>"
-"      <menu action='FileRecentMenu'>"
+"      <menu action='RecentMenu'>"
 "        <menuitem action='Stub'/>"	/* Dummy ... replaced on map */
 "      </menu>"
 "      <menuitem action='OpenExamples'/>"
@@ -1504,11 +1790,15 @@ static const char *mainw_menubar_ui_description =
 "      <menuitem action='SelectAll'/>"
 "      <menuitem action='Duplicate'/>"
 "      <menuitem action='Recalculate'/>"
+"      <menuitem action='AutoRecalculate'/>"
 "      <menuitem action='AlignColumns'/>"
 "      <separator/>"
 "      <menuitem action='Find'/>"
 "      <menuitem action='FindNext'/>"
 "      <menuitem action='NextError'/>"
+"      <menu action='JumpToColumnMenu'>"
+"        <menuitem action='Stub'/>"	/* Dummy ... replaced on map */
+"      </menu>"
 "      <separator/>"
 "      <menuitem action='Group'/>"
 "      <menuitem action='Ungroup'/>"
@@ -1575,7 +1865,7 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	GtkWidget *item;
 
 #ifdef DEBUG
-	printf( "mainw_init: 0x%x\n", (unsigned int) mainw );
+	printf( "mainw_init: %p\n", mainw );
 #endif /*DEBUG*/
 
         /* Make main menu bar
@@ -1620,11 +1910,18 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	 * for that dummy item.
 	 */
         item = gtk_ui_manager_get_widget( mainw->ui_manager,
-		"/MainwMenubar/FileMenu/FileRecentMenu/Stub" );
+		"/MainwMenubar/FileMenu/RecentMenu/Stub" );
 	mainw->recent_menu = gtk_widget_get_parent( GTK_WIDGET( item ) );
         gtk_signal_connect( GTK_OBJECT( mainw->recent_menu ), "map",
                 GTK_SIGNAL_FUNC( mainw_recent_map_cb ), mainw );
 
+	/* Same for the column jump menu.
+	 */
+        item = gtk_ui_manager_get_widget( mainw->ui_manager,
+		"/MainwMenubar/EditMenu/JumpToColumnMenu/Stub" );
+	mainw->jump_to_column_menu = 
+		gtk_widget_get_parent( GTK_WIDGET( item ) );
+
 	/* Attach toolbar.
   	 */
 	mainw->toolbar = gtk_ui_manager_get_widget( 
@@ -1654,6 +1951,9 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
         gtk_container_add( GTK_CONTAINER( frame ), mainw->space_free );
         gtk_signal_connect( GTK_OBJECT( mainw->space_free_eb ), "event",
                 GTK_SIGNAL_FUNC( mainw_space_free_event ), mainw );
+	set_tooltip_generate( mainw->space_free_eb,
+		(TooltipGenerateFn) mainw_space_free_tooltip_generate, 
+		mainw, NULL );
         gtk_widget_show( mainw->space_free );
 	mainw->imageinfo_changed_sid = g_signal_connect( main_imageinfogroup, 
 		"changed",
@@ -1679,10 +1979,9 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	/* Make toolkit/workspace displays.
 	 */
 	mainw->kitgview = TOOLKITGROUPVIEW( 
-		model_view_new( MODEL( mainw->ws->kitg ) ) );
+		model_view_new( MODEL( mainw->ws->kitg ), NULL ) );
 	g_object_ref( G_OBJECT( mainw->kitgview ) );
 	gtk_object_sink( GTK_OBJECT( mainw->kitgview ) );
-	view_link( VIEW( mainw->kitgview ), MODEL( mainw->ws->kitg ), NULL );
 	toolkitgroupview_set_mainw( mainw->kitgview, mainw );
 	gtk_menu_set_accel_group( GTK_MENU( mainw->kitgview->menu ),
 		iwnd->accel_group );
@@ -1692,12 +1991,27 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 		GTK_WIDGET( mainw->pane ), TRUE, TRUE, 0 );
 	gtk_widget_show( mainw->pane );
 
-	mainw->wsview = WORKSPACEVIEW( model_view_new( MODEL( mainw->ws ) ) );
-	view_link( VIEW( mainw->wsview ), MODEL( mainw->ws ), NULL );
+	mainw->wsview = WORKSPACEVIEW( 
+		model_view_new( MODEL( mainw->ws ), NULL ) );
 	gtk_paned_pack1( GTK_PANED( mainw->pane ), GTK_WIDGET( mainw->wsview ),
 		TRUE, FALSE );
 	gtk_widget_show( GTK_WIDGET( mainw->wsview ) );
 
+	mainw->popup = popup_build( _( "Workspace menu" ) );
+	popup_add_but( mainw->popup, _( "New C_olumn" ),
+		POPUP_FUNC( mainw_column_new_action_cb2 ) ); 
+	mainw->popup_jump =
+		popup_add_pullright( mainw->popup, _( "Jump to _Column" ) ); 
+	popup_add_but( mainw->popup, _( "Align _Columns" ),
+		POPUP_FUNC( mainw_layout_action_cb2 ) ); 
+	menu_add_sep( mainw->popup );
+	popup_add_but( mainw->popup, _( "_Group Selected" ),
+		POPUP_FUNC( mainw_group_action_cb2 ) ); 
+	menu_add_sep( mainw->popup );
+	popup_add_but( mainw->popup, STOCK_NEXT_ERROR,
+		POPUP_FUNC( mainw_next_error_action_cb2 ) ); 
+        popup_attach( mainw->wsview->fixed, mainw->popup, mainw );
+
 	/* Have to put toolkitbrowser in an ebox so the search entry gets
 	 * clipped to the pane size.
 	 */
@@ -1722,7 +2036,6 @@ static void
 mainw_popdown( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 {
 	Mainw *mainw = MAINW( iwnd );
-	Workspace *ws = mainw->ws;
 
 	if( mainw->pane_visible )
 		/* Re-read the position rather than looking at pane_position ...
@@ -1734,14 +2047,6 @@ mainw_popdown( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 		prefs_set( "MAINW_PANE_POSITION", "%d", 
 			mainw->pane_position );
 
-	if( ws->print_last &&
-		ws->last_symbol &&
-		ws->last_symbol->type == SYM_VALUE ) {
-		(void) set_output( &ws->last_symbol->expr->root );
-		(void) graph_value( &ws->last_symbol->expr->root );
-		ws->print_last = FALSE;
-	}
-
 	IM_FREEF( g_source_remove, mainw->compat_timeout );
 
 	filemodel_inter_savenclose_cb( IWINDOW( mainw ), 
@@ -1762,11 +2067,29 @@ mainw_link( Mainw *mainw, Workspace *ws )
 	iwindow_set_build( IWINDOW( mainw ), 
 		(iWindowBuildFn) mainw_build, ws, NULL, NULL );
 	iwindow_set_popdown( IWINDOW( mainw ), mainw_popdown, NULL );
-	iwindow_set_size_prefs( IWINDOW( mainw ), 
-		"MAINW_WINDOW_WIDTH", "MAINW_WINDOW_HEIGHT" );
+
+	/* If we have a saved size for this workspoace, set that. Otherwise,
+	 * default to the default.
+	 */
+	if( !ws->window_width ) {
+		iwindow_set_size_prefs( IWINDOW( mainw ), 
+			"MAINW_WINDOW_WIDTH", "MAINW_WINDOW_HEIGHT" );
+	}
+
 	iwindow_build( IWINDOW( mainw ) );
 
-	/* 10000: just a huge number, enough to get the gutter locked on to
+	if( ws->window_width ) {
+		GdkScreen *screen = 
+			gtk_widget_get_screen( GTK_WIDGET( mainw ) );
+
+		gtk_window_set_default_size( GTK_WINDOW( mainw ), 
+			IM_MIN( ws->window_width, 
+				gdk_screen_get_width( screen ) ),
+			IM_MIN( ws->window_height, 
+				gdk_screen_get_height( screen ) ) );
+	}
+
+	/* 10000: just a huge number; enough to get the gutter locked on to
 	 * the right edge.
 	 */
 	gtk_paned_set_position( GTK_PANED( mainw->pane ), 
@@ -1780,6 +2103,10 @@ mainw_new( Workspace *ws )
 {
 	Mainw *mainw;
 
+	/* We must have calced before we build the view so we get menus etc.
+	 */
+	symbol_recalculate_all_force( TRUE );
+
 	mainw = MAINW( g_object_new( TYPE_MAINW, NULL ) );
 	mainw_link( mainw, ws );
 
