@@ -84,6 +84,10 @@ struct _Mainw {
 	GtkUIManager *ui_manager;
 	GtkWidget *toolbar;
 	GtkWidget *recent_menu;
+	GtkWidget *jump_to_column_menu;
+
+	GtkWidget *popup;
+	GtkWidget *popup_jump;
 
 	GtkWidget *statusbar_main;
 	GtkWidget *statusbar;
@@ -109,6 +113,8 @@ extern GSList *mainw_recent_workspace;
 extern GSList *mainw_recent_image;
 extern GSList *mainw_recent_matrix;
 
+extern gboolean mainw_auto_recalc;
+
 void mainw_startup( void );
 void mainw_shutdown( void );
 void mainw_recent_freeze( void );
