@@ -37,10 +37,29 @@
 #include <gtkextra/gtksheet.h>
 #endif /*HAVE_GTK_EXTRA*/
 
+/* We have gtksheet patched into the nip sources as a temp measure, so call 
+ * directly.
+ */
+#define HAVE_GTK_EXTRA
+#include "gtksheet.h"
+
+/* Round N down to P boundary.
+ */
+#define ROUND_DOWN(N,P) ((N) - ((N) % P))
+
+/* Round N up to P boundary.
+ */
+#define ROUND_UP(N,P) (ROUND_DOWN( (N) + (P) - 1, (P) ))
+
+/* The size in cells at which we switch from displaying the whole matrix to
+ * displaying part of it in a scrolled window.
+ */
+static const int matrixview_max_width = 7;
+static const int matrixview_max_height = 10;
+
 /* Limit non-gtksheet display size with these ... could be prefs?
  */
-static const int matrixview_max_width = 10;
-static const int matrixview_max_height = 10;
+static const int matrixview_max_cells = 100;
 
 /* A HAVE_GTK_EXTRA we can use in expressions.
  */
@@ -162,7 +181,7 @@ matrixview_scan( View *view )
     		for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     			for( x = 0; x < matrixview->width; x++, p = p->next ) {
     				GtkWidget *item = GTK_WIDGET( p->data );
-				int i = x + y * matrix->width;
+				int i = x + y * matrix->value.width;
 
     				GtkWidget *entry;
 
@@ -174,7 +193,7 @@ matrixview_scan( View *view )
     					entry = item;
 
     				if( !matrixview_scan_text( matrixview, entry,
-    					&matrix->value[i], &changed ) ) {
+    					&matrix->value.coeff[i], &changed ) ) {
 					error_top( _( "Bad value." ) );
 					error_sub( _( "Cell (%d, %d):\n%s" ), 
     						x, y, error_get_sub() );
@@ -192,10 +211,10 @@ matrixview_scan( View *view )
     			for( x = 0; x < matrixview->width; x++ ) {
     				char *txt = gtk_sheet_cell_get_text(
     					sheet, y, x );
-				int i = x + y * matrix->width;
+				int i = x + y * matrix->value.width;
 
     				if( !matrixview_scan_string( txt,
-    					&matrix->value[i], &changed ) ) {
+    					&matrix->value.coeff[i], &changed ) ) {
 					error_top( _( "Bad value." ) );
 					error_sub( _( "Cell (%d, %d):\n%s" ), 
     						x, y, error_get_sub() );
@@ -223,7 +242,7 @@ matrixview_toggle_change_cb( GtkWidget *widget, Matrixview *matrixview )
     	int pos = g_slist_index( matrixview->items, widget );
 	int x = pos % matrixview->width;
 	int y = pos / matrixview->width;
-	int i = x + y * matrix->width;
+	int i = x + y * matrix->value.width;
 
 #ifdef DEBUG
     	printf( "matrixview_toggle_change_cb\n" );
@@ -231,17 +250,17 @@ matrixview_toggle_change_cb( GtkWidget *widget, Matrixview *matrixview )
 
     	/* Cycle value.
     	 */
-    	switch( (int) matrix->value[i] ) {
+    	switch( (int) matrix->value.coeff[i] ) {
     	case 0:
-    		matrix->value[i] = 128.0;
+    		matrix->value.coeff[i] = 128.0;
     		break;
 
     	case 255:
-    		matrix->value[i] = 0.0;
+    		matrix->value.coeff[i] = 0.0;
     		break;
 
     	default:
-    		matrix->value[i] = 255.0;
+    		matrix->value.coeff[i] = 255.0;
     		break;
     	}
 
@@ -268,10 +287,10 @@ matrixview_toggle_build( Matrixview *matrixview )
     	 * name; it is highlit by our resource file.
     	 */
     	cx = -1; cy = -1;
-    	if( matrix->height & 0x1 )
-    		cy = matrix->height >> 1;
-    	if( matrix->width & 0x1 )
-    		cx = matrix->width >> 1;
+    	if( matrix->value.height & 0x1 )
+    		cy = matrix->value.height >> 1;
+    	if( matrix->value.width & 0x1 )
+    		cx = matrix->value.width >> 1;
 
     	/* Build contents.
     	 */
@@ -310,14 +329,14 @@ matrixview_slider_change_cb( Tslider *tslider, Matrixview *matrixview )
     	int pos = g_slist_index( matrixview->items, tslider );
 	int x = pos % matrixview->width;
 	int y = pos / matrixview->width;
-	int i = x + y * matrix->width;
+	int i = x + y * matrix->value.width;
 
     	assert( pos >= 0 );
 
     	/* Install value.
     	 */
-    	if( matrix->value[i] != tslider->svalue ) {
-    		matrix->value[i] = tslider->svalue;
+    	if( matrix->value.coeff[i] != tslider->svalue ) {
+    		matrix->value.coeff[i] = tslider->svalue;
 
     		classmodel_update( CLASSMODEL( matrix ) );
     		symbol_recalculate_all();
@@ -464,7 +483,6 @@ matrixview_text_traverse( GtkSheet *sheet,
 {
     	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
     	Row *row = HEAPMODEL( matrix )->row;
-	Mainw *mainw = MAINW( idialog_get_root( widget ) );
 
     	char *new_text = gtk_sheet_cell_get_text( sheet, *new_row, *new_col );
 
@@ -474,7 +492,7 @@ matrixview_text_traverse( GtkSheet *sheet,
 	/* Make a note of what's in this cell before any editing ... "changed"
 	 * does a strcmp() on this to spot edit actions.
 	 */
-	SETSTR( matrixview->cell_text, new_text );
+	IM_SETSTR( matrixview->cell_text, new_text );
 	matrixview->cell_row = *new_row;
 	matrixview->cell_col = *new_col;
 
@@ -485,7 +503,7 @@ matrixview_text_traverse( GtkSheet *sheet,
 	 */
     	buf_appendf( &buf, _( ": cell (%d, %d): %s" ), 
 		*new_col, *new_row, new_text );
-	mainw_set_status( mainw, "%s", buf_all( &buf ) );
+	workspace_set_status( row->ws, "%s", buf_all( &buf ) );
 
     	return( TRUE );
 }
@@ -514,6 +532,27 @@ matrixview_text_changed( GtkSheet *sheet,
 	}
 }
 
+static void 
+matrixview_select_range( GtkSheet *sheet, 
+	GtkSheetRange *range, Matrixview *matrixview )
+{
+    	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
+
+	matrix_select( matrix,
+		range->col0, range->row0, 
+		(range->coli - range->col0) + 1, 
+		(range->rowi - range->row0) + 1 );
+}
+
+static void 
+matrixview_unselect_range( GtkSheet *sheet, 
+	GtkSheetRange *range, Matrixview *matrixview )
+{
+    	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
+
+	matrix_deselect( matrix );
+}
+
 static gboolean
 matrixview_text_event( GtkWidget *widget, 
 	GdkEvent *ev, Matrixview *matrixview )
@@ -537,19 +576,32 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 
     	matrixview->sheet = gtk_sheet_new( matrixview->height, 
 		matrixview->width, "" );
-    	gtk_sheet_hide_column_titles( GTK_SHEET( matrixview->sheet ) );
-    	gtk_sheet_hide_row_titles( GTK_SHEET( matrixview->sheet ) );
-    	gtk_sheet_set_selection_mode( GTK_SHEET( matrixview->sheet ), 
-    		GTK_SELECTION_SINGLE );
+
+	/* Cool, but causes too many resizing problems in an embedded widget.
+	 */
+	gtk_sheet_columns_set_resizable( GTK_SHEET( matrixview->sheet ), 
+		FALSE );
+	gtk_sheet_rows_set_resizable( GTK_SHEET( matrixview->sheet ), 
+		FALSE );
+
     	cell_width = GTK_SHEET( matrixview->sheet )->row_title_area.width;
     	cell_height = GTK_SHEET( matrixview->sheet )->column_title_area.height;
+
     	gtk_signal_connect( GTK_OBJECT( matrixview->sheet ), "traverse",
     		GTK_SIGNAL_FUNC( matrixview_text_traverse ), matrixview );
     	gtk_signal_connect( GTK_OBJECT( matrixview->sheet ), "changed",
     		GTK_SIGNAL_FUNC( matrixview_text_changed ), matrixview );
 
+    	gtk_sheet_set_selection_mode( GTK_SHEET( matrixview->sheet ), 
+		GTK_SELECTION_MULTIPLE );
+
+    	g_signal_connect( matrixview->sheet, "select-range",
+    		G_CALLBACK( matrixview_select_range ), matrixview );
+    	g_signal_connect( matrixview->sheet, "unselect-range",
+    		G_CALLBACK( matrixview_unselect_range ), matrixview );
+
 	/* We can't connect to "activate" on sheet's entry :-( most
-	 * gtk_sheet's fail to emit it. Have to parse events ourselves.
+	 * gtk_sheets fail to emit it. Have to parse events ourselves.
 	 */
 	entry = gtk_sheet_get_entry( GTK_SHEET( matrixview->sheet ) );
 	gtk_signal_connect( GTK_OBJECT( entry ), "event",
@@ -558,25 +610,81 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 
 	/* For large matricies, display in a scrolled window.
 	 */
-	if( matrixview->width * cell_width > 500 || 
-		matrixview->height * cell_height > 500 ) {
+	if( matrixview->width > matrixview_max_width || 
+		matrixview->height > matrixview_max_height ) {
+		GtkRequisition requisition;
+		gint spacing;
+		int border;
+		int width, height;
+
 		matrixview->swin = gtk_scrolled_window_new( NULL, NULL );
-		gtk_scrolled_window_set_policy( 
-			GTK_SCROLLED_WINDOW( matrixview->swin ),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+		gtk_box_pack_start( GTK_BOX( matrixview->box ), 
+			matrixview->swin, FALSE, FALSE, 0 );
+		gtk_container_add( GTK_CONTAINER( matrixview->swin ), 
+			matrixview->sheet );
 		hadj = gtk_scrolled_window_get_hadjustment( 
 			GTK_SCROLLED_WINDOW( matrixview->swin ) );
 		vadj = gtk_scrolled_window_get_vadjustment( 
 			GTK_SCROLLED_WINDOW( matrixview->swin ) );
 
-		gtk_widget_set_size_request( GTK_WIDGET( matrixview->swin ), 
-			IM_MIN( 500, matrixview->width * cell_width + 40 ), 
-			IM_MIN( 500, matrixview->height * cell_height + 40 ) );
+		/* Pick a size for the scrolled window. This is horrible! We
+		 * need the inner area of the swin to be a certain size, so we
+		 * need to allow extra space for the sccrollbars and the swin
+		 * padding.
+		 */
 
-		gtk_container_add( GTK_CONTAINER( matrixview->swin ), 
-			matrixview->sheet );
-		gtk_box_pack_start( GTK_BOX( matrixview->box ), 
-			matrixview->swin, FALSE, FALSE, 0 );
+		/* There is a thing in swin in 2.8+ to get the scrollbar out, 
+		 * but we want to work with earlier gtk as well. Yuk!
+		 */
+		gtk_widget_size_request( 
+			GTK_SCROLLED_WINDOW( matrixview->swin )->hscrollbar, 
+			&requisition);
+		gtk_widget_style_get( GTK_WIDGET( matrixview->swin ),
+			"scrollbar-spacing", &spacing,
+			NULL );
+		border = requisition.height + spacing;
+
+		/* Subarea of matrix we show, in cells.
+		 */
+		width = IM_MIN( matrixview->width, matrixview_max_width );
+		height = IM_MIN( matrixview->height, matrixview_max_height );
+
+		/* If we're showing row/column headers, need an extra
+		 * row/column.
+		 */
+		if( matrixview->width > matrixview_max_width )
+			height += 1;
+		if( matrixview->height > matrixview_max_height )
+			width += 1;
+
+		/* Convert to pixels.
+		 */
+		width *= cell_width;
+		height *= cell_height;
+
+		/* Will we be showing scrollbars? Need to add a bit.
+		 */
+		if( matrixview->width > matrixview_max_width )
+			height += border;
+		if( matrixview->height > matrixview_max_height )
+			width += border;
+
+		gtk_widget_set_size_request( GTK_WIDGET( matrixview->swin ), 
+			width, height );
+
+		/* If we're not showing scrollbars, we can hide the
+		 * row/column headers.
+		 */
+		if( matrixview->height <= matrixview_max_height )
+			gtk_sheet_hide_row_titles( 
+				GTK_SHEET( matrixview->sheet ) );
+		if( matrixview->width <= matrixview_max_width )
+			gtk_sheet_hide_column_titles( 
+				GTK_SHEET( matrixview->sheet ) );
+
+		gtk_scrolled_window_set_policy( 
+			GTK_SCROLLED_WINDOW( matrixview->swin ),
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
 	}	
 	else {
 		hadj = GTK_ADJUSTMENT( 
@@ -584,6 +692,11 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 		vadj = GTK_ADJUSTMENT( 
 			gtk_adjustment_new( 0, 0, 100, 10, 10, 10 ) );
 
+		/* No need for col/row titles for small matricies.
+		 */
+		gtk_sheet_hide_column_titles( GTK_SHEET( matrixview->sheet ) );
+		gtk_sheet_hide_row_titles( GTK_SHEET( matrixview->sheet ) );
+
 		gtk_widget_set_size_request( GTK_WIDGET( matrixview->sheet ), 
 			matrixview->width * cell_width, 
 			matrixview->height * cell_height );
@@ -647,9 +760,10 @@ matrixview_toggle_refresh( Matrixview *matrixview )
     	for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++, p = p->next ) {
     			GtkWidget *wid = GTK_WIDGET( p->data );
-    			int i = x + y * matrix->width;
+    			int i = x + y * matrix->value.width;
 
-    			matrixview_toggle_set_label( wid, matrix->value[i] );
+    			matrixview_toggle_set_label( wid, 
+				matrix->value.coeff[i] );
     		}
 }
 
@@ -666,10 +780,10 @@ matrixview_slider_refresh( Matrixview *matrixview )
     	for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++, p = p->next ) {
     			Tslider *tslider = TSLIDER( p->data );
-    			int i = x + y * matrix->width;
+    			int i = x + y * matrix->value.width;
 
-    			tslider->value = matrix->value[i];
-    			tslider->svalue = matrix->value[i];
+    			tslider->value = matrix->value.coeff[i];
+    			tslider->svalue = matrix->value.coeff[i];
 
     			tslider_changed( tslider );
     		}
@@ -701,10 +815,10 @@ matrixview_text_refresh( Matrixview *matrixview )
     	for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++, p = p->next ) {
     			GtkWidget *item = GTK_WIDGET( p->data );
-    			int i = x + y * matrix->width;
-    			double val = matrix->value[i];
+    			int i = x + y * matrix->value.width;
+    			double coeff = matrix->value.coeff[i];
 
-    			matrixview_text_set( matrixview, item, val );
+    			matrixview_text_set( matrixview, item, coeff );
     		}
 
     	matrixview_text_set( matrixview, matrixview->scale, matrix->scale );
@@ -717,22 +831,25 @@ static void
 matrixview_text_refresh( Matrixview *matrixview )
 {
     	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
+	GtkSheetRange *range = &GTK_SHEET( matrixview->sheet )->range;
 
     	int x, y;
 
 	/* width and height can be zero (during class construct), in which
 	 * case they'll be no gtk_sheet.
 	 */
-	if( matrixview->sheet )
-		gtk_sheet_freeze( GTK_SHEET( matrixview->sheet ) );
+	if( !matrixview->sheet )
+		return;
+
+	gtk_sheet_freeze( GTK_SHEET( matrixview->sheet ) );
 
     	for( y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++ ) {
-    			int i = x + y * matrixview->width;
-    			double val = matrix->value[i];
+    			int i = x + y * matrix->value.width;
+    			double coeff = matrix->value.coeff[i];
     			char buf[256];
 
-    			snprintf( buf, 256, "%g", val );
+    			snprintf( buf, 256, "%g", coeff );
 			gtk_signal_handler_block_by_data( 
 				GTK_OBJECT( matrixview->sheet ), matrixview );
     			gtk_sheet_set_cell_text( GTK_SHEET( matrixview->sheet ),
@@ -742,11 +859,33 @@ matrixview_text_refresh( Matrixview *matrixview )
 
 			if( x == matrixview->cell_col && 
 				y == matrixview->cell_row ) 
-				SETSTR( matrixview->cell_text, buf );
+				IM_SETSTR( matrixview->cell_text, buf );
     		}
 
-	if( matrixview->sheet )
-		gtk_sheet_thaw( GTK_SHEET( matrixview->sheet ) );
+	/* Is the model's selection different from ours? Update ours if it is.
+	 * Annoyingly, we can only change the selection when the widget's
+	 * realized.
+	 */
+	if( GTK_WIDGET_REALIZED( matrixview->sheet ) ) {
+		if( !matrix->selected )
+			gtk_sheet_unselect_range( 
+				GTK_SHEET( matrixview->sheet ) );
+		else if( range->col0 != matrix->range.left ||
+			range->coli != IM_RECT_RIGHT( &matrix->range ) - 1 ||
+			range->row0 != matrix->range.top ||
+			range->rowi != IM_RECT_BOTTOM( &matrix->range ) - 1 ) {
+			GtkSheetRange new_range;
+
+			new_range.col0 = matrix->range.left;
+			new_range.row0 = matrix->range.top;
+			new_range.coli = IM_RECT_RIGHT( &matrix->range );
+			new_range.rowi = IM_RECT_BOTTOM( &matrix->range );
+			gtk_sheet_select_range( GTK_SHEET( matrixview->sheet ),
+				&new_range );
+		}
+	}
+
+	gtk_sheet_thaw( GTK_SHEET( matrixview->sheet ) );
 
     	matrixview_text_set( matrixview, matrixview->scale, matrix->scale );
     	matrixview_text_set( matrixview, matrixview->offset, matrix->offset );
@@ -771,21 +910,44 @@ matrixview_refresh( vObject *vobject )
 	if( have_gtk_extra && 
 		(matrix->display == MATRIX_DISPLAY_TEXT || 
 		matrix->display == MATRIX_DISPLAY_TEXT_SCALE_OFFSET) ) {
-		width = matrix->width;
-		height = matrix->height;
+		width = matrix->value.width;
+		height = matrix->value.height;
 	}
 	else {
-		width = matrix->width;
-		height = matrix->height;
+		width = matrix->value.width;
+		height = matrix->value.height;
 
-		if( width > matrixview_max_width ) {
-			width = matrixview_max_width;
-			hclip = TRUE;
+		if( width * height > matrixview_max_cells ) {
+			if( width > height ) {
+				width = IM_CLIP( 1, 
+					matrixview_max_cells / height, 
+					matrix->value.width );
+				hclip = TRUE;
+			}
+			else {
+				height = IM_CLIP( 1, 
+					matrixview_max_cells / width, 
+					matrix->value.height );
+				vclip = TRUE;
+			}
 		}
 
-		if( height > matrixview_max_height ) {
-			height = matrixview_max_height;
-			vclip = TRUE;
+		/* Clip twice to make sure we clip in both directions if 
+		 * necessary.
+		 */
+		if( width * height > matrixview_max_cells ) {
+			if( width > height ) {
+				width = IM_CLIP( 1, 
+					matrixview_max_cells / height, 
+					matrix->value.width );
+				hclip = TRUE;
+			}
+			else {
+				height = IM_CLIP( 1, 
+					matrixview_max_cells / width, 
+					matrix->value.height );
+				vclip = TRUE;
+			}
 		}
 	}
 
@@ -801,8 +963,8 @@ matrixview_refresh( vObject *vobject )
     		matrixview->height != height ) {
     		/* Kill old UI stuff.
     		 */
-    		IM_FREEF( gtk_widget_destroy, matrixview->table );
     		IM_FREEF( gtk_widget_destroy, matrixview->sheet );
+    		IM_FREEF( gtk_widget_destroy, matrixview->table );
     		IM_FREEF( gtk_widget_destroy, matrixview->swin );
     		IM_FREEF( g_slist_free, matrixview->items );
     		IM_FREEF( gtk_widget_destroy, matrixview->cbox );
