@@ -154,8 +154,8 @@ model_loadstate_new( const char *filename )
 
 	xmlSetGenericErrorFunc( state, 
 		(xmlGenericErrorFunc) model_loadstate_error );
-	if( !(state->xdoc = (xmlDoc *) call_string_filename( 
-		(call_string_fn) xmlParseFile, 
+	if( !(state->xdoc = (xmlDoc *) callv_string_filename( 
+		(callv_string_fn) xmlParseFile, 
 		state->filename, NULL, NULL, NULL )) ) {
 		model_loadstate_error_get( state );
 		model_loadstate_destroy( state );
@@ -268,14 +268,18 @@ model_loadstate_rewrite( ModelLoadState *state, char *old_rhs, char *new_rhs )
 }
 
 View *
-model_view_new( Model *model )
+model_view_new( Model *model, View *parent )
 {
 	ModelClass *model_class = MODEL_GET_CLASS( model );
+	View *view;
 
-	if( model_class->view_new )
-		return( model_class->view_new() );
+	if( !model_class->view_new ) 
+		return( NULL );
 
-	return( NULL );
+	view = model_class->view_new( model, parent );
+	view_link( view, model, parent );
+
+	return( view );
 }
 
 /* Register a model subclass as loadable ... what we allow when we load an
@@ -289,11 +293,12 @@ model_register_loadable( ModelClass *model_class )
 }
 
 void
-model_scrollto( Model *model )
+model_scrollto( Model *model, ModelScrollPosition position )
 {
 	assert( IS_MODEL( model ) );
 
-	g_signal_emit( G_OBJECT( model ), model_signals[SIG_SCROLLTO], 0 );
+	g_signal_emit( G_OBJECT( model ), model_signals[SIG_SCROLLTO], 0, 
+		position );
 }
 
 void
@@ -417,10 +422,8 @@ model_empty( Model *model )
 }
 
 static void
-model_info( iObject *iobject, BufInfo *buf )
+model_real_scrollto( Model *model, ModelScrollPosition position )
 {
-	buf_appendf( buf, "display = \"%s\"\n", 
-		bool_to_char( MODEL( iobject )->display ) );
 }
 
 static xmlNode *
@@ -429,16 +432,12 @@ model_real_save( Model *model, xmlNode *xnode )
 	const char *tname = G_OBJECT_TYPE_NAME( model );
 	xmlNode *xthis;
 
-	if( !(xthis = xmlNewChild( xnode, NULL, tname, NULL )) ) {
+	if( !(xthis = xmlNewChild( xnode, NULL, (xmlChar *) tname, NULL )) ) {
 		error_top( _( "XML library error." ) );
 		error_sub( _( "model_save: xmlNewChild() failed" ) );
 		return( NULL );
 	}
 
-	if( !set_sprop( xthis, "name", IOBJECT( model )->name ) ||
-		!set_sprop( xthis, "caption", IOBJECT( model )->caption ) )
-		return( NULL );
-
 	if( icontainer_map( ICONTAINER( model ), 
 		(icontainer_map_fn) model_save, xthis, NULL ) )
 		return( NULL );
@@ -453,7 +452,7 @@ model_new_xml_sub( ModelClass *model_class,
 	GtkType type = GTK_CLASS_TYPE( model_class );
 	const char *tname = gtk_type_name( type );
 
-	if( strcasecmp( xnode->name, tname ) == 0 ) {
+	if( strcasecmp( (char *) xnode->name, tname ) == 0 ) {
 		Model *model = MODEL( g_object_new( type, NULL ) );
 
 		if( model_load( model, state, parent, xnode ) ) {
@@ -489,23 +488,15 @@ model_real_load( Model *model,
 	const char *tname = G_OBJECT_TYPE_NAME( model );
 	xmlNode *i;
 
-	char name[MAX_STRSIZE];
-	char caption[MAX_STRSIZE];
-
 	/* Should just be a sanity check.
 	 */
-	if( strcasecmp( xnode->name, tname ) != 0 ) {
+	if( strcasecmp( (char *) xnode->name, tname ) != 0 ) {
 		error_top( _( "XML load error." ) );
 		error_sub( _( "Can't load node of type \"%s\" into "
 			"object of type \"%s\"" ), xnode->name, tname );
 		return( FALSE );
 	}
 
-	if( get_sprop( xnode, "name", name, MAX_STRSIZE ) )
-		iobject_set( IOBJECT( model ), name, NULL );
-	if( get_sprop( xnode, "caption", caption, MAX_STRSIZE ) )
-		iobject_set( IOBJECT( model ), NULL, caption );
-
 	if( !ICONTAINER( model )->parent )
 		icontainer_child_add( ICONTAINER( parent ), 
 			ICONTAINER( model ), -1 );
@@ -537,11 +528,9 @@ model_class_init( ModelClass *class )
 
 	parent_class = g_type_class_peek_parent( class );
 
-	object_class->info = model_info;
-
 	class->view_new = NULL;
 	class->edit = NULL;
-	class->scrollto = NULL;
+	class->scrollto = model_real_scrollto;
 	class->layout = NULL;
 	class->reset = NULL;
 	class->save = model_real_save;
@@ -558,8 +547,9 @@ model_class_init( ModelClass *class )
 		G_SIGNAL_RUN_FIRST,
 		G_STRUCT_OFFSET( ModelClass, scrollto ),
 		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE, 0 );
+		g_cclosure_marshal_VOID__INT,
+		G_TYPE_NONE, 1,
+		G_TYPE_INT );
 	model_signals[SIG_LAYOUT] = g_signal_new( "layout",
 		G_OBJECT_CLASS_TYPE( object_class ),
 		G_SIGNAL_RUN_FIRST,
@@ -615,12 +605,12 @@ model_base_init( void )
 	/* We have to init some of our other classes to get them registered 
 	 * with the class loader.
 	 */
+	(void) g_type_class_ref( TYPE_CLOCK );
 	(void) g_type_class_ref( TYPE_COLOUR );
-	(void) g_type_class_ref( TYPE_COLUMN );
+	(void) g_type_class_ref( TYPE_EXPRESSION );
 	(void) g_type_class_ref( TYPE_FONTNAME );
 	(void) g_type_class_ref( TYPE_GROUP );
 	(void) g_type_class_ref( TYPE_IARROW );
-	(void) g_type_class_ref( TYPE_EXPRESSION );
 	(void) g_type_class_ref( TYPE_IIMAGE );
 	(void) g_type_class_ref( TYPE_IREGION );
 	(void) g_type_class_ref( TYPE_ITEXT );
@@ -628,13 +618,18 @@ model_base_init( void )
 	(void) g_type_class_ref( TYPE_NUMBER );
 	(void) g_type_class_ref( TYPE_OPTION );
 	(void) g_type_class_ref( TYPE_PATHNAME );
-	(void) g_type_class_ref( TYPE_RHS );
-	(void) g_type_class_ref( TYPE_ROW );
+	(void) g_type_class_ref( TYPE_PLOT );
+	(void) g_type_class_ref( TYPE_REAL );
 	(void) g_type_class_ref( TYPE_SLIDER );
 	(void) g_type_class_ref( TYPE_STRING );
-	(void) g_type_class_ref( TYPE_SUBCOLUMN );
 	(void) g_type_class_ref( TYPE_TOGGLE );
+	(void) g_type_class_ref( TYPE_VECTOR );
+
+	(void) g_type_class_ref( TYPE_RHS );
+	(void) g_type_class_ref( TYPE_ROW );
+	(void) g_type_class_ref( TYPE_SUBCOLUMN );
 	(void) g_type_class_ref( TYPE_WORKSPACE );
+	(void) g_type_class_ref( TYPE_COLUMN );
 }
 
 typedef struct {
