@@ -58,7 +58,7 @@ path_parse( const char *path )
 		/* Find the start of the next component, or the NULL
 		 * character.
 		 */
-		if( !(e = strchr( p, IM_PATH_SEP )) )
+		if( !(e = strchr( p, G_SEARCHPATH_SEPARATOR )) )
 			e = p + strlen( p );
 		len = e - p + 1;
 
@@ -70,9 +70,9 @@ path_parse( const char *path )
 		 */
 		op = g_slist_append( op, im_strdupn( name ) );
 
-		/* Skip IM_PATH_SEP.
+		/* Skip G_SEARCHPATH_SEPARATOR.
 		 */
-		if( *e == IM_PATH_SEP )
+		if( *e == G_SEARCHPATH_SEPARATOR )
 			e++;
 	}
 
@@ -102,7 +102,7 @@ static char *
 path_add_string( const char *str, char *buf )
 {
 	strcpy( buf, str );
-	strcat( buf, IM_PATH_SEP_STR );
+	strcat( buf, G_SEARCHPATH_SEPARATOR_S );
 
 	return( buf + strlen( buf ) );
 }
@@ -120,7 +120,7 @@ path_unparse( GSList *path )
 	 */
 	slist_fold( path, buf, (SListFoldFn) path_add_string, NULL );
 
-	/* Fix '\0' to remove trailing IM_PATH_SEP.
+	/* Fix '\0' to remove trailing G_SEARCHPATH_SEPARATOR.
 	 */
 	if( len > 0 )
 		buf[len - 1] = '\0';
@@ -143,7 +143,7 @@ path_match_pattern( Wild *wild, const char *dir_name, const char *name,
 		/* Make up whole path name.
 		 */
 		im_snprintf( buf, FILENAME_MAX, 
-			"%s" IM_DIR_SEP_STR "%s", dir_name, name );
+			"%s" G_DIR_SEPARATOR_S "%s", dir_name, name );
 
 		return( fn( buf, a, b, c ) );
 	}
@@ -162,8 +162,8 @@ path_scan_dir( const char *dir_name, Wild *wild,
 	const char *name;
 	void *ru;
 
-	if( !(dir = (GDir *) call_string_filename( (call_string_fn) g_dir_open,
-		dir_name, NULL, NULL, NULL )) ) 
+	if( !(dir = (GDir *) callv_string_filename( 
+		(callv_string_fn) g_dir_open, dir_name, NULL, NULL, NULL )) ) 
 		return( NULL );
 
 	while( (name = g_dir_read_name( dir )) ) 
@@ -329,7 +329,7 @@ path_add_file( const char *filename )
 	char buf[FILENAME_MAX];
 
 	im_strncpy( buf, filename, FILENAME_MAX );
-	if( (p = strrchr( buf, IM_DIR_SEP )) )
+	if( (p = strrchr( buf, G_DIR_SEPARATOR )) )
 		*p = '\0';
 
 	path_add_dir( buf );
@@ -347,22 +347,22 @@ path_init( void )
 	char buf[FILENAME_MAX];
 
 	im_snprintf( buf, FILENAME_MAX,
-		"%s" IM_DIR_SEP_STR "start" IM_PATH_SEP_STR
-		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR
-			PACKAGE IM_DIR_SEP_STR "start",
+		"%s" G_DIR_SEPARATOR_S "start" G_SEARCHPATH_SEPARATOR_S
+		"$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S
+			PACKAGE G_DIR_SEPARATOR_S "start",
 		get_savedir() );
 	path_start_default = path_parse( buf );
 
 	im_snprintf( buf, FILENAME_MAX,
-		"%s" IM_DIR_SEP_STR "data" IM_PATH_SEP_STR
-		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR 
-			PACKAGE IM_DIR_SEP_STR "data" IM_PATH_SEP_STR 
+		"%s" G_DIR_SEPARATOR_S "data" G_SEARCHPATH_SEPARATOR_S
+		"$VIPSHOME" G_DIR_SEPARATOR_S "share" G_DIR_SEPARATOR_S 
+		PACKAGE G_DIR_SEPARATOR_S "data" G_SEARCHPATH_SEPARATOR_S 
 		".",
 		get_savedir() );
 	path_search_default = path_parse( buf );
 
 	im_snprintf( buf, FILENAME_MAX, 
-		"%s" IM_DIR_SEP_STR "tmp", get_savedir() );
+		"%s" G_DIR_SEPARATOR_S "tmp", get_savedir() );
 	path_tmp_default = im_strdup( NULL, buf );
 #endif /*DEBUG*/
 }
