@@ -251,7 +251,7 @@ program_title( Program *program )
 	char txt[512];
 
 	buf_init_static( &buf, txt, 512 );
-	buf_appendf( &buf, _( "Program" ) );
+	buf_appendf( &buf, IOBJECT( program->kitg )->name );
 	if( program->kit ) {
 		buf_appendf( &buf, " - %s", IOBJECT( program->kit )->name );
 
@@ -639,7 +639,7 @@ program_edit_dia( Program *program, Tool *tool )
 		iwindow_true_cb, NULL, NULL, tool );
 	idialog_add_ok( IDIALOG( ss ), 
 		program_edit_dia_done_cb, _( "Set column item" ) );
-	idialog_set_parent( IDIALOG( ss ), GTK_WIDGET( program ) );
+	iwindow_set_parent( IWINDOW( ss ), GTK_WIDGET( program ) );
 	idialog_set_iobject( IDIALOG( ss ), IOBJECT( tool ) );
 	iwindow_build( IWINDOW( ss ) );
 
@@ -1086,8 +1086,8 @@ program_toolkit_new_done_cb( iWindow *iwnd, void *client,
 	/* Make a filename from the name ... user start directory.
 	 */
 	buf_init_static( &buf, str, 1024 );
-	buf_appendf( &buf, 
-		"$SAVEDIR" IM_DIR_SEP_STR "start" IM_DIR_SEP_STR "%s.def", 
+	buf_appendf( &buf, "$SAVEDIR" G_DIR_SEPARATOR_S 
+		"start" G_DIR_SEPARATOR_S "%s.def", 
 		name_text );
 	kit = toolkit_new_filename( main_toolkitgroup, buf_all( &buf ) );
 
@@ -1117,7 +1117,7 @@ program_toolkit_new_action_cb( GtkAction *action, Program *program )
 		iwindow_true_cb, NULL, NULL, program );
 	idialog_add_ok( IDIALOG( ss ), 
 		program_toolkit_new_done_cb, _( "Create" ) );
-	idialog_set_parent( IDIALOG( ss ), GTK_WIDGET( program ) );
+	iwindow_set_parent( IWINDOW( ss ), GTK_WIDGET( program ) );
 	iwindow_build( IWINDOW( ss ) );
 
 	gtk_widget_show( ss );
@@ -1195,7 +1195,7 @@ program_column_item_new_action_cb( GtkAction *action, Program *program )
 		iwindow_true_cb, NULL, NULL, program );
 	idialog_add_ok( IDIALOG( ss ), 
 		program_column_item_new_done_cb, _( "Create" ) );
-	idialog_set_parent( IDIALOG( ss ), GTK_WIDGET( program ) );
+	iwindow_set_parent( IWINDOW( ss ), GTK_WIDGET( program ) );
 	iwindow_build( IWINDOW( ss ) );
 
 	gtk_widget_show( ss );
@@ -1204,7 +1204,11 @@ program_column_item_new_action_cb( GtkAction *action, Program *program )
 static void
 program_program_new_action_cb( GtkAction *action, Program *program )
 {
-	(void) program_new( program->kitg );
+	Program *program2;
+
+	program2 = program_new( program->kitg );
+
+	gtk_widget_show( GTK_WIDGET( program2 ) ); 
 }
 
 static void
@@ -1256,7 +1260,7 @@ program_open_action_cb( GtkAction *action, Program *program )
 	iwindow_set_title( IWINDOW( filesel ), _( "Load Definition" ) );
 	filesel_set_flags( FILESEL( filesel ), FALSE, FALSE );
 	filesel_set_filetype( FILESEL( filesel ), filesel_type_definition, 0 ); 
-	idialog_set_parent( IDIALOG( filesel ), GTK_WIDGET( program ) );
+	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( program ) );
 	filesel_set_done( FILESEL( filesel ), program_load_file_cb, program );
 	filesel_set_multi( FILESEL( filesel ), TRUE );
 	iwindow_build( IWINDOW( filesel ) );
@@ -1391,7 +1395,8 @@ program_remove_tool_action_cb( GtkAction *action, Program *program )
 	if( model && IS_TOOL( model ) )
 		model_check_destroy( GTK_WIDGET( program ), model );
 	else 
-		box_info( GTK_WIDGET( program ), _( "No tool selected" ), "" );
+		box_info( GTK_WIDGET( program ), 
+			_( "No tool selected" ), "%s", "" );
 }
 
 static void
@@ -1468,7 +1473,7 @@ program_find_action_cb( GtkAction *action, Program *program )
 	idialog_set_callbacks( IDIALOG( find ), 
 		iwindow_true_cb, NULL, NULL, program );
 	idialog_add_ok( IDIALOG( find ), program_find_done_cb, GTK_STOCK_FIND );
-	idialog_set_parent( IDIALOG( find ), GTK_WIDGET( program ) );
+	iwindow_set_parent( IWINDOW( find ), GTK_WIDGET( program ) );
 	idialog_set_cancel_text( IDIALOG( find ), GTK_STOCK_CLOSE );
 	iwindow_build( IWINDOW( find ) );
 
@@ -1559,7 +1564,7 @@ program_goto_action_cb( GtkAction *action, Program *program )
 	idialog_add_ok( IDIALOG( ss ), 
 		program_goto_done_cb, GTK_STOCK_JUMP_TO );
 	idialog_set_pinup( IDIALOG( ss ), TRUE );
-	idialog_set_parent( IDIALOG( ss ), GTK_WIDGET( program ) );
+	iwindow_set_parent( IWINDOW( ss ), GTK_WIDGET( program ) );
 	iwindow_build( IWINDOW( ss ) );
 
 	gtk_widget_show( ss );
@@ -1620,7 +1625,7 @@ program_errorreport_action_cb( GtkAction *action, Program *program )
 
 	buf_init_static( &buf, txt, MAX_STRSIZE );
 	expr_error_print_all( &buf );
-	if( buf_isempty( &buf ) )
+	if( buf_is_empty( &buf ) )
 		buf_appendf( &buf, _( "No errors found." ) );
 
 	box_info( GTK_WIDGET( program ), _( "Error report." ), 
@@ -2144,7 +2149,7 @@ static PangoTabArray *
 program_tabs_new( void )
 {
 	const int ntabs = 20;
-	const int tab_width = 15;	/* in pixels */
+	const int tab_width = 30;	/* in pixels, about 4 chars */
 
 	PangoTabArray *tabs = pango_tab_array_new( ntabs, TRUE );
 	int i;
@@ -2317,8 +2322,6 @@ program_link( Program *program, Toolkitgroup *kitg )
 	program->kitgroup_destroy_sid = 
 		g_signal_connect( G_OBJECT( program->kitg ), "destroy",
 			G_CALLBACK( program_kitgroup_destroy ), program );
-
-	gtk_widget_show( GTK_WIDGET( program ) ); 
 }
 
 Program *
