@@ -73,8 +73,7 @@ void
 reduce_throw( Reduce *rc )
 {
 	if( !rc->running ) 
-		printf( "panic: uncaught exception in reduce_throw()! "
-			"expect a crash soon\n" );
+		error( "panic: uncaught exception in reduce_throw()!" );
 	else
 		longjmp( rc->error[--rc->running], -1 );
 }
@@ -223,12 +222,23 @@ reduce_get_string( Reduce *rc, PElement *base, char *buf, int n )
 {
 	int sz = n - 1;
 
-	(void) reduce_map_list( rc, base, 
-		(reduce_map_list_fn) reduce_add_char, &buf, &sz );
+	if( PEISSTATIC( base ) ) {
+		/* A static string ... rather than expanding to a list and
+		 * parsing, we can copy directly.
+		 */
+		HeapStaticString *string = PEGETSTATIC( base );
 
-	/* Add '\0' terminator.
-	 */
-	*buf = '\0';
+		im_strncpy( buf, string->text, n );
+		sz -= strlen( buf );
+	}
+	else {
+		(void) reduce_map_list( rc, base, 
+			(reduce_map_list_fn) reduce_add_char, &buf, &sz );
+
+		/* Add '\0' terminator.
+		 */
+		*buf = '\0';
+	}
 
 	return( n - sz - 1 );
 }
@@ -244,8 +254,7 @@ reduce_get_lstring_sub( Reduce *rc, PElement *base, GSList **labels, int *n )
 	return( NULL );
 }
 
-/* Evaluate to [[char]]. Return the number of strings we read. Append to
- * anything that's in the list already.
+/* Evaluate to [[char]]. Return the number of strings we read. 
  */
 int
 reduce_get_lstring( Reduce *rc, PElement *base, GSList **labels )
@@ -253,6 +262,7 @@ reduce_get_lstring( Reduce *rc, PElement *base, GSList **labels )
 	int n;
 
 	n = 0;
+	*labels = NULL;
 	(void) reduce_map_list( rc, base,
 		(reduce_map_list_fn) reduce_get_lstring_sub, labels, &n );
 
@@ -526,7 +536,7 @@ reduce_get_matrix( Reduce *rc,
 /* Test for object is the empty list.
  */
 gboolean
-reduce_iselist( Reduce *rc, PElement *base )
+reduce_is_elist( Reduce *rc, PElement *base )
 {
 	reduce_spine( rc, base );
 	if( PEISELIST( base ) ) 
@@ -538,7 +548,7 @@ reduce_iselist( Reduce *rc, PElement *base )
 /* Test for object is any list.
  */
 gboolean
-reduce_islist( Reduce *rc, PElement *base )
+reduce_is_list( Reduce *rc, PElement *base )
 {
 	reduce_spine( rc, base );
 	if( PEISLIST( base ) ) 
@@ -583,7 +593,7 @@ reduce_test_char( Reduce *rc, PElement *base, int *sz )
  * elements.
  */
 static gboolean
-reduce_n_isstring( Reduce *rc, PElement *base, int sz )
+reduce_n_is_string( Reduce *rc, PElement *base, int sz )
 {
 	void *result;
 
@@ -604,23 +614,23 @@ reduce_n_isstring( Reduce *rc, PElement *base, int sz )
  * allow infinite strings.
  */
 gboolean
-reduce_isstring( Reduce *rc, PElement *base )
+reduce_is_string( Reduce *rc, PElement *base )
 {
-	return( reduce_n_isstring( rc, base, 4 ) );
+	return( reduce_n_is_string( rc, base, 4 ) );
 }
 
 /* Test for list is a finite string. 
  */
 gboolean
-reduce_isfinitestring( Reduce *rc, PElement *base )
+reduce_is_finitestring( Reduce *rc, PElement *base )
 {
-	return( reduce_n_isstring( rc, base, -1 ) );
+	return( reduce_n_is_string( rc, base, -1 ) );
 }
 
 /* Test for list is realvec.
  */
 gboolean
-reduce_isrealvec( Reduce *rc, PElement *base )
+reduce_is_realvec( Reduce *rc, PElement *base )
 {
 	int sz = 4;
 
@@ -642,7 +652,7 @@ reduce_test_line( Reduce *rc, PElement *base, int *w, int *h )
 {
 	/* Test next line from matrix.
 	 */
-	if( !reduce_isrealvec( rc, base ) )
+	if( !reduce_is_realvec( rc, base ) )
 		return( base );
 
 	return( NULL );
@@ -651,7 +661,7 @@ reduce_test_line( Reduce *rc, PElement *base, int *w, int *h )
 /* Test for object is [[real]] .. don't test for rectangularness.
  */
 gboolean
-reduce_ismatrix( Reduce *rc, PElement *base )
+reduce_is_matrix( Reduce *rc, PElement *base )
 {
 	reduce_spine( rc, base );
 	if( !PEISLIST( base ) ) 
@@ -667,7 +677,7 @@ reduce_ismatrix( Reduce *rc, PElement *base )
 /* Test for object is a class.
  */
 gboolean
-reduce_isclass( Reduce *rc, PElement *klass )
+reduce_is_class( Reduce *rc, PElement *klass )
 {
 	reduce_spine( rc, klass );
 	if( PEISCLASS( klass ) )
@@ -682,12 +692,12 @@ reduce_isclass( Reduce *rc, PElement *klass )
 
  */
 gboolean
-reduce_isinstanceof_exact( Reduce *rc, const char *name, PElement *instance )
+reduce_is_instanceof_exact( Reduce *rc, const char *name, PElement *instance )
 {
         BufInfo buf;
         char txt[256];
 
-	if( !reduce_isclass( rc, instance ) )
+	if( !reduce_is_class( rc, instance ) )
 		return( FALSE );
 
         buf_init_static( &buf, txt, 256 );
@@ -701,17 +711,17 @@ reduce_isinstanceof_exact( Reduce *rc, const char *name, PElement *instance )
 /* Test for thing is an instance of the named class symbol.
  */
 gboolean
-reduce_isinstanceof( Reduce *rc, const char *name, PElement *instance )
+reduce_is_instanceof( Reduce *rc, const char *name, PElement *instance )
 {
 	PElement super;
 
 	reduce_spine( rc, instance );
 	if( !PEISCLASS( instance ) )
 		return( FALSE );
-	if( reduce_isinstanceof_exact( rc, name, instance ) )
+	if( reduce_is_instanceof_exact( rc, name, instance ) )
 		return( TRUE );
 	if( class_get_super( instance, &super ) && !PEISELIST( &super ) ) 
-		return( reduce_isinstanceof( rc, name, &super ) );
+		return( reduce_is_instanceof( rc, name, &super ) );
 
 	return( FALSE );
 }
@@ -844,7 +854,8 @@ is_WHNF( PElement *out )
 		PEISCOMPILEREF( out ) || PEISNOVAL( out ) || PEISSTATIC( out ) )
 		return( TRUE );
 
-	/* Must be a function ... loop down the spine, counting args.
+	/* Must be a function or generator ... loop down the spine, counting 
+	 * args.
 	 */
 	for( spine = *out, i = 0; PEGETTYPE( &spine ) == ELEMENT_NODE; i++ ) {
 		hn = PEGETVAL( &spine );
@@ -859,7 +870,7 @@ is_WHNF( PElement *out )
 		if( i > 1 )
 			return( FALSE );
 	}
-	else if( PEISDOT( &spine ) || PEISUNOP( &spine ) ) {
+	else if( PEISUNOP( &spine ) ) {
 		if( i > 0 )
 			return( FALSE );
 	}
@@ -948,7 +959,7 @@ reduce_start:
 	case ELEMENT_BOOL:
 	case ELEMENT_ELIST:
 	case ELEMENT_TAG:
-	case ELEMENT_IMAGE:
+	case ELEMENT_MANAGED:
 	case ELEMENT_SYMREF:
 	case ELEMENT_COMPILEREF:
 		/* Base type .. no more reduction needed.
@@ -1086,7 +1097,7 @@ reduce_start:
 			HeapNode **arg;
 			int na;
 
-			/* A VIPS unction.
+			/* A VIPS function.
 			 */
 			na = sym->fn_nargs;
 
@@ -1245,7 +1256,7 @@ reduce_start:
 			PEPOINTLEFT( hn, &np );
 
 			goto reduce_start;
-		
+
 		case TAG_GEN:
 		{
 			double d1;
@@ -1331,62 +1342,37 @@ reduce_start:
 			goto reduce_start;
 		}
 
-		case TAG_DOT:
+		case TAG_FILE:
 		{
-			HeapNode **arg;
-			PElement rhs1;
+			Managedfile *managedfile = MANAGEDFILE( GETLEFT( hn ) );
+			int ch = managedfile_getc( managedfile );
 
-			/* '.' extract operator ... should have 1 arg.
+			/* -1 means error, 0 means EOF.
 			 */
-			if( !RSCHECKARGS( rc, 1 ) )
-				/* Not enough ... function result.
+			if( ch == -1 )
+				reduce_throw( rc );
+			else if( ch == 0 ) {
+				/* Turn us into [].
 				 */
-				break;
-
-			arg = &RSGET( rc, 0 );
-
-			/* We don't use action_dispatch(), since dot is a bit
-			 * bizarre ... it's a family of uops, making tracing
-			 * slightly different from usual.
-			 */
-			PEPOINTRIGHT( arg[0], &rhs1 );
-			reduce_spine( rc, &rhs1 );
-
-			if( trace_flags & TRACE_OPERATOR ) {
-				BufInfo *buf = trace_push();
-				PElement rhs2;
-
-				buf_appendf( buf, "\"%s\" ", 
-					OPERATOR_NAME( BI_DOT ) );
-				PEPOINTLEFT( hn, &rhs2 );
-				trace_pelement( &rhs2 ); 
-				buf_appends( buf, " " );
-				trace_args( arg, 1 );
+				hn->type = TAG_APPL;
+				PPUT( hn, 
+					ELEMENT_COMB, COMB_I,
+					ELEMENT_ELIST, NULL ); 
 			}
+			else {
+				HeapNode *hn1;
 
-			action_proc_dot( rc, NULL,
-				BI_DOT, OPERATOR_NAME( BI_DOT ), 
-				arg, &rhs1 );
-
-			PPUTLEFT( arg[0], ELEMENT_COMB, COMB_I );
-
-			if( trace_flags & TRACE_OPERATOR ) {
-				trace_result( TRACE_OPERATOR, &rhs1 );
-				trace_pop();
+				/* Not at end ... make another CONS.
+				 */
+				if( NEWNODE( heap, hn1 ) )
+					reduce_throw( rc );
+				*hn1 = *hn;
+				hn->type = TAG_CONS;
+				PPUT( hn, 
+					ELEMENT_CHAR, GUINT_TO_POINTER( ch ), 
+					ELEMENT_NODE, hn1 );
 			}
 
-			/* Find output element.
-			 */
-			RSPOP( rc, 1 );
-			if( RSFRAMEEMPTY( rc ) )
-				np = RSGETWB( rc ); 
-			else
-				PEPOINTLEFT( RSGET( rc, 0 ), &np );
-
-			/* Write to above node.
-			 */
-			PEPUTP( &np, GETRT( arg[0] ), GETRIGHT( arg[0] ) );
-
 			/* Loop again with new np.
 			 */
 			goto reduce_start;
@@ -1728,7 +1714,7 @@ reduce_start:
 		Compile *compile;
 
 		/* Some unary operator. First arg is the compile that built
-		 * us, 2nd i atul arg that might need reducing.
+		 * us, 2nd is the actual arg that might need reducing.
 		 */
 		if( !RSCHECKARGS( rc, 2 ) )
 			/* Not enough ... function result.
@@ -1799,9 +1785,7 @@ reduce_spine_strict( Reduce *rc, PElement *np )
 
 	/* Make sure this element is reduced.
 	 */
-	if( PEGETTYPE( np ) == ELEMENT_NODE ||
-		PEGETTYPE( np ) == ELEMENT_SYMBOL ) 
-		reduce_spine( rc, np );
+	reduce_spine( rc, np );
 
 	/* If it's a non-empty list, may need to reduce inside.
 	 */
