@@ -84,6 +84,10 @@ static const int regionview_label_border = 5;
  */
 static const int regionview_crosshair_length = 5;
 
+/* The center of the crosshair is also sensitive for arrows.
+ */
+static const int regionview_crosshair_centre = 8;
+
 /* How close you need to get to switch the type.
  */
 static const int regionview_morph_threshold = 20;
@@ -458,7 +462,7 @@ regionview_queue_draw( Regionview *regionview )
 static void
 regionview_paint( Regionview *regionview )
 {
-        static char dash_list[] = { 10, 10 };
+        static gint8 dash_list[] = { 10, 10 };
 
 	Imagepresent *ip = regionview->ip;
 	Imagedisplay *id = ip->id;
@@ -530,8 +534,7 @@ regionview_detach( Regionview *regionview )
 		assert( regionview->ip->grabbed == regionview );
 
 #ifdef DEBUG_GRAB
-		printf( "regionview_detach: 0x%x\n", 
-			(unsigned int) regionview );
+		printf( "regionview_detach: %p\n", regionview );
 #endif /*DEBUG_GRAB*/
 
 		regionview->state = REGIONVIEW_WAIT;
@@ -550,7 +553,7 @@ regionview_destroy( GtkObject *object )
 	Imagedisplay *id;
 
 #ifdef DEBUG_MAKE
-	printf( "regionview_destroy: 0x%x\n", (unsigned int) regionview );
+	printf( "regionview_destroy: %p\n", regionview );
 #endif /*DEBUG_MAKE*/
 
 	g_return_if_fail( object != NULL );
@@ -746,8 +749,8 @@ regionview_get_model( Regionview *regionview )
 	return( model_area );
 }
 
-/* Are we in a ws in compatibility mode? If we are, we need to offset x/y by
- * image xoffset/yoffset.
+/* Are we in a ws in 7.8 compatibility mode? If we are, we need to offset x/y 
+ * by image xoffset/yoffset.
  */
 static gboolean
 regionview_get_compatibility( Regionview *regionview )
@@ -1141,7 +1144,7 @@ regionview_rect_touches_region( Regionview *regionview, Rect *expose )
 		tiny.top = canvas_area.top;
 		tiny.width = 1;
 		tiny.height = 1;
-		im_rect_marginadjust( &tiny, regionview_border_width );
+		im_rect_marginadjust( &tiny, regionview_crosshair_centre );
 		if( regionview_rect_touching( regionview, &tiny, expose ) )
 			return( TRUE );
 
@@ -1149,7 +1152,7 @@ regionview_rect_touches_region( Regionview *regionview, Rect *expose )
 		tiny.top = IM_RECT_BOTTOM( &canvas_area );
 		tiny.width = 1;
 		tiny.height = 1;
-		im_rect_marginadjust( &tiny, regionview_border_width );
+		im_rect_marginadjust( &tiny, regionview_crosshair_centre );
 		if( regionview_rect_touching( regionview, &tiny, expose ) )
 			return( TRUE );
 
@@ -1303,7 +1306,7 @@ regionview_find_resize( Regionview *regionview, int x, int y )
 		tiny.top = IM_RECT_BOTTOM( &canvas_area );
 		tiny.width = 1;
 		tiny.height = 1;
-		im_rect_marginadjust( &tiny, regionview_border_width );
+		im_rect_marginadjust( &tiny, regionview_crosshair_centre );
 		if( im_rect_includespoint( &tiny, x, y ) )
 			return( REGIONVIEW_RESIZE_BOTTOMRIGHT );
 
@@ -1311,7 +1314,7 @@ regionview_find_resize( Regionview *regionview, int x, int y )
 		tiny.top = canvas_area.top;
 		tiny.width = 1;
 		tiny.height = 1;
-		im_rect_marginadjust( &tiny, regionview_border_width );
+		im_rect_marginadjust( &tiny, regionview_crosshair_centre );
 		if( im_rect_includespoint( &tiny, x, y ) )
 			return( REGIONVIEW_RESIZE_TOPLEFT );
 
@@ -1369,7 +1372,7 @@ regionview_attach( Regionview *regionview, int x, int y )
 	assert( !regionview->ip->grabbed );
 
 #ifdef DEBUG_GRAB
-	printf( "regionview_attach: 0x%x\n", (unsigned int) regionview );
+	printf( "regionview_attach: %p\n", regionview );
 #endif /*DEBUG_GRAB*/
 
 	switch( regionview->resize ) {
@@ -1786,8 +1789,7 @@ regionview_motion( Regionview *regionview, GdkEvent *ev, int x, int y )
 	RegionviewResize resize;
 
 #ifdef EVENT
-	printf( "regionview_motion: 0x%x, %d x %d\n", 
-		(unsigned int) regionview, x, y );
+	printf( "regionview_motion: %p, %d x %d\n", regionview, x, y );
 #endif /*EVENT*/
 
 	/* We've got hints turned on, so we have to read the pointer.
@@ -2047,11 +2049,11 @@ regionview_set_type( Regionview *regionview, PElement *root )
 	gboolean result;
 	int i;
 
-	if( heap_isclass( root, &result ) && result )
+	if( heap_is_class( root, &result ) && result )
 		for( i = 0; i < IM_NUMBER( regionview_display_table ); i++ ) {
 			const char *name = regionview_display_table[i].name;
 
-			if( !heap_isinstanceof( name, root, &result ) )
+			if( !heap_is_instanceof( name, root, &result ) )
 				continue;
 			if( result ) {
 				regionview->type =
