@@ -141,47 +141,6 @@ rowview_set_name( Rowview *rview, const char *set_name )
 	}
 }
 
-static void *
-rowview_tip_sub( Link *link, BufInfo *buf )
-{
-	if( link->child->dirty ) {
-		symbol_qualified_name( link->child, buf );
-		buf_appends( buf, " " );
-	}
-
-	return( NULL );
-}
-
-static void
-rowview_tip( Rowview *rview, BufInfo *buf )
-{
-	Row *row = ROW( VOBJECT( rview )->iobject );
-
-	if( row == row->top_row && row->sym->dirty ) {
-		Symbol *sym = row->sym;
-
-		if( sym->ndirtychildren ) {
-			buf_appends( buf, ", " );
-			/* Expands to "A1, blocked on A27" ... tooltip on
-			 * dirty rows.
-			 */
-			buf_appends( buf, _( "blocked on" ) );
-			buf_appends( buf, " " );
-			slist_map( sym->topchildren,
-				(SListMapFn) rowview_tip_sub, buf );
-		}
-	}
-}
-
-static void
-rowview_set_tooltip( Rowview *rview, const char *tip )
-{
-	if( !rview->last_tooltip || strcmp( rview->last_tooltip, tip ) != 0 ) {
-		set_tooltip( rview->but, "%s", tip );
-		SETSTR( rview->last_tooltip, tip );
-	}
-}
-
 static void
 rowview_update_widgets( Rowview *rview )
 {
@@ -225,7 +184,7 @@ rowview_update_widgets( Rowview *rview )
 	/* Spin visible only if this is a class. 
 	 */
 	widget_visible( rview->spin, 
-		rview->visible && row->isclass && editable );
+		rview->visible && row->is_class && editable );
 
         /* Set colours.
          */
@@ -265,27 +224,6 @@ rowview_update_widgets( Rowview *rview )
 	}
 	widget_visible( rview->led, 
 		rview->visible && CALC_DISPLAY_LED && editable );
-
-	if( row->expr && row->expr->err ) 
-		/* Display error message in tooltip.
-		 */
-		rowview_set_tooltip( rview, row->expr->error_sub );
-	else if( row->expr ) {
-		BufInfo buf;
-		char txt[200];
-
-		buf_init_static( &buf, txt, 200 );
-		expr_tip( row->expr, &buf );
-		rowview_tip( rview, &buf );
-		rowview_set_tooltip( rview, buf_all( &buf ) );
-	}
-	else 
-		rowview_set_tooltip( rview, 
-			_( "Left-click to select, "
-			"shift-left-click to extend select, "
-			"double-left-click to edit, "
-			"right-click for menu, "
-			"left-drag to move" ) );
 }
 
 static void 
@@ -385,7 +323,7 @@ rowview_ungroup_cb( GtkWidget *menu, GtkWidget *button, Rowview *rview )
 
         workspace_deselect_all( row->ws );
         row_select( row );
-        if( workspace_selected_ungroup( row->ws ) )
+        if( !workspace_selected_ungroup( row->ws ) )
                 box_alert( button );
         symbol_recalculate_all();
 }
@@ -490,27 +428,17 @@ rowview_spin_down_cb( GtkWidget *widget, gpointer client )
 	filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
 }
 
-/* Scroll to make tally entry visible. If it's in a closed column, pop the
- * column open first.
+/* Scroll to make tally entry visible. 
  */
 static void
-rowview_scrollto( View *view )
+rowview_scrollto( View *view, ModelScrollPosition position )
 {
 	Rowview *rview = ROWVIEW( view );
-	Row *row = ROW( VOBJECT( rview )->iobject );
-	Column *col = row->top_col;
 	Columnview *cview = view_get_columnview( VIEW( rview ) );
 	Workspaceview *wview = cview->wview;
 
         int x, y, w, h;
 
-        /* If the column is closed, open it.
-         */
-        if( !col->open ) {
-                col->open  = TRUE;
-		iobject_changed( IOBJECT( col ) );
-        }
-
         /* Extract position of tally row in RC widget.
          */
         rowview_get_position( rview, &x, &y, &w, &h );
@@ -611,7 +539,7 @@ rowview_link( View *view, Model *model, View *parent )
 			GTK_SIGNAL_FUNC( rowview_drag_data_received ), rview );
 	}
 
-	rview->popup_sid = rowview_menu_attach( rview, rview->but );
+	rowview_menu_attach( rview, rview->but );
 }
 
 static void
@@ -674,67 +602,44 @@ rowview_class_init( RowviewClass *class )
 }
 
 static void
-rowview_flash_help( GtkWidget *widget, Rowview *rview )
+rowview_enter_cb( GtkWidget *widget, Rowview *rview )
 {
 	Row *row = ROW( VOBJECT( rview )->iobject );
-	Expr *expr = row->expr;
 
-	char str[MAX_LINELENGTH];
-	BufInfo buf;
-
-	/* No symbol? eg. on load error.
-	 */
-	if( !expr )
-		return;
-
-	buf_init_static( &buf, str, MAX_LINELENGTH );
-
-	row_qualified_name( row, &buf );
-
-	if( expr->err ) {
-		buf_appends( &buf, ": " );
-		buf_appends( &buf, expr->error_top );
-	}
-	else if( row->child_rhs->itext ) {
-		iText *itext = ITEXT( row->child_rhs->itext );
-
-		buf_appends( &buf, " = " );
-
-		if( row->ws->mode != WORKSPACE_MODE_FORMULA )
-			buf_appends( &buf, NN( itext->formula ) );
-		else 
-			buf_appends( &buf, buf_all( &itext->value ) );
-	}
-
-	workspace_set_status( row->ws, "%s", buf_firstline( &buf ) );
+	row_set_status( row );
+	row_show_dependants( row );
 }
 
 static void
-rowview_show_dependants_cb( GtkWidget *widget, Rowview *rview )
+rowview_leave_cb( GtkWidget *widget, Rowview *rview )
 {
-	row_show_dependants( ROW( VOBJECT( rview )->iobject ) );
-}
+	Row *row = ROW( VOBJECT( rview )->iobject );
 
-static void
-rowview_hide_dependants_cb( GtkWidget *widget, Rowview *rview )
-{
-	row_hide_dependants( ROW( VOBJECT( rview )->iobject ) );
+	row_hide_dependants( row );
 }
 
 static gboolean
 rowview_focus_cb( GtkWidget *widget, GtkDirectionType dir, Rowview *rview )
 {
-        view_scrollto( VIEW( rview ) );
+        view_scrollto( VIEW( rview ), MODEL_SCROLL_TOP );
 
         return( FALSE );
 }
 
+static void
+rowview_tooltip_generate( GtkWidget *widget, BufInfo *buf, Rowview *rview )
+{
+	Row *row = ROW( VOBJECT( rview )->iobject );
+
+	iobject_info( IOBJECT( row ), buf );
+	buf_removec( buf, '\n' );
+}
+
 static void
 rowview_init( Rowview *rview )
 {
         rview->visible = TRUE; 
 	rview->rnum = -1;
-        rview->popup_sid = 0; 
         rview->set_name_tid = 0; 
         rview->set_name = NULL; 
         rview->to_set_name = NULL; 
@@ -770,13 +675,13 @@ rowview_init( Rowview *rview )
         gtk_container_add( GTK_CONTAINER( rview->but ), rview->label );
         gtk_widget_show( rview->label );
         gtk_signal_connect( GTK_OBJECT( rview->but ), "enter",
-                GTK_SIGNAL_FUNC( rowview_flash_help ), rview );
-        gtk_signal_connect( GTK_OBJECT( rview->but ), "enter",
-                GTK_SIGNAL_FUNC( rowview_show_dependants_cb ), rview );
+                GTK_SIGNAL_FUNC( rowview_enter_cb ), rview );
         gtk_signal_connect( GTK_OBJECT( rview->but ), "leave",
-                GTK_SIGNAL_FUNC( rowview_hide_dependants_cb ), rview );
+                GTK_SIGNAL_FUNC( rowview_leave_cb ), rview );
         gtk_signal_connect( GTK_OBJECT( rview->but ), "focus",
                 GTK_SIGNAL_FUNC( rowview_focus_cb ), rview );
+	set_tooltip_generate( rview->but, 
+		(TooltipGenerateFn) rowview_tooltip_generate, rview, NULL );
 }
 
 GtkType
@@ -865,7 +770,7 @@ rowview_set_visible( Rowview *rview, gboolean visible )
 
 	if( rview->visible != visible ) {
 		widget_visible( rview->but, visible );
-		widget_visible( rview->spin, visible && row->isclass );
+		widget_visible( rview->spin, visible && row->is_class );
 		widget_visible( rview->led, visible && CALC_DISPLAY_LED );
 		if( rview->rhsview )
 			widget_visible( GTK_WIDGET( rview->rhsview ), visible );
