@@ -227,7 +227,7 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 
 	/* Must be a class display.
 	 */
-	assert( !scol->istop );
+	assert( !scol->is_top );
 	row = HEAPMODEL( scol )->row;
 	expr = row->expr;
 
@@ -258,13 +258,13 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 		redisplay? more reliable, but even slower :-(
 
 	 */
-	if( scol->scol->istop )
+	if( scol->scol->is_top )
 		heap_serial_new( reduce_context->heap );
 
 	/* Is it a class with a typecheck member? Go through
 	 * that. Do an isclass first to force eval.
 	 */
-	if( !heap_isclass( &instance, &result ) )
+	if( !heap_is_class( &instance, &result ) )
 		return( FALSE );
 	if( result &&
 		class_get_member( &instance, MEMBER_CHECK, NULL, &member ) ) {
@@ -274,7 +274,7 @@ subcolumn_class_new_heap( Subcolumn *scol, PElement *root )
 
 		/* Force eval of the typecheck member.
 		 */
-		if( !heap_isclass( &member, &result ) || !result )
+		if( !heap_is_class( &member, &result ) || !result )
 			return( FALSE );
 	}
 
@@ -369,7 +369,7 @@ subcolumn_new_heap( Heapmodel *heapmodel, PElement *root )
 
 	/* A bunch of locals? Update them all.
 	 */
-	if( !scol->istop && !subcolumn_class_new_heap( scol, root ) )
+	if( !scol->is_top && !subcolumn_class_new_heap( scol, root ) )
 		return( scol );
 
 	return( HEAPMODEL_CLASS( parent_class )->new_heap( heapmodel, root ) );
@@ -417,7 +417,7 @@ subcolumn_child_remove( iContainer *parent, iContainer *child )
 static Column *
 subcolumn_get_column( Subcolumn *scol )
 {
-	assert( scol->istop );
+	assert( scol->is_top );
 
 	return( COLUMN( ICONTAINER( scol )->parent ) );
 }
@@ -431,7 +431,7 @@ subcolumn_get_subcolumn( Subcolumn *scol )
 	Row *row;
 	Subcolumn *escol;
 
-	assert( !scol->istop );
+	assert( !scol->is_top );
 
 	rhs = HEAPMODEL( scol )->rhs;
 	row = HEAPMODEL( rhs )->row;
@@ -445,14 +445,14 @@ subcolumn_get_subcolumn( Subcolumn *scol )
 static Column *
 subcolumn_get_top_column( Subcolumn *scol )
 {
-	if( !scol->istop ) 
+	if( !scol->is_top ) 
 		return( subcolumn_get_top_column( 
 			subcolumn_get_subcolumn( scol ) ) );
 
 	return( subcolumn_get_column( scol ) );
 }
 
-/* Return the enclosing subcolumn ... but not the istop one. Ie. the enclosing
+/* Return the enclosing subcolumn ... but not the is_top one. Ie. the enclosing
  * subcolumn which has the base for this class tree.
  */
 static Subcolumn *
@@ -460,11 +460,11 @@ subcolumn_get_top_subcolumn( Subcolumn *scol )
 {
 	Subcolumn *enclosing;
 
-	if( scol->istop )
+	if( scol->is_top )
 		return( NULL );
 
 	enclosing = subcolumn_get_subcolumn( scol );
-	if( enclosing->istop )
+	if( enclosing->is_top )
 		return( scol );
 	else 
 		return( subcolumn_get_top_subcolumn( enclosing ) );
@@ -481,21 +481,21 @@ subcolumn_parent_add( iContainer *child )
 	assert( !IS_COLUMN( child->parent ) || 
 		g_slist_length( child->parent->children ) == 1 );
 
-	scol->istop = IS_COLUMN( child->parent );
+	scol->is_top = IS_COLUMN( child->parent );
 
 	/* For sub-columns, default to nothing visible.
 	 */
-	if( !scol->istop )
+	if( !scol->is_top )
 		scol->vislevel = 0;
 
 	/* Update context pointers.
 	 */
-	if( scol->istop )
+	if( scol->is_top )
 		scol->col = subcolumn_get_column( scol );
 	else
 		scol->col = NULL;
 
-	if( !scol->istop )
+	if( !scol->is_top )
 		scol->scol = subcolumn_get_subcolumn( scol );
 	else
 		scol->scol = NULL;
@@ -505,7 +505,13 @@ subcolumn_parent_add( iContainer *child )
 
 	/* Top level subcolumns default to display on, others to display off.
 	 */
-	MODEL( scol )->display = scol->istop;
+	MODEL( scol )->display = scol->is_top;
+}
+
+static View *
+subcolumn_view_new( Model *model, View *parent )
+{
+	return( subcolumnview_new() );
 }
 
 static gboolean
@@ -562,7 +568,7 @@ subcolumn_class_init( SubcolumnClass *class )
 	icontainer_class->child_remove = subcolumn_child_remove;
 	icontainer_class->parent_add = subcolumn_parent_add;
 
-	model_class->view_new = subcolumnview_new;
+	model_class->view_new = subcolumn_view_new;
 	model_class->load = subcolumn_load;
 	model_class->save = subcolumn_save;
 
@@ -625,7 +631,7 @@ subcolumn_link( Subcolumn *scol, Rhs *rhs, Column *col )
 {
 	assert( rhs == NULL || col == NULL );
 
-	/* parent_add() sets istop for us.
+	/* parent_add() sets is_top for us.
 	 */
 	if( rhs ) 
 		icontainer_child_add( ICONTAINER( rhs ), 
