@@ -38,6 +38,10 @@
 
 static FilemodelClass *parent_class = NULL;
 
+/* Largest string we let the user set for name/tip/etc.
+ */
+#define MAX_NAME (256)
+
 /* Remove a tool. Also strip the sym, if any.
  */
 static void 
@@ -51,7 +55,7 @@ tool_dispose( GObject *gobject )
 		symbol_name_print( tool->sym );
 	else
 		printf( "anonymous-tool" );
-	printf( " at addr 0x%x\n", (unsigned int) tool );
+	printf( " at addr %p\n", tool );
 #endif /*DEBUG*/
 
 	FREESID( tool->new_value_sid, tool->link_sym );
@@ -73,6 +77,12 @@ tool_dispose( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
 
+static View *
+tool_view_new( Model *model, View *parent )
+{
+	return( toolview_new() );
+}
+
 /* Save a tool's definition to a file. 
  */
 static gboolean
@@ -170,7 +180,7 @@ tool_class_init( ToolClass *class )
 
 	icontainer_class->parent_add = tool_parent_add;
 
-	model_class->view_new = toolview_new;
+	model_class->view_new = tool_view_new;
 	model_class->save_text = tool_save_text;
 }
 
@@ -254,7 +264,7 @@ toolitem_new( Toolitem *parent, Compile *compile, Tool *tool )
 {
 	Toolitem *toolitem;
 
-	if( !(toolitem = IM_NEW( NULL, Toolitem )) )
+	if( !(toolitem = INEW( NULL, Toolitem )) )
 		return( NULL );
 	toolitem->compile = compile;
 	toolitem->tool = tool;
@@ -284,7 +294,7 @@ toolitem_new( Toolitem *parent, Compile *compile, Tool *tool )
 
 /* Set label & name & icon.
 
-	FIXME ... we will do repeated heap_isinstanceof() during item build,
+	FIXME ... we will do repeated heap_is_instanceof() during item build,
 	do it once and set a flag instead
 
  */
@@ -292,49 +302,49 @@ static void
 toolitem_set_name( Toolitem *toolitem, PElement *root )
 {
 	gboolean result;
-	char value[100];
+	char value[MAX_NAME];
 	int i;
 
 	if( root && 
-		heap_isinstanceof( CLASS_MENUITEM, root, &result ) && 
+		heap_is_instanceof( CLASS_MENUITEM, root, &result ) && 
 		result ) {
 		if( class_get_member_string( root, 
-			MEMBER_LABEL, value, 100 ) ) {
+			MEMBER_LABEL, value, MAX_NAME ) ) {
 			/* Save the i18n-ed version.
 			 */
-			SETSTR( toolitem->label, _( value ) );
+			IM_SETSTR( toolitem->label, _( value ) );
 
 			/* Strip underscores (they mark mnemonics). Can't use
 			 * strrcpy(), we have overlapping blocks.
 			 */
-			im_strncpy( value, toolitem->label, 100 );
+			im_strncpy( value, toolitem->label, MAX_NAME );
 			for( i = 0; value[i]; i++ )
 				if( value[i] == '_' )
 					memccpy( value + i, value + i + 1,
-						0, 100 - i );
-			SETSTR( toolitem->name, value );
+						0, MAX_NAME - i );
+			IM_SETSTR( toolitem->name, value );
 		}
 
 		if( class_get_member_string( root, 
-			MEMBER_ICON, value, 100 ) ) 
-			SETSTR( toolitem->icon, value );
+			MEMBER_ICON, value, MAX_NAME ) ) 
+			IM_SETSTR( toolitem->icon, value );
 	}
 	else {
 		/* Remove underscores from the object name ... we don't want
 		 * them to be mnemonics.
 		 */
 		im_strncpy( value, 
-			IOBJECT( toolitem->compile->sym )->name, 100 );
+			IOBJECT( toolitem->compile->sym )->name, MAX_NAME );
 		for( i = 0; value[i]; i++ )
 			if( value[i] == '_' )
 				value[i] = ' ';
 
-		SETSTR( toolitem->label, value );
-		SETSTR( toolitem->name, toolitem->label );
+		IM_SETSTR( toolitem->label, value );
+		IM_SETSTR( toolitem->name, toolitem->label );
 	}
 
 	if( root && 
-		heap_isinstanceof( CLASS_MENUSEPARATOR, root, &result ) && 
+		heap_is_instanceof( CLASS_MENUSEPARATOR, root, &result ) && 
 		result ) 
 		toolitem->is_separator = TRUE;
 }
@@ -343,13 +353,14 @@ static void
 toolitem_set_tooltip( Toolitem *toolitem, PElement *root )
 {
 	gboolean result;
-	char value[100];
+	char value[MAX_NAME];
 
 	if( root && 
-		heap_isinstanceof( CLASS_MENUITEM, root, &result ) &&
+		heap_is_instanceof( CLASS_MENUITEM, root, &result ) &&
 		result &&
-		class_get_member_string( root, MEMBER_TOOLTIP, value, 100 ) ) {
-		SETSTR( toolitem->tooltip, _( value ) );
+		class_get_member_string( root, 
+			MEMBER_TOOLTIP, value, MAX_NAME ) ) {
+		IM_SETSTR( toolitem->tooltip, _( value ) );
 	}
 	else {
 		char *p;
@@ -372,15 +383,14 @@ toolitem_set_tooltip( Toolitem *toolitem, PElement *root )
 			while( isspace( (int)(*p) ) )
 				p++;
 
-			/* Limit to 100 chars or 1st line. Strip trailing
+			/* Limit to MAX_NAME chars or 1st line. Strip trailing
 			 * whitespace.
 			 */
-			im_strncpy( value, p, 100 );
+			im_strncpy( value, p, MAX_NAME );
 			if( (p = strchr( value, '\n' )) )
 				*p = '\0';
-			if( (p = (char *) my_strrspn( value, WHITESPACE )) )
-				*p = '\0';
-			SETSTR( toolitem->tooltip, value );
+			*((char *) my_strrspn( value, WHITESPACE )) = '\0';
+			IM_SETSTR( toolitem->tooltip, value );
 		}
 	}
 }
@@ -393,7 +403,7 @@ toolitem_set_pullright( Toolitem *toolitem, PElement *root )
 	/* New-style pullright?
 	 */
 	if( root && 
-		heap_isinstanceof( CLASS_MENUPULLRIGHT, root, &result ) &&
+		heap_is_instanceof( CLASS_MENUPULLRIGHT, root, &result ) &&
 		result ) 
 		toolitem->is_pullright = TRUE;
 	/* Old-style pullright?
@@ -421,7 +431,7 @@ toolitem_set_action( Toolitem *toolitem, PElement *root )
 	/* If this is a Menuaction, we need the action member.
 	 */
 	if( root && 
-		heap_isinstanceof( CLASS_MENUACTION, root, &result ) &&
+		heap_is_instanceof( CLASS_MENUACTION, root, &result ) &&
 		result ) {
 		PElement out;
 
@@ -435,7 +445,7 @@ toolitem_set_action( Toolitem *toolitem, PElement *root )
 	if( toolitem->is_action )
 		buf_appends( &buf, "." MEMBER_ACTION ); 
 
-	SETSTR( toolitem->action, buf_all( &buf ) );
+	IM_SETSTR( toolitem->action, buf_all( &buf ) );
 
 	/* No action member found and this is an item (ie. not a pullright)?
 	 * Default to the sym itself.
@@ -457,7 +467,7 @@ toolitem_set_path( Toolitem *toolitem )
 		buf_appendf( &buf, "<mainw>/Toolkits/%s", 
 			IOBJECT( toolitem->tool->kit )->name );
 	buf_appendf( &buf, "/%s", toolitem->name );
-	SETSTR( toolitem->path, buf_all( &buf ) );
+	IM_SETSTR( toolitem->path, buf_all( &buf ) );
 }
 
 static void
@@ -472,7 +482,7 @@ toolitem_set_user_path( Toolitem *toolitem )
 	else
 		buf_appends( &buf, IOBJECT( toolitem->tool->kit )->name );
 	buf_appendf( &buf, " / %s", toolitem->name );
-	SETSTR( toolitem->user_path, buf_all( &buf ) );
+	IM_SETSTR( toolitem->user_path, buf_all( &buf ) );
 }
 
 static void *
@@ -504,7 +514,7 @@ toolitem_set_help( Toolitem *toolitem )
 	buf_appends( &buf, ": " );
 	if( toolitem->tooltip )
 		buf_appends( &buf, toolitem->tooltip );
-	SETSTR( toolitem->help, buf_firstline( &buf ) );
+	IM_SETSTR( toolitem->help, buf_firstline( &buf ) );
 }
 
 static Toolitem *
@@ -571,7 +581,7 @@ toolitem_build_all( Tool *tool, Compile *compile, PElement *root,
 	/* If this is a dynamic pullright, walk the heap to find the members.
 	 */
 	if( toolitem->is_pullright && root && 
-		heap_isinstanceof( CLASS_MENUPULLRIGHT, root, &result ) &&
+		heap_is_instanceof( CLASS_MENUPULLRIGHT, root, &result ) &&
 		result ) {
 		PElement member;
 		HeapNode *p;
@@ -599,7 +609,7 @@ toolitem_build_all( Tool *tool, Compile *compile, PElement *root,
 				/* For dynamic menus, only make items for
 				 * things which are subclasses of menu.
 				 */
-				if( !heap_isinstanceof( CLASS_MENU, 
+				if( !heap_is_instanceof( CLASS_MENU, 
 					&v, &result ) || !result ) 
 					continue;
 
@@ -660,7 +670,8 @@ tool_toolitem_rebuild( Tool *tool )
 
 	case TOOL_DIA:
 		if( (tool->toolitem = toolitem_new( NULL, NULL, tool )) ) 
-			SETSTR( tool->toolitem->label, IOBJECT( tool )->name );
+			IM_SETSTR( tool->toolitem->label, 
+				IOBJECT( tool )->name );
 		break;
 
 	case TOOL_SEP:
@@ -732,7 +743,7 @@ tool_new_sym( Toolkit *kit, int pos, Symbol *sym )
 #ifdef DEBUG
 	printf( "tool_new_sym: new tool for " );
 	symbol_name_print( sym );
-	printf( " at 0x%x\n", (unsigned int) tool );
+	printf( " at %p\n", tool );
 #endif /*DEBUG*/
 
 	return( tool );
