@@ -86,6 +86,12 @@ toolkit_info( iObject *iobject, BufInfo *buf )
 	buf_appendf( buf, "group = \"%s\"\n", IOBJECT( kit->kitg )->name );
 }
 
+static View *
+toolkit_view_new( Model *model, View *parent )
+{
+	return( toolkitview_new() );
+}
+
 static gboolean
 toolkit_save_text( Model *model, iOpenFile *of )
 {
@@ -145,7 +151,7 @@ toolkit_class_init( ToolkitClass *class )
 	iobject_class->info = toolkit_info;
 	iobject_class->changed = toolkit_changed;
 
-	model_class->view_new = toolkitview_new;
+	model_class->view_new = toolkit_view_new;
 	model_class->save_text = toolkit_save_text;
 	model_class->load_text = toolkit_load_text;
 
@@ -290,7 +296,7 @@ toolkit_by_name( Toolkitgroup *kitg, const char *name )
 		char file[FILENAME_MAX];
 
 		im_snprintf( file, FILENAME_MAX,
-			"$SAVEDIR" IM_DIR_SEP_STR "start" IM_DIR_SEP_STR 
+			"$SAVEDIR" G_DIR_SEPARATOR_S "start" G_DIR_SEPARATOR_S 
 			"%s.def", 
 			name );
 		kit = toolkit_new_filename( kitg, file );
