@@ -68,7 +68,9 @@ static void *
 toolkitbrowser_rebuild_item3( Toolitem *toolitem,
 	Toolkitbrowser *toolkitbrowser )
 {
-	if( !toolitem->is_pullright && toolitem->compile ) {
+	if( !toolitem->is_pullright && 
+		!toolitem->is_separator &&
+		toolitem->compile ) {
 		char txt[256];
 		BufInfo buf;
 		GtkTreeIter iter;
