@@ -59,7 +59,7 @@ static void
 toolkitgroupview_finalize( GObject *gobject )
 {
 #ifdef DEBUG
-	printf( "toolkitgroupview_finalize: 0x%x\n", (unsigned int) gobject );
+	printf( "toolkitgroupview_finalize: %p\n", gobject );
 #endif /*DEBUG*/
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
