@@ -58,7 +58,6 @@ BufInfo error_sub_buf = BUF_STATIC( error_sub_text, MAX_STRSIZE );
 /* Useful: Error message and quit. Emergencies only ... we don't tidy up
  * properly.
  */
-/*VARARGS1*/
 void
 error( const char *fmt, ... )
 {
@@ -170,7 +169,7 @@ void
 error_vips( void )
 {	
 	if( !error_level && strlen( im_errorstring() ) > 0 ) {
-		if( !buf_isempty( &error_sub_buf ) )
+		if( !buf_is_empty( &error_sub_buf ) )
 			(void) buf_appendf( &error_sub_buf, "\n" );
 		(void) buf_appendf( &error_sub_buf, "%s", im_errorstring() );
 		im_clear_error_string();
@@ -199,7 +198,7 @@ set_prop( xmlNode *xnode, const char *name, const char *fmt, ... )
 	(void) im_vsnprintf( value, MAX_STRSIZE, fmt, ap );
         va_end( ap );
 
-	if( !xmlSetProp( xnode, name, value ) ) {
+	if( !xmlSetProp( xnode, (xmlChar *) name, (xmlChar *) value ) ) {
 		error_top( _( "Unable to set XML property." ) );
 		error_sub( _( "Unable to set property \"%s\" "
 			"to value \"%s\"." ),
@@ -247,6 +246,18 @@ set_slprop( xmlNode *xnode, const char *name, GSList *labels )
 	return( TRUE );
 }
 
+/* Set a double ... use non-localisable conversion, rather than %g.
+ */
+gboolean
+set_dprop( xmlNode *xnode, const char *name, double value )
+{
+	char buf[G_ASCII_DTOSTR_BUF_SIZE];
+
+	g_ascii_dtostr( buf, sizeof( buf ), value );
+
+	return( set_sprop( xnode, name, buf ) );
+}
+
 /* Save an array of double. For name=="fred" and n doubles in array, save as
  * "fredn" == n, "fred0" == array[0], etc.
  */
@@ -262,7 +273,7 @@ set_dlprop( xmlNode *xnode, const char *name, double *values, int n )
 
 	for( i = 0; i < n; i++ ) {
 		(void) im_snprintf( buf, 256, "%s%d", name, i );
-		if( !set_prop( xnode, buf, "%g", values[i] ) )
+		if( !set_dprop( xnode, buf, values[i] ) )
 			return( FALSE );
 	}
 
@@ -272,7 +283,7 @@ set_dlprop( xmlNode *xnode, const char *name, double *values, int n )
 gboolean
 get_sprop( xmlNode *xnode, const char *name, char *buf, int sz )
 {
-	char *value = xmlGetProp( xnode, name );
+	char *value = (char *) xmlGetProp( xnode, (xmlChar *) name );
 
 	if( !value ) 
 		return( FALSE );
@@ -286,7 +297,7 @@ get_sprop( xmlNode *xnode, const char *name, char *buf, int sz )
 gboolean
 get_spropb( xmlNode *xnode, const char *name, BufInfo *buf )
 {
-	char *value = xmlGetProp( xnode, name );
+	char *value = (char *) xmlGetProp( xnode, (xmlChar *) name );
 
 	if( !value ) 
 		return( FALSE );
@@ -318,7 +329,7 @@ get_dprop( xmlNode *xnode, const char *name, double *out )
 	if( !get_sprop( xnode, name, buf, 256 ) ) 
 		return( FALSE );
 
-	*out = atof( buf );
+	*out = g_ascii_strtod( buf, NULL );
 
 	return( TRUE );
 }
@@ -349,12 +360,13 @@ get_slprop( xmlNode *xnode, const char *name, GSList **out )
 	if( !get_iprop( xnode, buf, &n ) )
 		return( FALSE );
 
-	for( i = 0; i < n; i++ ) {
+	*out = NULL;
+	for( i = n - 1; i >= 0; i-- ) {
 		(void) im_snprintf( buf, 256, "%s%d", name, i );
 		if( !get_sprop( xnode, buf, buf, 256 ) )
 			return( FALSE );
 
-		*out = g_slist_append( *out, g_strdup( buf ) );
+		*out = g_slist_prepend( *out, g_strdup( buf ) );
 	}
 
 	return( TRUE );
@@ -393,7 +405,7 @@ get_node( xmlNode *base, const char *name )
 	xmlNode *i;
 
 	for( i = base->children; i; i = i->next )
-		if( strcmp( i->name, name ) == 0 )
+		if( strcmp( (char *) i->name, name ) == 0 )
 			return( i );
 
 	return( NULL );
@@ -414,9 +426,9 @@ prettify_tree_sub( xmlNode *xnode, int indent )
 		 	FIXME ... how odd
 
 		 */
-		txt = xmlNewText( "\n" );
+		txt = xmlNewText( (xmlChar *) "\n" );
 		xmlAddPrevSibling( xnode, txt );
-		txt = xmlNewText( spc( indent ) );
+		txt = xmlNewText( (xmlChar *) spc( indent ) );
 		xmlAddPrevSibling( xnode, txt );
 
 		if( xnode->children )
@@ -428,9 +440,9 @@ prettify_tree_sub( xmlNode *xnode, int indent )
 		xnode = next;
 	}
 
-	txt = xmlNewText( spc( indent - 2 ) );
+	txt = xmlNewText( (xmlChar *) spc( indent - 2 ) );
 	xmlAddNextSibling( xnode, txt );
-	txt = xmlNewText( "\n" );
+	txt = xmlNewText( (xmlChar *) "\n" );
 	xmlAddNextSibling( xnode, txt );
 }
 
@@ -766,7 +778,7 @@ queue_add( Queue *q, void *data )
 	else {
 		assert( q->list );
 
-		(void) g_slist_append( q->tail, data );
+		q->tail = g_slist_append( q->tail, data );
 		q->tail = q->tail->next;
 	}
 
@@ -972,7 +984,7 @@ buf_change( BufInfo *buf, const char *old, const char *new )
 	/* Find pos of old.
 	 */
 	for( i = buf->i - olen; i > 0; i-- )
-		if( isprefix( old, buf->base + i ) )
+		if( is_prefix( old, buf->base + i ) )
 			break;
 	assert( i >= 0 );
 
@@ -989,18 +1001,17 @@ buf_change( BufInfo *buf, const char *old, const char *new )
 	return( TRUE );
 }
 
-/* Remove the last character.
+/* Remove the last character, if it's ch.
  */
 gboolean
 buf_removec( BufInfo *buf, char ch )
 {
 	if( buf->full )
 		return( FALSE );
-
 	if( buf->i <= 0 ) 
 		return( FALSE );
-	buf->i -= 1;
-	assert( buf->base[buf->i] == ch );
+	if( buf->base[buf->i - 1] == ch )
+		buf->i -= 1;
 
 	return( TRUE );
 }
@@ -1032,6 +1043,18 @@ buf_vappendf( BufInfo *buf, const char *fmt, va_list ap )
 	return( buf_appends( buf, str ) );
 }
 
+/* Append a double, non-localised. Useful for config files etc.
+ */
+gboolean
+buf_appendg( BufInfo *buf, double g )
+{
+	char text[G_ASCII_DTOSTR_BUF_SIZE];
+
+	g_ascii_dtostr( text, sizeof( text ), g );
+
+	return( buf_appends( buf, text ) );
+}
+
 /* Append a number ... if the number is -ve, add brackets. Needed for
  * building function arguments.
  */
@@ -1092,6 +1115,16 @@ buf_appendi( BufInfo *buf, IMAGE *im )
 		im->Bands, decode_type( im->Type ) );
 }
 
+void
+buf_appendgv( BufInfo *buf, GValue *value )
+{
+	char *str_value;
+
+	str_value = g_strdup_value_contents( value );
+	buf_appends( buf, str_value );
+	g_free( str_value );
+}
+
 /* Read all text from buffer.
  */
 const char *
@@ -1118,7 +1151,7 @@ buf_firstline( BufInfo *buf )
 /* Test for buffer empty.
  */
 gboolean
-buf_isempty( BufInfo *buf )
+buf_is_empty( BufInfo *buf )
 {
 	return( buf->i == 0 );
 }
@@ -1126,7 +1159,7 @@ buf_isempty( BufInfo *buf )
 /* Test for buffer full.
  */
 gboolean
-buf_isfull( BufInfo *buf )
+buf_is_full( BufInfo *buf )
 {
 	return( buf->full );
 }
@@ -1142,7 +1175,7 @@ buf_len( BufInfo *buf )
 /* Test for string a ends string b. 
  */
 gboolean
-ispostfix( const char *a, const char *b )
+is_postfix( const char *a, const char *b )
 {	
 	int n = strlen( a );
 	int m = strlen( b );
@@ -1156,7 +1189,7 @@ ispostfix( const char *a, const char *b )
 /* Test for string a ends string b, case independant.
  */
 gboolean
-iscasepostfix( const char *a, const char *b )
+is_casepostfix( const char *a, const char *b )
 {	
 	int n = strlen( a );
 	int m = strlen( b );
@@ -1170,7 +1203,7 @@ iscasepostfix( const char *a, const char *b )
 /* Test for string a starts string b. 
  */
 gboolean
-isprefix( const char *a, const char *b )
+is_prefix( const char *a, const char *b )
 {
 	int n = strlen( a );
 	int m = strlen( b );
@@ -1188,7 +1221,7 @@ isprefix( const char *a, const char *b )
 /* Test for string a starts string b ... case insensitive. 
  */
 gboolean
-iscaseprefix( const char *a, const char *b )
+is_caseprefix( const char *a, const char *b )
 {
 	int n = strlen( a );
 	int m = strlen( b );
@@ -1203,10 +1236,7 @@ iscaseprefix( const char *a, const char *b )
 	return( TRUE );
 }
 
-/* Like strstr(), but case-insensitive. 
- * 
- * It's awkward to test for, since sometimes you need to add extra flags to
- * enable it in headers :-( Define our own for simplicity. 
+/* Like strstr(), but case-insensitive.
  */
 char *
 my_strcasestr( const char *haystack, const char *needle )
@@ -1216,7 +1246,7 @@ my_strcasestr( const char *haystack, const char *needle )
 	int nlen = strlen( needle );
 
 	for( i = 0; i <= hlen - nlen; i++ )
-		if( iscaseprefix( needle, haystack + i ) )
+		if( is_caseprefix( needle, haystack + i ) )
 			return( (char *) (haystack + i) );
 
 	return( NULL );
@@ -1354,7 +1384,7 @@ instr( char c, const char *spn )
 }
 
 /* Doh ... not everyone has strrspn(), define one. Return a pointer to the
- * start of the trailing segment of p which contains only chars in spn.
+ * start of the trailing segment of p which contains only chars in spn. 
  */
 const char *
 my_strrspn( const char *p, const char *spn )
@@ -1363,12 +1393,9 @@ my_strrspn( const char *p, const char *spn )
 
 	for( p1 = p + strlen( p ) - 1; p1 >= p && instr( *p1, spn ); p1-- )
 		;
-
 	p1++;
-	if( *p1 == '\0' )
-		return( NULL );
-	else
-		return( p1 );
+
+	return( p1 );
 }
 
 /* Find a pointer to the start of the trailing segment of p which contains
@@ -1381,12 +1408,9 @@ my_strrcspn( const char *p, const char *spn )
 
 	for( p1 = p + strlen( p ) - 1; p1 >= p && !instr( *p1, spn ); p1-- )
 		;
-
 	p1++;
-	if( *p1 == '\0' )
-		return( NULL );
-	else
-		return( p1 );
+
+	return( p1 );
 }
 
 /* Find the rightmost occurence of string a in string b.
@@ -1585,7 +1609,9 @@ static const char *type_names[] = {
 	"LabS",			/* IM_TYPE_LABS		21 */
 	"sRGB",			/* IM_TYPE_sRGB		22 */
 	"Yxy",			/* IM_TYPE_YXY		23 */
-	"Fourier"		/* IM_TYPE_FOURIER	24 */
+	"Fourier",		/* IM_TYPE_FOURIER	24 */
+	"RGB16",		/* IM_TYPE_RGB16	25 */
+	"GREY16"		/* IM_TYPE_GREY16	26 */
 };
 static const int ntype_names = IM_NUMBER( type_names );
 
@@ -1604,61 +1630,66 @@ void
 get_image_info( BufInfo *buf, const char *name )
 {
 	char name2[FILENAME_MAX];
-	char *type;
+	struct stat st;
 
-	type = NULL;
 	expand_variables( name, name2 );
-	if( im_istiff( name2 ) )
-		type = _( "TIFF image" );
-	else if( im_isjpeg( name2 ) )
-		type = _( "JPEG image" );
-	else if( im_ispng( name2 ) )
-		type = _( "PNG image" );
-	else if( im_isppm( name2 ) )
-		type = _( "PPM/PGM/PBM image" );
-	else if( im_ismagick( name2 ) )
-		type = _( "libMagick-supported image" );
-	else if( im_isvips( name2 ) ) 
-		type = _( "VIPS image" );
+	buf_appendf( buf, "%s, ", im_skip_dir( name ) );
 
-	if( type ) {
-		IMAGE *im;
-
-		if( !(im = im_open( name2, "r" )) ) {
-			buf_appends( buf, im_errorstring() );
-			return;
-		}
-
-		buf_appendf( buf, ngettext( 
-				/* Expands to (eg) "fred.jpeg, JPEG image,
-				 * 8-bit unsigned integer, 100x100, 3 bands"
-				 */
-				"%s, %s, %s, %dx%d, %d band",
-				"%s, %s, %s, %dx%d, %d bands",
-				im->Bands ),
-			im_skip_dir( name ), type, 
-			decode_bandfmt( im->BandFmt ),
-			im->Xsize, im->Ysize, im->Bands );
-
-		im_close( im );
+	/* Read size and file/dir.
+	 */
+	if( stat( name2, &st ) == -1 ) {
+		buf_appendf( buf, "%s", g_strerror( errno ) );
+		return;
 	}
-	else {
-		struct stat st;
 
-		/* Just read size.
-		 */
-		if( stat( name2, &st ) != -1 ) {
-			buf_appendf( buf, "%s, ", im_skip_dir( name ) );
-			if( S_ISDIR( st.st_mode ) )
-				/* Used as file info, eg. "fred, directory"
-				 */
-				buf_appends( buf, _( "directory" ) );
-			else
-				to_size( buf, st.st_size );
+	if( S_ISDIR( st.st_mode ) )
+		buf_appends( buf, _( "directory" ) );
+	else if( S_ISREG( st.st_mode ) ) {
+		char *type;
+
+		type = NULL;
+		if( im_istiff( name2 ) )
+			type = _( "TIFF image" );
+		else if( im_isjpeg( name2 ) )
+			type = _( "JPEG image" );
+		else if( im_ispng( name2 ) )
+			type = _( "PNG image" );
+		else if( im_isppm( name2 ) )
+			type = _( "PPM/PGM/PBM image" );
+		else if( im_isexr( name2 ) )
+			type = _( "OpenEXR image" );
+		else if( im_isanalyze( name2 ) )
+			type = _( "Analyze image" );
+		else if( im_ismagick( name2 ) )
+			type = _( "libMagick-supported image" );
+		else if( im_isvips( name2 ) ) 
+			type = _( "VIPS image" );
+
+		if( type ) {
+			IMAGE *im;
+
+			if( !(im = im_open( name2, "r" )) ) {
+				buf_appends( buf, im_errorstring() );
+				return;
+			}
+			buf_appendi( buf, im ); 
+			im_close( im );
 		}
+		else 
+			/* No idea wtf this is, just put size in.
+			 */
+			to_size( buf, st.st_size );
 	}
 }
 
+/* A char that can be part of an environment variable name? A-Za-z0-9_
+ */
+static gboolean
+isvariable( int ch )
+{
+	return( isalnum( ch ) || ch == '_' );
+}
+
 /* Expand environment variables from in to out. Return true if we performed an
  * expansion, false for no variables there.
  */
@@ -1683,7 +1714,7 @@ expand_once( char *in, char *out )
 			 */
 			p++;
 			for( r = vname; 
-				isalnum( (int)(*r = *p) ) && 
+				isvariable( (int)(*r = *p) ) && 
 					(r - vname) < FILENAME_MAX;
 				p++, r++ )
 				;
@@ -1770,15 +1801,26 @@ swap_chars( char *buf, char from, char to )
 			buf[i] = to;
 }
 
-/* If we use '/' seps, swap all '\' for '/' ... likewise vice versa.
+/* If we use '/' seps, swap all '\' for '/' ... likewise vice versa. Only in
+ * the filename part, though. We don't want to junk '\,', for example.
  */
 void
 nativeize_path( char *buf )
 {
-	if( IM_DIR_SEP == '/' )
-		swap_chars( buf, '\\', '/' );
+	char filename[FILENAME_MAX];
+	char mode[FILENAME_MAX];
+
+	im_filename_split( buf, filename, mode );
+
+	if( G_DIR_SEPARATOR == '/' )
+		swap_chars( filename, '\\', '/' );
 	else
-		swap_chars( buf, '/', '\\' );
+		swap_chars( filename, '/', '\\' );
+
+	if( strcmp( mode, "" ) != 0 )
+		im_snprintf( buf, FILENAME_MAX, "%s:%s", filename, mode );
+	else
+		im_snprintf( buf, FILENAME_MAX, "%s", filename );
 }
 
 /* Change all occurences of "from" into "to". This will loop if "to" contains
@@ -1812,12 +1854,12 @@ canonicalize_path( char *path )
 	/* Any "/./" can go.
 	 */
 	swap_string( path, 
-		IM_DIR_SEP_STR "." IM_DIR_SEP_STR, IM_DIR_SEP_STR );
+		G_DIR_SEPARATOR_S "." G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S );
 
 	/* Any "//" can go.
 	 */
 	swap_string( path, 
-		IM_DIR_SEP_STR IM_DIR_SEP_STR, IM_DIR_SEP_STR );
+		G_DIR_SEPARATOR_S G_DIR_SEPARATOR_S, G_DIR_SEPARATOR_S );
 
 	/* Repeatedly search for "/[^/]+/../" and remove it.
 
@@ -1830,12 +1872,12 @@ canonicalize_path( char *path )
 
 		found = FALSE;
 
-		for( p = path; (p = strchr( p, IM_DIR_SEP )); p++ ) {
+		for( p = path; (p = strchr( p, G_DIR_SEPARATOR )); p++ ) {
 			char *q;
 
-			q = strchr( p + 1, IM_DIR_SEP );
+			q = strchr( p + 1, G_DIR_SEPARATOR );
 
-			if( q && isprefix( IM_DIR_SEP_STR "..", q ) ) {
+			if( q && is_prefix( G_DIR_SEPARATOR_S "..", q ) ) {
 				memmove( p, q + 3, strlen( q + 3 ) + 1 );
 				found = TRUE;
 				break;
@@ -1846,7 +1888,7 @@ canonicalize_path( char *path )
 	/* We may have the empty string ... that's just '/'.
 	 */
 	if( strcmp( path, "" ) == 0 )
-		strcpy( path, IM_DIR_SEP_STR );
+		strcpy( path, G_DIR_SEPARATOR_S );
 }
 
 /* Absoluteize a path. Must be FILENAME_MAX chars available.
@@ -1861,7 +1903,7 @@ absoluteize_path( char *path )
 		im_strncpy( buf, path, FILENAME_MAX );
 		cwd = g_get_current_dir();
 		im_snprintf( path, FILENAME_MAX,
-			"%s%s%s", cwd, IM_DIR_SEP_STR, buf );
+			"%s%s%s", cwd, G_DIR_SEPARATOR_S, buf );
 		g_free( cwd );
 		canonicalize_path( path );
 	}
@@ -1893,11 +1935,11 @@ get_vipshome( const char *argv0 )
 	return( prefix_buffer );
 }
 
-/* Call a function, building a string arg. We expand env. variables, but
- * that's all.
+/* Call a void*-valued function, building a string arg. We expand env. 
+ * variables, but that's all.
  */
 void *
-call_string( call_string_fn fn, const char *arg, void *a, void *b, void *c )
+callv_string( callv_string_fn fn, const char *arg, void *a, void *b, void *c )
 {
 	char buf[FILENAME_MAX];
 
@@ -1907,24 +1949,24 @@ call_string( call_string_fn fn, const char *arg, void *a, void *b, void *c )
 }
 
 void *
-call_stringva( call_string_fn fn, 
+callv_stringva( callv_string_fn fn, 
 	const char *fmt, va_list ap, void *a, void *b, void *c )
 {
 	char buf[FILENAME_MAX];
 
         (void) im_vsnprintf( buf, FILENAME_MAX, fmt, ap );
 
-	return( call_string( fn, buf, a, b, c ) );
+	return( callv_string( fn, buf, a, b, c ) );
 }
 
 void *
-call_stringf( call_string_fn fn, const char *fmt, ... )
+callv_stringf( callv_string_fn fn, const char *fmt, ... )
 {
 	va_list ap;
 	void *res;
 
         va_start( ap, fmt );
-        res = call_stringva( fn, fmt, ap, NULL, NULL, NULL );
+        res = callv_stringva( fn, fmt, ap, NULL, NULL, NULL );
         va_end( ap );
 
 	return( res );
@@ -1933,7 +1975,7 @@ call_stringf( call_string_fn fn, const char *fmt, ... )
 /* Call a function, building a filename arg. Nativize and absoluteize too.
  */
 void *
-call_string_filename( call_string_fn fn, 
+callv_string_filename( callv_string_fn fn, 
 	const char *filename, void *a, void *b, void *c )
 {
 	char buf[FILENAME_MAX];
@@ -1946,24 +1988,100 @@ call_string_filename( call_string_fn fn,
 }
 
 void *
-call_string_filenameva( call_string_fn fn, 
+callv_string_filenameva( callv_string_fn fn, 
 	const char *fmt, va_list ap, void *a, void *b, void *c )
 {
 	char buf[FILENAME_MAX];
 
         (void) im_vsnprintf( buf, FILENAME_MAX, fmt, ap );
 
-	return( call_string_filename( fn, buf, a, b, c ) );
+	return( callv_string_filename( fn, buf, a, b, c ) );
 }
 
 void *
-call_string_filenamef( call_string_fn fn, const char *fmt, ... )
+callv_string_filenamef( callv_string_fn fn, const char *fmt, ... )
 {
 	va_list ap;
 	void *res;
 
         va_start( ap, fmt );
-        res = call_string_filenameva( fn, fmt, ap, NULL, NULL, NULL );
+        res = callv_string_filenameva( fn, fmt, ap, NULL, NULL, NULL );
+        va_end( ap );
+
+	return( res );
+}
+
+/* Call an int-valued function, building a string arg. We expand env. 
+ * variables, but that's all.
+ */
+int
+calli_string( calli_string_fn fn, const char *arg, void *a, void *b, void *c )
+{
+	char buf[FILENAME_MAX];
+
+	expand_variables( arg, buf );
+
+	return( fn( buf, a, b, c ) );
+}
+
+int
+calli_stringva( calli_string_fn fn, 
+	const char *fmt, va_list ap, void *a, void *b, void *c )
+{
+	char buf[FILENAME_MAX];
+
+        (void) im_vsnprintf( buf, FILENAME_MAX, fmt, ap );
+
+	return( calli_string( fn, buf, a, b, c ) );
+}
+
+int
+calli_stringf( calli_string_fn fn, const char *fmt, ... )
+{
+	va_list ap;
+	int res;
+
+        va_start( ap, fmt );
+        res = calli_stringva( fn, fmt, ap, NULL, NULL, NULL );
+        va_end( ap );
+
+	return( res );
+}
+
+/* Call a function, building a filename arg. Nativize and absoluteize too.
+ */
+int
+calli_string_filename( calli_string_fn fn, 
+	const char *filename, void *a, void *b, void *c )
+{
+	char buf[FILENAME_MAX];
+
+	expand_variables( filename, buf );
+        nativeize_path( buf );
+	absoluteize_path( buf );
+
+	return( fn( buf, a, b, c ) );
+}
+
+int
+calli_string_filenameva( calli_string_fn fn, 
+	const char *fmt, va_list ap, void *a, void *b, void *c )
+{
+	char buf[FILENAME_MAX];
+
+        (void) im_vsnprintf( buf, FILENAME_MAX, fmt, ap );
+
+	return( calli_string_filename( fn, buf, a, b, c ) );
+}
+
+int
+calli_string_filenamef( calli_string_fn fn, const char *fmt, ... )
+{
+	va_list ap;
+	int res;
+
+        va_start( ap, fmt );
+        res = calli_string_filenameva( fn, fmt, ap, NULL, NULL, NULL );
         va_end( ap );
 
 	return( res );
@@ -1995,6 +2113,14 @@ setenvf( const char *name, const char *fmt, ... )
 	g_setenv( name, buf, TRUE );
 }
 
+int
+check( const char *filename )
+{
+	/* Need to work on filenames containing %.
+	 */
+	return( im_existsf( "%s", filename ) );
+}
+
 /* File exists? 
  */
 gboolean
@@ -2004,9 +2130,8 @@ existsf( const char *name, ... )
 	gboolean res;
 
         va_start( ap, name );
-        res = GPOINTER_TO_INT( call_string_filenameva( 
-		(call_string_fn) im_existsf, 
-			name, ap, NULL, NULL, NULL ) );
+        res = calli_string_filenameva( 
+		(calli_string_fn) check, name, ap, NULL, NULL, NULL );
         va_end( ap );
 
 	return( res );
@@ -2019,10 +2144,9 @@ mkdirf( const char *name, ... )
 	gboolean res;
 
         va_start( ap, name );
-        res = GPOINTER_TO_INT( call_string_filenameva( 
-		(call_string_fn) mkdir, 
-			name, ap, GINT_TO_POINTER( 0755 ), NULL, NULL ) == 
-			NULL );
+        res = calli_string_filenameva( 
+		(calli_string_fn) mkdir, 
+			name, ap, GINT_TO_POINTER( 0755 ), NULL, NULL ) == 0;
         va_end( ap );
 
 	return( res );
@@ -2037,8 +2161,8 @@ systemf( const char *fmt, ... )
 	int res;
 
         va_start( ap, fmt );
-        res = GPOINTER_TO_INT( call_stringva( 
-		(call_string_fn) system, fmt, ap, NULL, NULL, NULL ) );
+        res = calli_stringva( 
+		(calli_string_fn) system, fmt, ap, NULL, NULL, NULL );
         va_end( ap );
 
 	return( res );
@@ -2051,10 +2175,10 @@ touchf( const char *fmt, ... )
 	int fd;
 
         va_start( ap, fmt );
-        fd = GPOINTER_TO_INT( call_string_filenameva( 
-		(call_string_fn) creat, 
+        fd = calli_string_filenameva( 
+		(calli_string_fn) creat, 
 			fmt, ap, GINT_TO_POINTER( S_IRUSR | S_IWUSR ), 
-				NULL, NULL ) );
+				NULL, NULL );
         va_end( ap );
 	(void) close( fd );
 
@@ -2068,9 +2192,8 @@ unlinkf( const char *fmt, ... )
 	int res;
 
         va_start( ap, fmt );
-        res = GPOINTER_TO_INT( call_string_filenameva( 
-		(call_string_fn) unlink, 
-			fmt, ap, NULL, NULL, NULL ) );
+        res = calli_string_filenameva( 
+		(calli_string_fn) unlink, fmt, ap, NULL, NULL, NULL );
         va_end( ap );
 
 	return( res );
@@ -2144,7 +2267,7 @@ file_build( const char *fname )
 	of->fname_real = NULL;
 	of->last_errno = 0;
 
-	SETSTR( of->fname, fname );
+	IM_SETSTR( of->fname, fname );
 	if( !of->fname ) {
 		file_close( of );
 		return( NULL );
@@ -2177,7 +2300,7 @@ file_open_read( const char *name, ... )
 		return( NULL );
 	}
 
-	if( !(of->fp = (FILE *) call_string_filename( (call_string_fn) fopen, 
+	if( !(of->fp = (FILE *) callv_string_filename( (callv_string_fn) fopen, 
 		of->fname_real, "r", NULL, NULL )) ) {
 		error_top( _( "Unable to open." ) );
 		error_sub( _( "Unable to open file \"%s\" for reading.\n%s." ),
@@ -2200,7 +2323,7 @@ file_open_read_stdin()
 
 	if( !(of = file_build( "stdin" )) )
 		return( NULL );
-	SETSTR( of->fname_real, of->fname );
+	IM_SETSTR( of->fname_real, of->fname );
 	if( !of->fname_real ) {
 		file_close( of );
 		return( NULL );
@@ -2227,12 +2350,12 @@ file_open_write( const char *name, ... )
 
 	if( !of )
 		return( NULL );
-	SETSTR( of->fname_real, of->fname );
+	IM_SETSTR( of->fname_real, of->fname );
 	if( !of->fname_real ) {
 		file_close( of );
 		return( NULL );
 	}
-	if( !(of->fp = (FILE *) call_string_filename( (call_string_fn) fopen,
+	if( !(of->fp = (FILE *) callv_string_filename( (callv_string_fn) fopen,
 		of->fname_real, "w", NULL, NULL )) ) {
 		error_top( _( "Unable to open." ) );
 		error_sub( _( "Unable to open file \"%s\" for writing.\n%s." ),
@@ -2316,7 +2439,7 @@ file_read( iOpenFile *of )
 }
 
 /* Load a file into a buffer. Useful for OpenFiles we can't seek in, like
- * stdin.
+ * stdin. 
  */
 char *
 file_read_buffer( iOpenFile *of, char *buffer, size_t max )
@@ -2327,7 +2450,7 @@ file_read_buffer( iOpenFile *of, char *buffer, size_t max )
 	 */
 	len = fread( buffer, sizeof( char ), max - 1, of->fp );
 	if( !feof( of->fp ) ) {
-		/* Too large for buffer.
+		/* File too large for buffer.
 		 */
 		of->last_errno = errno;
 		error_top( _( "Unable to read." ) );
@@ -2340,6 +2463,23 @@ file_read_buffer( iOpenFile *of, char *buffer, size_t max )
 	return( buffer );
 }
 
+/* Return '\0' for EOF, -1 for error.
+ */
+int
+file_getc( iOpenFile *of )
+{
+	int ch;
+
+	ch = fgetc( of->fp );
+
+	if( ch == EOF && feof( of->fp ) )
+		return( 0 );
+	else if( ch == EOF )
+		return( -1 );
+	else 
+		return( ch );
+}
+
 off_t
 statf( const char *fmt, ... )
 {
@@ -2348,9 +2488,8 @@ statf( const char *fmt, ... )
 	int result;
 
         va_start( ap, fmt );
-        result = GPOINTER_TO_INT( call_string_filenameva( 
-		(call_string_fn) stat, 
-			fmt, ap, &st, NULL, NULL ) );
+        result = calli_string_filenameva( 
+		(calli_string_fn) stat, fmt, ap, &st, NULL, NULL );
         va_end( ap );
 
 	if( result == -1 || S_ISDIR( st.st_mode ) )
@@ -2431,6 +2570,27 @@ escape_markup( const char *in, char *out, int len )
 	return( out );
 }
 
+/* VIPS filenames can have embedded modes. Mode strings are punctuated with
+ * ',' and ':' chars. So strings in modes must have these chars escaped.
+ */
+char *
+escape_mode( const char *in, char *out, int len )
+{
+	const char *p;
+	char *q;
+
+	for( p = in, q = out; *p && q - out < len - 5; p++, q++ ) {
+		if( *p == ':' || *p == ',' )
+			*q++ = '\\';
+
+		*q = *p;
+	}
+
+	*q = '\0';
+
+	return( out );
+}
+
 /* Return a string of n characters. Buffer is zapped each time!
  */
 const char *
@@ -2519,7 +2679,7 @@ find_space( const char *name )
 	struct statvfs st;
 	double sz;
 
-	if( call_string_filename( (call_string_fn) statvfs, 
+	if( calli_string_filename( (calli_string_fn) statvfs, 
 		(gpointer) name, &st, NULL, NULL ) )
 		/* Set to error value. 
 		 */
@@ -2536,7 +2696,7 @@ find_space( const char *name )
 	struct statfs st;
 	double sz;
 
-	if( call_string_filename( (call_string_fn) statfs, 
+	if( calli_string_filename( (calli_string_fn) statfs, 
 		(gpointer) name, &st, NULL, NULL ) )
 		sz = -1;
 	else
@@ -2544,7 +2704,7 @@ find_space( const char *name )
 
 	return( sz );
 }
-#elif defined HAVE_WINDOWS_H
+#elif defined OS_WIN32
 double
 find_space( const char *name )
 {
@@ -2554,14 +2714,17 @@ find_space( const char *name )
 	double sz;
 	char name2[FILENAME_MAX];
 
-       expand_variables( name, name2 );
-       if( name2[1] == ':' )
-               name2[3] = 0;
+	expand_variables( name, name2 );
+
+	/* Truncate to just the drive letter.
+	 */
+	if( name2[1] == ':' )
+		name2[3] = 0;
 
 	if( !GetDiskFreeSpaceEx( name2, &avail, &total, &free ) )
 		sz = -1;
 	else
-		sz = IM_MAX( 0, (double) (__int64) avail.QuadPart );
+		sz = IM_MAX( 0, (double) free.QuadPart );
 
 	return( sz );
 }
@@ -2632,9 +2795,9 @@ temp_name( char *name, const char *type )
 
 	dir = PATH_TMP;
 	if( !existsf( "%s", dir ) )
-		dir = IM_DIR_SEP_STR;
+		dir = G_DIR_SEPARATOR_S;
 
-	im_snprintf( name, FILENAME_MAX, "%s" IM_DIR_SEP_STR 
+	im_snprintf( name, FILENAME_MAX, "%s" G_DIR_SEPARATOR_S 
 		"untitled-" PACKAGE "-%d-XXXXXXX", 
 		dir, n++ );
 	expand_variables( name, buf );
@@ -2716,14 +2879,15 @@ increment_name( char *buf )
 	int n;
 	char fmt[256];
 
-	strcpy( fmt, "%d" );
-
-        if( (p = (char *) my_strrspn( buf, NUMERIC )) ) {
+	/* If there's no number, p will point at the '\0'.
+	 */
+        p = (char *) my_strrspn( buf, NUMERIC );
+	if( *p ) {
                 n = atoi( p );
-		im_snprintf( fmt, 256, "%%0%dd", strlen( p ) );
+		im_snprintf( fmt, 256, "%%0%dd", (int) strlen( p ) );
 	}
 	else {
-		p = buf + strlen( buf );
+		strcpy( fmt, "%d" );
 		n = 0;
 	}
 
@@ -2754,12 +2918,19 @@ increment_filename( char *filename )
 	/* Also save any chars to the right of the number component (if any) of
 	 * the filename.
 	 */
-	if( !(p = (char *) my_strrcspn( file, NUMERIC )) )
+	p = (char *) my_strrcspn( file, NUMERIC );
+
+	/* No numbers there? Take nothing as the tail and put the number at
+	 * the end.
+	 */
+	if( p == file ) 
 		p = file + strlen( file );
+
 	im_strncpy( tail, p, FILENAME_MAX );
 	*p = '\0';
 
 	increment_name( buf );
+
 	strcpy( filename, buf );
 	strcat( filename, tail );
 	strcat( filename, suf );
@@ -2790,28 +2961,32 @@ extract_first_line( char *buf, char *str, int len )
         return( n );
 }
 
-/* Make a valid ip name from a filename. Skip leading stuff, from trailing
- * stuff, filter remaining chars for allowed in names set.
+/* Make a valid ip name from a filename. 
  */
 void
 name_from_filename( const char *in, char *out )
 {
-	const char *p, *q;
-	int len;
+	const char *p;
 
-	/* Make up a name ... skip leading path prefix, and any non-alpha.
+	/* Skip leading path prefix, and any non-alpha.
+	 * Don't use isalnum(), since we don't want leading digits.
 	 */
 	p = im_skip_dir( in ); 
-	for( q = p; *q && (isalnum( *q ) || *q == '\'' || *q == '_'); q++ )
-		;
+	while( *p && !(isalpha( *p ) || *p == '_') )
+		p += 1;
 
-	len = IM_MIN( FILENAME_MAX - 1, q - p );
-
-	if( len == 0 )
+	if( !*p )
 		strcpy( out, "untitled" );
 	else {
-		im_strncpy( out, p, len + 1 );
-		out[len] = '\0';
+		char *q;
+
+		/* Filter non-identifier chars. Stop at the first '.' 
+		 * character, so we don't get the suffix in there too.
+		 */
+		for( q = out; *p && *p != '.'; p++ )
+			if( isalnum( *p ) || *p == '_' || *p == '\'' )
+				*q++ = *p;
+		*q = '\0';
 	}
 }
 
@@ -2898,7 +3073,7 @@ recent_load( const char *filename )
 
 	recent = NULL;
 
-	if( (of = file_open_read( "%s" IM_DIR_SEP_STR "%s", 
+	if( (of = file_open_read( "%s" G_DIR_SEPARATOR_S "%s", 
 		get_savedir(), filename )) ) {
 		char buf[256];
 
@@ -2962,7 +3137,7 @@ recent_save( GSList *recent, const char *filename )
 	slist_map_rev( recent, 
 		(SListMapFn) recent_save_sub, &old_recent );
 
-	if( (of = file_open_write( "%s" IM_DIR_SEP_STR "%s", 
+	if( (of = file_open_write( "%s" G_DIR_SEPARATOR_S "%s", 
 		get_savedir(), filename )) ) {
 		slist_map_rev( old_recent,
 			(SListMapFn) recent_save_sub2, of );
@@ -3012,21 +3187,22 @@ set_symbol_drag_type( GtkWidget *widget )
 const char *
 get_savedir( void )
 {
-#ifdef HAVE_WINDOWS_H
+#ifdef OS_WIN32
 	/* If APPDATA is not defined, default to HOME, we know that will
 	 * exist (since we make it if necessary in main()).
 	 */
 	if( g_getenv( "APPDATA" ) && existsf( "%s", g_getenv( "APPDATA" ) ) )
-		return( "$APPDATA" IM_DIR_SEP_STR IP_NAME );
+		return( "$APPDATA" G_DIR_SEPARATOR_S IP_NAME );
 	else
-		return( "$HOME" IM_DIR_SEP_STR "." IP_NAME );
-#else /*!HAVE_WINDOWS_H*/
-	/* *nix-style system .. .dot file in home area.
-
-		FIXME ... do something better for Mac
-
+		return( "$HOME" G_DIR_SEPARATOR_S "." IP_NAME );
+#elif OS_DARWIN
+	/* Darwin ... in ~/Library
 	 */
-	return( "$HOME" IM_DIR_SEP_STR "." IP_NAME );
-#endif /*HAVE_WINDOWS_H*/
+	return( "$HOME" G_DIR_SEPARATOR_S "Library" G_DIR_SEPARATOR_S IP_NAME );
+#else 
+	/* *nix-style system .. .dot file in home area.
+	 */
+	return( "$HOME" G_DIR_SEPARATOR_S "." IP_NAME );
+#endif /*OS_WIN32*/
 }
 
