@@ -30,11 +30,22 @@
 #include "ip.h"
 
 /*
-#define DEBUG_HISTORY_MISS
+#define DEBUG_TIME
 #define DEBUG
+#define DEBUG_HISTORY_MISS
 #define DEBUG_HISTORY
  */
 
+/* This is usually turned on from a -D in cflags.
+#define DEBUG_LEAK
+ */
+
+/* Often want it off ... we get spurious complaints about leaks if an
+ * operation has no images in or out (eg. im_version) becausew it'll never
+ * get GCed.
+ */
+#undef DEBUG_LEAK
+
 /* Maxiumum number of args to a VIPS function.
  */
 #define MAX_VIPS_ARGS (100)
@@ -53,6 +64,7 @@ typedef struct _VipsInfo {
 	im_object *vargv;		/* vargv we build for VIPS */
 	int nargs;			/* Number of args needed from ip */
 	int nres;			/* Number of objects we write back */
+	int nires;			/* Number of images we write back */
 	int inpos[MAX_VIPS_ARGS];	/* Positions of inputs */
 	int outpos[MAX_VIPS_ARGS];	/* Positions of outputs */
 
@@ -82,7 +94,7 @@ typedef struct _VipsInfo {
 	gboolean in_cache;
 } VipsInfo;
 
-/* VIPS argument types we support. Keep order in sync with VipsArgument.
+/* VIPS argument types we support. Keep order in sync with VipsArgumentType.
  */
 static im_arg_type vips_supported[] = {
 	IM_TYPE_DOUBLE,		
@@ -93,7 +105,9 @@ static im_arg_type vips_supported[] = {
 	IM_TYPE_DOUBLEVEC,
 	IM_TYPE_DMASK,
 	IM_TYPE_IMASK,
-	IM_TYPE_IMAGEVEC
+	IM_TYPE_IMAGEVEC,
+	IM_TYPE_INTVEC,
+	IM_TYPE_GVALUE
 };
 
 typedef enum _VipsArgumentType {
@@ -106,25 +120,41 @@ typedef enum _VipsArgumentType {
 	VIPS_DOUBLEVEC,
 	VIPS_DMASK,
 	VIPS_IMASK,
-	VIPS_IMAGEVEC
+	VIPS_IMAGEVEC,
+	VIPS_INTVEC,
+	VIPS_GVALUE
 } VipsArgumentType;
 
 /* The previous function calls we are caching, plus an LRU queue for flushing.
  */
 static GHashTable *vips_history_table = NULL;
 static Queue *vips_history_lru = NULL;
+int vips_history_size = 0;
 
-/* All the VipsInfo we make ... for leak testing. Build this file with DEBUG
- * to enable add/remove. to this list.
+#ifdef DEBUG_LEAK
+/* All the VipsInfo we make ... for leak testing. Build this file with 
+ * DEBUG_LEAK to enable add/remove. to this list.
  */
 static GSList *vips_info_all = NULL;
+#endif /*DEBUG_LEAK*/
 
 void
 vips_check_all_destroyed( void )
 {
-	if( vips_info_all != NULL )
+#ifdef DEBUG_LEAK
+	if( vips_info_all != NULL ) {
+		GSList *p;
+
 		printf( "** %d VipsInfo leaked!\n", 
 			g_slist_length( vips_info_all ) );
+
+		for( p = vips_info_all; p; p = p->next ) {
+			VipsInfo *vi = (VipsInfo *) p->data;
+
+			printf( "\t%s\n", vi->name );
+		}
+	}
+#endif /*DEBUG_LEAK*/
 }
 
 /* Error early on .. we can't print args yet.
@@ -217,6 +247,18 @@ vips_hash( VipsInfo *vi )
 				break;
 			}
 
+			case VIPS_INTVEC:
+			{
+				im_intvec_object *v = 
+					(im_intvec_object *) vi->vargv[i];
+				int j;
+
+				for( j = 0; j < v->n; j++ )
+					HASH_I( v->vec[j] );
+
+				break;
+			}
+
 			case VIPS_DMASK:
 			{
 				im_mask_object *mo = vi->vargv[i];
@@ -259,6 +301,10 @@ vips_hash( VipsInfo *vi )
 				break;
 			}
 
+			case VIPS_GVALUE:
+				HASH_P( vi->vargv[i] );
+				break;
+
 			default:
 			case VIPS_NONE:
 				break;
@@ -339,6 +385,21 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 				break;
 			}
 
+			case VIPS_INTVEC:
+			{
+				im_intvec_object *v1 = 
+					(im_intvec_object *) vi1->vargv[i];
+				im_intvec_object *v2 = 
+					(im_intvec_object *) vi2->vargv[i];
+				int j;
+
+				for( j = 0; j < v1->n; j++ )
+					if( v1->vec[j] != v2->vec[j] )
+						return( FALSE );
+
+				break;
+			}
+
 			case VIPS_DMASK:
 			{
 				im_mask_object *mo1 = 
@@ -408,6 +469,15 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 				break;
 			}
 
+			/* Very strict. Could be more generous here: we'd need
+			 * to have a pspec for each argument type and then use 
+			 * g_param_values_cmp() to test equality.
+			 */
+			case VIPS_GVALUE:
+				if( vi1->vargv[i] != vi2->vargv[i] )
+					return( FALSE );
+				break;
+
 			default:
 			case VIPS_NONE:
 				break;
@@ -521,6 +591,7 @@ vips_history_insert( VipsInfo *vi )
 	assert( !vi->in_cache );
 
 	g_hash_table_insert( vips_history_table, vi, vi );
+	vips_history_size += 1;
 
 	assert( g_hash_table_lookup( vips_history_table, vi ) );
 
@@ -533,23 +604,6 @@ vips_history_insert( VipsInfo *vi )
 #endif /*DEBUG_HISTORY*/
 }
 
-static VipsInfo *
-vips_history_remove_lru( void )
-{
-	VipsInfo *vi;
-
-	vi = (VipsInfo *) queue_head( vips_history_lru );
-	g_hash_table_remove( vips_history_table, vi );
-	vi->in_cache = FALSE;
-
-#ifdef DEBUG_HISTORY
-	vips_history_sanity();
-	printf( "vips_history_remove_lru: flushing \"%s\"\n", vi->name );
-#endif /*DEBUG_HISTORY*/
-
-	return( vi );
-}
-
 /* Are we in the history? Remove us.
  */
 static void
@@ -558,6 +612,7 @@ vips_history_remove( VipsInfo *vi )
 	if( vi->in_cache ) {
 		queue_remove( vips_history_lru, vi );
 		g_hash_table_remove( vips_history_table, vi );
+		vips_history_size -= 1;
 		vi->in_cache = FALSE;
 
 #ifdef DEBUG_HISTORY
@@ -582,9 +637,9 @@ vips_destroy( VipsInfo *vi )
 	/* Are we in the history? Remove us.
 	 */
 	vips_history_remove( vi ); 
-#ifdef DEBUG
+#ifdef DEBUG_LEAK
 	vips_info_all = g_slist_remove( vips_info_all, vi );
-#endif /*DEBUG*/
+#endif /*DEBUG_LEAK*/
 
 	/* Break "destroy" links to iis.
 	 */
@@ -618,6 +673,7 @@ vips_destroy( VipsInfo *vi )
 		case VIPS_INT: 	
 		case VIPS_COMPLEX: 
 		case VIPS_IMAGEVEC: 	/* Input only, so no freeing reqd */
+		case VIPS_GVALUE:
 			/* Do nothing.
 			 */
 			break;
@@ -641,6 +697,10 @@ vips_destroy( VipsInfo *vi )
 			IM_FREE( ((im_doublevec_object *) obj)->vec );
 			break;
 
+		case VIPS_INTVEC:
+			IM_FREE( ((im_intvec_object *) obj)->vec );
+			break;
+
 		case VIPS_DMASK:
 			IM_FREE( ((im_mask_object *) obj)->name );
 			IM_FREEF( im_free_dmask, 
@@ -665,6 +725,24 @@ vips_destroy( VipsInfo *vi )
 	IM_FREE( vi );
 }
 
+static void
+vips_history_remove_lru( void )
+{
+	VipsInfo *vi;
+
+	vi = (VipsInfo *) queue_head( vips_history_lru );
+
+#ifdef DEBUG_HISTORY
+	printf( "vips_history_remove_lru: flushing \"%s\"\n", vi->name );
+#endif /*DEBUG_HISTORY*/
+
+	vips_destroy( vi );
+
+#ifdef DEBUG_HISTORY
+	vips_history_sanity();
+#endif /*DEBUG_HISTORY*/
+}
+
 static void
 vips_history_destroy_cb( Imageinfo *ii, VipsInfo *vi )
 {
@@ -730,7 +808,7 @@ vips_history_add( VipsInfo *vi )
 	/* History too big? Flush!
 	 */
 	if( queue_length( vips_history_lru ) > VIPS_HISTORY_MAX ) 
-		vips_destroy( vips_history_remove_lru() );
+		vips_history_remove_lru();
 }
 
 /* VIPS types -> a buffer. For tracing calls.
@@ -776,10 +854,25 @@ vips_tochar( VipsInfo *vi, int i, BufInfo *buf )
 		buf_appendf( buf, "doublevec" );
 		break;
 
+	case VIPS_INTVEC:
+		buf_appendf( buf, "intvec" );
+		break;
+
 	case VIPS_IMAGEVEC:
 		buf_appendf( buf, "imagevec" );
 		break;
 
+	case VIPS_GVALUE:
+	{
+		GValue *value = (GValue *) obj;
+
+		buf_appends( buf, "(gvalue" );
+		buf_appendgv( buf, value );
+		buf_appendf( buf, ")" );
+
+		break;
+	}
+
 	default:
 		assert( FALSE );
 	}
@@ -975,14 +1068,15 @@ vips_new( Reduce *rc, im_function *fn )
 	vi->vargv = NULL;
 	vi->nargs = 0;
 	vi->nres = 0;
+	vi->nires = 0;
 	vi->ninii = 0;
 	vi->noutii = 0;
 	vi->use_lut = FALSE;		/* Set this properly later */
 	vi->found_hash = FALSE;
 	vi->in_cache = FALSE;
-#ifdef DEBUG
+#ifdef DEBUG_LEAK
 	vips_info_all = g_slist_prepend( vips_info_all, vi );
-#endif /*DEBUG*/
+#endif /*DEBUG_LEAK*/
 
 	/* Look over the args ... count the number of inputs we need, and 
 	 * the number of outputs we generate. Note the position of each.
@@ -994,13 +1088,18 @@ vips_new( Reduce *rc, im_function *fn )
 			/* Found an output object.
 			 */
 			vi->outpos[vi->nres] = i;
-			vi->nres++; 
+			vi->nres += 1; 
+
+			/* Image output.
+			 */
+			if( strcmp( ty->type, IM_TYPE_IMAGE ) == 0 ) 
+				vi->nires += 1;
 		}
 		else if( strcmp( ty->type, IM_TYPE_DISPLAY ) != 0 ) {
 			/* Found an input object.
 			 */
 			vi->inpos[vi->nargs] = i;
-			vi->nargs++; 
+			vi->nargs += 1; 
 		}
 	}
 
@@ -1033,6 +1132,7 @@ vips_is_callable( im_function *fn )
 {
 	int i;
 	int nout;
+	int nin;
 
 	if( fn->argc >= MAX_VIPS_ARGS )
 		return( FALSE );
@@ -1055,12 +1155,21 @@ vips_is_callable( im_function *fn )
 
         /* Must be at least one output argument.
          */
-        for( nout = 0, i = 0; i < fn->argc; i++ )
+        nin = nout = 0;
+        for( i = 0; i < fn->argc; i++ )
                 if( fn->argv[i].desc->flags & IM_TYPE_OUTPUT )
                         nout++;
+		else
+                        nin++;
         if( nout == 0 ) 
                 return( FALSE );
 
+	/* Need at least 1 input argument: we reply on having an application
+	 * node to overwrite with (I result).
+	 */
+        if( nin == 0 ) 
+                return( FALSE );
+
 	return( TRUE );
 }
 
@@ -1108,6 +1217,23 @@ vips_make_doublevec( im_doublevec_object *dv, int n, double *vec )
 	return( 0 );
 }
 
+/* Make an im_intvec_object. Make from a vec of doubles, because that's what
+ * we get from nip.
+ */
+static int
+vips_make_intvec( im_intvec_object *dv, int n, double *vec )
+{
+	int i;
+
+	dv->n = n;
+	if( !(dv->vec = IARRAY( NULL, n, int )) )
+		return( -1 );
+	for( i = 0; i < n; i++ )
+		dv->vec[i] = vec[i];
+
+	return( 0 );
+}
+
 /* Make an im_imagevec_object.
  */
 static int
@@ -1203,7 +1329,9 @@ vips_fromip( Reduce *rc, PElement *arg,
 		if( !PEISIMAGE( arg ) )
 			return( FALSE );
 
-		*im = PEGETII( arg );
+		/* Do an IMAGEINFO() as well as GETII to validate the pointer.
+		 */
+		*im = IMAGEINFO( PEGETII( arg ) );
 		break;
 	}
 
@@ -1219,6 +1347,18 @@ vips_fromip( Reduce *rc, PElement *arg,
 		break;
 	}
 
+	case VIPS_INTVEC:
+	{
+		double buf[100];
+		int n;
+
+		n = reduce_get_realvec( rc, arg, buf, 100 );
+		if( vips_make_intvec( *obj, n, buf ) )
+			return( FALSE );
+
+		break;
+	}
+
 	case VIPS_IMAGEVEC:
 	{
 		Imageinfo *buf[100];
@@ -1259,14 +1399,21 @@ vips_fromip( Reduce *rc, PElement *arg,
 		break;
 	}
 
+	case VIPS_GVALUE:
+	{
+		GValue *value = *obj;
+
+		memset( value, 0, sizeof( GValue ) );
+		if( !heap_ip_to_gvalue( arg, value ) )
+			return( FALSE );
+
+		break;
+	}
+
 	default:
 		assert( FALSE );
 	}
 
-#ifdef DEBUG
-	pgraph( arg );
-#endif /*DEBUG*/
-
 	return( TRUE );
 }
 
@@ -1313,7 +1460,7 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 		outii = vi->outii[*outiiindex];
 		*outiiindex += 1;
 
-		PEPUTP( arg, ELEMENT_IMAGE, outii );
+		PEPUTP( arg, ELEMENT_MANAGED, outii );
 		break;
 	}
 
@@ -1339,6 +1486,11 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 		break;
 	}
 
+	case VIPS_GVALUE:
+		if( !heap_gvalue_to_ip( (GValue *) obj, arg ) )
+			reduce_throw( vi->rc );
+		break;
+
 	case VIPS_DOUBLEVEC:
 	case VIPS_IMAGEVEC:
 	default:
@@ -1350,6 +1502,107 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 #endif /*DEBUG*/
 }
 
+/* VIPS types -> a string buffer. Used to update image history. Yuk! Should be
+ * a method on object type.
+ */
+static void
+vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
+{
+	im_object obj = vi->vargv[i];
+	im_type_desc *vips = vi->fn->argv[i].desc;
+	VipsArgumentType vt = vips_type_find( vips->type );
+
+	switch( vt ) {
+	case VIPS_DOUBLE:
+		buf_appendf( buf, "%g", *((double*)obj) );
+		break;
+
+	case VIPS_INT:
+		buf_appendf( buf, "%d", *((int*)obj) );
+		break;
+
+	case VIPS_COMPLEX:
+		buf_appendf( buf, "(%g, %g)", 
+			((double*)obj)[0], ((double*)obj)[1] );
+		break;
+
+	case VIPS_STRING:
+		buf_appendf( buf, "\"%s\"", (char*)obj );
+		break;
+
+	case VIPS_IMAGE:
+		buf_appendf( buf, "%s", NN( ((IMAGE*)obj)->filename ) );
+		break;
+
+	case VIPS_DMASK:
+	case VIPS_IMASK:
+	{
+		im_mask_object *mo = obj;
+
+		buf_appendf( buf, "%s", NN( mo->name ) );
+		break;
+	}
+
+	case VIPS_DOUBLEVEC:
+	{
+		im_doublevec_object *v = (im_doublevec_object *) obj;
+		int j;
+
+		buf_appendf( buf, "\"" );
+		for( j = 0; j < v->n; j++ )
+			buf_appendf( buf, "%g ", v->vec[j] );
+		buf_appendf( buf, "\"" );
+
+		break;
+	}
+
+	case VIPS_INTVEC:
+	{
+		im_intvec_object *v = (im_intvec_object *) obj;
+		int j;
+
+		buf_appendf( buf, "\"" );
+		for( j = 0; j < v->n; j++ )
+			buf_appendf( buf, "%d ", v->vec[j] );
+		buf_appendf( buf, "\"" );
+
+		break;
+	}
+
+	case VIPS_IMAGEVEC:
+	{
+		im_imagevec_object *v = (im_imagevec_object *) obj;
+		int j;
+
+		buf_appendf( buf, "\"" );
+		for( j = 0; j < v->n; j++ )
+			buf_appendf( buf, "%s ", v->vec[j]->filename );
+		buf_appendf( buf, "\"" );
+
+		break;
+	}
+
+	case VIPS_GVALUE:
+	{
+		GValue *value = (GValue *) obj;
+
+		buf_appendgv( buf, value );
+
+		break;
+	}
+
+	case VIPS_NONE:
+		if( strcmp( vips->type, IM_TYPE_DISPLAY ) == 0 ) 
+			/* Just assume sRGB.
+			 */
+			buf_appendf( buf, "sRGB" );
+		break;
+
+	default:
+		assert( FALSE );
+	}
+}
+
 /* Sort out the input images. Can we do this with a LUT?
  */
 static void
@@ -1374,7 +1627,7 @@ vips_gather( VipsInfo *vi )
 				(im_imagevec_object *) vi->vargv[i];
 
 			for( j = 0; j < iv->n; j++ ) {
-				vi->inii[vi->ninii] = (Imageinfo *) iv->vec[j];
+				vi->inii[vi->ninii] = IMAGEINFO( iv->vec[j] );
 				vi->ninii += 1;
 			}
 		}
@@ -1417,7 +1670,8 @@ vips_gather( VipsInfo *vi )
 			 * in the vector.
 			 */
 			for( j = 0; j < iv->n; j++ ) {
-				Imageinfo *ii = (Imageinfo *) iv->vec[j];
+				Imageinfo *ii = IMAGEINFO( iv->vec[j] );
+
 				if( !(iv->vec[j] = imageinfo_get( 
 					vi->use_lut, ii )) )
 					reduce_throw( vi->rc );
@@ -1435,8 +1689,7 @@ vips_build_output( VipsInfo *vi, int i )
 	VipsArgumentType vt = vips_type_find( ty->type );
 	char tname[FILENAME_MAX];
 
-	/* Provide output objects for the function to write
-	 * to.
+	/* Provide output objects for the function to write to.
 	 */
 	switch( vt ) {
 	case VIPS_DOUBLE:
@@ -1464,10 +1717,20 @@ vips_build_output( VipsInfo *vi, int i )
 		break;
 	}
 
+	case VIPS_GVALUE:
+	{
+		GValue *value = vi->vargv[i];
+
+		memset( value, 0, sizeof( GValue ) );
+
+		break;
+	}
+
 	case VIPS_DOUBLEVEC:
+	case VIPS_INTVEC:
 		/* 
 
-			FIXME ... add output doublevec
+			FIXME ... add these as output args
 
 		 */
 	default:
@@ -1528,17 +1791,71 @@ vips_fill_spine( VipsInfo *vi, HeapNode **arg )
 	}
 }
 
+static gboolean
+vips_build_argv( VipsInfo *vi, char **argv )
+{
+	int i;
+
+	for( i = 0; i < vi->fn->argc; i++ ) {
+		BufInfo buf;
+		char txt[512];
+
+		buf_init_static( &buf, txt, 512 );
+		vips_tobuf( vi, i, &buf );
+		if( !(argv[i] = im_strdup( NULL, buf_all( &buf ) )) )
+			return( FALSE );
+	}
+
+#ifdef DEBUG
+	printf( "vips_build_argv: argv for %s is:\n  ", vi->fn->name );
+	for( i = 0; i < vi->fn->argc; i++ )
+		printf( "%s ", NN( argv[i] ) );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	return( TRUE );
+}
+
+static void
+vips_free_argv( int argc, char **argv )
+{
+	int i;
+
+	for( i = 0; i < argc; i++ ) {
+		IM_FREE( argv[i] );
+	}
+	IM_FREE( argv );
+}
+
 /* Update the VIPS hist for all output images.
  */
 static void
 vips_update_hist( VipsInfo *vi )
 {
+	int argc = vi->fn->argc;
+	char **argv;
 	int i;
 
 #ifdef DEBUG
-	printf( "vips_update_history: %s\n", vi->name );
+	printf( "vips_update_hist: %s\n", vi->name );
 #endif /*DEBUG*/
 
+	/* No output images? Nothing to do.
+	 */
+	if( vi->nires == 0 )
+		return;
+
+	/* Build an argv for this call. +1 for NULL termination.
+	 */
+	if( !(argv = IM_ARRAY( NULL, argc + 1, char * )) )
+		return;
+	for( i = 0; i < argc + 1; i++ )
+		argv[i] = NULL;
+	if( !vips_build_argv( vi, argv ) ) {
+		vips_free_argv( argc, argv );
+		return;
+	}
+
 	for( i = 0; i < vi->nres; i++ ) {
 		int j = vi->outpos[i];
 		im_type_desc *ty = vi->fn->argv[j].desc;
@@ -1548,20 +1865,18 @@ vips_update_hist( VipsInfo *vi )
 		 */
 		if( vt == VIPS_IMAGE ) {
 #ifdef DEBUG
-			printf( "vips_update_history: adding to arg %d\n", j );
+			printf( "vips_update_hist: adding to arg %d\n", j );
 #endif /*DEBUG*/
 
-			/* Can't do args! Groan.
-			 */
-			im_updatehist( vi->vargv[j], 1, &vi->fn->name );
+			im_updatehist( vi->vargv[j], vi->fn->name, argc, argv );
 		}
 	}
+
+	vips_free_argv( argc, argv );
 }
 
-/* Write the results back to the heap.
- */
-static void
-vips_write_result( VipsInfo *vi, PElement *out )
+static void *
+vips_write_result_sub( Reduce *rc, PElement *safe, VipsInfo *vi, PElement *out )
 {
 	int outiiindex;
 
@@ -1574,23 +1889,45 @@ vips_write_result( VipsInfo *vi, PElement *out )
 	if( vi->nres == 1 )
 		/* Single result.
 		 */
-		vips_toip( vi, vi->outpos[0], &outiiindex, out );
+		vips_toip( vi, vi->outpos[0], &outiiindex, safe );
 	else {
 		/* Have to build a list of results.
 		 */
-		PElement list = *out;
+		PElement list;
 		PElement t;
 		int i;
 
+		list = *safe;
 		heap_list_init( &list );
 		for( i = 0; i < vi->nres; i++ ) {
 			if( !heap_list_add( vi->rc->heap, &list, &t ) )
-				reduce_throw( vi->rc );
+				return( out );
 			vips_toip( vi, vi->outpos[i], &outiiindex, &t );
 
 			(void) heap_list_next( &list );
 		}
 	}
+
+	/* Now overwrite out with safe.
+	 */
+	PEPUTPE( out, safe );
+
+	return( NULL );
+}
+
+/* Write the results back to the heap. We have to so this in two stages:
+ * build the output object linked off a new managed Element, then once it's
+ * built, overwrite our output
+ */
+static gboolean
+vips_write_result( VipsInfo *vi, PElement *out )
+{
+	if( reduce_safe_pointer( vi->rc, 
+		(reduce_safe_pointer_fn) vips_write_result_sub, 
+		vi, out, NULL, NULL ) )
+		return( FALSE );
+
+	return( TRUE );
 }
 
 /* Loop over the output IMAGEs, transforming them into Imageinfo.
@@ -1625,7 +1962,8 @@ vips_wrap_output( VipsInfo *vi )
 
 		/* This output ii depends upon all of the input images.
 		 */
-		imageinfo_sub_add_all( outii, vi->ninii, vi->inii );
+		managed_sub_add_all( MANAGED( outii ), 
+			vi->ninii, (Managed **) vi->inii );
 
 		/* Junk the pointer in vargv to stop im_close() on vips end.
 		 */
@@ -1685,6 +2023,16 @@ vips_fillva( VipsInfo *vi, va_list ap )
 			if( trace_flags & TRACE_VIPS ) 
 				buf_appendf( trace_current(), "%d ", v );
 		}
+		else if( vt == VIPS_GVALUE ) {
+			GValue *value = va_arg( ap, GValue * );
+
+			vi->vargv[i] = value;
+
+			if( trace_flags & TRACE_VIPS ) {
+				buf_appendgv( trace_current(), value );
+				buf_appends( trace_current(), " " );
+			}
+		}
 		else if( vt == VIPS_IMAGE ) {
 			Imageinfo *ii = va_arg( ap, Imageinfo * );
 
@@ -1708,9 +2056,9 @@ vips_fillva( VipsInfo *vi, va_list ap )
 	    	}
 		else if( vt == VIPS_DOUBLEVEC ) {
 			int n = va_arg( ap, int );
-			double *buf = va_arg( ap, double * );
+			double *vec = va_arg( ap, double * );
 
-			if( vips_make_doublevec( vi->vargv[i], n, buf ) )
+			if( vips_make_doublevec( vi->vargv[i], n, vec ) )
 				reduce_throw( vi->rc );
 			if( trace_flags & TRACE_VIPS ) {
 				BufInfo *buf = trace_current();
@@ -1719,10 +2067,15 @@ vips_fillva( VipsInfo *vi, va_list ap )
 				buf_appendf( buf, "<" );
 				buf_appendf( buf, _( "doublevec" ) );
 				for( i = 0; i < n; i++ )
-					buf_appendf( buf, " %g", buf[i] );
+					buf_appendf( buf, " %g", vec[i] );
 				buf_appends( buf, "> " );
 			}
 		}
+		/* 
+
+			FIXME ... add intvec perhaps
+
+		 */
 		else if( vt == VIPS_IMAGEVEC ) {
 			int n = va_arg( ap, int );
 			Imageinfo **vec = va_arg( ap, Imageinfo ** );
@@ -1752,7 +2105,7 @@ vips_fillva( VipsInfo *vi, va_list ap )
 }
 
 static void
-vips_call( VipsInfo *vi, PElement *out )
+vips_dispatch( VipsInfo *vi, PElement *out )
 {
 	Reduce *rc = vi->rc;
 	VipsInfo *old_vi;
@@ -1774,17 +2127,34 @@ vips_call( VipsInfo *vi, PElement *out )
 			buf_appendf( trace_current(), "(from cache) " );
 
 #ifdef DEBUG_HISTORY
-		printf( "vips_call: found %s in history\n", vi->name );
+		printf( "vips_dispatch: found %s in history\n", vi->name );
 #endif /*DEBUG_HISTORY*/
 	}
 	else {
 		/* No: call function and save result in history.
 		 */
+		int result;
+
+#ifdef DEBUG_TIME
+		static GTimer *timer = NULL;
+
+		if( !timer )
+			timer = g_timer_new();
+		g_timer_reset( timer );
+#endif /*DEBUG_TIME*/
+
 #ifdef DEBUG_HISTORY_MISS
-		printf( "vips_call: calling %s\n", vi->name );
+		printf( "vips_dispatch: calling %s\n", vi->name );
 #endif /*DEBUG_HISTORY_MISS*/
 
-		if( vi->fn->disp( vi->vargv ) ) {
+		result = vi->fn->disp( vi->vargv );
+
+#ifdef DEBUG_TIME
+		printf( "vips_dispatch: %s - %g\n", 
+			vi->name, g_timer_elapsed( timer, NULL ) );
+#endif /*DEBUG_TIME*/
+
+		if( result ) {
 			vips_error_fn_vips( vi );
 			vips_destroy( vi );
 			reduce_throw( rc );
@@ -1798,9 +2168,12 @@ vips_call( VipsInfo *vi, PElement *out )
 
 	/* Write result back to heap. 
 	 */
-	vips_write_result( vi, out );
+	if( !vips_write_result( vi, out ) ) {
+		vips_destroy( vi );
+		reduce_throw( rc );
+	}
 
-	/* Can we cache this operation? Memorise or destroy.
+	/* Can we cache this operation? Memoise or destroy.
 	 */
 	if( vi->fn->flags & IM_FN_NOCACHE )
 		vips_destroy( vi );
@@ -1817,7 +2190,7 @@ vips_spine_sub( VipsInfo *vi, PElement *out, HeapNode **arg )
 
 	vips_fill_spine( vi, arg );
 
-	vips_call( vi, out );
+	vips_dispatch( vi, out );
 
 	REDUCE_CATCH_STOP;
 
@@ -1914,7 +2287,7 @@ vipsva_sub( VipsInfo *vi, PElement *out, va_list ap )
 	if( trace_flags & TRACE_VIPS ) 
 		buf_appends( trace_current(), " ->\n" ); 
 
-	vips_call( vi, out );
+	vips_dispatch( vi, out );
 
 	REDUCE_CATCH_STOP;
 
