@@ -198,7 +198,27 @@ workspace_selected_names_sub( Row *row, NamesInfo *names )
 {
 	if( !names->first )
 		buf_appends( names->buf, names->separator );
-	row_qualified_name( row, names->buf );
+
+	/* Hack: if this is a matrix with selected cells, use an extract to
+	 * get those cells out. We should really have a row method for this I
+	 * guess :-(
+	 */
+	if( row->child_rhs && row->child_rhs->graphic &&
+		IS_MATRIX( row->child_rhs->graphic ) &&
+		MATRIX( row->child_rhs->graphic )->selected ) {
+		Matrix *matrix = MATRIX( row->child_rhs->graphic );
+
+		buf_appends( names->buf, "(" );
+		row_qualified_name( row, names->buf );
+		buf_appendf( names->buf, ".extract %d %d %d %d)",
+			matrix->range.left, 
+			matrix->range.top, 
+			matrix->range.width, 
+			matrix->range.height );
+	}
+	else 
+		row_qualified_name( row, names->buf );
+
 	names->first = FALSE;
 
 	return( NULL );
@@ -253,7 +273,7 @@ workspace_is_one_empty( Workspace *ws )
 		return( NULL );
 
 	col = COLUMN( children->data );
-	if( !column_isempty( col ) )
+	if( !column_is_empty( col ) )
 		return( NULL );
 
 	return( col );
@@ -311,7 +331,7 @@ workspace_column_name_new( Workspace *ws, xmlNode *columns )
 			for( i = columns; i; i = i->next ) {
 				char name[MAX_STRSIZE];
 
-				if( strcmp( i->name, "Column" ) == 0 &&
+				if( strcmp( (char *) i->name, "Column" ) == 0 &&
 					get_sprop( i, "name", 
 						name, MAX_STRSIZE ) )
 					if( strcmp( name, buf ) == 0 )
@@ -428,7 +448,7 @@ workspace_add_def( Workspace *ws, const char *str )
 	symbol_made( sym );
 	filemodel_set_modified( FILEMODEL( ws ), TRUE );
 
-	if( symbol_recalculate( sym ) ) {
+	if( symbol_recalculate_check( sym ) ) {
 		/* Eval error.
 		 */
 		expr_error_get( sym->expr );
@@ -441,7 +461,7 @@ workspace_add_def( Workspace *ws, const char *str )
 
 	/* Jump to column containing object.
 	 */
-	model_scrollto( MODEL( col ) );
+	model_scrollto( MODEL( col ), MODEL_SCROLL_BOTTOM );
 
 	return( sym );
 }
@@ -648,7 +668,7 @@ workspace_auto_recover_load( iWindow *iwnd,
 	void *client, iWindowNotifyFn nfn, void *sys )
 {
 	char *filename = (char *) client;
-	Mainw *mainw = MAINW( idialog_get_root( GTK_WIDGET( iwnd ) ) );
+	Mainw *mainw = MAINW( iwindow_get_root_noparent( GTK_WIDGET( iwnd ) ) );
 	Filemodel *filemodel;
 
 	/* Load ws file.
@@ -690,7 +710,7 @@ workspace_auto_recover( GtkWidget *parent )
 		return;
 	}
 
-	if( (p = strrchr( buf, IM_DIR_SEP )) )
+	if( (p = strrchr( buf, G_DIR_SEPARATOR )) )
 		*p = '\0';
 
 	date_sofar = -1;
@@ -736,7 +756,6 @@ workspace_dispose( GObject *gobject )
 
 	ws = WORKSPACE( gobject );
 
-	ws->last_column = NULL;
 	IDESTROY( ws->sym );
 	IM_FREEF( g_source_remove, ws->auto_save_timeout );
 
@@ -799,6 +818,10 @@ workspace_link( Workspace *ws, Workspacegroup *wsg, const char *name )
 {
 	Symbol *sym;
 
+#ifdef DEBUG
+	printf( "workspace_link: naming ws as %s\n", name );
+#endif /*DEBUG*/
+
 	sym = symbol_new_defining( wsg->sym->expr->compile, name );
 
 	ws->sym = sym;
@@ -807,6 +830,7 @@ workspace_link( Workspace *ws, Workspacegroup *wsg, const char *name )
 	sym->expr = expr_new( sym );
 	(void) compile_new( sym->expr );
 	symbol_made( sym );
+	iobject_set( IOBJECT( ws ), name, NULL );
 }
 
 static const char *
@@ -840,6 +864,12 @@ char_to_workspacemode( const char *mode )
 		return( (WorkspaceMode) -1 );
 }
 
+static View *
+workspace_view_new( Model *model, View *parent )
+{
+	return( workspaceview_new() );
+}
+
 static gboolean
 workspace_load( Model *model, 
 	ModelLoadState *state, Model *parent, xmlNode *xnode )
@@ -866,6 +896,16 @@ workspace_load( Model *model,
 	(void) get_dprop( xnode, "scale", &ws->scale );
 	(void) get_dprop( xnode, "offset", &ws->offset );
 
+	(void) get_iprop( xnode, "window_width", &ws->window_width );
+	(void) get_iprop( xnode, "window_height", &ws->window_height );
+
+	if( get_sprop( xnode, "name", buf, 256 ) ) {
+		IM_SETSTR( IOBJECT( ws )->name, buf );
+	}
+	if( get_sprop( xnode, "caption", buf, 256 ) ) {
+		IM_SETSTR( IOBJECT( ws )->caption, buf );
+	}
+
 	if( !MODEL_CLASS( parent_class )->load( model, state, parent, xnode ) )
 		return( FALSE );
 
@@ -882,8 +922,12 @@ workspace_save( Model *model, xmlNode *xnode )
 		return( NULL );
 
 	if( !set_prop( xthis, "view", workspacemode_to_char( ws->mode ) ) ||
-		!set_prop( xthis, "scale", "%g", ws->scale ) ||
-		!set_prop( xthis, "offset", "%g", ws->offset ) )
+		!set_dprop( xthis, "scale", ws->scale ) ||
+		!set_dprop( xthis, "offset", ws->offset ) ||
+		!set_prop( xthis, "window_width", "%d", ws->window_width ) ||
+		!set_prop( xthis, "window_height", "%d", ws->window_height ) ||
+		!set_sprop( xthis, "name", IOBJECT( ws )->name ) ||
+		!set_sprop( xthis, "caption", IOBJECT( ws )->caption ) ) 
 		return( NULL );
 
 	return( xthis );
@@ -909,7 +953,7 @@ workspace_rename_row_node( ModelLoadState *state, Column *col, xmlNode *xnode )
 {
 	char name[MAX_STRSIZE];
 
-	if( strcmp( xnode->name, "Row" ) == 0 &&
+	if( strcmp( (char *) xnode->name, "Row" ) == 0 &&
 		get_sprop( xnode, "name", name, MAX_STRSIZE ) ) {
 		char *new_name;
 
@@ -928,7 +972,7 @@ workspace_rename_column_node( Workspace *ws,
 {
 	char name[MAX_STRSIZE];
 
-	if( strcmp( xnode->name, "Column" ) == 0 &&
+	if( strcmp( (char *) xnode->name, "Column" ) == 0 &&
 		get_sprop( xnode, "name", name, MAX_STRSIZE ) &&
 		icontainer_map( ICONTAINER( ws ), 
 			(icontainer_map_fn) iobject_test_name, name, NULL ) ) {
@@ -998,10 +1042,8 @@ workspace_load_compat( Workspace *ws, int major, int minor )
 	}
 	path_free( path );
 
-	/* Recalc to build all menus/toolitem etc. before we make the mainw
-	 * for this workspace.
-	 */
-	symbol_recalculate_all_force();
+	ws->compat_major = major;
+	ws->compat_minor = minor;
 
 	return( TRUE );
 }
@@ -1043,15 +1085,28 @@ workspace_top_load( Filemodel *filemodel,
 		 */
 		if( filemodel->major != MAJOR_VERSION || 
 			filemodel->minor != MINOR_VERSION ) {
-			if( !workspace_load_compat( ws, 
+			if( existsf( "$VIPSHOME/share/" PACKAGE "/compat/%d.%d",
 				filemodel->major, filemodel->minor ) ) 
-				return( FALSE );
-			ws->compatibility = TRUE;
+				if( !workspace_load_compat( ws, 
+					filemodel->major, filemodel->minor ) ) 
+					return( FALSE );
+
+			/* Do we need broken region handling?
+			 */
+			if( filemodel->major == 7 || 
+				filemodel->minor == 8 ) 
+				ws->compatibility = TRUE;
 		}
 
 		if( model_load( MODEL( ws ), state, parent, xnode ) )
 			return( FALSE );
 
+		/* The model_load() will set the name from the name saved in
+		 * the XML. We want to override that with the name from the
+		 * filename.
+		 */
+		iobject_set( IOBJECT( ws ), name, NULL );
+
 		break;
 
 	case WORKSPACE_LOAD_COLUMNS:
@@ -1168,7 +1223,7 @@ workspace_class_init( WorkspaceClass *class )
 	icontainer_class->child_add = workspace_child_add;
 	icontainer_class->child_remove = workspace_child_remove;
 
-	model_class->view_new = workspaceview_new;
+	model_class->view_new = workspace_view_new;
 	model_class->load = workspace_load;
 	model_class->save = workspace_save;
 	model_class->empty = workspace_empty;
@@ -1201,27 +1256,27 @@ workspace_init( Workspace *ws )
 	ws->selected = NULL;
 	ws->errors = NULL;
         ws->mode = WORKSPACE_MODE_REGULAR;
+
 	ws->compatibility = FALSE;
+	ws->compat_major = 0;
+	ws->compat_minor = 0;
 
 	ws->load_type = WORKSPACE_LOAD_TOP;
 	ws->save_type = WORKSPACE_SAVE_ALL;
 
-	ws->last_symbol = NULL;
-	ws->last_column = NULL;
-	ws->print_last = FALSE;
-
 	ws->area.left = 0;
 	ws->area.top = 0;
 	ws->area.width = 0;
 	ws->area.height = 0;
 	ws->vp = ws->area;
+	ws->window_width = 0;
+	ws->window_height = 0;
 
 	ws->auto_save_timeout = 0;
 
 	ws->status = NULL;
-	im_snprintf( txt, 256, 
-		_( "%s: (c) 2005 The National Gallery" ), PACKAGE );
-	SETSTR( ws->status, txt );
+	im_snprintf( txt, 256, _( NIP_COPYRIGHT ), PACKAGE );
+	IM_SETSTR( ws->status, txt );
 
 	ws->scale = 1.0;
 	ws->offset = 0.0;
@@ -1273,7 +1328,6 @@ workspace_new( Workspacegroup *wsg, const char *name )
 	}
 
 	ws = WORKSPACE( g_object_new( TYPE_WORKSPACE, NULL ) );
-	iobject_set( IOBJECT( ws ), name, NULL );
 	icontainer_child_add( ICONTAINER( wsg ), ICONTAINER( ws ), -1 );
 	workspace_link( ws, wsg, name );
 	(void) workspace_column_pick( ws );
@@ -1433,7 +1487,7 @@ workspace_add_action( Workspace *ws,
 
 		buf_appends( &buf, " " );
 		workspace_selected_names( ws, &buf, " " );
-		if( buf_isfull( &buf ) ) {
+		if( buf_is_full( &buf ) ) {
 			error_top( _( "Overflow error." ) );
 			error_sub( _( "Too many names selected." ) );
 			return( FALSE );
@@ -1507,7 +1561,7 @@ workspace_selected_recalc( Workspace *ws )
 
 	/* Recalc even if autorecomp is off.
 	 */
-	symbol_recalculate_all_force();
+	symbol_recalculate_all_force( FALSE );
 
 	workspace_deselect_all( ws );
 
@@ -1657,7 +1711,7 @@ workspace_ungroup_row( Row *row )
 	int length;
 	int i;
 
-	if( !heap_isinstanceof( CLASS_GROUP, root, &result ) )
+	if( !heap_is_instanceof( CLASS_GROUP, root, &result ) )
 		return( row );
 	if( result ) {
 		if( !class_get_member( root, MEMBER_VALUE, NULL, &value ) || 
@@ -1670,7 +1724,7 @@ workspace_ungroup_row( Row *row )
 				return( row );
 	}
 	else {
-		if( !heap_islist( root, &result ) )
+		if( !heap_is_list( root, &result ) )
 			return( row );
 		if( result ) {
 			if( (length = heap_list_length( root )) < 0 ) 
@@ -1737,7 +1791,7 @@ workspace_set_status( Workspace *ws, const char *fmt, ... )
 	(void) im_vsnprintf( buf, 256, fmt, ap );
 	va_end( ap );
 
-	SETSTR( ws->status, buf );
+	IM_SETSTR( ws->status, buf );
 	iobject_changed( IOBJECT( ws ) );
 }
 
