@@ -39,6 +39,14 @@
 
 static ViewClass *parent_class = NULL;
 
+/* Params for "Align Columns" function.
+ */
+static const int workspaceview_layout_snap_threshold = 20;
+static const int workspaceview_layout_hspacing = 10;
+static const int workspaceview_layout_vspacing = 30;
+static const int workspaceview_layout_left = 3;
+static const int workspaceview_layout_top = 3;
+
 static void
 workspaceview_scroll_to( Workspaceview *wview, int x, int y )
 {
@@ -85,6 +93,8 @@ workspaceview_scroll( Workspaceview *wview, int x, int y, int w, int h )
         ny = vadj->value;
         if( y + h > IM_RECT_BOTTOM( vp ) )
                 ny = IM_MAX( 0, (y + h) - vp->height );
+        if( y < ny )
+                ny = y;
 
 #ifdef DEBUG
         printf( "workspaceview_scroll: x=%d, y=%d, w=%d, h=%d, "
@@ -148,6 +158,48 @@ workspaceview_watch_changed_cb( Watchgroup *watchgroup, Watch *watch,
 		}
 }
 
+/* Scroll events ... handle mousewheel shortcuts here. Do this ourselves
+ * (rather than just relying on the scrollbars) so we can do shift + wheel == 
+ * left/right.
+ */
+static gboolean
+workspaceview_scroll_event_cb( GtkWidget *widget, 
+	GdkEventScroll *ev, Workspaceview *wview )
+{
+	gboolean handled = FALSE;
+
+	/* Gimp uses page_incr / 4 I think, but then scroll speed varies with
+	 * window size, which is pretty odd. Just use a constant.
+	 */
+	const int incr = 50;
+
+	if( ev->direction == GDK_SCROLL_UP || 
+		ev->direction == GDK_SCROLL_DOWN ) {
+		if( ev->state & GDK_SHIFT_MASK ) {
+			if( ev->direction == GDK_SCROLL_UP )
+				workspaceview_scroll_to( wview, 
+					wview->vp.left + incr, wview->vp.top );
+			else
+				workspaceview_scroll_to( wview, 
+					wview->vp.left - incr, wview->vp.top );
+
+			handled = TRUE;
+		}
+		else {
+			if( ev->direction == GDK_SCROLL_UP )
+				workspaceview_scroll_to( wview, 
+					wview->vp.left, wview->vp.top - incr );
+			else
+				workspaceview_scroll_to( wview, 
+					wview->vp.left, wview->vp.top + incr );
+
+			handled = TRUE;
+		}
+	}
+
+	return( handled );
+}
+
 static void
 workspaceview_realize_cb( GtkWidget *wid, Workspaceview *wview )
 {
@@ -303,13 +355,6 @@ static void
 workspaceview_scroll_adjustment_cb( GtkAdjustment *adj, Workspaceview *wview )
 {
 	workspaceview_scroll_update( wview );
-
-	/* Not really the place to do this, but seems harmless. Fix the
-	 * page_increment, since we want the scroll wheel to always scroll at
-	 * the same speed no matter what the window size is.
-	 */
-	adj->step_increment = 10;
-	adj->page_increment = 100;
 }
 
 /* Timer callback for background scroll.
@@ -485,7 +530,7 @@ workspaceview_child_size_cb( Columnview *cview,
 
 	assert( IS_WORKSPACEVIEW( wview ) );
 
-	/* Compute a new boundung box for our children.
+	/* Compute a new bounding box for our children.
 	 */
 	wview->bounding.left = 0;
 	wview->bounding.top = 0;
@@ -525,13 +570,65 @@ workspaceview_child_size_cb( Columnview *cview,
 static void
 workspaceview_child_add( View *parent, View *child )
 {
-	assert( IS_WORKSPACEVIEW( parent ) );
-	assert( IS_COLUMNVIEW( child ) );
+	Columnview *cview = COLUMNVIEW( child );
+	Column *column = COLUMN( VOBJECT( cview )->iobject );
+	Workspaceview *wview = WORKSPACEVIEW( parent );
 
 	gtk_signal_connect( GTK_OBJECT( child ), "size_allocate", 
 		GTK_SIGNAL_FUNC( workspaceview_child_size_cb ), parent );
 
 	VIEW_CLASS( parent_class )->child_add( parent, child );
+
+	/* Pick start xy pos. 
+	 */
+        workspaceview_pick_xy( wview, &column->x, &column->y );
+	gtk_fixed_put( GTK_FIXED( wview->fixed ),
+		GTK_WIDGET( cview ), column->x, column->y );
+	cview->lx = column->x;
+	cview->ly = column->y;
+	wview->front = cview;
+}
+
+static void
+workspaceview_child_remove( View *parent, View *child )
+{
+	Columnview *cview = COLUMNVIEW( child );
+	Workspaceview *wview = WORKSPACEVIEW( parent );
+
+	if( wview->front == cview )
+		wview->front = NULL;
+
+	VIEW_CLASS( parent_class )->child_remove( parent, child );
+}
+
+static void
+workspaceview_child_position( View *parent, View *child )
+{
+	Workspaceview *wview = WORKSPACEVIEW( parent );
+	Columnview *cview = COLUMNVIEW( child );
+
+	gtk_fixed_move( GTK_FIXED( wview->fixed ),
+		GTK_WIDGET( cview ), cview->lx, cview->ly );
+
+	VIEW_CLASS( parent_class )->child_position( parent, child );
+}
+
+static void
+workspaceview_child_front( View *parent, View *child )
+{
+	Workspaceview *wview = WORKSPACEVIEW( parent );
+	Columnview *cview = COLUMNVIEW( child );
+
+	if( wview->front != cview ) {
+		gtk_widget_ref( GTK_WIDGET( cview ) );
+		gtk_container_remove( GTK_CONTAINER( wview->fixed ),
+			GTK_WIDGET( cview ) );
+		gtk_fixed_put( GTK_FIXED( wview->fixed ),
+			GTK_WIDGET( cview ), cview->lx, cview->ly );
+		gtk_widget_unref( GTK_WIDGET( cview ) );
+
+		wview->front = cview;
+	}
 }
 
 static void 
@@ -572,12 +669,6 @@ typedef struct _WorkspaceLayout {
 	Columnview *cview;
 } WorkspaceLayout;
 
-static const int workspaceview_layout_snap_threshold = 20;
-static const int workspaceview_layout_hspacing = 10;
-static const int workspaceview_layout_vspacing = 15;
-static const int workspaceview_layout_left = 3;
-static const int workspaceview_layout_top = 3;
-
 static void *
 workspaceview_layout_add( View *view, WorkspaceLayout *layout )
 {
@@ -632,6 +723,7 @@ workspaceview_layout_set_pos( Columnview *cview, WorkspaceLayout *layout )
 		layout->area.width = GTK_WIDGET( cview )->allocation.width;
 
 	iobject_changed( IOBJECT( column ) );
+	filemodel_set_modified( FILEMODEL( column->ws ), TRUE );
 
 	return( NULL );
 }
@@ -676,7 +768,7 @@ workspaceview_layout_loop( WorkspaceLayout *layout )
 
 /* Autolayout ... try to rearrange columns so they don't overlap. 
 
-	Algorithm:
+	Strategy:
 
 	search for top-left-most column
 
@@ -726,6 +818,9 @@ workspaceview_class_init( WorkspaceviewClass *class )
 	vobject_class->refresh = workspaceview_refresh;
 
 	view_class->child_add = workspaceview_child_add;
+	view_class->child_remove = workspaceview_child_remove;
+	view_class->child_position = workspaceview_child_position;
+	view_class->child_front = workspaceview_child_front;
 	view_class->layout = workspaceview_layout;
 }
 
@@ -783,7 +878,8 @@ workspaceview_init( Workspaceview *wview )
 		"watch_changed",
 		G_CALLBACK( workspaceview_watch_changed_cb ), wview );
 
-	/* Size set in refresh.
+	/* Ask for our own window so we can spot events on the window 
+	 * background.
 	 */
 	wview->fixed = gtk_fixed_new();
 	gtk_fixed_set_has_window( GTK_FIXED( wview->fixed ), TRUE );
@@ -795,6 +891,8 @@ workspaceview_init( Workspaceview *wview )
 	gtk_viewport_set_shadow_type( 
 		GTK_VIEWPORT( GTK_BIN( wview->window )->child ), 
 		GTK_SHADOW_NONE );
+	gtk_signal_connect( GTK_OBJECT( wview->window ), "scroll_event",
+		GTK_SIGNAL_FUNC( workspaceview_scroll_event_cb ), wview );
 	gtk_signal_connect( GTK_OBJECT( wview->fixed ), "realize", 
 		GTK_SIGNAL_FUNC( workspaceview_realize_cb ), wview );
         gtk_signal_connect( GTK_OBJECT( wview->fixed ), "event",
@@ -833,10 +931,10 @@ workspaceview_init( Workspaceview *wview )
 GtkType
 workspaceview_get_type( void )
 {
-	static GtkType workspaceview_type = 0;
+	static GtkType type = 0;
 
-	if (!workspaceview_type) {
-		static const GtkTypeInfo wviewt = {
+	if( !type ) {
+		static const GtkTypeInfo info = {
 			"Workspaceview",
 			sizeof( Workspaceview ),
 			sizeof( WorkspaceviewClass ),
@@ -847,10 +945,10 @@ workspaceview_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		workspaceview_type = gtk_type_unique( TYPE_VIEW, &wviewt );
+		type = gtk_type_unique( TYPE_VIEW, &info );
 	}
 
-	return( workspaceview_type );
+	return( type );
 }
 
 View *
@@ -885,3 +983,4 @@ workspaceview_pick_xy( Workspaceview *wview, int *x, int *y )
 	if( wview->next_y > 200 )
 		wview->next_y = 3;
 }
+
